/*
 * gani : The Realtek RTL816x Gigabit Ethernet Driver for Solaris
 *
 * Copyright (c) 2004-2013 Masayuki Murayama.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the author nor the names of its contributors may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#pragma	ident	"%Z%%M% %I%     %E%"

/*
 * TODO
 *	offloading seem to cause tx timeout
 *	tx hang on 8169
 *	rx hang on 8169
 *
 *  BUGS
 */

/* ======================================================= */

/*
 * Solaris system header files and macros
 */
#include <sys/types.h>
#include <sys/conf.h>
#include <sys/debug.h>
#include <sys/kmem.h>
#include <sys/modctl.h>
#include <sys/errno.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/byteorder.h>
#include <sys/ethernet.h>
#include <sys/pci.h>

#include "gem_mii.h"
#include "gem.h"
#include "rtl8169reg.h"

char	ident[] = "rtl816x nic driver v" VERSION;

/*
 * Useful macros
 */
#define	ROUNDUP2(x, y)	(((x)+(y)-1) & ~((y)-1))
#define	ETHERHEADERL	(ETHERMAX - ETHERMTU)
#define	VTAG_SIZE	4
#define	RMS_PCI(m)	(ETHERHEADERL + VTAG_SIZE + (m) + ETHERFCSL + 4)
#define	RMS_PCIE_GIGA(m)	\
		(ETHERHEADERL + VTAG_SIZE + (m) + ETHERFCSL + 1)

#define	RMS_PCIE_FAST(m)	\
	(((m) == R8101_11) ||	\
	((m) == R8101_12) ||	\
	((m) == R8101_13) ||	\
	((m) == R8101_14) ||	\
	((m) == R8101_15) ||	\
	((m) == R8101_16) ? 0x5f3 : 0x5ef)

#define	ONESEC		(drv_usectohz(1*1000000))

#define	DESC_BASE_ALIGN	256	/* it should be defined in rtl8169reg.h */
#define	DESC_MAX_SIZE	(16*1024)	/* in byte */

#define	TALLY_OFFSET(dp)	\
	(((intptr_t)(dp)->io_area) - ((intptr_t)(dp)->rx_ring))
#define	TALLY_VADDR(dp)		((void *)((dp)->io_area))
#define	TALLY_DMA_ADDR(dp)	((dp)->io_area_dma)

#ifdef MAP_MEM
#define	FLSHB(dp, reg)	(void) INB(dp, reg)
#define	FLSHW(dp, reg)	(void) INW(dp, reg)
#define	FLSHL(dp, reg)	(void) INL(dp, reg)
#else
#define	FLSHB(dp, reg)
#define	FLSHW(dp, reg)
#define	FLSHL(dp, reg)
#endif

#define	TXDESC(p)	((struct tx_desc *)(void *)(p))
#define	RXDESC(p)	((struct rx_desc *)(void *)(p))

/* additional PCI related macros for compatibility */
#ifndef PCI_CAP_ID_PCI_E
#define	PCI_CAP_ID_PCI_E	0x10
#endif

#ifdef GEM3
#define	IS_MAC_ONLINE(dp)	((dp)->mac_state == MAC_STATE_ONLINE)
#else
#define	IS_MAC_ONLINE(dp)	((dp)->mac_active)
#endif

#define	BIT_0	(1U << 0)
#define	BIT_1	(1U << 1)
#define	BIT_2	(1U << 2)
#define	BIT_3	(1U << 3)
#define	BIT_4	(1U << 4)
#define	BIT_5	(1U << 5)
#define	BIT_6	(1U << 6)
#define	BIT_7	(1U << 7)
#define	BIT_8	(1U << 8)
#define	BIT_9	(1U << 9)

#define	BIT_10	(1U << 10)
#define	BIT_11	(1U << 11)
#define	BIT_12	(1U << 12)
#define	BIT_13	(1U << 13)
#define	BIT_14	(1U << 14)
#define	BIT_15	(1U << 15)
#define	BIT_16	(1U << 16)
#define	BIT_17	(1U << 17)
#define	BIT_18	(1U << 18)
#define	BIT_19	(1U << 19)

#define	BIT_20	(1U << 20)
#define	BIT_21	(1U << 21)
#define	BIT_22	(1U << 22)
#define	BIT_23	(1U << 23)
#define	BIT_24	(1U << 24)
#define	BIT_25	(1U << 25)
#define	BIT_26	(1U << 26)
#define	BIT_27	(1U << 27)
#define	BIT_28	(1U << 28)
#define	BIT_29	(1U << 29)

#define	BIT_30	(1U << 30)
#define	BIT_31	(1U << 31)

#define	ARRAY_SIZE(x)	(sizeof (x) / sizeof ((x)[0]))

/*
 * Debugging
 */
#ifdef DEBUG_LEVEL
static int gani_debug = DEBUG_LEVEL;
#define	DPRINTF(n, args)	if (gani_debug > (n)) cmn_err args
#else
#define	DPRINTF(n, args)
#endif

#define	DUMP_ETHER(n, dp, m) \
	DPRINTF(n, (CE_CONT, "!%s: %s: mac: %02x:%02x:%02x:%02x:%02x:%02x", \
		(dp)->name, __func__, \
		(m)[0], (m)[1], (m)[2], (m)[3], (m)[4], (m)[5]));
/*
 * Our configration for rtl816x
 */
static boolean_t gani_64bit_addr = B_FALSE;

#ifndef GEM_CONFIG_TX_DIRECT
#undef TX_MAX_FRAGS
#define	TX_MAX_FRAGS	1
#undef TX_BUF_SIZE
#define	TX_BUF_SIZE	256
#undef TX_RING_SIZE
#define	TX_RING_SIZE	TX_BUF_SIZE
#else
#define	TX_MAX_FRAGS	GEM_MAXTXFRAGS
#endif

#ifdef TEST_TXDESC_FULL
#define	TX_RING_SIZE	(DESC_BASE_ALIGN / sizeof (struct tx_desc))
#endif
#ifdef TEST_RX_EMPTY
#define	RX_BUF_SIZE	1
#endif

#ifndef	TX_BUF_SIZE
#define	TX_BUF_SIZE	256
#endif
#ifndef	TX_RING_SIZE
#define	TX_RING_SIZE	(DESC_MAX_SIZE / sizeof (struct tx_desc))
#endif

#ifndef	RX_BUF_SIZE
#define	RX_BUF_SIZE	256
#endif
#ifndef	RX_RING_SIZE
#define	RX_RING_SIZE	RX_BUF_SIZE
#endif

#define	OUR_INTR_MASK	\
	(INTR_SERR | INTR_SWInt | INTR_FOVW | INTR_PUN | INTR_RDU | \
	INTR_TER | INTR_TOK | INTR_RER | INTR_ROK)

#if defined(sun4u) || defined(CONFIG_TX_COPY)
static int	gani_tx_copy_thresh = ETHERMAX + 4;
#else
static int	gani_tx_copy_thresh = 256;
#endif
#if defined(sun4u) || defined(CONFIG_RX_COPY)
static int	gani_rx_copy_thresh = ETHERMAX + 4;
#else
static int	gani_rx_copy_thresh = 256;
#endif
static int gani_pcie_req_max = 5;

static int gani_eee_enable = 0;

/*
 * Local device definitions
 */
struct chip_info {
	uint32_t	tcr_val; /* from RTL8169/RTL8168 docs */
	uint32_t	tcr_mask;
	uint32_t	rcr_mask;
	int		max_mtu;
#define	MTU4K	(4*1024 - (ETHERMAX - ETHERMTU) - ETHERFCSL - 4)
#define	MTU6K	(6*1024 - (ETHERMAX - ETHERMTU) - ETHERFCSL - 4)
#define	MTU7K	(7*1024 - (ETHERMAX - ETHERMTU) - ETHERFCSL - 4)
#define	MTU9K	(9*1024 - (ETHERMAX - ETHERMTU) - ETHERFCSL - 4)
	uint_t		type;
	char		*name;
};

#define	GANI_TYPE_MASK		0xf000U
#define	GANI_TYPE_PCI		0x1000U
#define	GANI_TYPE_PCIE_GIGA	0x2000U
#define	GANI_TYPE_PCIE_FAST	0x4000U

#define	R8169_1	(0 | GANI_TYPE_PCI)
#define	R8169_2	(1 | GANI_TYPE_PCI)
#define	R8169_3	(2 | GANI_TYPE_PCI)
#define	R8169_4	(3 | GANI_TYPE_PCI)
#define	R8169_5	(4 | GANI_TYPE_PCI)
#define	R8169_6	(5 | GANI_TYPE_PCI)

#define	R8168_1 (0 | GANI_TYPE_PCIE_GIGA)
#define	R8168_2 (1 | GANI_TYPE_PCIE_GIGA)
#define	R8168_3 (2 | GANI_TYPE_PCIE_GIGA)
#define	R8168_4 (3 | GANI_TYPE_PCIE_GIGA)
#define	R8168_5 (4 | GANI_TYPE_PCIE_GIGA)
#define	R8168_6 (5 | GANI_TYPE_PCIE_GIGA)
#define	R8168_7 (6 | GANI_TYPE_PCIE_GIGA)
#define	R8168_8 (7 | GANI_TYPE_PCIE_GIGA)
#define	R8168_9 (8 | GANI_TYPE_PCIE_GIGA)
#define	R8168_10 (9 | GANI_TYPE_PCIE_GIGA)
#define	R8168_11 (10 | GANI_TYPE_PCIE_GIGA)
#define	R8168_12 (11 | GANI_TYPE_PCIE_GIGA)
#define	R8168_13 (12 | GANI_TYPE_PCIE_GIGA)
#define	R8168_14 (13 | GANI_TYPE_PCIE_GIGA)
#define	R8168_15 (14 | GANI_TYPE_PCIE_GIGA)
#define	R8168_16 (15 | GANI_TYPE_PCIE_GIGA)
#define	R8168_17 (16 | GANI_TYPE_PCIE_GIGA)
#define	R8168_18 (17 | GANI_TYPE_PCIE_GIGA)
#define	R8168_19 (18 | GANI_TYPE_PCIE_GIGA)
#define	R8168_20 (19 | GANI_TYPE_PCIE_GIGA)
#define	R8168_21 (20 | GANI_TYPE_PCIE_GIGA)
#define	R8168_22 (21 | GANI_TYPE_PCIE_GIGA)
#define	R8168_23 (22 | GANI_TYPE_PCIE_GIGA)
#define	R8168_24 (23 | GANI_TYPE_PCIE_GIGA)

#define	R8101_1	(0 | GANI_TYPE_PCIE_FAST)
#define	R8101_2	(1 | GANI_TYPE_PCIE_FAST)
#define	R8101_3	(2 | GANI_TYPE_PCIE_FAST)
#define	R8101_4	(3 | GANI_TYPE_PCIE_FAST)
#define	R8101_5	(4 | GANI_TYPE_PCIE_FAST)
#define	R8101_6	(5 | GANI_TYPE_PCIE_FAST)
#define	R8101_7	(6 | GANI_TYPE_PCIE_FAST)
#define	R8101_8	(7 | GANI_TYPE_PCIE_FAST)
#define	R8101_9	(8 | GANI_TYPE_PCIE_FAST)
#define	R8101_10 (9 | GANI_TYPE_PCIE_FAST)
#define	R8101_11 (10 | GANI_TYPE_PCIE_FAST)
#define	R8101_12 (11 | GANI_TYPE_PCIE_FAST)
#define	R8101_13 (12 | GANI_TYPE_PCIE_FAST)
#define	R8101_14 (13 | GANI_TYPE_PCIE_FAST)
#define	R8101_15 (14 | GANI_TYPE_PCIE_FAST)
#define	R8101_16 (15 | GANI_TYPE_PCIE_FAST)

#define	GANI_PCIE(x)	\
	((x)->chip->type & (GANI_TYPE_PCIE_GIGA | GANI_TYPE_PCIE_FAST))

#define	GANI_PCI(x)	\
	((x)->chip->type & GANI_TYPE_PCI)

#define	GANI_PCIE_GIGA(x)	\
	((x)->chip->type & GANI_TYPE_PCIE_GIGA)

#define	GANI_PCIE_FAST(x)	\
	((x)->chip->type & GANI_TYPE_PCIE_FAST)

/* pci gigabit ethernet devices */
static struct chip_info chiptbl_8169[] = {
{
	0x00000000, 0xfc800000, 0xff7e1880, MTU7K, R8169_1,
	"RTL8169"
},
{
	0x00800000, 0xfc800000, 0xff7e1880, MTU7K, R8169_2,
	"RTL8169S/8110S rev.D"
},
{
	0x04000000, 0xfc800000, 0xff7e1880, MTU7K, R8169_3,
	"RTL8169S/8110S rev.E" },
{
	0x10000000, 0xfc800000, 0xff7e1880, MTU7K, R8169_4,
	"RTL8169SB/8110SB"
},
{
	0x18000000, 0xfc800000, 0xff7e1880, MTU7K, R8169_5,
	"RTL8169/8110SC rev.D"
},
{
	0x98000000, 0xfc800000, 0xff7e1880, MTU7K, R8169_6,
	"RTL8169/8110SC rev.E"
},
};

/* pci express gigabit ethernet devices */
static struct chip_info chiptbl_8168[] = {
{
	0x30000000, 0x7c800000, 0xff7e1880, MTU4K, R8168_1,
	"RTL8168B/8111B rev.B"
},
{
	0x38000000, 0x7cf00000, 0xff7e1880, MTU4K, R8168_2,
	"RTL8168B/8111B rev.E"
},
{
	0x38500000, 0x7cf00000, 0xff7e1880, MTU4K, R8168_3,
	"RTL8168B/8111B rev.F"
},
{
	0x38000000, 0x7c800000, 0xff7e1880, MTU4K, R8168_3,
	"RTL8168B/8111B unknown"
},
/* ---- */
{
	0x3c000000, 0x7cf00000, 0xff7e1880, MTU6K, R8168_4,
	"RTL8168C/8111C"
},
{
	0x3c200000, 0x7cf00000, 0xff7e1880, MTU6K, R8168_5,
	"RTL8168C/8111C rev.B"
},
{
	0x3c400000, 0x7cf00000, 0xff7e1880, MTU6K, R8168_6,
	"RTL8168C/8111C rev.C"
},
{
	0x3c000000, 0x7c800000, 0xff7e1880, MTU6K, R8168_6,
	"RTL8168C/8111C unknown"
},
/* ---- */
{
	0x3c900000, 0x7cf00000, 0xff7e1880, MTU6K, R8168_7,
	"RTL8168CP/8111CP rev.B"
},
{
	0x3cb00000, 0x7cf00000, 0xff7e1880, MTU6K, R8168_8,
	"RTL8168CP/8111CP rev.C"
},
{
	0x3c800000, 0x7c800000, 0xff7e1880, MTU6K, R8168_8,
	"RTL8168CP/8111CP"
},
/* ---- */
{
	0x28100000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_9,
	"RTL8168D/8111D"
},
{
	0x28300000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_10,
	"RTL8168D/8111D rev.C"
},
{
	0x28000000, 0x7c800000, 0xff7e1880, MTU9K, R8168_10,
	"RTL8168D/8111D unknown"
},
/* ---- */
{
	0x28800000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_11,
	"RTL8168DP/8111DP"
},
{
	0x28a00000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_12,
	"RTL8168DP/8111DP rev.B"
},
{
	0x28800000, 0x7c800000, 0xff7e1880, MTU9K, R8168_13,
	"RTL8168DP/8111DP rev.C"
},
/* ---- */
{
	0x2c100000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_14,
	"RTL8168E/8111E"
},
{
	0x2c200000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_15,
	"RTL8168E/8111E rev.B"
},
{
	0x2c000000, 0x7c800000, 0xff7e1880, MTU9K, R8168_15,
	"RTL8168E/8111E unknown"
},
/* ---- */
{
	0x2c800000, 0x7cf00000, 0xff7e0080, MTU9K, R8168_16,
	"RTL8168E-VL/8111E-VL"
},
{
	0x2c900000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_17,
	"RTL8168E-VL/8111E-VL rev.B"
},
{
	0x2c800000, 0x7c800000, 0xff7e0080, MTU9K, R8168_17,
	"RTL8168E-VL/8111E-VL unknown"
},
/* ---- */
{
	0x48000000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_18,
	"RTL8168E/8111F"
},
{
	0x48100000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_19,
	"RTL8168E/8111F rev.B"
},
{
	0x48000000, 0x7c800000, 0xff7e1880, MTU9K, R8168_19,
	"RTL8168E/8111F unknown"
},
/* ---- */
{
	0x48800000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_20,
	"RTL8411"
},
{
	0x48800000, 0x7c800000, 0xff7e1880, MTU9K, R8168_20,
	"RTL8411 unknown"
},
/* ---- */
{
	0x4c000000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_21,
	"RTL8168G/8111G"
},
{
	0x4c100000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_22,
	"RTL8168G/8111G rev.B"
},
{
	0x50000000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_23,
	"RTL8168EP/8111EP"
},
{
	0x50800000, 0x7cf00000, 0xff7e1880, MTU9K, R8168_24,
	"RTL8168GU/8111GU"
},
{
	0x50800000, 0x7c800000, 0xff7e1880, MTU9K, R8168_24,
	"RTL8168GU/8111GU unknown"
},
};

/* pci express fast ethernet devices */
static struct chip_info chiptbl_8101[] = {
{
	0x34000000, 0x7cf00000, 0xff7e1880, 1500, R8101_1,
	"RTL8101E rev.B"
},
{
	0x34200000, 0x7cf00000, 0xff7e1880, 1500, R8101_2,
	"RTL8101E rev.E"
},
{
	0x34300000, 0x7cf00000, 0xff7e1880, 1500, R8101_3,
	"RTL8101E rev.F"
},
{
	0x34000000, 0x7c800000, 0xff7e1880, 1500, R8101_3,
	"RTL8101E unknown"
},
/* ---- */
{
	0x24900000, 0x6cf00000, 0xff7e1880, 1500, R8101_4,
	"RTL8102E"
},
{
	0x24a00000, 0x6cf00000, 0xff7e1880, 1500, R8101_5,
	"RTL8102E rev.B"
},
{
	0x24c00000, 0x6cf00000, 0xff7e1880, 1500, R8101_6,
	"RTL8103E"
},
{
	0x24d00000, 0x6cf00000, 0xff7e1880, 1500, R8101_7,
	"RTL8103E rev.B"
},
{
	0x24e00000, 0x6cf00000, 0xff7e1880, 1500, R8101_8,
	"RTL8103E rev.C"
},
{
	0x24800000, 0x6c800000, 0xff7e1880, 1500, R8101_8,
	"RTL8103E unknown"
},
/* ---- */
{
	0x24000000, 0x7c800000, 0xff7e1880, 1500, R8101_9,
	"RTL8401E"
},
/* ---- */
{
	0x2c000000, 0x7c800000, 0xff7e1880, 1500, R8101_10,
	"RTL8105E"
},
/* ---- */
{
	0x40900000, 0x7cf00000, 0xff7e1880, 1500, R8101_11,
	"RTL8105E rev.B"
},
{
	0x40a00000, 0x7cf00000, 0xff7e1880, 1500, R8101_12,
	"RTL8105E rev.C"
},
{
	0x40b00000, 0x7cf00000, 0xff7e1880, 1500, R8101_13,
	"RTL8105E rev.D"
},
{
	0x40c00000, 0x7cf00000, 0xff7e1880, 1500, R8101_13,
	"RTL8105E rev.D"
},
{
	0x40800000, 0x7c800000, 0xff7e1880, 1500, R8101_13,
	"RTL8105E unknown"
},
/* ---- */
{
	0x44000000, 0x7c800000, 0xff7e1880, 1500, R8101_14,
	"RTL8402"
},
/* ---- */
{
	0x44800000, 0x7cf00000, 0xff7e1880, 1500, R8101_15,
	"RTL8106E"
},
{
	0x44900000, 0x7cf00000, 0xff7e1880, 1500, R8101_16,
	"RTL8106E rev.B"
},
{
	0x44800000, 0x7c800000, 0xff7e1880, 1500, R8101_16,
	"RTL8106E unknown"
},
};

struct gani_dev {
	/*
	 * Misc HW information
	 */
	struct chip_info	*chip;
	uint16_t		svid;
	uint16_t		ssid;
	uint8_t			mac[ETHERADDRL];
	uint8_t			busclk;
#define	BUSCLK_PCI33	0
#define	BUSCLK_PCI66	1
#define	BUSCLK_PCIE	2
	uint8_t			pcie_cap;

	uint32_t		rcr;
	uint32_t		tcr;
	boolean_t		need_to_reset;
	boolean_t		last_stats_valid;
	uint16_t		imr;
	uint16_t		isr_pended;
	boolean_t		msi;
#ifdef notdef
	boolean_t		initialized;
#endif
#ifdef GEM_CONFIG_VLAN_HW
#ifdef CONFIG_CKSUM_OFFLOAD
	boolean_t		new_cksum;
#endif
#endif

#ifdef TEST_FOVW
	int			rx_pkt_cnt;
#endif
#ifdef CONFIG_POLLING
	int			last_poll_interval; /* polling interval in nS */
#endif

	struct rtl8169_tally_counters	last_stat;

	kmutex_t		stat_lock;
	timeout_id_t		stat_to_id;
#ifdef WA_NEW_TX_HANG
	int			tx_last_desc;
#endif
	/* phy access */
	uint_t			cur_page;

	boolean_t		wol_enabled;
};

/*
 * private functions
 */

#ifdef DEBUG_LEVEL
/* debugging support */
static void gani_tx_desc_dump(struct gem_dev *dp);
#endif

/* mii operations */
static void  gani_mii_sync(struct gem_dev *);
static uint16_t	gani_mii_read_raw(struct gem_dev *, uint_t);
static void gani_mii_write_raw(struct gem_dev *, uint_t, uint16_t);
static uint16_t	gani_mii_read(struct gem_dev *, uint_t);
static void gani_mii_write(struct gem_dev *, uint_t, uint16_t);

/* nic operations */
static int gani_attach_chip(struct gem_dev *);
static int gani_reset_chip(struct gem_dev *);
static int gani_init_chip(struct gem_dev *);
static int gani_start_chip(struct gem_dev *);
static int gani_stop_chip(struct gem_dev *);
static int gani_set_media(struct gem_dev *);
static int gani_set_rx_filter(struct gem_dev *);
static int gani_get_stats(struct gem_dev *);

/* descriptor operations */
static int gani_tx_desc_write(struct gem_dev *dp, int slot,
    ddi_dma_cookie_t *dmacookie, int frags, uint64_t flag);
static void gani_tx_start(struct gem_dev *dp, int slot, int frags);
static void gani_rx_desc_write(struct gem_dev *dp, int slot,
    ddi_dma_cookie_t *dmacookie, int frags);
static uint_t gani_tx_desc_stat(struct gem_dev *dp, int slot, int ndesc);
static uint64_t gani_rx_desc_stat(struct gem_dev *dp, int slot, int ndesc);

static void gani_tx_desc_init(struct gem_dev *dp, int slot);
static void gani_rx_desc_init(struct gem_dev *dp, int slot);

/* interrupt handler */
static uint_t gani_interrupt(struct gem_dev *dp);

/* private functions */
static boolean_t gani_disable_EEE(struct gem_dev *dp);
static boolean_t gani_enable_EEE(struct gem_dev *dp);
static void gani_powerup_pll(struct gem_dev *dp);
static void gani_powerdown_pll(struct gem_dev *dp);

/* ======================================================== */

/* mapping attributes */
/* Data access requirements. */
static struct ddi_device_acc_attr gani_dev_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_STRUCTURE_LE_ACC,
	DDI_STRICTORDER_ACC
};

/* On sparc, the buffers should be native endianness */
static struct ddi_device_acc_attr gani_buf_attr = {
	DDI_DEVICE_ATTR_V0,
	DDI_NEVERSWAP_ACC,	/* native endianness */
	DDI_STRICTORDER_ACC
};

static ddi_dma_attr_t gani_dma_attr_buf32 = {
	DMA_ATTR_V0,		/* dma_attr_version */
	0,			/* dma_attr_addr_lo */
	0xffffffffull,		/* dma_attr_addr_hi */
	0xffffull,		/* dma_attr_count_max */
	1,			/* dma_attr_align */
	0xffff,			/* dma_attr_burstsizes */
	1,			/* dma_attr_minxfer */
	0x0000ffffull,		/* dma_attr_maxxfer */
	0xffffffffull,		/* dma_attr_seg */
	0 /* pached later */,	/* dma_attr_sgllen */
	1,			/* dma_attr_granular */
	0			/* dma_attr_flags */
};

static ddi_dma_attr_t gani_dma_attr_buf64 = {
	DMA_ATTR_V0,		/* dma_attr_version */
	0,			/* dma_attr_addr_lo */
	0xffffffffffffull,	/* dma_attr_addr_hi */
	0xffffull,		/* dma_attr_count_max */
	1,			/* dma_attr_align */
	0xffff,			/* dma_attr_burstsizes */
	1,			/* dma_attr_minxfer */
	0x00000000ffffull,	/* dma_attr_maxxfer */
	0xffffffffffffull,	/* dma_attr_seg */
	0 /* pached later */,	/* dma_attr_sgllen */
	1,			/* dma_attr_granular */
	0			/* dma_attr_flags */
};

static ddi_dma_attr_t gani_dma_attr_desc = {
	DMA_ATTR_V0,		/* dma_attr_version */
	0,			/* dma_attr_addr_lo */
	0xffffffffull,		/* dma_attr_addr_hi */
	0xffffffffull,		/* dma_attr_count_max */
	DESC_BASE_ALIGN,	/* dma_attr_align */
	0,			/* dma_attr_burstsizes */
	1,			/* dma_attr_minxfer */
	0xffffffffull,		/* dma_attr_maxxfer */
	0xffffffffull,		/* dma_attr_seg */
	1,			/* dma_attr_sgllen */
	1,			/* dma_attr_granular */
	0			/* dma_attr_flags */
};

/* =============================================================== */
/*
 * Debugging support
 */
/* =============================================================== */
#ifdef DEBUG_HANG
static void
gani_dump_regs(struct gem_dev *dp, char *m)
{
	int		i;

	cmn_err(CE_CONT, "!%s: register dump (%s)", dp->name, m);

	for (i = 0; i < 256; i += 8) {
		cmn_err(CE_CONT,
		    "!%02x: %02x %02x %02x %02x %02x %02x %02x %02x",
		    i,
		    INB(dp, i+0), INB(dp, i+1),
		    INB(dp, i+2), INB(dp, i+3),
		    INB(dp, i+4), INB(dp, i+5),
		    INB(dp, i+6), INB(dp, i+7));
	}
}
#endif
/* =============================================================== */
/*
 * Hardware manupilation
 */
/* =============================================================== */

static uint16_t
gani_map_phy_ocp_addr(uint16_t page, uint16_t reg)
{
	if (page != OCP_STD_PHY_BASE_PAGE) {
		reg -= 16;
	}

	page <<= 4;
	reg <<= 1;

	return (page + reg);
}

static void
gani_mac_ocp_write(struct gem_dev *dp, uint_t reg_addr, uint_t value)
{
	OUTL(dp, MACOCP,
	    (reg_addr >> 1) << OCPR_ADDR_REG_SHIFT | OCPR_WRITE | value);
}

static uint16_t
gani_mac_ocp_read(struct gem_dev *dp, uint_t reg_addr)
{
	OUTL(dp, MACOCP, (reg_addr >> 1) << OCPR_ADDR_REG_SHIFT);

	return (INL(dp, MACOCP));
}

uint32_t
gani_OCP_read(struct gem_dev *dp, uint_t mask, uint_t reg)
{
	int	i;
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_23) {
		OUTL(dp, ERIAR,
		    ERIAR_READ
		    | (ERIAR_OOB << ERIAR_TYPE_SHIFT)
		    | (mask << ERIAR_ByteEn_SHIFT)
		    | reg);

		for (i = 0; i < 10; i++) {
			drv_usecwait(100);
			/* Check if the RTL8168 has completed ERI read */
			if (INL(dp, ERIAR) & ERIAR_FLAG) {
				break;
			}
		}

		return (INL(dp, ERIDR));
	} else {
		ASSERT((reg & ~0xfffU) == 0);
		ASSERT((mask & ~0xfU) == 0);
		OUTL(dp, OCPAR, (mask << 12) | reg);
		for (i = 0; i < 20; i++) {
			drv_usecwait(100);
			if (INL(dp, OCPAR) & OCPAR_FLAG) {
				break;
			}
		}
		return (INL(dp, OCPDR));
	}
}

static void
gani_OCP_write(struct gem_dev *dp, uint_t mask, uint_t reg, uint_t data)
{
	int	i;
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_23) {
		OUTL(dp, ERIDR, data);
		OUTL(dp, ERIAR,
		    ERIAR_WRITE
		    | (ERIAR_OOB << ERIAR_TYPE_SHIFT)
		    | (mask << ERIAR_ByteEn_SHIFT)
		    | reg);

		for (i = 0; i < 10; i++) {
			drv_usecwait(100);
			/* Check if the RTL8168 has completed ERI write */
			if (!(INL(dp, ERIAR) & ERIAR_FLAG)) {
				break;
			}
		}

	} else {
		ASSERT((reg & ~0xfffU) == 0);
		ASSERT((mask & ~0xfU) == 0);

		OUTL(dp, OCPDR, data);
		OUTL(dp, OCPAR, OCPAR_FLAG | (mask << 12) | reg);
		for (i = 0; i < 20; i++) {
			drv_usecwait(100);
			if ((INL(dp, OCPAR) & OCPAR_FLAG) == 0) {
				break;
			}
		}
	}
}

static void
gani_OOB_mutex_lock(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_23) {
		uint32_t	tmp_value;

		tmp_value = gani_OCP_read(dp, 0xF, 0x110);
		tmp_value |= BIT_0;
		gani_OCP_write(dp, 0x1, 0x110, tmp_value);
		while (gani_OCP_read(dp, 0xF, 0x114) & BIT_0) {
			if (gani_OCP_read(dp, 0xF, 0x11C) & BIT_0) {
				tmp_value &= ~BIT_0;
				gani_OCP_write(dp, 0x1, 0x110, tmp_value);
				while (gani_OCP_read(dp, 0xF, 0x11C) & BIT_0)
					;
				tmp_value |= BIT_0;
				gani_OCP_write(dp, 0x1, 0x110, tmp_value);
			}
		}
	} else {
		uint32_t	reg;

		if (lp->chip->type == R8168_13) {
			reg = 0x04;
		} else {
			reg = 0x14;
		}
		gani_OCP_write(dp, 0x8, reg, 0x01000000);

		while (gani_OCP_read(dp, 0xF, reg) & 0x00FF0000) {
			if (gani_OCP_read(dp, 0xF, 0x09C) & 0x000000FF) {
				gani_OCP_write(dp, 0x8, reg, 0x00000000);

				while (gani_OCP_read(dp, 0xF, 0x09C) & 0xFF)
					;

				gani_OCP_write(dp, 0x8, reg, 0x01000000);
			}
		}
	}
}

static void
gani_OOB_mutex_unlock(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_23) {
		uint32_t	tmp_value;

		tmp_value = gani_OCP_read(dp, 0xF, 0x11C);
		tmp_value |= BIT_0;
		gani_OCP_write(dp, 0x1, 0x11C, tmp_value);

		tmp_value = gani_OCP_read(dp, 0xF, 0x110);
		tmp_value &= ~BIT_0;
		gani_OCP_write(dp, 0x1, 0x110, tmp_value);
	} else {
		uint32_t	reg;

		if (lp->chip->type == R8168_13) {
			reg = 0x04;
		} else {
			reg = 0x14;
		}
		gani_OCP_write(dp, 0x1, 0x9C, 0x00000001);
		gani_OCP_write(dp, 0x8, reg, 0x00000000);
	}
}

static void
gani_OOB_notify(struct gem_dev *dp, uint8_t cmd)
{
	int	i;

	OUTB(dp, ERIDR, cmd);
	OUTL(dp, ERIAR, 0x800010E8);
	drv_usecwait(2000);
	for (i = 0; i < 5; i++) {
		drv_usecwait(100);
		if ((INL(dp, ERIAR) & ERIAR_FLAG) == 0) {
			break;
		}
	}

	gani_OCP_write(dp, 0x1, 0x30, 0x00000001);
}

static boolean_t
gani_check_dash(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_23) {
		return ((gani_OCP_read(dp, 0xF, 0x128) & BIT_0) != 0);
	} else {
		uint32_t	reg;

		reg = (lp->chip->type == R8168_13) ? 0xb8 : 0x10;
		return ((gani_OCP_read(dp, 0xF, reg) & 0x00008000) != 0);
	}
}

static void
gani_driver_start(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_23) {
		if (!gani_check_dash(dp)) {
			return;
		}
		gani_OCP_write(dp, 0x1, 0x180, OOB_CMD_DRIVER_START);
		gani_OCP_write(dp,
		    0x1, 0x30,
		    gani_OCP_read(dp, 0xF, 0x30) | BIT_0);
	} else {
		int	timeout;
		uint32_t	reg;

		gani_OOB_notify(dp, OOB_CMD_DRIVER_START);

		if (lp->chip->type == R8168_13) {
			reg = 0xB8;
		} else {
			reg = 0x10;
		}
		for (timeout = 0; timeout < 10; timeout++) {
			delay(drv_usectohz(10000));
			if (gani_OCP_read(dp, 0xF, reg) & BIT_11) {
				break;
			}
		}
	}
}

static void
gani_driver_stop(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_23) {
		uint32_t	tmp_value;

		if (!gani_check_dash(dp)) {
			return;
		}
		gani_OCP_write(dp, 0x1, 0x180, OOB_CMD_DRIVER_STOP);
		tmp_value = gani_OCP_read(dp, 0xF, 0x30);
		tmp_value |= BIT_0;
		gani_OCP_write(dp, 0x1, 0x30, tmp_value);
	} else {
		int timeout;
		uint32_t	reg;

		gani_OOB_notify(dp, OOB_CMD_DRIVER_STOP);

		if (lp->chip->type == R8168_13) {
			reg = 0xB8;
		} else {
			reg = 0x10;
		}
		for (timeout = 0; timeout < 10; timeout++) {
			delay(drv_usectohz(10000));
			if ((gani_OCP_read(dp, 0xF, reg) & BIT_11) == 0) {
				break;
			}
		}
	}
}

/* IO routines for PCI-E phy */
static uint16_t
gani_ephy_read(struct gem_dev *dp, uint32_t location)
{
	int	i;
	uint16_t	uval;

	OUTL(dp, EPHYAR, location << EPHYAR_REGADDR_SHIFT);
	drv_usecwait(100);

	for (i = 0; (INL(dp, EPHYAR) & EPHYAR_FLAG) == 0; i++) {
		if (i > 10) {
			cmn_err(CE_CONT, "%s: %s: timeout",
			    dp->name, __func__);
			uval = 0xffffU;
			goto x;
		}
		drv_usecwait(100);
	}

	uval = (uint16_t)(INL(dp, EPHYAR) & EPHYAR_DATA);
x:
	drv_usecwait(20);

	return (uval);
}

static void
gani_ephy_write(struct gem_dev *dp, uint32_t location, uint16_t val)
{
	int	i;

	OUTL(dp, EPHYAR,
	    EPHYAR_FLAG | (location << EPHYAR_REGADDR_SHIFT) | val);
	drv_usecwait(100);

	for (i = 0; INL(dp, EPHYAR) & EPHYAR_FLAG; i++) {
		if (i > 10) {
			cmn_err(CE_CONT, "%s: %s: timeout",
			    dp->name, __func__);
			break;
		}
		drv_usecwait(100);
	}

	drv_usecwait(20);
}

static uint32_t
gani_csi_read(struct gem_dev *dp, uint_t addr)
{
	int	i;
	uint32_t	uval;
	struct gani_dev	*lp = dp->private;

	ASSERT((addr & ~CSIAR_ADDR) == 0);

	uval = CSIAR_ByteEn | addr;
	if (lp->chip->type == R8168_20 || lp->chip->type == R8101_14) {
		uval |= 0x00020000U;
	}
	OUTL(dp, CSIAR, uval);
	drv_usecwait(100);

	/* Check if the RTL8101 has completed CSI read */
	for (i = 0; (INL(dp, CSIAR) & CSIAR_FLAG) == 0; i++) {
		if (i > 10) {
			cmn_err(CE_WARN, "%s: %s timeout",
			    dp->name, __func__);
			uval = 0;
			goto x;
		}
		drv_usecwait(100);
	}

	uval = INL(dp, CSIDR);
x:
	drv_usecwait(20);
	return (uval);
}

static void
gani_csi_write(struct gem_dev *dp, uint_t addr, uint32_t val)
{
	int	i;
	uint32_t	uval;
	struct gani_dev	*lp = dp->private;

	ASSERT((addr & ~CSIAR_ADDR) == 0);

	OUTL(dp, CSIDR, val);
	uval = CSIAR_FLAG | CSIAR_ByteEn | addr;
	if (lp->chip->type == R8168_20 || lp->chip->type == R8101_14) {
		uval |= 0x00020000U;
	}
	OUTL(dp, CSIAR, uval);
	drv_usecwait(100);

	for (i = 0; INL(dp, CSIAR) & CSIAR_FLAG; i++) {
		if (i > 10) {
			break;
		}
		drv_usecwait(100);
	}
	drv_usecwait(20);
}

static uint32_t
gani_eri_read(struct gem_dev *dp, uint_t addr, int len, uint_t type)
{
	int	i;
	int	val_shift;
	int	shift = 0;
	uint32_t	value1 = 0;
	uint32_t	value2 = 0;
	uint32_t	mask;

	if (len > 4 || len <= 0) {
		return (0xffffffffU);
	}

	while (len > 0) {
		val_shift = addr % ERIAR_ADDR_ALIGN;
		addr = addr & ~0x3;

		OUTL(dp, ERIAR,
		    ERIAR_READ |
		    type << ERIAR_TYPE_SHIFT |
		    ERIAR_ByteEn << ERIAR_ByteEn_SHIFT |
		    addr);

		for (i = 0; i < 10; i++) {
			drv_usecwait(100);

			/* Check if the RTL8168 has completed ERI read */
			if (INL(dp, ERIAR) & ERIAR_FLAG) {
				break;
			}
		}

		if (len == 1) {
			mask = 0xFFU << (val_shift * 8);
		} else if (len == 2) {
			mask = 0xFFFFU << (val_shift * 8);
		} else if (len == 3) {
			mask = 0xFFFFFFU << (val_shift * 8);
		} else {
			mask = 0xFFFFFFFFU << (val_shift * 8);
		}

		value1 = INL(dp, ERIDR) & mask;
		value2 |= (value1 >> val_shift * 8) << shift * 8;

		if (len <= 4 - val_shift) {
			len = 0;
		} else {
			len -= (4 - val_shift);
			shift = 4 - val_shift;
			addr += 4;
		}
	}

	return (value2);
}

static void
gani_eri_write(struct gem_dev *dp, int addr, int len, uint32_t value, int type)
{
	int	i;
	int	val_shift;
	int	shift = 0;
	uint32_t	value1 = 0;
	uint32_t	mask;

	if (len > 4 || len <= 0) {
		return;
	}

	while (len > 0) {
		val_shift = addr % ERIAR_ADDR_ALIGN;
		addr = addr & ~0x3;

		if (len == 1) {
			mask = 0xFFU << (val_shift * 8);
		} else if (len == 2) {
			mask = 0xFFFFU << (val_shift * 8);
		} else if (len == 3) {
			mask = 0xFFFFFFU << (val_shift * 8);
		} else {
			mask = 0xFFFFFFFFU << (val_shift * 8);
		}

		value1 = gani_eri_read(dp, addr, 4, type) & ~mask;
		value1 |= ((value << val_shift * 8) >> shift * 8);

		OUTL(dp, ERIDR, value1);
		OUTL(dp, ERIAR,
		    ERIAR_WRITE |
		    type << ERIAR_TYPE_SHIFT |
		    ERIAR_ByteEn << ERIAR_ByteEn_SHIFT |
		    addr);

		for (i = 0; i < 10; i++) {
			drv_usecwait(100);

			/* Check if the RTL8168 has completed ERI write */
			if ((INL(dp, ERIAR) & ERIAR_FLAG) == 0) {
				break;
			}
		}

		if (len <= 4 - val_shift) {
			len = 0;
		} else {
			len -= (4 - val_shift);
			shift = 4 - val_shift;
			addr += 4;
		}
	}
}

static uint8_t
gani_efuse_read(struct gem_dev *dp, uint_t index)
{
	int	i;
	uint8_t	val;
	struct gani_dev	*lp = dp->private;

	OUTL(dp, EFUSEAR, EFUSE_READ | (index << EFUSE_REG_SHIFT));
	drv_usecwait(100);

	for (i = 0; (INL(dp, EFUSEAR) & EFUSE_READ_OK) == 0; i++) {
		if (i > 300) {
			/* time out */
			val = 0xff;
			cmn_err(CE_WARN, "!%s: %s: timeout",
			    dp->name, __func__);
			goto x;
		}
		drv_usecwait(100);
	}

	val = (uint8_t)(INL(dp, EFUSEAR) & EFUSE_DATA);
x:
	return (val);
}

static int
gani_reset_chip(struct gem_dev *dp)	/* not yet 035 */
{
	int		i;
	uint32_t	val;
	struct gani_dev	*lp = dp->private;

	DPRINTF(0, (CE_CONT, "!%s: %s: called", dp->name, __func__));

#ifdef DEBUG_HANG
	gani_dump_regs(dp, "reset_chip 1");
#endif
#ifdef notdef
	if (!lp->initialized) {
		OUTB(dp, CR9346, CR9346_EEM_WE);
		OUTB(dp, CFG1, INB(dp, CFG1) | CFG1_PMEn);
		OUTB(dp, CFG5, INB(dp, CFG5) & CFG5_PME_STS);
		OUTB(dp, CR9346, CR9346_EEM_NORMAL);

		if (lp->busclk == BUSCLK_PCI66) {
			if (lp->chip->type == R8169_5) {
				OUTL(dp, 0x7c, 0x000fffff);
			} else if (lp->chip->type == R8169_6) {
				OUTL(dp, 0x7c, 0x003fffff);
			}
		} else if (lp->busclk == BUSCLK_PCI33) {
			if (lp->chip->type == R8169_5) {
				OUTL(dp, 0x7c, 0x000fff00);
			} else if (lp->chip->type == R8169_6) {
				OUTL(dp, 0x7c, 0x003fff00);
			}
		}

		lp->initialized = B_TRUE;
	}
#endif
	/*
	 * Disable Rx filter
	 */
	switch (lp->chip->type) {
	case R8168_16:
	case R8168_17:
	case R8168_18:
	case R8168_19:
	case R8168_20:
	case R8168_21:
	case R8168_22:
	case R8168_23:
		/* R8168_16 or later */
		OUTL(dp, RCR, RCR_128_INT | RCR_MXDMA);
		break;

	default:
		/*  pci, pci-e fast and old pci-e gaga chipset */
		val = INL(dp, RCR);
		if (val & RCR_ACCEPT_MODE) {
			OUTL(dp, RCR, val & ~RCR_ACCEPT_MODE);
		}
		break;
	}

	/*
	 * Disable tx and rx if possible
	 */
	if (GANI_PCIE(lp)) {
		switch (lp->chip->type) {
		case R8168_1:
		case R8168_2:
		case R8168_3:

		case R8101_1:
		case R8101_2:
		case R8101_3:
			/* EMPTY */
			break;

		case R8168_4:
		case R8168_5:
		case R8168_6:
		case R8168_7:
		case R8168_8:
		case R8168_9:
		case R8168_10:
		case R8168_14:
		case R8168_15:

		case R8101_4:
		case R8101_5:
		case R8101_6:
		case R8101_7:
		case R8101_8:
		case R8101_9:
		case R8101_10:
		case R8101_11:
		case R8101_12:
		case R8101_13:
		case R8101_15:
		case R8101_16:
			OUTB(dp, CR, CR_STOP | CR_RE | CR_TE);
			drv_usecwait(100);
			break;

		case R8168_11:
		case R8168_12:
		case R8168_13:
			for (i = 0;
			    (INB(dp, TPPoll) & TPPoll_NPQ) && i < 1000;
			    i++) {
				drv_usecwait(20);
			}
			break;

		case R8168_23:
			OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_3);
			drv_usecwait(2000);
			for (i = 0; i < 10; i++) {
				if ((INB(dp, MCUCmd) &
				    (MCUCmd_TXFIFO_EMPTY
				    | MCUCmd_RXFIFO_EMPTY))
				    == (MCUCmd_TXFIFO_EMPTY
				    | MCUCmd_RXFIFO_EMPTY)) {
					break;
				}
				drv_usecwait(100);
			}
			drv_usecwait(2000);
			break;

		default:
			delay(drv_usectohz(10000));
			/* FALLTHRU */

		case R8101_14:
			OUTB(dp, CR, CR_STOP | CR_RE | CR_TE);
			while ((INL(dp, TCR) & BIT_11) == 0) {
				drv_usecwait(100);
			}
			break;
		}
	}

	/* disable interrupts */
	OUTW(dp, IMR, 0);
	FLSHW(dp, IMR);

	OUTB(dp, CR, CR_RST);

	for (i = 0; INB(dp, CR) & CR_RST; i++) {
		if (i > 1000) {
			/* timeout */
			cmn_err(CE_WARN, "!%s: failed to reset: timeout",
			    dp->name);
			return (GEM_FAILURE);
		}
		drv_usecwait(100);
	}

	/* 013 */
	if (lp->chip->type == R8168_11) {
		gani_OOB_mutex_lock(dp);
		gani_OCP_write(dp, 0x3, 0x10,
		    gani_OCP_read(dp, 0xF, 0x010) & ~0x00004000);
		gani_OOB_mutex_unlock(dp);

		gani_OOB_notify(dp, OOB_CMD_RESET);

		for (i = 0; i < 10; i++) {
			delay(drv_usectohz(10000));
			if (gani_OCP_read(dp, 0xF, 0x010) & 0x00004000) {
				break;
			}
		}

		for (i = 0; i < 5; i++) {
			if ((gani_OCP_read(dp, 0xF, 0x034) & 0xFFFF) == 0) {
				break;
			}
		}
	} else if (lp->chip->type == R8168_23) {
		for (i = 0; i < 10; i++) {
			drv_usecwait(100);
			if (INW(dp, 0xD2) & BIT_9) {
				break;
			}
		}
		OUTB(dp, 0xF2, INB(dp, 0xF2) & ~BIT_3);
	}

	/* clear bogus interrupts */
	lp->isr_pended = INW(dp, ISR);
	OUTW(dp, ISR, 0xffffU);
	FLSHW(dp, ISR);

	if (GANI_PCI(lp)) {
		OUTB(dp, 0x82, 0x01);

		switch (lp->chip->type) {
		case R8169_2: /* RTL8169S rev.D */
			OUTB(dp, 0x82, 0x01);
			break;
		}
	}

#ifdef DEBUG_HANG
	gani_dump_regs(dp, "reset_chip 2");
#endif
	return (GEM_SUCCESS);
}

/*
 * Setup rtl8169
 */
static int
gani_init_8169(struct gem_dev *dp, ddi_acc_handle_t conf_ha)
{
	uint32_t		val;
	uint32_t		cpcmd;
	struct gani_dev		*lp = dp->private;

	DPRINTF(0, (CE_CONT, "!%s: %s: called", dp->name, __func__));
	
	OUTW(dp, RMS, RMS_PCI(dp->mtu));
	DPRINTF(0, (CE_CONT, "!%s: RMS %d", dp->name, INW(dp, RMS)));

	/* make configuration registers writable */
	OUTB(dp, CR9346, CR9346_EEM_WE);

	OUTB(dp, ETThR, 0x3f);

	/* configure CPlus command register */
	cpcmd = INW(dp, CpCR);
	cpcmd &= ~(CpCR_RxVLAN | CpCR_RxChkSum);
	cpcmd |= CpCR_MulRW;
	if (gani_64bit_addr) {
		cpcmd |= CpCR_DAC;
	}
#ifdef GEM_CONFIG_VLAN_HW
	cpcmd |= CpCR_RxVLAN;
#ifdef GEM_CONFIG_CKSUM_OFFLOAD
	cpcmd |= CpCR_RxChkSum;
#endif /* GEM_CONFIG_CKSUM_OFFLOAD */
#endif /* GEM_CONFIG_VLAN_HW */
	OUTW(dp, CpCR, cpcmd);

	/* fix pci configuration registers */
	pci_config_put8(conf_ha, PCI_CONF_CACHE_LINESZ, 8);

	if (lp->chip->type == R8169_2 /* RTL8169S rev.D */) {
		cpcmd |= CpCR_EnAnaPLL;
	} else {
		cpcmd &= ~CpCR_EnAnaPLL;
	}
	OUTW(dp, CpCR, cpcmd);

	pci_config_put8(conf_ha, PCI_CONF_LATENCY_TIMER, 0x40);

	/* XXX - undocumented */
	OUTW(dp, 0xe2, 0);

	/* Transmit Normal Priority Descriptors */
	DPRINTF(4, (CE_CONT, "!%s: TNPDS 0x%llx",
	    dp->name, (unsigned long long)dp->tx_ring_dma));
	OUTL(dp, TNPDSL, (uint32_t)dp->tx_ring_dma);
	OUTL(dp, TNPDSH, (uint32_t)(dp->tx_ring_dma >> 32));

	/* Transmit High Priority Descriptors: Do nothing */
	OUTL(dp, THPDSL, 0);
	OUTL(dp, THPDSH, 0);

	/* Receive Descriptor Start Address register */
	OUTL(dp, RDSARL, (uint32_t)dp->rx_ring_dma);
	OUTL(dp, RDSARH, (uint32_t)(dp->rx_ring_dma >> 32));
	DPRINTF(4, (CE_CONT, "!%s: RDSAR 0x%llx",
	    dp->name, (unsigned long long)dp->rx_ring_dma));

	/* Missed packet counter: clear it */
	OUTL(dp, MPC, 0);

	/* MulInt Register */
	OUTW(dp, MULINT, INW(dp, MULINT) & 0xf000);

	/* Interrupt status register: clear bogus interrupts */
	OUTW(dp, ISR, 0xffffU);
	FLSHW(dp, ISR);

	/* enable tx and rx before writing rcr and tcr */
	OUTB(dp, CR, CR_RE | CR_TE);

	OUTL(dp, RCR, lp->rcr);
	OUTL(dp, TCR, lp->tcr);

	/* make config registers read only */
	OUTB(dp, CR9346, CR9346_EEM_NORMAL);
	drv_usecwait(10);

	/* TimerInt Register */
	OUTL(dp, TimerInt, 0);

	/* Timer count register: test count rate */
	OUTL(dp, TCTR, 0);
#if DEBUG_LEVEL > 3
	drv_usecwait(1000);
	cmn_err(CE_CONT, "!%s: TCTR:%d in 1mS", dp->name, INL(dp, TCTR));
#endif
	return (GEM_SUCCESS);
}

static int
gani_init_8168(struct gem_dev *dp, ddi_acc_handle_t conf_ha)
{
	uint32_t	val;
	uint32_t	cpcmd;
	uint32_t	devcsr;
	boolean_t	aspm = B_FALSE;
	struct gani_dev	*lp = dp->private;

	DPRINTF(0, (CE_CONT, "!%s: %s: called", dp->name, __func__));

	pci_config_put8(conf_ha, PCI_CONF_LATENCY_TIMER, 0x40);

	/* make configuration registers writable */
	OUTB(dp, CR9346, CR9346_EEM_WE);

	OUTB(dp, ETThR, 0x3f);

	OUTW(dp, RMS, RMS_PCIE_GIGA(dp->mtu));
	DPRINTF(0, (CE_CONT, "!%s: RMS %d", dp->name, INW(dp, RMS)));

	/* configure CPlus command register */
	cpcmd = INW(dp, CpCR);
	cpcmd |= CpCR_StatDis | CpCR_INTT_1;
	cpcmd &= ~(CpCR_RxVLAN | CpCR_RxChkSum);

#ifdef GEM_CONFIG_VLAN_HW
	/* common capability for CpCR */
	cpcmd |= CpCR_RxVLAN;
#ifdef GEM_CONFIG_CKSUM_OFFLOAD
	if (dp->mtu <= ETHERMTU) {
		cpcmd |= CpCR_RxChkSum;
	} else {
		cpcmd &= ~CpCR_RxChkSum;
	}
#endif /* GEM_CONFIG_CKSUM_OFFLOAD */
#endif /* GEM_CONFIG_VLAN_HW */
	OUTW(dp, CpCR, cpcmd);

	OUTW(dp, 0xe2, 0x5f51);
	OUTB(dp, CFG5, INB(dp, CFG5) & ~BIT_7);

	/* Transmit Normal Priority Descriptors */
	DPRINTF(2, (CE_CONT, "!%s: TNPDS 0x%llx",
	    dp->name, (uint64_t)dp->tx_ring_dma));
	OUTL(dp, TNPDSH, (uint32_t)(dp->tx_ring_dma >> 32));
	OUTL(dp, TNPDSL, (uint32_t)dp->tx_ring_dma);
#ifdef SANITY
	/* Transmit High Priority Descriptors: Do nothing */
	OUTL(dp, THPDSH, 0);
	OUTL(dp, THPDSL, 0);
#endif
	/* Receive Descriptor Start Address register */
	DPRINTF(2, (CE_CONT, "!%s: RDSAR 0x%llx",
	    dp->name, (uint64_t)dp->rx_ring_dma));
	OUTL(dp, RDSARH, (uint32_t)(dp->rx_ring_dma >> 32));
	OUTL(dp, RDSARL, (uint32_t)dp->rx_ring_dma);
#ifdef notdef
	OUTL(dp, RCR, lp->rcr);
#endif
	OUTL(dp, TCR, lp->tcr);
	DPRINTF(0, (CE_CONT, "!%s: %s: rcr:%b, tcr:%x",
	    dp->name, __func__,
	    INL(dp, RCR), RCR_BITS, INL(dp, TCR)));

	/* Interrupt status register: clear bogus interrupts */
	OUTW(dp, ISR, 0xffffU);
	FLSHW(dp, ISR);

	devcsr = pci_config_get32(conf_ha, lp->pcie_cap + 8);
	DPRINTF(0, (CE_CONT, "!%s: %s: devcsr:0x%x",
	    dp->name, __func__, devcsr));
	devcsr &= ~(7 << 12);

	switch (lp->chip->type) {
	case R8168_11: case R8168_12: case R8168_13: case R8168_14:
	case R8168_15: case R8168_16: case R8168_17: case R8168_18:
	case R8168_19: case R8168_20: case R8168_21: case R8168_22:
	case R8168_23: case R8168_24:
		devcsr |= gani_pcie_req_max << 12;
		break;
	default:
		if (dp->mtu > ETHERMTU) {
			devcsr |= 2 << 12;
		} else {
			devcsr |= gani_pcie_req_max << 12;
		}
		break;
	}

	switch (lp->chip->type) {
	case R8168_1:
	case R8168_2:
	case R8168_3:
		devcsr |= 0x800;
		break;
	}
	pci_config_put32(conf_ha, lp->pcie_cap + 8, devcsr);

	switch (lp->chip->type) {
	case R8168_4: /* RTL8168C */
		/* 012 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		OUTB(dp, DBGREG, 0xe0 | DBGREG_FIX_NAK_1 | DBGREG_FIX_NAK_2);

		gani_ephy_write(dp, 2,
		    (gani_ephy_read(dp, 2) & ~0x0800) | 0x1000);
		gani_ephy_write(dp, 3, gani_ephy_read(dp, 3) | 0x0002);
		gani_ephy_write(dp, 6, gani_ephy_read(dp, 6) & ~0x0080);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		pci_config_put8(conf_ha, 0x81, 0);

		val = cpcmd;
		val &= CpCR_NORMAL | CpCR_RxVLAN | CpCR_RxChkSum | 0x3;
		OUTW(dp, CpCR, val);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) | CFG4_JUMBO);
			/* don't use cksum offload */
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) & ~CFG4_JUMBO);
		}
		break;

	case R8168_5: /* RTL8168C rev.B */
		/* 012 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		gani_ephy_write(dp, 1,
		    gani_ephy_read(dp, 1) | 0x0001);
		gani_ephy_write(dp, 3,
		    (gani_ephy_read(dp, 3) & ~0x0400) | 0x0220);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		/* disable clock request */
		pci_config_put8(conf_ha, 0x81, 0);

		val = cpcmd;
		val &= CpCR_NORMAL | CpCR_RxVLAN | CpCR_RxChkSum | 0x3;
		OUTW(dp, CpCR, val);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) | CFG4_JUMBO);
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) & ~CFG4_JUMBO);
		}
		break;

	case R8168_6: /* RTL8168C rev.C */
		/* 013 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		/* disable clock request */
		pci_config_put8(conf_ha, 0x81, 0);

		val = cpcmd;
		val &= CpCR_NORMAL | CpCR_RxVLAN | CpCR_RxChkSum | 0x3;
		OUTW(dp, CpCR, val);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) | CFG4_JUMBO);
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) & ~CFG4_JUMBO);
		}
		break;

	case R8168_7: /* RTL8168CP rev.B */
		/* 013 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		gani_eri_write(dp, 0x1EC, 1, 0x07, ERIAR_ASF);

		/* disable clock request */
		pci_config_put8(conf_ha, 0x81, 0);

		val = cpcmd;
		val &= CpCR_NORMAL | CpCR_RxVLAN | CpCR_RxChkSum | 0x3;
		OUTW(dp, CpCR, val);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) | CFG4_JUMBO);
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) & ~CFG4_JUMBO);
		}
		break;

	case R8168_8: /* RTL8168 rev.C */
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		gani_eri_write(dp, 0x1EC, 1, 0x07, ERIAR_ASF);

		/* disable clock request */
		pci_config_put8(conf_ha, 0x81, 0);

		val = cpcmd;
		val &= CpCR_NORMAL | CpCR_RxVLAN | CpCR_RxChkSum | 0x3;
		OUTW(dp, CpCR, val);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);
		OUTB(dp, 0xD1, 0x20);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) | CFG4_JUMBO);
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) & ~CFG4_JUMBO);
		}
		break;

	case R8168_9: /* RTL8168D */
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		/* disable clock request */
		pci_config_put8(conf_ha, 0x81, 0);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~BIT_4);
		OUTB(dp, DBGREG, INB(dp, DBGREG) | BIT_7 | BIT_1);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) | CFG4_JUMBO);
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) & ~CFG4_JUMBO);
		}
		gani_ephy_write(dp, 0x01, 0x7c7d);
		gani_ephy_write(dp, 0x02, 0x091f);
		gani_ephy_write(dp, 0x06, 0xb271);
		gani_ephy_write(dp, 0x07, 0xce00);
		break;

	case R8168_10: /* RTL8168D rev.C */
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		OUTB(dp, DBGREG, INB(dp, DBGREG) | BIT_7 | BIT_1);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) | CFG4_JUMBO);
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) & ~CFG4_JUMBO);
		}
		OUTB(dp, CFG1, INB(dp, CFG1) | 0x10);

		gani_ephy_write(dp, 0x01, 0x6C7F);
		gani_ephy_write(dp, 0x02, 0x011F);
		gani_ephy_write(dp, 0x03, 0xC1B2);
		gani_ephy_write(dp, 0x1A, 0x0546);
		gani_ephy_write(dp, 0x1C, 0x80C4);
		gani_ephy_write(dp, 0x1D, 0x78E4);
		gani_ephy_write(dp, 0x0A, 0x8100);

		/* disable clock request */
		pci_config_put8(conf_ha, 0x81, 0);

		OUTB(dp, 0xf3, INB(dp, 0xf3) | BIT_2);
		break;

	case R8168_11: /* RTL8168DP */
	case R8168_13:
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x17000000);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
		}
		/* disable clock request */
		pci_config_put8(conf_ha, 0x81, 0);

		OUTB(dp, CFG1, INB(dp, CFG1) | 0x10);
		break;

	case R8168_12: /* RTL8168DP */
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x17000000);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
		} else {
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
		}

		val = gani_ephy_read(dp, 0x0B);
		gani_ephy_write(dp, 0x0B, val | 0x48);

		val = gani_ephy_read(dp, 0x19) & ~0x20;
		gani_ephy_write(dp, 0x19, val | 0x50);

		val = gani_ephy_read(dp, 0x0C) & ~0x100;
		gani_ephy_write(dp, 0x0C, val | 0x20);

		pci_config_put8(conf_ha, 0x81, 1);

		OUTB(dp, CFG1, INB(dp, CFG1) | 0x10);
		break;

	case R8168_14:
	case R8168_15:
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		/* set EPHY registers */
		val = gani_ephy_read(dp, 0x00) & ~0x0200;
		val |= 0x0100;
		gani_ephy_write(dp, 0x00, val);

		val = gani_ephy_read(dp, 0x00);
		val |= 0x0004;
		gani_ephy_write(dp, 0x00, val);

		val = gani_ephy_read(dp, 0x06) & ~0x0002;
		val |= 0x0001;
		gani_ephy_write(dp, 0x06, val);

		val = gani_ephy_read(dp, 0x06);
		val |= 0x0030;
		gani_ephy_write(dp, 0x06, val);

		val = gani_ephy_read(dp, 0x07);
		val |= 0x2000;
		gani_ephy_write(dp, 0x07, val);

		val = gani_ephy_read(dp, 0x00);
		val |= 0x0020;
		gani_ephy_write(dp, 0x00, val);

		val = gani_ephy_read(dp, 0x03) & ~0x5800;
		val |= 0x2000;
		gani_ephy_write(dp, 0x03, val);

		val = gani_ephy_read(dp, 0x03);
		val |= 0x0001;
		gani_ephy_write(dp, 0x03, val);

		val = gani_ephy_read(dp, 0x01) & ~0x0800;
		val |= 0x1000;
		gani_ephy_write(dp, 0x01, val);

		val = gani_ephy_read(dp, 0x07);
		val |= 0x4000;
		gani_ephy_write(dp, 0x07, val);

		val = gani_ephy_read(dp, 0x1E);
		val |= 0x2000;
		gani_ephy_write(dp, 0x1E, val);

		gani_ephy_write(dp, 0x19, 0xFE6C);

		val = gani_ephy_read(dp, 0x0A);
		val |= 0x0040;
		gani_ephy_write(dp, 0x0A, val);

		cpcmd &= 0x2063;
		if (dp->mtu > ETHERMTU) {
			OUTB(dp, ETThR, 0x24);
			OUTB(dp, CFG3, INB(dp, CFG3) | CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_0);
		} else {
			OUTB(dp, ETThR, 0x0C);
			OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_JUMBO);
			OUTB(dp, CFG4, INB(dp, CFG4) & ~BIT_0);
		}

		OUTB(dp, 0xF3, INB(dp, 0xF3) | BIT_5);
		OUTB(dp, 0xF3, INB(dp, 0xF3) & ~BIT_5);

		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_7 | BIT_6);

		OUTB(dp, 0xD1, INB(dp, 0xD1) | BIT_2 | BIT_3);

		OUTB(dp, 0xF1,
		    INB(dp, 0xF1) |
		    BIT_7 | BIT_6 | BIT_5 | BIT_4 | BIT_2 | BIT_1);

		OUTB(dp, CFG5, (INB(dp, CFG5) & ~0x08) | BIT_0);
		OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);
		break;

	case R8168_16:
	case R8168_17:
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x17000000);

		val = gani_eri_read(dp, 0xD5, 1, ERIAR_ExGMAC)
		    | BIT_3 | BIT_2;
		gani_eri_write(dp, 0xD5, 1, val, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xC0, 2, 0x00000000, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xB8, 2, 0x00000000, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xC8, 4, 0x00100002, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xE8, 4, 0x00100006, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x1D0, 4, ERIAR_ExGMAC);
		val |= BIT_1;
		gani_eri_write(dp, 0x1D0, 1, val, ERIAR_ExGMAC);

		OUTL(dp, TCR, INL(dp, TCR) | BIT_7);
		OUTB(dp, 0xD3, INB(dp, 0xD3) & ~BIT_7);
		OUTB(dp, 0x1B, INB(dp, 0x1B) & ~0x07);

		if (lp->chip->type == R8168_16) {
			OUTL(dp, 0xB0, 0xEE480010);
			OUTB(dp, 0x1A, INB(dp, 0x1A) & ~(BIT_2 | BIT_3));
			gani_eri_write(dp, 0x1DC, 1, 0x64, ERIAR_ExGMAC);

			gani_ephy_write(dp, 0x06, 0xF020);
			gani_ephy_write(dp, 0x07, 0x01FF);
			gani_ephy_write(dp, 0x00, 0x5027);
			gani_ephy_write(dp, 0x01, 0x0003);
			gani_ephy_write(dp, 0x02, 0x2D16);
			gani_ephy_write(dp, 0x03, 0x6D49);
			gani_ephy_write(dp, 0x08, 0x0006);
			gani_ephy_write(dp, 0x0A, 0x00C8);
		} else {
			val = gani_eri_read(dp, 0x1B0, 4, ERIAR_ExGMAC);
			val |= BIT_4;
			gani_eri_write(dp, 0x1B0, 1, val, ERIAR_ExGMAC);
			gani_eri_write(dp, 0xCC, 4, 0x00000050, ERIAR_ExGMAC);
			gani_eri_write(dp, 0xd0, 4, 0x07ff0060, ERIAR_ExGMAC);
			OUTB(dp, TDFNR, (INB(dp, TDFNR) & ~0x3F) | 0x8);
		}

		val = gani_ephy_read(dp, 0x09);
		val |= BIT_7;
		gani_ephy_write(dp, 0x09, val);

		val = gani_ephy_read(dp, 0x19);
		val |= (BIT_2 | BIT_5 | BIT_9);
		gani_ephy_write(dp, 0x19, val);

		if (aspm) {
			OUTB(dp, CFG5, INB(dp, CFG5) | BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
		} else {
			OUTB(dp, CFG5, INB(dp, CFG5) & ~BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) & ~BIT_7);
		}

		OUTB(dp, CFG2, INB(dp, CFG2) & ~BIT_5);

		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_6);
		OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_6);

		cpcmd &= 0x2063;
		if (dp->mtu > ETHERMTU) {
			OUTB(dp, ETThR, 0x27);
		} else {
			OUTB(dp, ETThR, 0x0C);
		}

		/* disable clock request. */
		pci_config_put8(conf_ha, 0x81, 0);
		break;

	case R8168_18:
	case R8168_19:
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x17000000);

		gani_eri_write(dp, 0xC8, 4, 0x00100002, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xE8, 4, 0x00100006, ERIAR_ExGMAC);
		OUTL(dp, TCR, INL(dp, TCR) | BIT_7);
		OUTB(dp, 0xD3, INB(dp, 0xD3) & ~BIT_7);
		val = gani_eri_read(dp, 0xDC, 1, ERIAR_ExGMAC);
		val &= ~BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);
		val |= BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);

		if (lp->chip->type == R8168_18) {
			val = gani_ephy_read(dp, 0x06);
			val |= BIT_5;
			val &= ~(BIT_7 | BIT_6);
			gani_ephy_write(dp, 0x06, val);

			val = gani_ephy_read(dp, 0x08);
			val |= BIT_1;
			val &= ~BIT_0;
			gani_ephy_write(dp, 0x08, val);
		}

		val = gani_ephy_read(dp, 0x09);
		val |= BIT_7;
		gani_ephy_write(dp, 0x09, val);

		val = gani_ephy_read(dp, 0x19);
		val |= (BIT_2 | BIT_5 | BIT_9);
		gani_ephy_write(dp, 0x19, val);

		if (aspm) {
			OUTB(dp, CFG5, INB(dp, CFG5) | BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
		} else {
			OUTB(dp, CFG5, INB(dp, CFG5) & ~BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) & ~BIT_7);
		}

		cpcmd &= 0x2063;
		if (dp->mtu > ETHERMTU) {
			OUTB(dp, ETThR, 0x27);
		} else {
			OUTB(dp, ETThR, 0x0C);
		}

		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_6);
		OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_6);
		gani_eri_write(dp, 0xC0, 2, 0x00000000, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xB8, 2, 0x00000000, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0xD5, 1, ERIAR_ExGMAC);
		val |= BIT_3 | BIT_2;
		gani_eri_write(dp, 0xD5, 1, val, ERIAR_ExGMAC);
		OUTB(dp, 0x1B, INB(dp, 0x1B) & ~0x07);

		val = gani_eri_read(dp, 0x1B0, 1, ERIAR_ExGMAC);
		val |= BIT_4;
		gani_eri_write(dp, 0x1B0, 1, val, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x1d0, 1, ERIAR_ExGMAC);
		val |= BIT_4 | BIT_1;
		gani_eri_write(dp, 0x1d0, 1, val, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xCC, 4, 0x00000050, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xd0, 4, 0x00000060, ERIAR_ExGMAC);

		if (INB(dp, 0x8c) & BIT_28) {
			gani_mii_write_raw(dp, 0x1F, 0x0007);
			gani_mii_write_raw(dp, 0x1E, 0x002C);
			val = gani_mii_read_raw(dp, 0x16);
			val |= BIT_10;
			gani_mii_write_raw(dp, 0x16, val);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8B80);
			val = gani_mii_read_raw(dp, 0x06);
			val |= BIT_7;
			gani_mii_write_raw(dp, 0x06, val);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		}
		break;

	case R8168_20:
		/* 035 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x17000000);

		gani_eri_write(dp, 0xC8, 4, 0x00100002, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xE8, 4, 0x00100006, ERIAR_ExGMAC);
		OUTL(dp, TCR, INL(dp, TCR) | BIT_7);
		OUTB(dp, 0xD3, INB(dp, 0xD3) & ~BIT_7);
		val = gani_eri_read(dp, 0xDC, 1, ERIAR_ExGMAC);
		val &= ~BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);
		val |= BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);

		val = gani_ephy_read(dp, 0x06);
		val |= BIT_5;
		val &= ~(BIT_7 | BIT_6);
		gani_ephy_write(dp, 0x06, val);

		gani_ephy_write(dp, 0x0f, 0x5200);

		val = gani_ephy_read(dp, 0x19);
		val |= (BIT_2 | BIT_5 | BIT_9);
		gani_ephy_write(dp, 0x19, val);

		if (aspm) {
			OUTB(dp, CFG5, INB(dp, CFG5) | BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
		} else {
			OUTB(dp, CFG5, INB(dp, CFG5) & ~BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) & ~BIT_7);
		}

		cpcmd &= 0x2063;
		if (dp->mtu > ETHERMTU) {
			OUTB(dp, ETThR, 0x27);
		} else {
			OUTB(dp, ETThR, 0x0C);
		}

		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_6);
		OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_6);
		gani_eri_write(dp, 0xC0, 2, 0x00000000, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xB8, 2, 0x00000000, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0xD5, 1, ERIAR_ExGMAC);
		val |= BIT_3 | BIT_2;
		gani_eri_write(dp, 0xD5, 1, val, ERIAR_ExGMAC);

		val = gani_eri_read(dp, 0x1B0, 1, ERIAR_ExGMAC);
		val |= BIT_4;
		gani_eri_write(dp, 0x1B0, 1, val, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x1d0, 1, ERIAR_ExGMAC);
		val |= BIT_4 | BIT_1;
		gani_eri_write(dp, 0x1d0, 1, val, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xCC, 4, 0x00000050, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xd0, 4, 0x00000060, ERIAR_ExGMAC);
		break;

	case R8168_21:
	case R8168_22:
	case R8168_24:
		gani_eri_write(dp, 0xC8, 1, 0x02, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xCA, 1, 0x08, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xCC, 1, 0x38, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xD0, 1, 0x48, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xE8, 4, 0x00100006, ERIAR_ExGMAC);

		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x17000000);

		val = gani_eri_read(dp, 0xDC, 1, ERIAR_ExGMAC);
		val &= ~BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);
		val |= BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		cpcmd = INW(dp, CpCR) &
		    ~(CpCR_BIST | CpCR_Macdbgo_oe | CpCR_HALFDUP |
		    CpCR_RXFLOW | CpCR_TXFLOW |
		    CpCR_CXPL | CpCR_ASF | CpCR_StatDis |
		    CpCR_Macdbgo_sel);

		gani_eri_write(dp, 0xC0, 2, 0x00000000, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xB8, 2, 0x00000000, ERIAR_ExGMAC);
		OUTB(dp, 0x1B, INB(dp, 0x1B) & ~0x07);

		if (aspm) {
			OUTB(dp, CFG5, INB(dp, CFG5) | BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
		} else {
			OUTB(dp, CFG5, INB(dp, CFG5) & ~BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) & ~BIT_7);
		}

		OUTB(dp, 0xF1, INB(dp, 0xF1) | BIT_7);

		val = gani_eri_read(dp, 0x2FC, 1, ERIAR_ExGMAC);
		val &= ~(BIT_0 | BIT_1 | BIT_2);
		val |= BIT_0;
		gani_eri_write(dp, 0x2FC, 1, val, ERIAR_ExGMAC);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, ETThR, 0x27);
		} else {
			OUTB(dp, ETThR, 0x0C);
		}

		break;

	case R8168_23:
		gani_eri_write(dp, 0xC8, 1, 0x02, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xCA, 1, 0x08, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xCC, 1, 0x2f, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xD0, 1, 0x5f, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xE8, 4, 0x00100006, ERIAR_ExGMAC);

		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x17000000);

		val = gani_eri_read(dp, 0xDC, 1, ERIAR_ExGMAC);
		val &= ~BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);
		val |= BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);

		gani_ephy_write(dp, 0x00, 0x10a3);
		gani_ephy_write(dp, 0x06, 0xf030);
		gani_ephy_write(dp, 0x08, 0x2006);
		gani_ephy_write(dp, 0x0d, 0x1666);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		cpcmd = INW(dp, CpCR) &
		    ~(CpCR_BIST | CpCR_Macdbgo_oe | CpCR_HALFDUP |
		    CpCR_RXFLOW | CpCR_TXFLOW |
		    CpCR_CXPL | CpCR_ASF | CpCR_StatDis |
		    CpCR_Macdbgo_sel);

		gani_eri_write(dp, 0xC0, 2, 0x00000000, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xB8, 2, 0x00000000, ERIAR_ExGMAC);
		OUTB(dp, 0x1B, INB(dp, 0x1B) & ~0x07);

		if (aspm) {
			OUTB(dp, CFG5, INB(dp, CFG5) | BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
		} else {
			OUTB(dp, CFG5, INB(dp, CFG5) & ~BIT_0);
			OUTB(dp, CFG2, INB(dp, CFG2) & ~BIT_7);
		}

		val = gani_eri_read(dp, 0x2FC, 1, ERIAR_ExGMAC);
		val |= BIT_2;
		gani_eri_write(dp, 0x2FC, 1, val, ERIAR_ExGMAC);

		val = gani_eri_read(dp, 0x1d0, 1, ERIAR_ExGMAC);
		val |= BIT_1;
		gani_eri_write(dp, 0x1d0, 1, val, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x1B0, 1, ERIAR_ExGMAC);
		val |= BIT_1 | BIT_0;
		gani_eri_write(dp, 0x1B0, 1, val, ERIAR_ExGMAC);

		val = gani_eri_read(dp, 0x3F2, 2, ERIAR_ExGMAC);
		val &= ~(BIT_8 | BIT_11);
		val |= (BIT_0 | BIT_1 | BIT_9 | BIT_10 | BIT_12
		    | BIT_13 | BIT_14);
		gani_eri_write(dp, 0x3F2, 2, val, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x3F5, 1, ERIAR_ExGMAC);
		val |= BIT_6 | BIT_7;
		gani_eri_write(dp, 0x3F5, 1, val, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x2E8, 2, 0x883C, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x2EA, 2, 0x8C12, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x2EC, 2, 0x9003, ERIAR_ExGMAC);

		val = gani_eri_read(dp, 0xd4, 2, ERIAR_ExGMAC);
		val |= 0x1f80;
		gani_eri_write(dp, 0xd4, 2, val, ERIAR_ExGMAC);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, ETThR, 0x27);
		} else {
			OUTB(dp, ETThR, 0x0C);
		}

		break;

	case R8168_1: /* RTL8168B rev.B */
		/* 035 */
		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		val = cpcmd;
		val &= CpCR_NORMAL | CpCR_RxVLAN | CpCR_RxChkSum | 0x3;
		OUTW(dp, CpCR, val);
		break;

	case R8168_2: /* RTL8168B rev.E */
		/* 035 */
		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		val = cpcmd;
		val &= CpCR_NORMAL | CpCR_RxVLAN | CpCR_RxChkSum | 0x3;
		OUTW(dp, CpCR, val);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_0);
		} else {
			OUTB(dp, CFG4, INB(dp, CFG4) & ~BIT_0);
		}
		break;

	case R8168_3: /* RTL8168B rev.F */
		/* 035 */
		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		val = cpcmd;
		val &= CpCR_NORMAL | CpCR_RxVLAN | CpCR_RxChkSum | 0x3;
		OUTW(dp, CpCR, val);

		OUTB(dp, ETThR, 0x3f);

		if (dp->mtu > ETHERMTU) {
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_0);
		} else {
			OUTB(dp, CFG4, INB(dp, CFG4) & ~BIT_0);
		}
		break;
	}

	OUTB(dp, CR, CR_RE | CR_TE);

	/* make config registers read only */
	OUTB(dp, CR9346, CR9346_EEM_NORMAL);
	drv_usecwait(10);

	/* TimerInt Register */
	OUTL(dp, TimerInt, 0);

	/* Timer count register: test count rate */
	OUTL(dp, TCTR, 0);
#if DEBUG_LEVEL > 3
	drv_usecwait(1000);
	cmn_err(CE_CONT, "!%s: TCTR:%d in 1mS", dp->name, INL(dp, TCTR));
#endif
	return (GEM_SUCCESS);
}

static int
gani_init_8101(struct gem_dev *dp, ddi_acc_handle_t conf_ha)
{
	uint32_t	val;
	uint16_t	data16;
	uint32_t	cpcmd;
	struct gani_dev	*lp = dp->private;

	DPRINTF(0, (CE_CONT, "!%s: %s: called", dp->name, __func__));

	OUTW(dp, RMS, RMS_PCIE_FAST(lp->chip->type));
	DPRINTF(2, (CE_CONT, "!%s: RMS %d", dp->name, INW(dp, RMS)));

	OUTL(dp, TCR, lp->tcr);

	/* make configuration registers writable */
	OUTB(dp, CR9346, CR9346_EEM_WE);

	/* configure CPlus command register */
	cpcmd = INW(dp, CpCR);
	cpcmd &= ~(CpCR_RxVLAN | CpCR_RxChkSum);
#ifdef GEM_CONFIG_VLAN_HW
	cpcmd &= ~CpCR_RxVLAN;
#ifdef GEM_CONFIG_CKSUM_OFFLOAD
	cpcmd |= CpCR_RxChkSum;
#endif /* GEM_CONFIG_CKSUM_OFFLOAD */
#endif /* GEM_CONFIG_VLAN_HW */

	cpcmd &= 0x2063;

	switch (lp->chip->type) {
	case R8101_4: /* RTL8102E */
		/* set PCI configuration space offset 0x70F to 0x17 */
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x17000000);

		if (pci_config_get8(conf_ha, 0x81) == 1) {
			pci_config_put8(conf_ha, 0x81, 0);

			OUTB(dp, DBGREG, 0x98);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_2);

			pci_config_put8(conf_ha, 0x81, 1);
		}
#if 0
		/*
		 * Set PCI configuration space offset 0x70 to 0x50
		 * Increase the Tx performance
		 */
		pci_config_get8(conf_ha, 0x79, &device_control);
		device_control &= ~0x70;
		device_control |= 0x50;
		pci_config_put8(conf_ha, 0x79, device_control);
#endif

		OUTB(dp, CFG1, 0x0f);
		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		gani_ephy_write(dp, 0x03, 0xc2f9);
		break;

	case R8101_5: /* RTL8102E rev.B */
		if (pci_config_get8(conf_ha, 0x81) == 1) {
			pci_config_put8(conf_ha, 0x81, 0);

			OUTB(dp, DBGREG, 0x98);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_2);

			pci_config_put8(conf_ha, 0x81, 1);
		}
#if 0
		/*
		 * Set PCI configuration space offset 0x70 to 0x50
		 * Increase the Tx performance
		 */
		pci_config_get8(conf_ha, 0x79, &device_control);
		device_control &= ~0x70;
		device_control |= 0x50;
		pci_config_put8(conf_ha, 0x79, device_control);
#endif

		OUTB(dp, CFG1, 0x0f);
		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		gani_ephy_write(dp, 0x01, 0x6fe5);
		gani_ephy_write(dp, 0x03, 0xd7d9);
		break;

	case R8101_6: /* RTL8103E */
		if (pci_config_get8(conf_ha, 0x81) == 1) {
			pci_config_put8(conf_ha, 0x81, 0);

			OUTB(dp, DBGREG, 0x98);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_2);

			pci_config_put8(conf_ha, 0x81, 1);
		}
#if 0
		/*
		 * Set PCI configuration space offset 0x70 to 0x50
		 * Increase the Tx performance
		 */
		pci_config_get8(conf_ha, 0x79, &device_control);
		device_control &= ~0x70;
		device_control |= 0x50;
		pci_config_put8(conf_ha, 0x79, device_control);
#endif
		OUTB(dp, 0xf4, 0x01);
		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		gani_ephy_write(dp, 0x06, 0xaf35);
		break;

	case R8101_7: /* RTL8103E */
		if (pci_config_get8(conf_ha, 0x81) == 1) {
			pci_config_put8(conf_ha, 0x81, 0);

			OUTB(dp, DBGREG, 0x98);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_2);

			pci_config_put8(conf_ha, 0x81, 1);
		}
#if 0
		/*
		 * Set PCI configuration space offset 0x70 to 0x50
		 * Increase the Tx performance
		 */
		pci_config_get8(conf_ha, 0x79, &device_control);
		device_control &= ~0x70;
		device_control |= 0x50;
		pci_config_put8(conf_ha, 0x79, device_control);
#endif
		OUTB(dp, 0xf4, 0x01);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		OUTB(dp, 0xf5, INB(dp, 0xf5) | BIT_2);
		gani_ephy_write(dp, 0x19, 0xec90);
		gani_ephy_write(dp, 0x01, 0x6fe5);
		gani_ephy_write(dp, 0x03, 0x05d9);
		gani_ephy_write(dp, 0x06, 0xaf35);
		break;

	case R8101_8: /* RTL8103E */
		if (pci_config_get8(conf_ha, 0x81) == 1) {
			pci_config_put8(conf_ha, 0x81, 0);

			OUTB(dp, DBGREG, 0x98);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_2);
			OUTB(dp, 0xF4, INB(dp, 0xF4) | BIT_3);
			OUTB(dp, 0xF5, INB(dp, 0xF5) | BIT_2);

			pci_config_put8(conf_ha, 0x81, 1);
			if (gani_ephy_read(dp, 0x10) == 0x0008) {
				gani_ephy_write(dp, 0x10, 0x000C);
			}
		}

		if (pci_config_get8(conf_ha, 0x80) & 3) {
			gani_ephy_write(dp, 0x02, 0x011F);
		}
#if 0
		/*
		 * Set PCI configuration space offset 0x70 to 0x50
		 * Increase the Tx performance
		 */
		pci_config_get8(conf_ha, 0x79, &device_control);
		device_control &= ~0x70;
		device_control |= 0x50;
		pci_config_put8(conf_ha, 0x79, device_control);
#endif
		OUTB(dp, 0xf4, 0x01);
		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		gani_ephy_write(dp, 0x01, 0x6fe5);
		gani_ephy_write(dp, 0x03, 0x05d9);
		gani_ephy_write(dp, 0x06, 0xaf35);
		gani_ephy_write(dp, 0x19, 0xecfa);
		break;

	case R8101_9: /* RTL8401 */
		if (pci_config_get8(conf_ha, 0x81) == 1) {
			pci_config_put8(conf_ha, 0x81, 0);

			OUTB(dp, DBGREG, 0x98);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
			OUTB(dp, CFG4, INB(dp, CFG4) | BIT_2);

			pci_config_put8(conf_ha, 0x81, 1);
		}
#if 0
		/*
		 * Set PCI configuration space offset 0x70 to 0x50
		 * Increase the Tx performance
		 */
		pci_config_get8(conf_ha, 0x79, &device_control);
		device_control &= ~0x70;
		device_control |= 0x50;
		pci_config_put8(conf_ha, 0x79, device_control);
#endif
		OUTB(dp, 0xf4, 0x01);
		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		gani_ephy_write(dp, 0x06, 0xaf25);
		gani_ephy_write(dp, 0x07, 0x8e68);
		break;

	case R8101_10:
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);

		OUTB(dp, MTPS, 0x0C);
		/* Set PCI configuration space offset 0x79 to 0x50 */
		pci_config_put8(conf_ha, 0x79, 0x50);

		/* tx checksum offload enable */

		OUTB(dp, 0xF3, INB(dp, 0xF3) | BIT_5);
		OUTB(dp, 0xF3, INB(dp, 0xF3) & ~BIT_5);

		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_7 | BIT_6);

		OUTB(dp, 0xF1, INB(dp, 0xF1)
		    | BIT_7 | BIT_6 | BIT_5 | BIT_4 | BIT_2 | BIT_1);

		OUTB(dp, CFG5, (INB(dp, CFG5)&~0x08) | BIT_0);
		OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);

		OUTB(dp, CFG3, INB(dp, CFG3) & ~CFG3_BEACON_EN);

		/* set EPHY registers */
		data16 = gani_ephy_read(dp, 0x00) & ~0x0200;
		data16 |= 0x0100;
		gani_ephy_write(dp, 0x00, data16);

		data16 = gani_ephy_read(dp, 0x00);
		data16 |= 0x0004;
		gani_ephy_write(dp, 0x00, data16);

		data16 = gani_ephy_read(dp, 0x06) & ~0x0002;
		data16 |= 0x0001;
		gani_ephy_write(dp, 0x06, data16);

		data16 = gani_ephy_read(dp, 0x06);
		data16 |= 0x0030;
		gani_ephy_write(dp, 0x06, data16);

		data16 = gani_ephy_read(dp, 0x07);
		data16 |= 0x2000;
		gani_ephy_write(dp, 0x07, data16);

		data16 = gani_ephy_read(dp, 0x00);
		data16 |= 0x0020;
		gani_ephy_write(dp, 0x00, data16);

		data16 = gani_ephy_read(dp, 0x03) & ~0x5800;
		data16 |= 0x2000;
		gani_ephy_write(dp, 0x03, data16);

		data16 = gani_ephy_read(dp, 0x03);
		data16 |= 0x0001;
		gani_ephy_write(dp, 0x03, data16);

		data16 = gani_ephy_read(dp, 0x01) & ~0x0800;
		data16 |= 0x1000;
		gani_ephy_write(dp, 0x01, data16);

		data16 = gani_ephy_read(dp, 0x07);
		data16 |= 0x4000;
		gani_ephy_write(dp, 0x07, data16);

		data16 = gani_ephy_read(dp, 0x1E);
		data16 |= 0x2000;
		gani_ephy_write(dp, 0x1E, data16);

		gani_ephy_write(dp, 0x19, 0xFE6C);

		data16 = gani_ephy_read(dp, 0x0A);
		data16 |= 0x0040;
		gani_ephy_write(dp, 0x0A, data16);

		break;

	case R8101_11:
	case R8101_12:
	case R8101_13:
#if 0
		tp->cp_cmd &= 0x2063;

		/* tx checksum offload enable */
#endif
		if (pci_config_get8(conf_ha, 0x80) & 0x03) {
			OUTB(dp, CFG5, INB(dp, CFG5) | BIT_0);
			OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_7);
			OUTB(dp, 0xF1, INB(dp, 0xF1) | BIT_7);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
		}

		OUTB(dp, 0xF1, INB(dp, 0xF1) | BIT_5 | BIT_3);
		OUTB(dp, 0xF2, INB(dp, 0xF2) & ~BIT_0);
		OUTB(dp, 0xD3, INB(dp, 0xD3) | BIT_3 | BIT_2);
		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_6);
		OUTW(dp, 0xE0, INW(dp, 0xE0) & ~0xDF9C);

		data16 = gani_ephy_read(dp, 0x07);
		data16 |= 0x4000;
		gani_ephy_write(dp, 0x07, data16);

		data16 = gani_ephy_read(dp, 0x19);
		data16 |= 0x0200;
		gani_ephy_write(dp, 0x19, data16);

		data16 = gani_ephy_read(dp, 0x19);
		data16 |= 0x0020;
		gani_ephy_write(dp, 0x19, data16);

		data16 = gani_ephy_read(dp, 0x1E);
		data16 |= 0x2000;
		gani_ephy_write(dp, 0x1E, data16);

		data16 = gani_ephy_read(dp, 0x03);
		data16 |= 0x0001;
		gani_ephy_write(dp, 0x03, data16);

		data16 = gani_ephy_read(dp, 0x19);
		data16 |= 0x0100;
		gani_ephy_write(dp, 0x19, data16);

		data16 = gani_ephy_read(dp, 0x19);
		data16 |= 0x0004;
		gani_ephy_write(dp, 0x19, data16);

		data16 = gani_ephy_read(dp, 0x0A);
		data16 |= 0x0020;
		gani_ephy_write(dp, 0x0A, data16);

		if (lp->chip->type == R8101_11) {
			OUTB(dp, CFG5, INB(dp, CFG5) & ~BIT_0);
		} else if (lp->chip->type == R8101_12 ||
		    lp->chip->type == R8101_13) {
			data16 = gani_ephy_read(dp, 0x1E);
			data16 |= 0x8000;
			gani_ephy_write(dp, 0x1E, data16);
		}

		if (lp->chip->type == R8101_13) {
			if ((INB(dp, 0x8c) & BIT_28) &&
			    (INB(dp, 0xEF) & BIT_2) == 0) {
				gani_mii_write_raw(dp, 0x1F, 0x0005);
				data16 = gani_mii_read_raw(dp, 0x1a);
				data16 &= ~(BIT_8 | BIT_0);
				gani_mii_write_raw(dp, 0x1a, data16);
				gani_mii_write_raw(dp, 0x1f, 0x0000);
			} else if (INB(dp, 0xEF) & BIT_2) {
				gani_mii_write_raw(dp, 0x1F, 0x0001);
				data16 = gani_mii_read_raw(dp, 0x1B) | BIT_2;
				gani_mii_write_raw(dp, 0x1B, data16);
				gani_mii_write_raw(dp, 0x1F, 0x0000);
			}
		}
		break;

	case R8101_14:
		val = gani_csi_read(dp, 0x70c) & 0x00ffffff;
		gani_csi_write(dp, 0x70c, val | 0x27000000);
#if 0
		/*
		 * Set PCI configuration space offset 0x70 to 0x50
		 * Increase the Tx performance
		 */
		pci_config_get8(conf_ha, 0x79, &device_control);
		device_control &= ~0x70;
		device_control |= 0x50;
		pci_config_put8(conf_ha, 0x79, device_control);
#endif
		gani_eri_write(dp, 0xC8, 4, 0x00000002, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xE8, 4, 0x00000006, ERIAR_ExGMAC);
		OUTL(dp, TCR, INL(dp, TCR) | BIT_7);
		OUTB(dp, 0xD3, INB(dp, 0xD3) & ~BIT_7);

		val = gani_eri_read(dp, 0xDC, 1, ERIAR_ExGMAC);
		val &= ~BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);
		val |= BIT_0;
		gani_eri_write(dp, 0xDC, 1, val, ERIAR_ExGMAC);

		gani_ephy_write(dp, 0x19, 0xff64);

		OUTB(dp, CFG5, INB(dp, CFG5) | BIT_0);
		OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);

		gani_eri_write(dp, 0xC0, 2, 0x00000000, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xB8, 2, 0x00000000, ERIAR_ExGMAC);
		gani_eri_write(dp, 0xD5, 1, 0x0000000E, ERIAR_ExGMAC);

		break;

	case R8101_15:
	case R8101_16:
#if 0
		tp->cp_cmd &= 0x2063;

		/* tx checksum offload enable */
		dev->features |= NETIF_F_IP_CSUM;
#endif
		if (pci_config_get8(conf_ha, 0x80) & 0x3) {
			OUTB(dp, CFG5, INB(dp, CFG5) | BIT_0);
			OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_7);
			OUTB(dp, 0xF1, INB(dp, 0xF1) | BIT_7);
			OUTB(dp, CFG2, INB(dp, CFG2) | BIT_7);
		}

		OUTB(dp, 0xF1, INB(dp, 0xF1) | BIT_5 | BIT_3);
		OUTB(dp, 0xF2, INB(dp, 0xF2) & ~BIT_0);
		OUTB(dp, 0xD3, INB(dp, 0xD3) | BIT_3 | BIT_2);
		OUTB(dp, 0xD0, INB(dp, 0xD0) & ~BIT_6);
		OUTW(dp, 0xE0, INW(dp, 0xE0) & ~0xDF9C);

		break;
	}

	OUTB(dp, ETThR, 0x3f);

	OUTW(dp, CpCR, cpcmd);

	/* intermitigate */
	OUTW(dp, 0xe2, 0);

	/* Transmit Normal Priority Descriptors */
	DPRINTF(4, (CE_CONT, "!%s: TNPDS 0x%llx",
	    dp->name, (unsigned long long)dp->tx_ring_dma));
	OUTL(dp, TNPDSL, (uint32_t)dp->tx_ring_dma);
	OUTL(dp, TNPDSH, (uint32_t)(dp->tx_ring_dma >> 32U));

	/* Receive Descriptor Start Address register */
	OUTL(dp, RDSARL, (uint32_t)dp->rx_ring_dma);
	OUTL(dp, RDSARH, (uint32_t)(dp->rx_ring_dma >> 32U));
	DPRINTF(4, (CE_CONT, "!%s: RDSAR 0x%llx",
	    dp->name, (unsigned long long)dp->rx_ring_dma));

	/* need to enable tx and rx before writing rcr and tcr */
	OUTB(dp, CR, CR_RE | CR_TE);

	OUTL(dp, RCR, lp->rcr);
	OUTL(dp, TCR, lp->tcr);

	/* make config registers read only */
	OUTB(dp, CR9346, CR9346_EEM_NORMAL);
	drv_usecwait(10);

	/* TimerInt Register */
	OUTL(dp, TimerInt, 0);

	/* Timer count register: test count rate */
	OUTL(dp, TCTR, 0);
#if DEBUG_LEVEL > 3
	drv_usecwait(1000);
	cmn_err(CE_CONT, "!%s: TCTR:%d in 1mS", dp->name, INL(dp, TCTR));
#endif

	if (gani_eee_enable) {
		(void) gani_enable_EEE(dp);
	} else {
		(void) gani_disable_EEE(dp);
	}

	return (GEM_SUCCESS);
}

static int
gani_init_chip(struct gem_dev *dp)
{
	int	ret;
	ddi_acc_handle_t	conf_ha;
	struct gani_dev	*lp = dp->private;

	DPRINTF(0, (CE_CONT, "!%s: %s: called", dp->name, __func__));

	if (pci_config_setup(dp->dip, &conf_ha) != DDI_SUCCESS) {
		cmn_err(CE_WARN, "!%s: pci_config_setup failed", dp->name);
		return (GEM_FAILURE);
	}

	gani_powerup_pll(dp);

	if (GANI_PCI(lp)) {
		ret = gani_init_8169(dp, conf_ha);
	} else if (GANI_PCIE_GIGA(lp)) {
		ret = gani_init_8168(dp, conf_ha);
	} else {
		ret = gani_init_8101(dp, conf_ha);
	}
	pci_config_teardown(&conf_ha);

#ifdef DEBUG_HANG
	gani_dump_regs(dp, "init_chip done");
#endif
#ifdef CONFIG_POLLING
	lp->last_poll_interval = 0; /* polling interval in nS */
#endif
	lp->last_stats_valid = B_FALSE;

	return (ret);
}

static uint_t
gani_mcast_hash(struct gem_dev *dp, uint8_t *addr)
{
	uint_t	h;
	struct gani_dev	*lp = dp->private;

	h = gem_ether_crc_be(addr, ETHERADDRL) >> (32 - 6);

	return (GANI_PCIE(lp) ? (h ^ 0x38) : h);
}

static int
gani_set_rx_filter(struct gem_dev *dp)
{
	uint32_t	mode;
	uint64_t	mhash;
	int		i;
	uint8_t		*m;
	struct gani_dev	*lp = dp->private;

	DPRINTF(1, (CE_CONT, "!%s: %s: called", dp->name, __func__));

	mode = RCR_AB	/* accept broadcast */
	    | RCR_APM	/* accept physical match  */
	    | RCR_AM;	/* accept multicast */

	mhash = 0;

	if ((dp->rxmode & RXMODE_ENABLE) == 0) {
		mode = 0;

	} else if (dp->rxmode & RXMODE_PROMISC) {
		/* promiscious mode implies all multicast and all physical */
		mode |= RCR_AAP;
		mhash = ~0ULL;

	} else if ((dp->rxmode & RXMODE_ALLMULTI) || dp->mc_count > 32) {
		/* accept all multicast packets */
		mhash = ~0ULL;

	} else if (dp->mc_count > 0) {
		/*
		 * make the hash table to select the interresting
		 * multicast address only.
		 */
		for (i = 0; i < dp->mc_count; i++) {
			/* hash table is 64 = 2^6 bit width */
			mhash |= 1ULL << dp->mc_list[i].hash;
		}
	}

	/* 035 */
	if (lp->chip->type == R8168_16 && dp->speed == GEM_SPD_10) {
		mode |= RCR_AAP;
	}

	/*
	 * Don't disable the rx filter not to drop packets
	 * while we are changing rx filer mode.
	 */

	m = &dp->cur_addr.ether_addr_octet[0];
	if (bcmp(m, lp->mac, ETHERADDRL)) {
		/*
		 * make config registers writable before
		 * changing IDRs. (undocumented)
		 */
		OUTB(dp, CR9346, CR9346_EEM_WE);

		/* set mac address */
		OUTL(dp, IDR + 0,
		    (m[3] << 24) | (m[2] << 16) | (m[1] << 8) | m[0]);
		OUTL(dp, IDR + 4, (m[5] <<  8) | m[4]);

		OUTB(dp, CR9346, CR9346_EEM_NORMAL);
		drv_usecwait(10);
	}

	if (mode & RCR_AM) {
		/* need to set up multicast hash table */
		OUTL(dp, MAR + 4, (uint32_t)(mhash >> 32U));
		OUTL(dp, MAR + 0, (uint32_t)mhash);
	}

	/* update rcr */
	OUTL(dp, RCR, lp->rcr | mode);

	DPRINTF(4, (CE_CONT, "!%s: returned", __func__));
#ifdef DEBUG_HANG
	gani_dump_regs(dp, "set_rxfilter done");
#endif

	return (GEM_SUCCESS);
}


#define	MII_ABILITY_SPEED_DUPLEX	\
	(MII_ABILITY_100BASE_T4	|	\
	MII_ABILITY_100BASE_TX_FD |	\
	MII_ABILITY_100BASE_TX |	\
	MII_ABILITY_10BASE_T |	\
	MII_ABILITY_10BASE_T_FD)

static int
gani_set_media(struct gem_dev *dp)
{
	uint32_t	data32;
	struct gani_dev	*lp = dp->private;
#ifdef GEM3
	DPRINTF(0, (CE_CONT, "!%s: %s: phys: %b, mac_state: %d",
	    dp->name, __func__, INB(dp, PHYS), PHYS_BITS, dp->mac_state));
#else
	DPRINTF(0, (CE_CONT, "!%s: %s: phys: %b, mac_active: %d",
	    dp->name, __func__, INB(dp, PHYS), PHYS_BITS, dp->mac_active));
#endif
	switch (lp->chip->type) {
	/* 017 */
	case R8169_4:
		if (dp->speed == GEM_SPD_10 &&
		    (INB(dp, CFG2) & CFG2_PCICLKF_66)) {
			lp->tcr &= ~TCR_MXDMA;
		} else {
			lp->tcr |= TCR_MXDMA;
		}
		OUTL(dp, TCR, lp->tcr);
		break;

	case R8169_5:
	case R8169_6:
		if (dp->full_duplex) {
			lp->tcr = (lp->tcr & ~TCR_IFG) | TCR_IFG_802_3;
		} else {
			lp->tcr = (lp->tcr & ~TCR_IFG) | TCR_IFG_HALF_8169;
		}
		OUTL(dp, TCR, lp->tcr);
		break;
#if 0
	case R8101_4:
	case R8101_5:
		if (gani_csi_read(dp, 0x70c) != 0x27000000) {
			gani_csi_write(dp, 0x70c, 0x27000000);
		}
		break;
#endif
	case R8101_11:
	case R8101_12:
	case R8101_13:
		if (!dp->full_duplex && gani_eee_enable == 1) {
			(void) gani_disable_EEE(dp);
		}

		if (dp->speed == GEM_SPD_10) {
			gani_eri_write(dp, 0x1D0, 2, 0x4D02, ERIAR_ExGMAC);
			gani_eri_write(dp, 0x1DC, 2, 0x0060, ERIAR_ExGMAC);

			gani_eri_write(dp, 0x1B0, 2, 0, ERIAR_ExGMAC);
			gani_mii_write_raw(dp, 0x1F, 0x0004);
			data32 = gani_mii_read_raw(dp, 0x10);
			data32 |= 0x0400;
			data32 &= ~0x0800;
			gani_mii_write_raw(dp, 0x10, data32);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		} else {
			gani_eri_write(dp, 0x1D0, 2, 0, ERIAR_ExGMAC);
			if (gani_eee_enable == 1 &&
			    (INB(dp, 0xEF) & BIT_0) == 0) {
				gani_eri_write(dp,
				    0x1B0, 2, 0xED03, ERIAR_ExGMAC);
			}
		}
		break;

	case R8101_14:
	case R8101_15:
	case R8101_16:
		if (dp->speed == GEM_SPD_10) {
			gani_eri_write(dp, 0x1D0, 2, 0x4d02, ERIAR_ExGMAC);
			gani_eri_write(dp, 0x1DC, 2, 0x0060, ERIAR_ExGMAC);
		} else {
			gani_eri_write(dp, 0x1D0, 2, 0, ERIAR_ExGMAC);
		}
		break;

	case R8168_11:
		if (dp->speed == GEM_SPD_10) {
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x10, 0x04ee);
		} else {
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x10, 0x01ee);
		}
		break;

	case R8168_18:
	case R8168_19:
	case R8168_20:
		if (dp->speed == GEM_SPD_1000) {
			gani_eri_write(dp, 0x1bc, 4, 0x11, ERIAR_ExGMAC);
			gani_eri_write(dp, 0x1dc, 4, 0x05, ERIAR_ExGMAC);
		} else {
			gani_eri_write(dp, 0x1bc, 4, 0x1f, ERIAR_ExGMAC);
			gani_eri_write(dp, 0x1dc, 4, 0x3f, ERIAR_ExGMAC);
		}
#if 0
		if ((INB(dp, ChipCmd) & (CmdRxEnb | CmdTxEnb)) == 0) {
			int timeout;
			for (timeout = 0; timeout < 10; timeout++) {
				if ((gani_eri_read(dp,
				    0x1AE, 4, ERIAR_ExGMAC) & BIT_13) == 0) {
					break;
				}
				drv_usecwait(1000);
			}
			rtl8168_init_ring_indexes(tp);
			OUTB(dp, ChipCmd, CmdRxEnb | CmdTxEnb);
		}
#endif
		break;

	case R8168_16:
	case R8168_17: {
		uint32_t eri_data;

		if (lp->chip->type == R8168_16) {
			(void) gani_set_rx_filter(dp);
		} else if (lp->chip->type == R8168_17) {
			if (dp->speed == GEM_SPD_1000) {
				gani_eri_write(dp,
				    0x1bc, 4, 0x00000011, ERIAR_ExGMAC);
				gani_eri_write(dp,
				    0x1dc, 4, 0x00000005, ERIAR_ExGMAC);
			} else if (dp->speed == GEM_SPD_100) {
				gani_eri_write(dp,
				    0x1bc, 4, 0x0000001f, ERIAR_ExGMAC);
				gani_eri_write(dp,
				    0x1dc, 4, 0x00000005, ERIAR_ExGMAC);
			} else {
				gani_eri_write(dp,
				    0x1bc, 4, 0x0000001f, ERIAR_ExGMAC);
				gani_eri_write(dp,
				    0x1dc, 4, 0x0000003f, ERIAR_ExGMAC);
			}
		}

		eri_data = gani_eri_read(dp, 0xDC, 1, ERIAR_ExGMAC);
		eri_data &= ~BIT_0;
		gani_eri_write(dp, 0xDC, 1, eri_data, ERIAR_ExGMAC);
		eri_data |= BIT_0;
		gani_eri_write(dp, 0xDC, 1, eri_data, ERIAR_ExGMAC);
#if 0
		if ((INB(dp, ChipCmd) & (CmdRxEnb | CmdTxEnb)) == 0) {
			int timeout;
			for (timeout = 0; timeout < 10; timeout++) {
				if ((gani_eri_read(dp, 0x1AE, 4, ERIAR_ExGMAC)
				    & BIT_13) == 0) {
					break;
				}
				mdelay(1);
			}
			rtl8168_init_ring_indexes(tp);
			OUTB(dp, ChipCmd, CmdRxEnb | CmdTxEnb);
		}
#endif
		break;
	}
	case R8168_14:
	case R8168_15:
		if (gani_eee_enable) {
			/* patch inter gap */
			if (dp->full_duplex) {
				gani_mii_write_raw(dp, 0x1F, 0x0006);
				gani_mii_write_raw(dp, 0x00, 0x5a30);
				gani_mii_write_raw(dp, 0x1F, 0x0000);

				if (dp->speed == GEM_SPD_10 ||
				    dp->speed == GEM_SPD_100) {
					lp->tcr = (lp->tcr & ~TCR_IFG)
					    | BIT_25;
				} else {
					lp->tcr = (lp->tcr & ~TCR_IFG)
					    | TCR_IFG_802_3;
				}
			} else {
				gani_mii_write_raw(dp, 0x1F, 0x0006);
				gani_mii_write_raw(dp, 0x00, 0x5a00);
				gani_mii_write_raw(dp, 0x1F, 0x0000);

				lp->tcr = (lp->tcr & ~TCR_IFG)
				    | TCR_IFG_802_3;
			}
			OUTL(dp, TCR, lp->tcr);
		}
		break;

	case R8168_21:
	case R8168_22:
	case R8168_24:
#if 0
		if ((INB(dp, ChipCmd) & (CmdRxEnb | CmdTxEnb)) == 0) {
			rtl8168_init_ring_indexes(tp);
			rtl8168_desc_addr_fill(tp);
			OUTB(dp, ChipCmd, CmdRxEnb | CmdTxEnb);
		}
#endif
		break;

	case R8168_23:
		OUTL(dp, ERIDR, 0x00000000);
		OUTL(dp, ERIAR, 0x8042f108);
		break;
#ifdef notdef
		/*
		 * following code are required on link down
		 */
		if (netif_msg_ifdown(tp))
			printk(KERN_INFO PFX "%s: link down\n", dev->name);
		netif_carrier_off(dev);
		if (lp->chip->type == R8168_23) {
			OUTL(dp, ERIDR, 0x00000001);
			OUTL(dp, ERIAR, 0x8042f108);
		}
#endif
	}
#if 0
	if (GANI_PCIE_FAST(lp)) {
		/* from rtl8101_aspm_fix2(struct net_device *dev) */
		uint32_t csi_tmp;

		csi_tmp = rtl8101_csi_read(tp, 0x70c) & 0x00ffffff;
		if (B_TRUE /* linkup */) {
			rtl8101_csi_write(tp, 0x70c, csi_tmp | 0x3F000000);
		} else {
			rtl8101_csi_write(tp, 0x70c, csi_tmp | 0x17000000);
		}
	}
#endif
	return (GEM_SUCCESS);
}

static int
gani_start_chip(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	DPRINTF(0, (CE_CONT, "!%s: %s: called", dp->name, __func__));

	ASSERT(dp->mii_state == MII_STATE_LINKUP ||
	    dp->mii_state == MII_STATE_LINKDOWN);

	/* prepare to dump Tally Count Registers */
	OUTL(dp, DTCCRH, (uint32_t)(TALLY_DMA_ADDR(dp) >> 32));
#ifdef SANITY
	OUTL(dp, DTCCRL, 0);
#endif
	/* clear tally counter area */
	DPRINTF(2, (CE_CONT, "!%s: tally base: vaddr:0x%p, dma addr:0x%llx",
	    dp->name, TALLY_VADDR(dp),
	    (unsigned long long)TALLY_DMA_ADDR(dp)));
	bzero(TALLY_VADDR(dp), TC_SIZE);

	/* Enable interrupts */
	lp->imr = OUR_INTR_MASK;
	if (GANI_PCIE(lp)) {
		lp->imr |= INTR_TDU;
	}
#ifdef CONFIG_POLLING
	lp->imr |= INTR_TimeOut;
#endif
	if ((dp->misc_flag & GEM_NOINTR) == 0) {
		OUTW(dp, IMR, lp->imr);
		FLSHW(dp, IMR);
	}
#ifdef DEBUG_HANG
	gani_dump_regs(dp, "start_chip done");
#endif
	return (GEM_SUCCESS);
}

/*
 * gani_stop_chip: disable interrupts and ensure rx has stopped.
 */
static int
gani_stop_chip(struct gem_dev *dp)
{
	int		i;
	uint8_t		*m;
	struct gani_dev	*lp = dp->private;

	DPRINTF(0, (CE_CONT, "!%s: %s: called", dp->name, __func__));
#if DEBUG_LEVEL > 1
	if (dp->tx_desc_head != dp->tx_desc_tail) {
		gani_tx_desc_dump(dp);
	}
#endif
	/* wait until dumping tally counters stops. */
	for (i = 0; INL(dp, DTCCRL) & DTCCR_CMD; i++) {
		if (i > 100) {
			cmn_err(CE_WARN, "!%s: timeout: dumping tally counters",
			    dp->name);
			break;
		}
		drv_usecwait(10);
	}

	/* disable interrupts */
	/* XXX - don't clear pended interrupts */
	OUTW(dp, IMR, 0);
	FLSHW(dp, IMR);

	switch (lp->chip->type) {
	case R8101_4:
	case R8101_5:
		if (gani_csi_read(dp, 0x70c) != 0x17000000) {
			gani_csi_write(dp, 0x70c, 0x17000000);
		}
		break;
	}

	/* stop nic core */
	(void) (dp->gc.gc_reset_chip)(dp);

	/* restore factory mac address */
	OUTB(dp, CR9346, CR9346_EEM_WE);
	m = dp->dev_addr.ether_addr_octet;
	OUTL(dp, IDR + 0,
	    (m[3] << 24) | (m[2] << 16) | (m[1] << 8) | m[0]);
	OUTL(dp, IDR + 4, (m[5] << 8) | m[4]);
	OUTB(dp, CR9346, CR9346_EEM_NORMAL);

	gani_powerdown_pll(dp);
#ifdef DEBUG_HANG
	gani_dump_regs(dp, "stop_chip done");
#endif
	return (GEM_SUCCESS);
}

/* private function */
static boolean_t
gani_update_stats(struct gem_dev *dp)
{
	uint32_t			x;
	struct rtl8169_tally_counters	*tbp;
	struct rtl8169_tally_counters	*ls;
	struct rtl8169_tally_counters	new;
	struct gani_dev			*lp = dp->private;

	DPRINTF(4, (CE_CONT, "!%s: %s: called", dp->name, __func__));

	ASSERT(mutex_owned(&lp->stat_lock));

	if (!IS_MAC_ONLINE(dp)) {
		/* nic isn't running yet */
		return (B_FALSE);
	}

	if (INL(dp, DTCCRL) & DTCCR_CMD) {
		/* Now dumping tally counters, do nothing */
		return (B_FALSE);
	}

	(void) ddi_dma_sync(dp->desc_dma_handle,
	    (off_t)TALLY_OFFSET(dp), (size_t)TC_SIZE, DDI_DMA_SYNC_FORKERNEL);

	tbp = TALLY_VADDR(dp);

	new.tc_txok = LE_64(tbp->tc_txok);		/* 8byte */
	new.tc_rxok = LE_64(tbp->tc_rxok);		/* 8byte */
	new.tc_txer = LE_64(tbp->tc_txer);		/* 8byte */
	new.tc_rxer = LE_32(tbp->tc_rxer);		/* 4byte */
	new.tc_misspkt = LE_16(tbp->tc_misspkt);	/* 2byte */
	new.tc_fae = LE_16(tbp->tc_fae);		/* 2byte */
	new.tc_tx1col = LE_32(tbp->tc_tx1col);		/* 4byte */
	new.tc_txmcol = LE_32(tbp->tc_txmcol);		/* 4byte */
	new.tc_rxokphy = LE_64(tbp->tc_rxokphy);	/* 8byte */
	new.tc_rxokbrd = LE_64(tbp->tc_rxokbrd);	/* 8byte */
	new.tc_rxokmu = LE_32(tbp->tc_rxokmu);		/* 4byte */
	new.tc_txabt = LE_16(tbp->tc_txabt);		/* 2byte */
	new.tc_txundrn = LE_16(tbp->tc_txundrn);	/* 2byte */

	if (lp->last_stats_valid) {
		struct gem_stats	*gsp = &dp->stats;
#if 0
		ls = &lp->last_stat;
		gsp->errxmt += new.tc_txer - ls->tc_txer;
		gsp->errrcv += new.tc_rxer - ls->tc_rxer;
		gsp->missed += new.tc_misspkt - ls->tc_misspkt;
		gsp->frame += new.tc_fae - ls->tc_fae;
		gsp->first_coll +=	x = new.tc_tx1col - ls->tc_tx1col;
		gsp->collisions +=	x;
		gsp->multi_coll +=	x = new.tc_txmcol - ls->tc_txmcol;
		gsp->collisions +=	x*2;
		gsp->excoll += new.tc_txabt - ls->tc_txabt;
		gsp->underflow += new.tc_txundrn - ls->tc_txundrn;
#else
		uint64_t	txer;
		uint64_t	txok;
		uint64_t	rxer;
		uint64_t	rxok;

		ls = &lp->last_stat;

		txer = new.tc_txer - ls->tc_txer;
		txok = new.tc_txok - ls->tc_txok;
		rxer = new.tc_rxer - ls->tc_rxer;
		rxok = new.tc_rxok - ls->tc_rxok;

		gsp->errrcv += rxer;
		gsp->frame += new.tc_fae - ls->tc_fae;
		gsp->missed += new.tc_misspkt - ls->tc_misspkt;

		gsp->errxmt += txer;
		gsp->first_coll += x = new.tc_tx1col - ls->tc_tx1col;
		gsp->collisions += x;
		gsp->multi_coll += x = new.tc_txmcol - ls->tc_txmcol;
		gsp->collisions += x*2;
		gsp->excoll += new.tc_txabt - ls->tc_txabt;

		gsp->underflow += new.tc_txundrn - ls->tc_txundrn;
#ifdef NEVER
		gsp->rpackets += rxok + rxer;
		gsp->opackets += txok + txer;
#endif
#endif
	}

	lp->last_stat = new;
	lp->last_stats_valid = B_TRUE;

	/* issue Dump_Tally_Counters_Cmd for the next call */
	bzero(tbp, sizeof (struct rtl8169_tally_counters));
	OUTL(dp, DTCCRL, DTCCR_CMD | (uint32_t)TALLY_DMA_ADDR(dp));

	return (B_TRUE);
}

/* private function */
static void
gani_stat_timeout(void *arg)
{
	struct gem_dev	*dp = arg;
	struct gani_dev	*lp = dp->private;

	mutex_enter(&lp->stat_lock);
	(void) gani_update_stats(dp);
	lp->stat_to_id = timeout(gani_stat_timeout, dp, ONESEC);
	mutex_exit(&lp->stat_lock);
}

static int
gani_get_stats(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	DPRINTF(4, (CE_CONT, "!%s: %s: called", dp->name, __func__));

	mutex_enter(&lp->stat_lock);
	(void) gani_update_stats(dp);
	mutex_exit(&lp->stat_lock);

	return (GEM_SUCCESS);
}

/*
 * discriptor  manupiration
 */
static int
gani_tx_desc_write(struct gem_dev *dp, int slot,
    ddi_dma_cookie_t *dmacookie, int frags, uint64_t flag)
{
	int			i;
	struct tx_desc		*tdp;
	uint32_t		mark;
	uint32_t		vlan_tag = 0;
	struct gani_dev		*lp = dp->private;
	int			curslot;
	uint_t			tx_ring_size = dp->gc.gc_tx_ring_size;
#ifdef GEM_CONFIG_VLAN_HW
#ifdef GEM_CONFIG_CKSUM_OFFLOAD
	uint32_t		mss;
#endif /* GEM_CONFIG_CKSUM_OFFLOAD */
#endif /* GEM_CONFIG_VLAN_HW */
	uint32_t		tmp0;
	uint32_t		tmp1;
	uint32_t		tmp2;

	/*
	 * write tx descriptor(s) in reversed order
	 */
	mark = TXD0_OWN | TXD0_LS;
#ifdef GEM_CONFIG_VLAN_HW
#ifdef CONFIG_CKSUM_OFFLOAD
	mss = (GEM_TXFLAG_MSS & flag) >> GEM_TXFLAG_MSS_SHIFT;
	if (mss) {
		mark |= (mss << TXD0_LGSMSS_SHIFT) | TXD0_LGSEN;
	} else if (!lp->new_cksum) {
		if (flag & GEM_TXFLAG_IPv4) {
			mark |= TXD0_IPCS;
		}
		if (flag & GEM_TXFLAG_TCP) {
			mark |= TXD0_TCPCS;
		} else if (flag & GEM_TXFLAG_UDP) {
			mark |= TXD0_UDPCS;
		}
	} else {
		if (flag & GEM_TXFLAG_IPv4) {
			vlan_tag |= TXD1_IPCS;
		}
		if (flag & GEM_TXFLAG_TCP) {
			vlan_tag |= TXD1_TCPCS;
		} else if (flag & GEM_TXFLAG_UDP) {
			vlan_tag |= TXD1_UDPCS;
		}
	}
#endif /* GEM_CONFIG_CKSUM_OFFLOAD */
	if (flag & GEM_TXFLAG_VTAG) {
		uint32_t	vtag;
		vtag = (flag & GEM_TXFLAG_VTAG) >> GEM_TXFLAG_VTAG_SHIFT;
		vlan_tag |= TXD1_TAGC | TXD1_VID(vtag);
	}
#endif /* GEM_CONFIG_VLAN_HW */

#if DEBUG_LEVEL > 100
if (flag & (GEM_TXFLAG_IPv4 | GEM_TXFLAG_UDP | GEM_TXFLAG_TCP)) {
	cmn_err(CE_CONT,
	    "!%s: %s: seqnum %d, slot %d, frags %d flag 0x%llx, mss %d",
	    dp->name, __func__, dp->tx_active_tail, slot, frags,
	    (unsigned long long)flag, mss);
	for (i = 0; i < frags; i++) {
		cmn_err(CE_CONT, "!%d: addr 0x%llx, len 0x%x",
		    i, dmacookie[i].dmac_laddress, dmacookie[i].dmac_size);
	}
}
#endif

#if TX_MAX_FRAGS > 1
	for (i = frags - 1; i > 0; i--) {

		curslot = SLOT(slot + i, tx_ring_size);

		/*
		 * specify descriptor control flags
		 */
		if (curslot == tx_ring_size - 1) {
			mark |= TXD0_EOR;
		}
		tdp = &TXDESC(dp->tx_ring)[curslot];
		tmp0 = (uint32_t)(dmacookie[i].dmac_laddress >> 32);
		tmp1 = (uint32_t)dmacookie[i].dmac_laddress;
		tmp2 = mark | (uint32_t)dmacookie[i].dmac_size;

		tdp->txd3 = LE_32(tmp0);
		tdp->txd2 = LE_32(tmp1);
		tdp->txd1 = LE_32(vlan_tag);
		tdp->txd0 = LE_32(tmp2);
		mark &= ~(TXD0_LS | TXD0_EOR);
	}
#endif /* TX_MAX_FRAGS > 1 */
	/*
	 * specify descriptor control flags for first fragment.
	 */
	mark |= TXD0_FS;
	if (flag & GEM_TXFLAG_HEAD) {
		mark &= ~TXD0_OWN;
	}
	if (slot == tx_ring_size - 1) {
		mark |= TXD0_EOR;
	}

	tdp = &TXDESC(dp->tx_ring)[slot];
	tmp0 = (uint32_t)(dmacookie[i].dmac_laddress >> 32);
	tmp1 = (uint32_t)dmacookie[i].dmac_laddress;
	tmp2 = mark | (uint32_t)dmacookie[i].dmac_size;

	tdp->txd3 = LE_32(tmp0);
	tdp->txd2 = LE_32(tmp1);
	tdp->txd1 = LE_32(vlan_tag);
	tdp->txd0 = LE_32(tmp2);

	return (frags);
}

static void
gani_tx_start(struct gem_dev *dp, int start_slot, int nslot)
{
	uint32_t	own;
	uint_t		tx_ring_size = dp->gc.gc_tx_ring_size;
	struct gani_dev	*lp = dp->private;

	DPRINTF(5, (CE_CONT, "!%s: %s: start_slot %d, nslot %d",
	    dp->name, __func__, start_slot, nslot));

	ASSERT(nslot > 0);

	own = LE_32(TXD0_OWN);

	if (nslot > 1) {
		gem_tx_desc_dma_sync(dp,
		    SLOT(start_slot + 1, tx_ring_size),
		    nslot - 1, DDI_DMA_SYNC_FORDEV);
	}

	TXDESC(dp->tx_ring)[start_slot].txd0 |= own;
	gem_tx_desc_dma_sync(dp, start_slot, 1, DDI_DMA_SYNC_FORDEV);
#ifdef WA_NEW_TX_HANG
	lp->tx_last_desc = start_slot + nslot - 1;
#endif

	/* kick Tx engine */
	ASSERT(IS_MAC_ONLINE(dp));
	OUTB(dp, TPPoll, TPPoll_NPQ);
}

static void
gani_rx_desc_write(struct gem_dev *dp, int slot,
		ddi_dma_cookie_t *dmacookie, int frags)
{
	uint32_t		mark;
	struct rx_desc		*rdp;
	uint32_t		tmp0;
	uint32_t		tmp1;
	uint32_t		tmp2;

#if DEBUG_LEVEL > 4
{
	int	i;
	cmn_err(CE_CONT,
	    "!%s: %s: seqnum %d, slot %d, frags %d",
	    dp->name, __func__, dp->rx_desc_tail, slot, frags);
	for (i = 0; i < frags; i++) {
		cmn_err(CE_CONT, "!%d: addr 0x%llx, len 0x%x",
		    i, dmacookie[i].dmac_laddress, dmacookie[i].dmac_size);
	}
}
#endif
	mark = RXD0_OWN;
	if (slot == dp->gc.gc_rx_ring_size - 1) {
		mark = RXD0_OWN | RXD0_EOR;
	}

	rdp = &RXDESC(dp->rx_ring)[slot];

	ASSERT(dmacookie->dmac_size == dp->rx_buf_len);
	tmp0 = (uint32_t)(dmacookie->dmac_laddress >> 32);
	tmp1 = (uint32_t)dmacookie->dmac_laddress;
	tmp2 = mark | (uint32_t)dmacookie->dmac_size;

	rdp->rxd3 = LE_32(tmp0);
	rdp->rxd2 = LE_32(tmp1);
	rdp->rxd1 = 0;
	rdp->rxd0 = LE_32(tmp2);
}

static uint_t
gani_tx_desc_stat(struct gem_dev *dp, int slot, int ndesc)
{
	uint32_t	tsr;
	struct tx_desc	*tdp;
	struct gani_dev	*lp = dp->private;
	int		frags = ndesc;
	uint_t		tx_ring_size = dp->gc.gc_tx_ring_size;

	/* XXX - we must check the last descripor of the packet. */
	tdp = &TXDESC(dp->tx_ring)[SLOT(slot + frags - 1, tx_ring_size)];

	tsr = tdp->txd0;
	tsr = LE_32(tsr);
#ifndef lint
	DPRINTF(2, (CE_CONT,
	    "!%s: %s: slot:%d ndesc:%d isr:%b tsr:%b",
	    dp->name, __func__, slot, ndesc,
	    INW(dp, ISR), INTR_BITS, tsr, TXD0_BITS));
#endif
	if (tsr & TXD0_OWN) {
		/* not transmitted yet */
		if (GANI_PCIE(lp)) {
			/* XXX - restart potentially stopped tx engine. */
			OUTB(dp, TPPoll, TPPoll_NPQ);
		}
		return (0);
	}

#ifdef DEBUG_LEVEL
	if (slot + ndesc != tx_ring_size && (tsr & TXD0_EOR)) {
		/* the descriptor is corrupted */
#ifndef lint
		cmn_err(CE_WARN,
		    "!%s: %s: tx descriptor corrupted, slot:%d ndesc:%d tsr:%b",
		    dp->name, __func__, slot, ndesc, tsr, TXD0_BITS);
#endif
		gani_tx_desc_dump(dp);
		return (GEM_TX_DONE | GEM_TX_ERR);
	}
#endif /* DEBUG_LEVEL */
	return (GEM_TX_DONE);
}

#ifdef DEBUG_LEVEL
static void
gani_tx_desc_dump(struct gem_dev *dp)
{
	int		i;
	uint_t		tx_ring_size = dp->gc.gc_tx_ring_size;
	struct tx_desc	*tdp;

	cmn_err(CE_CONT,
	    "!%s: tx descriptor dump: current head:%d[%d], tail:%d[%d]",
	    dp->name,
	    dp->tx_desc_head, (int)SLOT(dp->tx_desc_head, tx_ring_size),
	    dp->tx_desc_tail, (int)SLOT(dp->tx_desc_tail, tx_ring_size));

	tdp = &TXDESC(dp->tx_ring)[0];
	for (i = 0; i < tx_ring_size; i++, tdp++) {
#ifdef lint
		cmn_err(CE_CONT, "%d: %x %x %x %x",
		    i,
		    LE_32(tdp->txd0),
		    LE_32(tdp->txd1),
		    LE_32(tdp->txd2),
		    LE_32(tdp->txd3));
#else
		cmn_err(CE_CONT, "%d: %b %x %x %x",
		    i,
		    LE_32(tdp->txd0), TXD0_BITS,
		    LE_32(tdp->txd1),
		    LE_32(tdp->txd2),
		    LE_32(tdp->txd3));
#endif /* lint */
	}
}
#endif /* DEBUG_LEVEL */

static uint64_t
gani_rx_desc_stat(struct gem_dev *dp, int slot, int ndesc)
{
	uint_t		len;
	uint32_t	rsr;
	struct rx_desc	*rdp;
	uint64_t	flags = GEM_RX_DONE;
#ifdef GEM_CONFIG_VLAN_HW
	uint64_t	vtag;
#endif
	struct gani_dev	*lp = dp->private;

	rdp = &RXDESC(dp->rx_ring)[slot];
	rsr = rdp->rxd0;
	rsr = LE_32(rsr);
#ifndef lint
	DPRINTF(2, (CE_CONT, "!%s: %s: slot:%d rxd0:0x%b, rxd1:0x%x",
	    dp->name, __func__, slot, rsr, RXD0_BITS, LE_32(rdp->rxd1)));
#endif
	if (rsr & RXD0_OWN) {
		/* not transmitted yet */
		return (0);
	}
#ifndef lint
	DPRINTF(2, (CE_CONT, "!%s: %s: slot:%d rxd0:0x%b, rxd1:0x%x",
	    dp->name, __func__, slot, rsr, RXD0_BITS, LE_32(rdp->rxd1)));
#endif
	if ((rsr & (RXD0_RES | RXD0_LS | RXD0_FS)) != (RXD0_LS | RXD0_FS)) {
		if (rsr & RXD0_RES) {
			if (rsr & RXD0_CRC) {
				dp->stats.crc++;
			}
			if (rsr & RXD0_RUNT) {
				dp->stats.runt++;
			}
			dp->stats.errrcv++;
		}

		if ((rsr & (RXD0_LS | RXD0_FS)) != (RXD0_LS | RXD0_FS)) {
			/* received frame is too long */
			if (rsr & RXD0_FS) {
				dp->stats.frame_too_long++;
				dp->stats.errrcv++;
			}
		}
		return (GEM_RX_ERR);
	}

	/* eliminate fcs */
	if ((len = rsr & RXD0_FRAMELEN) < ETHERFCSL) {
		return (GEM_RX_ERR);
	}
	len -= ETHERFCSL;

#ifdef GEM_CONFIG_VLAN_HW
	if ((vtag = LE_32(rdp->rxd1)) & RXD1_TAVA) {
		/* fix vlan tag format */
		flags |= ((uint64_t)BSWAP_16(vtag & 0xffff))
		    << GEM_RX_VTAG_SHIFT;
		DPRINTF(2, (CE_CONT, "%s: %s: vtag detected %llx",
		    dp->name, __func__, (unsigned long long)flags));
	}
#ifdef GEM_CONFIG_CKSUM_OFFLOAD
	if ((rsr & (RXD0_PID | RXD0_TCPF)) == RXD0_PID_TCP) {
		flags |= GEM_RX_CKSUM_TCP;
	} else if ((rsr & (RXD0_PID | RXD0_UDPF)) == RXD0_PID_UDP) {
		flags |= GEM_RX_CKSUM_UDP;
	}
	if (lp->new_cksum) {
		uint32_t	rsr1;
		rsr1 = rdp->rxd1;
		rsr1 = LE_32(rsr1);

		if ((rsr & (RXD0_PID | RXD0_IPF)) == RXD0_PID_IP_NEW &&
		    (rsr1 & RXD1_IPv4)) {
			flags |= GEM_RX_CKSUM_IPv4;
		}
	} else {
		if ((rsr & (RXD0_PID | RXD0_IPF)) == RXD0_PID_IP) {
			flags |= GEM_RX_CKSUM_IPv4;
		}
	}
#endif /* GEM_CONFIG_CKSUM_OFFLOAD */
#endif /* GEM_CONFIG_VLAN_HW */
#ifdef notdef
{
	int	i;
	uint8_t	*bp = (uint8_t *)&dp->rx_buf_head->rxb_buf[0];

	cmn_err(CE_CONT, "!%s: len:%d", dp->name, len);

	for (i = 0; i < len; i += 10) {
		cmn_err(CE_CONT,
		    "!%02x %02x %02x %02x %02x"
		    " %02x %02x %02x %02x %02x",
		    bp[0], bp[1], bp[2], bp[3], bp[4],
		    bp[5], bp[6], bp[7], bp[8], bp[9]);
		bp += 10;
	}
}
#endif
	return (flags | GEM_RX_DONE | len);
}

static void
gani_tx_desc_init(struct gem_dev *dp, int slot)
{
	TXDESC(dp->tx_ring)[slot].txd0 = 0;
}

static void
gani_rx_desc_init(struct gem_dev *dp, int slot)
{
	RXDESC(dp->rx_ring)[slot].rxd0 = 0;
}

/*
 * Device depend interrupt handler
 */
static uint_t
gani_interrupt(struct gem_dev *dp)
{
	int		i;
	int		nread;
	uint16_t	isr;
	uint16_t	bogus_isr;
	uint16_t	imr_hw;
	uint_t		restart_tx = 0;
	struct gani_dev	*lp = dp->private;
	uint_t		tx_ring_size = dp->gc.gc_tx_ring_size;

	isr = INW(dp, ISR);
	bogus_isr = lp->isr_pended;
	lp->isr_pended = 0;

	DPRINTF(4, (CE_CONT, "!%s: time:%ld %s: isr:%b",
	    dp->name, ddi_get_lbolt(), __func__, isr, INTR_BITS));

	imr_hw = lp->imr;
	if (((isr | bogus_isr) & lp->imr) == 0) {
		/* not for us */
		return (DDI_INTR_UNCLAIMED);
	}

	DPRINTF(2, (CE_CONT, "!%s: time:%ld %s: isr:%b imr:%b",
	    dp->name, ddi_get_lbolt(), __func__, isr, INTR_BITS,
	    INW(dp, IMR), INTR_BITS));

	if (!IS_MAC_ONLINE(dp)) {
		/* disable further interrupts */
		OUTW(dp, IMR, 0);
		FLSHW(dp, IMR);

		/* ack to all interrupts */
		OUTW(dp, ISR, 0xffff);
		FLSHW(dp, ISR);
		return (DDI_INTR_CLAIMED);
	}

	/* clear interrupt sources explicitly */
	if (lp->pcie_cap) {
		/* disable interrupts */
		OUTW(dp, IMR, imr_hw = 0);
	}
	OUTW(dp, ISR, isr);

#ifdef CONFIG_POLLING
	if (dp->poll_interval != lp->last_poll_interval) {
		if (dp->poll_interval) {
			/* polling mode */
			lp->imr &= ~(INTR_ROK | INTR_TDU | INTR_TOK);

			switch (lp->busclk) {
			case BUSCLK_PCIE:
				i = dp->poll_interval / (1000 / 125);
				break;

			case BUSCLK_PCI66:
			case BUSCLK_PCI33:
			default:
				i = dp->poll_interval / (1000 / 33);
				break;
			}

			/*
			 * To schedule the next timer interrupt correctly,
			 * we need to reset PCI clock counter. So, we pretend
			 * as we were interrupted from polling timer.
			 */
			isr |= INTR_TimeOut;

			DPRINTF(3, (CE_CONT,
			    "%s: poll_interval:%d", dp->name, i));
		} else {
			/* normal mode, restore interrupt mask */
			lp->imr |= INTR_ROK | INTR_TOK;
			if (GANI_PCIE(lp)) {
				lp->imr |= INTR_TDU;
			}
			i = 0;
		}
		OUTL(dp, TimerInt, i);

		lp->last_poll_interval = dp->poll_interval;
	}

	if (isr & INTR_TimeOut) {
		/*
		 * Reset PCI clock counter to schedule the next polling
		 * timer interrupt.
		 */
		OUTL(dp, TCTR, 0);
	}
#endif /* CONFIG_POLLING */

	/* PCI read/write barrier to ensure the interrupt is ack'ed. */
	/* XXX - it costs 5%-10% of cpu utilazation on 1.8GHz AMD athlon */
	FLSHW(dp, ISR);

	if (isr & INTR_PUN) {
		/*
		 * Link or PHY status has changed
		 */
		DPRINTF(1, (CE_CONT, "!%s: isr:%b", dp->name, isr, INTR_BITS));
#ifdef GEM3
		gem_mii_link_check(dp);
#else
		if (gem_mii_link_check(dp)) {
			restart_tx |= INTR_RESTART_TX;
		}
#endif
	}

#ifdef GEM3
	if (isr & INTR_TER) {
		cmn_err(CE_WARN, "!%s: Tx error, isr:%b",
		    dp->name, isr, INTR_BITS);
		lp->need_to_reset = B_TRUE;
	} else if ((GANI_PCIE(lp) && (isr & INTR_TDU)) ||
	    (isr & INTR_TOK) && dp->tx_free_bufs < dp->gc.gc_tx_buf_size / 8) {
		/* Need to kick potentially blocked downstream */
		gem_tx_done(dp);
	}
#endif /* !GEM3 */

	if (isr & (INTR_ROK | INTR_RER | INTR_RDU | INTR_FOVW)) {
		if (isr & INTR_FOVW) {
			if (lp->chip->type == R8168_1 /* RTL8168B rev.B */) {
				lp->need_to_reset = B_TRUE;
				lp->imr &= ~INTR_FOVW;
			}
			dp->stats.overflow++;
			dp->stats.errrcv++;
		}
		nread = gem_receive(dp);
#ifdef TEST_FOVW
		lp->rx_pkt_cnt += nread;
		if ((lp->rx_pkt_cnt % 100000) == 99999) {
			isr |= INTR_FOVW;
		}
#endif
#ifdef CONFIG_POLLING
		if (isr & INTR_RDU) {
			if (dp->poll_interval && dp->poll_pkt_delay > 1 &&
			    GANI_PCIE(lp)) {
				/*
				 * current polling interval is too long.
				 * recompute it.
				 */
				dp->poll_pkt_delay--;
				DPRINTF(-1, (CE_CONT,
				    "!%s: pkt-delay decreased to %d",
				    dp->name, dp->poll_pkt_delay));
			}
			dp->stats.errrcv++;
			dp->stats.norcvbuf++;
		}
#endif /* CONFIG_POLLING */
	}

#ifndef GEM3
	if (isr & INTR_TER) {
		cmn_err(CE_WARN, "!%s: Tx error, isr:%b",
		    dp->name, isr, INTR_BITS);
		lp->need_to_reset = B_TRUE;
	} else if ((GANI_PCIE(lp) && (isr & INTR_TDU)) ||
	    (isr & INTR_TOK)) {
		/* Need to kick potentially blocked down stream */
		if (gem_tx_done(dp)) {
			restart_tx |= INTR_RESTART_TX;
		}
	}
#endif /* !GEM3 */

	if (isr & INTR_SERR) {
		cmn_err(CE_WARN, "!%s: unexpected interrupt: isr:%b",
		    dp->name, isr, INTR_BITS);
		lp->need_to_reset = B_TRUE;
	}

	if (lp->need_to_reset) {
		(void) gem_restart_nic(dp, GEM_RESTART_KEEP_BUF);

		restart_tx = INTR_RESTART_TX;
		lp->need_to_reset = B_FALSE;
	}

	if ((dp->misc_flag & GEM_NOINTR) == 0 && lp->imr != imr_hw) {
		/* enable interrupts again */
		OUTW(dp, IMR, lp->imr);
	}

	return (DDI_INTR_CLAIMED | restart_tx);
}

/*
 * MII Interfaces
 */
#define	MII_DELAY(dp, r) { (void) INL(dp, (r)); (void) INL(dp, (r)); }

static void
gani_mii_sync(struct gem_dev *dp)
{
	/* do nothing */
}

#define	GANI_MII_MAX_RETRY	10

static uint16_t
gani_mii_read_raw(struct gem_dev *dp, uint_t index)
{
	int	i;
	int	retry = 0;
	uint_t	ret;
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_11) {
		OUTL(dp, OCPDR,
		    OCPDR_READ |
		    (index & OCPDR_REG) << OCPDR_GPHY_REG_SHIFT);
		OUTL(dp, OCPAR, OCPAR_GPHY_WRITE);
		OUTL(dp, EPHY_RXER_NUM, 0);

		for (i = 0; i < 100; i++) {
			drv_usecwait(1000);
			if ((INL(dp, OCPAR) & OCPAR_FLAG) == 0) {
				break;
			}
		}

		drv_usecwait(1000);
		OUTL(dp, OCPAR, OCPAR_GPHY_READ);
		OUTL(dp, EPHY_RXER_NUM, 0);

		for (i = 0; i < 100; i++) {
			drv_usecwait(1000);
			if (INL(dp, OCPAR) & OCPAR_FLAG) {
				break;
			}
		}

		ret = INL(dp, OCPDR) & OCPDR_DATA;

	} else if (
	    lp->chip->type == R8168_21 || lp->chip->type == R8168_22 ||
	    lp->chip->type == R8168_23 || lp->chip->type == R8168_24) {

		uint32_t	data32;
		uint16_t	ocp_addr;

		ocp_addr = gani_map_phy_ocp_addr(lp->cur_page, index);

		data32 = ocp_addr/2;
		data32 <<= OCPR_ADDR_REG_SHIFT;

		OUTL(dp, PHYOCP, data32);
		for (i = 0; i < 10; i++) {
			drv_usecwait(100);

			if (INL(dp, PHYOCP) & OCPR_FLAG) {
				break;
			}
		}
		ret = INL(dp, PHYOCP) & OCPDR_DATA;

	} else {
		if (lp->chip->type == R8168_12 ||
		    lp->chip->type == R8168_13) {
			/* RTL8168DP */
			OUTL(dp, 0xD0, INL(dp, 0xD0) & ~0x00020000);
		}
again:
		OUTL(dp, PHYAR, index << PHYAR_REGADDR_SHIFT);
		drv_usecwait(20);

		for (i = 0; ((ret = INL(dp, PHYAR)) & PHYAR_FLAG) == 0; i++) {
			if (i > 50) {
				if (retry++ < GANI_MII_MAX_RETRY) {
					goto again;
				}
				cmn_err(CE_WARN, "!%s: %s: timeout, reg:%d",
				    dp->name, __func__, index);
				ret = 0;
				goto x;
			}
			drv_usecwait(20);
		}
		ret &= PHYAR_DATA;
x:
		drv_usecwait(100);
		if (lp->chip->type == R8168_12 ||
		    lp->chip->type == R8168_13) {
			/* RTL8168DP */
			OUTL(dp, 0xD0, INL(dp, 0xD0) | 0x00020000);
		}
	}

	return ((uint16_t)ret);
}

/*
 * for R8101 series
 */
static uint16_t
gani_phyio_read(struct gem_dev *dp, uint_t index)
{
	int	i;
	uint16_t	ret;

	OUTL(dp, PHYIO, PHYIO_READ | (index << PHYIO_REG_SHIFT));
	drv_usecwait(100);

	for (i = 0; (INL(dp, PHYIO) & PHYIO_FLAG) == 0; i++) {
		if (i > 10) {
			cmn_err(CE_WARN, "!%s: %s: timeout, reg:%d",
			    dp->name, __func__, index);
			return (0);
		}
		drv_usecwait(100);
	}
	ret = (uint16_t)(INL(dp, PHYIO) & PHYIO_DATA);
	return (ret);
}

static void
gani_mii_write_raw(struct gem_dev *dp, uint_t index, uint16_t val)
{
	int	i;
	int	retry = 0;
	struct gani_dev	*lp = dp->private;

	if (lp->chip->type == R8168_11) {
		OUTL(dp, OCPDR, OCPDR_WRITE
		    | (index & OCPDR_REG) << OCPDR_GPHY_REG_SHIFT
		    | (val & OCPDR_DATA));
		OUTL(dp, OCPAR, OCPAR_GPHY_WRITE);
		OUTL(dp, EPHY_RXER_NUM, 0);

		for (i = 0; i < 100; i++) {
			drv_usecwait(1000);
			if (!(INL(dp, OCPAR) & OCPAR_FLAG)) {
				break;
			}
		}

	} else if (
	    lp->chip->type == R8168_21 || lp->chip->type == R8168_22 ||
	    lp->chip->type == R8168_23 || lp->chip->type == R8168_24) {
		uint32_t	data32;
		uint16_t	ocp_addr;

		if (index == 0x1F) {
			lp->cur_page = val ? val : OCP_STD_PHY_BASE_PAGE;
			return;
		}

		ocp_addr = gani_map_phy_ocp_addr(lp->cur_page, index);

		data32 = ocp_addr/2;
		data32 <<= OCPR_ADDR_REG_SHIFT;
		data32 |= OCPR_WRITE | val;

		OUTL(dp, PHYOCP, data32);
		for (i = 0; i < 10; i++) {
			drv_usecwait(100);

			if (!(INL(dp, PHYOCP) & OCPR_FLAG)) {
				break;
			}
		}

	} else {
		if (lp->chip->type == R8168_12 ||
		    lp->chip->type == R8168_13) {
			/* RTL8168DP */
			OUTL(dp, 0xD0, INL(dp, 0xD0) & ~0x00020000);
		}
again:
		OUTL(dp, PHYAR,
		    PHYAR_FLAG | (index << PHYAR_REGADDR_SHIFT) | val);
		drv_usecwait(100);

		for (i = 0; (INL(dp, PHYAR) & PHYAR_FLAG) != 0; i++) {
			if (i > 50) {
				if (retry++ < GANI_MII_MAX_RETRY) {
					goto again;
				}
				cmn_err(CE_WARN, "!%s: %s: timeout, reg:%d",
				    dp->name, __func__, index);
				break;
			}
			drv_usecwait(100);
		}
		drv_usecwait(20);

		if (lp->chip->type == R8168_12 ||
		    lp->chip->type == R8168_13) {
			/* RTL8168DP */
			OUTL(dp, 0xD0, INL(dp, 0xD0) | 0x00020000);
		}
	}
}
#undef	MII_DELAY

static void
gani_phyio_write(struct gem_dev *dp, uint_t index, uint16_t val)
{
	int	i;

	OUTL(dp, PHYIO,
	    PHYIO_WRITE | (index << PHYIO_REG_SHIFT) | val);
	drv_usecwait(100);

	for (i = 0; INL(dp, PHYIO) & PHYIO_FLAG; i++) {
		if (i > 10) {
			cmn_err(CE_WARN, "!%s: %s: timeout, reg:%d",
			    dp->name, __func__, index);
			break;
		}
		drv_usecwait(100);
	}
	drv_usecwait(100);
}

#ifdef notdef
/*
 * not required for 035
 */
static void
gani_mii_write_ocp(struct gem_dev *dp, uint_t index, uint16_t	val)
{
	int	i;

	OUTL(dp, OCPDR, OCPDR_WRITE | (index << OCPDR_GPHY_REG_SHIFT) | val);
	OUTL(dp, OCPAR, OCPAR_GPHY_WRITE);
	OUTL(dp, EPHY_RXER_NUM, 0);

	for (i = 0; INL(dp, OCPAR) & OCPAR_FLAG; i++) {
		if (i > 100) {
			/* timeout */
			cmn_err(CE_WARN,
			    "!%s:%s: timeout", dp->name, __func__);
			break;
		}
		drv_usecwait(2000);
	}
}

static uint16_t
gani_mii_read_ocp(struct gem_dev *dp, uint_t index)
{
	int	i;
	uint16_t	val;

	OUTL(dp, OCPDR, OCPDR_READ | (index << OCPDR_GPHY_REG_SHIFT));
	OUTL(dp, OCPAR, OCPAR_GPHY_WRITE);
	OUTL(dp, EPHY_RXER_NUM, 0);

	for (i = 0; (INL(dp, OCPAR) & OCPAR_FLAG); i++) {
		if (i > 100) {
			/* timeout */
			cmn_err(CE_WARN,
			    "!%s:%s: timeout", dp->name, __func__);
			break;
		}
		drv_usecwait(2000);
	}

	OUTL(dp, OCPAR, OCPAR_GPHY_READ);
	val = INL(dp, OCPDR) & OCPDR_DATA;

	return (val);
}
#endif

static uint16_t
gani_mii_read(struct gem_dev *dp, uint_t index)
{
	struct gani_dev	*lp = dp->private;

	/* select page 0 */
	gani_mii_write_raw(dp, 0x1f, 0);

	return (gani_mii_read_raw(dp, index));
}

static void
gani_mii_write(struct gem_dev *dp, uint_t index, uint16_t val)
{
	struct gani_dev	*lp = dp->private;

	/* select page 0 */
	gani_mii_write_raw(dp, 0x1f, 0);

	/* don't change ANE bit on RESET */
	if (index == MII_CONTROL && val == MII_CONTROL_RESET) {
		val |= gani_mii_read_raw(dp, index) & MII_CONTROL_ANE;
	}

	/* RSAN also needs RESET */
	if ((GANI_PCIE_GIGA(lp) || lp->chip->type == R8101_10) &&
	    index == MII_CONTROL && (val & MII_CONTROL_RSAN)) {
		val |= MII_CONTROL_RESET;
	}
	gani_mii_write_raw(dp, index, val);
}

static boolean_t
gani_enable_EEE(struct gem_dev *dp)
{
	boolean_t	ret;
	uint16_t	data;
	struct gani_dev	*lp = dp->private;

	ret = B_TRUE;
	switch (lp->chip->type) {
	case R8168_14:
	case R8168_15:
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x0020);
		data = gani_mii_read_raw(dp, 0x15) | 0x0100;
		gani_mii_write_raw(dp, 0x15, data);
		gani_mii_write_raw(dp, 0x1F, 0x0006);
		gani_mii_write_raw(dp, 0x00, 0x5A30);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0006);
		gani_mii_write_raw(dp, 0x0D, 0x0000);
		if ((INB(dp, CFG4) & 0x40) && (INB(dp, 0x6D) & BIT_7)) {
			data = INW(dp, CustomLED);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8AC8);
			gani_mii_write_raw(dp, 0x06, data);
			gani_mii_write_raw(dp, 0x05, 0x8B82);
			data = gani_mii_read_raw(dp, 0x06) | 0x0010;
			gani_mii_write_raw(dp, 0x05, 0x8B82);
			gani_mii_write_raw(dp, 0x06, data);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		}
		break;

	case R8168_16:
	case R8168_17:
		data = gani_eri_read(dp, 0x1B0, 4, ERIAR_ExGMAC) | 0x0003;
		gani_eri_write(dp, 0x1B0, 4, data, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x0020);
		data = gani_mii_read_raw(dp, 0x15)|0x0100;
		gani_mii_write_raw(dp, 0x15, data);
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B85);
		data = gani_mii_read_raw(dp, 0x06)|0x2000;
		gani_mii_write_raw(dp, 0x06, data);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0006);
		gani_mii_write_raw(dp, 0x1D, 0x0000);
		break;

	case R8168_18:
	case R8168_19:
	case R8168_20:
		data = gani_eri_read(dp, 0x1B0, 4, ERIAR_ExGMAC);
		data |= BIT_1 | BIT_0;
		gani_eri_write(dp, 0x1B0, 4, data, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0020);
		data = gani_mii_read_raw(dp, 0x15);
		data |= BIT_8;
		gani_mii_write_raw(dp, 0x15, data);
		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B85);
		data = gani_mii_read_raw(dp, 0x06);
		data |= BIT_13;
		gani_mii_write_raw(dp, 0x06, data);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0006);
		gani_mii_write_raw(dp, 0x0D, 0x0000);
		break;

	case R8168_21:
	case R8168_22:
	case R8168_24:
		data = gani_eri_read(dp, 0x1B0, 4, ERIAR_ExGMAC);
		data |= BIT_1 | BIT_0;
		gani_eri_write(dp, 0x1B0, 4, data, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		data = gani_mii_read_raw(dp, 0x11);
		gani_mii_write_raw(dp, 0x11, data | BIT_4);
		gani_mii_write_raw(dp, 0x1F, 0x0A5D);
		gani_mii_write_raw(dp, 0x10, 0x0006);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		break;

	case R8101_11:
	case R8101_12:
	case R8101_13:
		gani_eri_write(dp, 0x1B0, 2, 0xED03, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0004);
		if (INB(dp, 0xEF) & 0x02) {
			gani_mii_write_raw(dp, 0x10, 0x731F);
			gani_mii_write_raw(dp, 0x19, 0x7630);
		} else {
			gani_mii_write_raw(dp, 0x10, 0x711F);
			gani_mii_write_raw(dp, 0x19, 0x7030);
		}
		gani_mii_write_raw(dp, 0x1A, 0x1506);
		gani_mii_write_raw(dp, 0x1B, 0x0551);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0002);
		gani_mii_write_raw(dp, 0x0D, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0003);
		gani_mii_write_raw(dp, 0x0E, 0x0015);
		gani_mii_write_raw(dp, 0x0D, 0x4003);
		gani_mii_write_raw(dp, 0x0E, 0x0002);
		gani_mii_write_raw(dp, 0x0D, 0x0000);

		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_ANE | MII_CONTROL_RSAN);
		break;

	case R8101_14:
	case R8101_15:
	case R8101_16:
		gani_eri_write(dp, 0x1B0, 2, 0xED03, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x10, 0x731F);
		gani_mii_write_raw(dp, 0x19, 0x7630);
		gani_mii_write_raw(dp, 0x1A, 0x1506);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0002);
		gani_mii_write_raw(dp, 0x0D, 0x0000);

		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_ANE | MII_CONTROL_RSAN);
		break;

	default:
		/* Not Support EEE */
		ret = B_FALSE;
	}

	return (ret);
}

static boolean_t
gani_disable_EEE(struct gem_dev *dp)
{
	int	ret;
	uint16_t	data;
	struct gani_dev	*lp = dp->private;

	ret = B_TRUE;
	switch (lp->chip->type) {
	case R8168_14:
	case R8168_15:
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x0020);
		data = gani_mii_read_raw(dp, 0x15) & ~0x0100;
		gani_mii_write_raw(dp, 0x15, data);
		gani_mii_write_raw(dp, 0x1F, 0x0006);
		gani_mii_write_raw(dp, 0x00, 0x5A00);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		if (INB(dp, CFG4) & 0x40) {
			data = INW(dp, CustomLED);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8B82);
			data = gani_mii_read_raw(dp, 0x06) & ~0x0010;
			gani_mii_write_raw(dp, 0x05, 0x8B82);
			gani_mii_write_raw(dp, 0x06, data);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		}
		break;

	case R8168_16:
	case R8168_17:
		data = gani_eri_read(dp, 0x1B0, 4, ERIAR_ExGMAC) & ~0x0003;
		gani_eri_write(dp, 0x1B0, 4, data, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B85);
		data = gani_mii_read_raw(dp, 0x06) & ~0x2000;
		gani_mii_write_raw(dp, 0x06, data);
		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x0020);
		data = gani_mii_read_raw(dp, 0x15) & ~0x0100;
		gani_mii_write_raw(dp, 0x15, data);
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		break;

	case R8168_18:
	case R8168_19:
	case R8168_20:
		data = gani_eri_read(dp, 0x1B0, 4, ERIAR_ExGMAC);
		data &= ~(BIT_1 | BIT_0);
		gani_eri_write(dp, 0x1B0, 4, data, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B85);
		data = gani_mii_read_raw(dp, 0x06);
		data &= ~BIT_13;
		gani_mii_write_raw(dp, 0x06, data);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0020);
		data = gani_mii_read_raw(dp, 0x15);
		data &= ~BIT_8;
		gani_mii_write_raw(dp, 0x15, data);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		break;

	case R8168_21:
	case R8168_22:
	case R8168_24:
		data = gani_eri_read(dp, 0x1B0, 4, ERIAR_ExGMAC);
		data &= ~(BIT_1 | BIT_0);
		gani_eri_write(dp, 0x1B0, 4, data, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		data = gani_mii_read_raw(dp, 0x11);
		gani_mii_write_raw(dp, 0x11, data & ~BIT_4);
		gani_mii_write_raw(dp, 0x1F, 0x0A5D);
		gani_mii_write_raw(dp, 0x10, 0x0000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		break;

	case R8101_11:
	case R8101_12:
	case R8101_13:
		gani_eri_write(dp, 0x1B0, 2, 0, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x10, 0x401F);
		gani_mii_write_raw(dp, 0x19, 0x7030);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0003);
		gani_mii_write_raw(dp, 0x0E, 0x0015);
		gani_mii_write_raw(dp, 0x0D, 0x4003);
		gani_mii_write_raw(dp, 0x0E, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0000);

		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_ANE | MII_CONTROL_RSAN);
		break;

	case R8101_14:
		gani_eri_write(dp, 0x1B0, 2, 0, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x10, 0x401F);
		gani_mii_write_raw(dp, 0x19, 0x7030);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0000);

		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_ANE | MII_CONTROL_RSAN);
		break;

	case R8101_15:
	case R8101_16:
		gani_eri_write(dp, 0x1B0, 2, 0, ERIAR_ExGMAC);
		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x10, 0xC07F);
		gani_mii_write_raw(dp, 0x19, 0x7030);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0007);
		gani_mii_write_raw(dp, 0x0E, 0x003C);
		gani_mii_write_raw(dp, 0x0D, 0x4007);
		gani_mii_write_raw(dp, 0x0E, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0x0000);

		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_ANE | MII_CONTROL_RSAN);
		break;

	default:
		/* Not Support EEE */
		ret = B_FALSE;
		break;
	}

	return (ret);
}

struct gani_patch {
	uint_t	addr:8;
	uint_t	val:16;
};

static struct gani_patch gani_8169_m2_patch[] = {
	/* 012 */
	0x1f, 0x0001, 0x06, 0x006e, 0x08, 0x0708,
	0x15, 0x4000, 0x18, 0x65c7,

	0x1f, 0x0001, 0x03, 0x00a1, 0x02, 0x0008,
	0x01, 0x0120, 0x00, 0x1000, 0x04, 0x0800,
	0x04, 0x0000,

	0x03, 0xff41, 0x02, 0xdf60, 0x01, 0x0140,
	0x00, 0x0077, 0x04, 0x7800, 0x04, 0x7000,

	0x03, 0x802f, 0x02, 0x4f02, 0x01, 0x0409,
	0x00, 0xf0f9, 0x04, 0x9800, 0x04, 0x9000,

	0x03, 0xdf01, 0x02, 0xdf20, 0x01, 0xff95,
	0x00, 0xba00, 0x04, 0xa800, 0x04, 0xa000,

	0x03, 0xff41, 0x02, 0xdf20, 0x01, 0x0140,
	0x00, 0x00bb, 0x04, 0xb800, 0x04, 0xb000,

	0x03, 0xdf41, 0x02, 0xdc60, 0x01, 0x6340,
	0x00, 0x007d, 0x04, 0xd800, 0x04, 0xd000,

	0x03, 0xdf01, 0x02, 0xdf20, 0x01, 0x100a,
	0x00, 0xa0ff, 0x04, 0xf800, 0x04, 0xf000,

	0x1f, 0x0000, 0x0b, 0x0000, 0x00, 0x9200,
};

static struct gani_patch gani_8169_m4_patch[] = {
	/* 012 */
	0x1f, 0x0002, 0x01, 0x90d0, 0x1f, 0x0000,
};

static struct gani_patch gani_8169_m5_patch0[] = {
	/* 012 */
	0x1f, 0x0001, 0x04, 0x0000, 0x03, 0x00a1, 0x02, 0x0008,
	0x01, 0x0120, 0x00, 0x1000, 0x04, 0x0800, 0x04, 0x9000,
	0x03, 0x802f, 0x02, 0x4f02, 0x01, 0x0409, 0x00, 0xf099,
	0x04, 0x9800, 0x04, 0xa000, 0x03, 0xdf01, 0x02, 0xdf20,
	0x01, 0xff95, 0x00, 0xba00, 0x04, 0xa800, 0x04, 0xf000,
	0x03, 0xdf01, 0x02, 0xdf20, 0x01, 0x101a, 0x00, 0xa0ff,
	0x04, 0xf800, 0x04, 0x0000, 0x1f, 0x0000,

	0x1f, 0x0001, 0x10, 0xf41b, 0x14, 0xfb54, 0x18, 0xf5c7,
	0x1f, 0x0000,

	0x1f, 0x0001, 0x17, 0x0cc0, 0x1f, 0x0000,
};

static struct gani_patch gani_8169_m5_patch1[] = {
	/* 012 */
	0x1f, 0x0001, 0x10, 0xf01b, 0x1f, 0x0000,
};

static struct gani_patch gani_8169_m6_patch[] = {
	/* 009 */
	0x1f, 0x0001, 0x04, 0x0000, 0x03, 0x00a1,
	0x02, 0x0008, 0x01, 0x0120, 0x00, 0x1000,
	0x04, 0x0800, 0x04, 0x9000, 0x03, 0x802f,
	0x02, 0x4f02, 0x01, 0x0409, 0x00, 0xf099,
	0x04, 0x9800, 0x04, 0xa000, 0x03, 0xdf01,
	0x02, 0xdf20, 0x01, 0xff95, 0x00, 0xba00,
	0x04, 0xa800, 0x04, 0xf000, 0x03, 0xdf01,
	0x02, 0xdf20, 0x01, 0x101a, 0x00, 0xa0ff,
	0x04, 0xf800, 0x04, 0x0000, 0x1f, 0x0000,

	0x1f, 0x0001, 0x0b, 0x8480, 0x1f, 0x0000,

	0x1f, 0x0001, 0x18, 0x67c7, 0x04, 0x2000,
	0x03, 0x002f, 0x02, 0x4360, 0x01, 0x0109,
	0x00, 0x3022, 0x04, 0x2800, 0x1f, 0x0000,

	0x1f, 0x0001, 0x17, 0x0cc0, 0x1f, 0x0000,
};

static struct gani_patch gani_8168_m1_patch[] = {
	/* 035 */
	0x1F, 0x0001, 0x0B, 0x94B0,

	0x1F, 0x0003, 0x12, 0x6096, 0x1F, 0x0000,

	0x0D, 0xF8A0,
#if 1
	0x1F, 0x0000,	/* bug ? */
#endif
};

static struct gani_patch gani_8168_m2_patch[] = {
	/* 035 */
	0x1F, 0x0001, 0x0B, 0x94B0,

	0x1F, 0x0003, 0x12, 0x6096,

	0x1F, 0x0000,
};

static struct gani_patch gani_8168_m3_patch[] = {
	/* 035 */
	0x1F, 0x0001, 0x0B, 0x94B0,

	0x1F, 0x0003, 0x12, 0x6096,

	0x1F, 0x0000,
};

static struct gani_patch gani_8168_m4_patch[] = {
	/* 035 */
	0x1F, 0x0001, 0x12, 0x2300, 0x1F, 0x0000, 0x1F, 0x0003,
	0x16, 0x000A, 0x1F, 0x0000,

	0x1F, 0x0003, 0x12, 0xC096, 0x1F, 0x0000,

	0x1F, 0x0002, 0x00, 0x88DE, 0x01, 0x82B1, 0x1F, 0x0000,

	0x1F, 0x0002, 0x08, 0x9E30, 0x09, 0x01F0, 0x1F, 0x0000,

	0x1F, 0x0002, 0x0A, 0x5500, 0x1F, 0x0000,

	0x1F, 0x0002, 0x03, 0x7002, 0x1F, 0x0000,

	0x1F, 0x0002, 0x0C, 0x00C8, 0x1F, 0x0000,

	0x1F, 0x0000,
};

static struct gani_patch gani_8168_m5_patch0[] = {
	/* 035 */
	0x1F, 0x0001, 0x12, 0x2300, 0x1F, 0x0003, 0x16, 0x0F0A,
	0x1F, 0x0000,

	0x1F, 0x0002, 0x00, 0x88DE, 0x01, 0x82B1, 0x1F, 0x0000,

	0x1F, 0x0002, 0x0C, 0x7EB8, 0x1F, 0x0000,

	0x1F, 0x0002, 0x06, 0x0761, 0x1F, 0x0000,

	0x1F, 0x0001, 0x03, 0x802F, 0x02, 0x4F02, 0x01, 0x0409,
	0x00, 0xF099, 0x04, 0x9800, 0x04, 0x9000, 0x1F, 0x0000,

	0x1F, 0x0000,
};

static struct gani_patch gani_8168_m5_patch1[] = {
	/* 035 */
	0x1F, 0x0001, 0x1D, 0x3D98, 0x1F, 0x0000,

	0x1F, 0x0001, 0x17, 0x0CC0, 0x1F, 0x0000,
};

static struct gani_patch gani_8168_m6_patch0[] = {
	/* 035 */
	0x1F, 0x0001, 0x12, 0x2300, 0x1F, 0x0003, 0x16, 0x0F0A,
	0x1F, 0x0000,

	0x1F, 0x0002, 0x00, 0x88DE, 0x01, 0x82B1, 0x1F, 0x0000,

	0x1F, 0x0002, 0x0C, 0x7EB8, 0x1F, 0x0000,

	0x1F, 0x0002, 0x06, 0x5461, 0x1F, 0x0000,

	0x1F, 0x0002, 0x06, 0x5461, 0x1F, 0x0000,

	0x1F, 0x0000,
};

static struct gani_patch gani_8168_m6_patch1[] = {
	/* 035 */
	0x1F, 0x0001, 0x1D, 0x3D98, 0x1F, 0x0000,

	0x1f, 0x0001, 0x17, 0x0CC0, 0x1F, 0x0000,
};

static struct gani_patch gani_8168_m7_patch[] = {
	/* 035 */
	0x1F, 0x0001, 0x1D, 0x3D98,

	0x1F, 0x0001, 0x14, 0xCAA3, 0x1C, 0x000A, 0x18, 0x65D0,

	0x1F, 0x0003, 0x17, 0xB580, 0x18, 0xFF54, 0x19, 0x3954,

	0x1F, 0x0002, 0x0D, 0x310C, 0x0E, 0x310C, 0x0F, 0x311C,
	0x06, 0x0761,

	0x1F, 0x0003, 0x18, 0xFF55, 0x19, 0x3955, 0x18, 0xFF54,
	0x19, 0x3954,

	0x1F, 0x0001, 0x17, 0x0CC0,

	0x1F, 0x0000,
};

static struct gani_patch gani_8168_m8_patch[] = {
	/* 035 */
	0x1F, 0x0001, 0x14, 0xCAA3, 0x1C, 0x000A, 0x18, 0x65D0,

	0x1F, 0x0003, 0x17, 0xB580, 0x18, 0xFF54, 0x19, 0x3954,

	0x1F, 0x0002, 0x0D, 0x310C, 0x0E, 0x310C, 0x0F, 0x311C,
	0x06, 0x0761,

	0x1F, 0x0003, 0x18, 0xFF55, 0x19, 0x3955, 0x18, 0xFF54,
	0x19, 0x3954,

	0x1F, 0x0001, 0x17, 0x0CC0,

	0x1F, 0x0000,
};

static struct gani_patch gani_8168_m9_patch0[] = {
	/* 035 */
	0x1F, 0x0001, 0x06, 0x4064, 0x07, 0x2863, 0x08, 0x059C,
	0x09, 0x26B4, 0x0A, 0x6A19, 0x0B, 0xDCC8, 0x10, 0xF06D,
	0x14, 0x7F68, 0x18, 0x7FD9, 0x1C, 0xF0FF, 0x1D, 0x3D9C,
	0x1F, 0x0003, 0x12, 0xF49F, 0x13, 0x070B, 0x1A, 0x05AD,
	0x14, 0x94C0,
};

static struct gani_patch gani_8168_m9_patch1[] = {
	/* 035 */
	0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080, 0x05, 0x8000,
	0x06, 0xf8f9, 0x06, 0xfaef, 0x06, 0x59ee, 0x06, 0xf8ea,
	0x06, 0x00ee, 0x06, 0xf8eb, 0x06, 0x00e0, 0x06, 0xf87c,
	0x06, 0xe1f8, 0x06, 0x7d59, 0x06, 0x0fef, 0x06, 0x0139,
	0x06, 0x029e, 0x06, 0x06ef, 0x06, 0x1039, 0x06, 0x089f,
	0x06, 0x2aee, 0x06, 0xf8ea, 0x06, 0x00ee, 0x06, 0xf8eb,
	0x06, 0x01e0, 0x06, 0xf87c, 0x06, 0xe1f8, 0x06, 0x7d58,
	0x06, 0x409e, 0x06, 0x0f39, 0x06, 0x46aa, 0x06, 0x0bbf,
	0x06, 0x8290, 0x06, 0xd682, 0x06, 0x9802, 0x06, 0x014f,
	0x06, 0xae09, 0x06, 0xbf82, 0x06, 0x98d6, 0x06, 0x82a0,
	0x06, 0x0201, 0x06, 0x4fef, 0x06, 0x95fe, 0x06, 0xfdfc,
	0x06, 0x05f8, 0x06, 0xf9fa, 0x06, 0xeef8, 0x06, 0xea00,
	0x06, 0xeef8, 0x06, 0xeb00, 0x06, 0xe2f8, 0x06, 0x7ce3,
	0x06, 0xf87d, 0x06, 0xa511, 0x06, 0x1112, 0x06, 0xd240,
	0x06, 0xd644, 0x06, 0x4402, 0x06, 0x8217, 0x06, 0xd2a0,
	0x06, 0xd6aa, 0x06, 0xaa02, 0x06, 0x8217, 0x06, 0xae0f,
	0x06, 0xa544, 0x06, 0x4402, 0x06, 0xae4d, 0x06, 0xa5aa,
	0x06, 0xaa02, 0x06, 0xae47, 0x06, 0xaf82, 0x06, 0x13ee,
	0x06, 0x834e, 0x06, 0x00ee, 0x06, 0x834d, 0x06, 0x0fee,
	0x06, 0x834c, 0x06, 0x0fee, 0x06, 0x834f, 0x06, 0x00ee,
	0x06, 0x8351, 0x06, 0x00ee, 0x06, 0x834a, 0x06, 0xffee,
	0x06, 0x834b, 0x06, 0xffe0, 0x06, 0x8330, 0x06, 0xe183,
	0x06, 0x3158, 0x06, 0xfee4, 0x06, 0xf88a, 0x06, 0xe5f8,
	0x06, 0x8be0, 0x06, 0x8332, 0x06, 0xe183, 0x06, 0x3359,
	0x06, 0x0fe2, 0x06, 0x834d, 0x06, 0x0c24, 0x06, 0x5af0,
	0x06, 0x1e12, 0x06, 0xe4f8, 0x06, 0x8ce5, 0x06, 0xf88d,
	0x06, 0xaf82, 0x06, 0x13e0, 0x06, 0x834f, 0x06, 0x10e4,
	0x06, 0x834f, 0x06, 0xe083, 0x06, 0x4e78, 0x06, 0x009f,
	0x06, 0x0ae0, 0x06, 0x834f, 0x06, 0xa010, 0x06, 0xa5ee,
	0x06, 0x834e, 0x06, 0x01e0, 0x06, 0x834e, 0x06, 0x7805,
	0x06, 0x9e9a, 0x06, 0xe083, 0x06, 0x4e78, 0x06, 0x049e,
	0x06, 0x10e0, 0x06, 0x834e, 0x06, 0x7803, 0x06, 0x9e0f,
	0x06, 0xe083, 0x06, 0x4e78, 0x06, 0x019e, 0x06, 0x05ae,
	0x06, 0x0caf, 0x06, 0x81f8, 0x06, 0xaf81, 0x06, 0xa3af,
	0x06, 0x81dc, 0x06, 0xaf82, 0x06, 0x13ee, 0x06, 0x8348,
	0x06, 0x00ee, 0x06, 0x8349, 0x06, 0x00e0, 0x06, 0x8351,
	0x06, 0x10e4, 0x06, 0x8351, 0x06, 0x5801, 0x06, 0x9fea,
	0x06, 0xd000, 0x06, 0xd180, 0x06, 0x1f66, 0x06, 0xe2f8,
	0x06, 0xeae3, 0x06, 0xf8eb, 0x06, 0x5af8, 0x06, 0x1e20,
	0x06, 0xe6f8, 0x06, 0xeae5, 0x06, 0xf8eb, 0x06, 0xd302,
	0x06, 0xb3fe, 0x06, 0xe2f8, 0x06, 0x7cef, 0x06, 0x325b,
	0x06, 0x80e3, 0x06, 0xf87d, 0x06, 0x9e03, 0x06, 0x7dff,
	0x06, 0xff0d, 0x06, 0x581c, 0x06, 0x551a, 0x06, 0x6511,
	0x06, 0xa190, 0x06, 0xd3e2, 0x06, 0x8348, 0x06, 0xe383,
	0x06, 0x491b, 0x06, 0x56ab, 0x06, 0x08ef, 0x06, 0x56e6,
	0x06, 0x8348, 0x06, 0xe783, 0x06, 0x4910, 0x06, 0xd180,
	0x06, 0x1f66, 0x06, 0xa004, 0x06, 0xb9e2, 0x06, 0x8348,
	0x06, 0xe383, 0x06, 0x49ef, 0x06, 0x65e2, 0x06, 0x834a,
	0x06, 0xe383, 0x06, 0x4b1b, 0x06, 0x56aa, 0x06, 0x0eef,
	0x06, 0x56e6, 0x06, 0x834a, 0x06, 0xe783, 0x06, 0x4be2,
	0x06, 0x834d, 0x06, 0xe683, 0x06, 0x4ce0, 0x06, 0x834d,
	0x06, 0xa000, 0x06, 0x0caf, 0x06, 0x81dc, 0x06, 0xe083,
	0x06, 0x4d10, 0x06, 0xe483, 0x06, 0x4dae, 0x06, 0x0480,
	0x06, 0xe483, 0x06, 0x4de0, 0x06, 0x834e, 0x06, 0x7803,
	0x06, 0x9e0b, 0x06, 0xe083, 0x06, 0x4e78, 0x06, 0x049e,
	0x06, 0x04ee, 0x06, 0x834e, 0x06, 0x02e0, 0x06, 0x8332,
	0x06, 0xe183, 0x06, 0x3359, 0x06, 0x0fe2, 0x06, 0x834d,
	0x06, 0x0c24, 0x06, 0x5af0, 0x06, 0x1e12, 0x06, 0xe4f8,
	0x06, 0x8ce5, 0x06, 0xf88d, 0x06, 0xe083, 0x06, 0x30e1,
	0x06, 0x8331, 0x06, 0x6801, 0x06, 0xe4f8, 0x06, 0x8ae5,
	0x06, 0xf88b, 0x06, 0xae37, 0x06, 0xee83, 0x06, 0x4e03,
	0x06, 0xe083, 0x06, 0x4ce1, 0x06, 0x834d, 0x06, 0x1b01,
	0x06, 0x9e04, 0x06, 0xaaa1, 0x06, 0xaea8, 0x06, 0xee83,
	0x06, 0x4e04, 0x06, 0xee83, 0x06, 0x4f00, 0x06, 0xaeab,
	0x06, 0xe083, 0x06, 0x4f78, 0x06, 0x039f, 0x06, 0x14ee,
	0x06, 0x834e, 0x06, 0x05d2, 0x06, 0x40d6, 0x06, 0x5554,
	0x06, 0x0282, 0x06, 0x17d2, 0x06, 0xa0d6, 0x06, 0xba00,
	0x06, 0x0282, 0x06, 0x17fe, 0x06, 0xfdfc, 0x06, 0x05f8,
	0x06, 0xe0f8, 0x06, 0x60e1, 0x06, 0xf861, 0x06, 0x6802,
	0x06, 0xe4f8, 0x06, 0x60e5, 0x06, 0xf861, 0x06, 0xe0f8,
	0x06, 0x48e1, 0x06, 0xf849, 0x06, 0x580f, 0x06, 0x1e02,
	0x06, 0xe4f8, 0x06, 0x48e5, 0x06, 0xf849, 0x06, 0xd000,
	0x06, 0x0282, 0x06, 0x5bbf, 0x06, 0x8350, 0x06, 0xef46,
	0x06, 0xdc19, 0x06, 0xddd0, 0x06, 0x0102, 0x06, 0x825b,
	0x06, 0x0282, 0x06, 0x77e0, 0x06, 0xf860, 0x06, 0xe1f8,
	0x06, 0x6158, 0x06, 0xfde4, 0x06, 0xf860, 0x06, 0xe5f8,
	0x06, 0x61fc, 0x06, 0x04f9, 0x06, 0xfafb, 0x06, 0xc6bf,
	0x06, 0xf840, 0x06, 0xbe83, 0x06, 0x50a0, 0x06, 0x0101,
	0x06, 0x071b, 0x06, 0x89cf, 0x06, 0xd208, 0x06, 0xebdb,
	0x06, 0x19b2, 0x06, 0xfbff, 0x06, 0xfefd, 0x06, 0x04f8,
	0x06, 0xe0f8, 0x06, 0x48e1, 0x06, 0xf849, 0x06, 0x6808,
	0x06, 0xe4f8, 0x06, 0x48e5, 0x06, 0xf849, 0x06, 0x58f7,
	0x06, 0xe4f8, 0x06, 0x48e5, 0x06, 0xf849, 0x06, 0xfc04,
	0x06, 0x4d20, 0x06, 0x0002, 0x06, 0x4e22, 0x06, 0x0002,
	0x06, 0x4ddf, 0x06, 0xff01, 0x06, 0x4edd, 0x06, 0xff01,
	0x06, 0xf8fa, 0x06, 0xfbef, 0x06, 0x79bf, 0x06, 0xf822,
	0x06, 0xd819, 0x06, 0xd958, 0x06, 0x849f, 0x06, 0x09bf,
	0x06, 0x82be, 0x06, 0xd682, 0x06, 0xc602, 0x06, 0x014f,
	0x06, 0xef97, 0x06, 0xfffe, 0x06, 0xfc05, 0x06, 0x17ff,
	0x06, 0xfe01, 0x06, 0x1700, 0x06, 0x0102, 0x05, 0x83d8,
	0x06, 0x8051, 0x05, 0x83d6, 0x06, 0x82a0, 0x05, 0x83d4,
	0x06, 0x8000, 0x02, 0x2010, 0x03, 0xdc00, 0x1f, 0x0000,
	0x0b, 0x0600, 0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x00fc,
	0x1f, 0x0000,
};

static struct gani_patch gani_8168_m10_patch0[] = {
	/* 035 */
	0x1F, 0x0001, 0x06, 0x4064, 0x07, 0x2863, 0x08, 0x059C,
	0x09, 0x26B4, 0x0A, 0x6A19, 0x0B, 0xDCC8, 0x10, 0xF06D,
	0x14, 0x7F68, 0x18, 0x7FD9, 0x1C, 0xF0FF, 0x1D, 0x3D9C,
	0x1F, 0x0003, 0x12, 0xF49F, 0x13, 0x070B, 0x1A, 0x05AD,
	0x14, 0x94C0,

	0x1F, 0x0002, 0x06, 0x5561, 0x1F, 0x0005, 0x05, 0x8332,
	0x06, 0x5561,
};

static struct gani_patch gani_8168_m10_patch1[] = {
	/* 035 */
	0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080, 0x05, 0x8000,
	0x06, 0xf8f9, 0x06, 0xfaee, 0x06, 0xf8ea, 0x06, 0x00ee,
	0x06, 0xf8eb, 0x06, 0x00e2, 0x06, 0xf87c, 0x06, 0xe3f8,
	0x06, 0x7da5, 0x06, 0x1111, 0x06, 0x12d2, 0x06, 0x40d6,
	0x06, 0x4444, 0x06, 0x0281, 0x06, 0xc6d2, 0x06, 0xa0d6,
	0x06, 0xaaaa, 0x06, 0x0281, 0x06, 0xc6ae, 0x06, 0x0fa5,
	0x06, 0x4444, 0x06, 0x02ae, 0x06, 0x4da5, 0x06, 0xaaaa,
	0x06, 0x02ae, 0x06, 0x47af, 0x06, 0x81c2, 0x06, 0xee83,
	0x06, 0x4e00, 0x06, 0xee83, 0x06, 0x4d0f, 0x06, 0xee83,
	0x06, 0x4c0f, 0x06, 0xee83, 0x06, 0x4f00, 0x06, 0xee83,
	0x06, 0x5100, 0x06, 0xee83, 0x06, 0x4aff, 0x06, 0xee83,
	0x06, 0x4bff, 0x06, 0xe083, 0x06, 0x30e1, 0x06, 0x8331,
	0x06, 0x58fe, 0x06, 0xe4f8, 0x06, 0x8ae5, 0x06, 0xf88b,
	0x06, 0xe083, 0x06, 0x32e1, 0x06, 0x8333, 0x06, 0x590f,
	0x06, 0xe283, 0x06, 0x4d0c, 0x06, 0x245a, 0x06, 0xf01e,
	0x06, 0x12e4, 0x06, 0xf88c, 0x06, 0xe5f8, 0x06, 0x8daf,
	0x06, 0x81c2, 0x06, 0xe083, 0x06, 0x4f10, 0x06, 0xe483,
	0x06, 0x4fe0, 0x06, 0x834e, 0x06, 0x7800, 0x06, 0x9f0a,
	0x06, 0xe083, 0x06, 0x4fa0, 0x06, 0x10a5, 0x06, 0xee83,
	0x06, 0x4e01, 0x06, 0xe083, 0x06, 0x4e78, 0x06, 0x059e,
	0x06, 0x9ae0, 0x06, 0x834e, 0x06, 0x7804, 0x06, 0x9e10,
	0x06, 0xe083, 0x06, 0x4e78, 0x06, 0x039e, 0x06, 0x0fe0,
	0x06, 0x834e, 0x06, 0x7801, 0x06, 0x9e05, 0x06, 0xae0c,
	0x06, 0xaf81, 0x06, 0xa7af, 0x06, 0x8152, 0x06, 0xaf81,
	0x06, 0x8baf, 0x06, 0x81c2, 0x06, 0xee83, 0x06, 0x4800,
	0x06, 0xee83, 0x06, 0x4900, 0x06, 0xe083, 0x06, 0x5110,
	0x06, 0xe483, 0x06, 0x5158, 0x06, 0x019f, 0x06, 0xead0,
	0x06, 0x00d1, 0x06, 0x801f, 0x06, 0x66e2, 0x06, 0xf8ea,
	0x06, 0xe3f8, 0x06, 0xeb5a, 0x06, 0xf81e, 0x06, 0x20e6,
	0x06, 0xf8ea, 0x06, 0xe5f8, 0x06, 0xebd3, 0x06, 0x02b3,
	0x06, 0xfee2, 0x06, 0xf87c, 0x06, 0xef32, 0x06, 0x5b80,
	0x06, 0xe3f8, 0x06, 0x7d9e, 0x06, 0x037d, 0x06, 0xffff,
	0x06, 0x0d58, 0x06, 0x1c55, 0x06, 0x1a65, 0x06, 0x11a1,
	0x06, 0x90d3, 0x06, 0xe283, 0x06, 0x48e3, 0x06, 0x8349,
	0x06, 0x1b56, 0x06, 0xab08, 0x06, 0xef56, 0x06, 0xe683,
	0x06, 0x48e7, 0x06, 0x8349, 0x06, 0x10d1, 0x06, 0x801f,
	0x06, 0x66a0, 0x06, 0x04b9, 0x06, 0xe283, 0x06, 0x48e3,
	0x06, 0x8349, 0x06, 0xef65, 0x06, 0xe283, 0x06, 0x4ae3,
	0x06, 0x834b, 0x06, 0x1b56, 0x06, 0xaa0e, 0x06, 0xef56,
	0x06, 0xe683, 0x06, 0x4ae7, 0x06, 0x834b, 0x06, 0xe283,
	0x06, 0x4de6, 0x06, 0x834c, 0x06, 0xe083, 0x06, 0x4da0,
	0x06, 0x000c, 0x06, 0xaf81, 0x06, 0x8be0, 0x06, 0x834d,
	0x06, 0x10e4, 0x06, 0x834d, 0x06, 0xae04, 0x06, 0x80e4,
	0x06, 0x834d, 0x06, 0xe083, 0x06, 0x4e78, 0x06, 0x039e,
	0x06, 0x0be0, 0x06, 0x834e, 0x06, 0x7804, 0x06, 0x9e04,
	0x06, 0xee83, 0x06, 0x4e02, 0x06, 0xe083, 0x06, 0x32e1,
	0x06, 0x8333, 0x06, 0x590f, 0x06, 0xe283, 0x06, 0x4d0c,
	0x06, 0x245a, 0x06, 0xf01e, 0x06, 0x12e4, 0x06, 0xf88c,
	0x06, 0xe5f8, 0x06, 0x8de0, 0x06, 0x8330, 0x06, 0xe183,
	0x06, 0x3168, 0x06, 0x01e4, 0x06, 0xf88a, 0x06, 0xe5f8,
	0x06, 0x8bae, 0x06, 0x37ee, 0x06, 0x834e, 0x06, 0x03e0,
	0x06, 0x834c, 0x06, 0xe183, 0x06, 0x4d1b, 0x06, 0x019e,
	0x06, 0x04aa, 0x06, 0xa1ae, 0x06, 0xa8ee, 0x06, 0x834e,
	0x06, 0x04ee, 0x06, 0x834f, 0x06, 0x00ae, 0x06, 0xabe0,
	0x06, 0x834f, 0x06, 0x7803, 0x06, 0x9f14, 0x06, 0xee83,
	0x06, 0x4e05, 0x06, 0xd240, 0x06, 0xd655, 0x06, 0x5402,
	0x06, 0x81c6, 0x06, 0xd2a0, 0x06, 0xd6ba, 0x06, 0x0002,
	0x06, 0x81c6, 0x06, 0xfefd, 0x06, 0xfc05, 0x06, 0xf8e0,
	0x06, 0xf860, 0x06, 0xe1f8, 0x06, 0x6168, 0x06, 0x02e4,
	0x06, 0xf860, 0x06, 0xe5f8, 0x06, 0x61e0, 0x06, 0xf848,
	0x06, 0xe1f8, 0x06, 0x4958, 0x06, 0x0f1e, 0x06, 0x02e4,
	0x06, 0xf848, 0x06, 0xe5f8, 0x06, 0x49d0, 0x06, 0x0002,
	0x06, 0x820a, 0x06, 0xbf83, 0x06, 0x50ef, 0x06, 0x46dc,
	0x06, 0x19dd, 0x06, 0xd001, 0x06, 0x0282, 0x06, 0x0a02,
	0x06, 0x8226, 0x06, 0xe0f8, 0x06, 0x60e1, 0x06, 0xf861,
	0x06, 0x58fd, 0x06, 0xe4f8, 0x06, 0x60e5, 0x06, 0xf861,
	0x06, 0xfc04, 0x06, 0xf9fa, 0x06, 0xfbc6, 0x06, 0xbff8,
	0x06, 0x40be, 0x06, 0x8350, 0x06, 0xa001, 0x06, 0x0107,
	0x06, 0x1b89, 0x06, 0xcfd2, 0x06, 0x08eb, 0x06, 0xdb19,
	0x06, 0xb2fb, 0x06, 0xfffe, 0x06, 0xfd04, 0x06, 0xf8e0,
	0x06, 0xf848, 0x06, 0xe1f8, 0x06, 0x4968, 0x06, 0x08e4,
	0x06, 0xf848, 0x06, 0xe5f8, 0x06, 0x4958, 0x06, 0xf7e4,
	0x06, 0xf848, 0x06, 0xe5f8, 0x06, 0x49fc, 0x06, 0x044d,
	0x06, 0x2000, 0x06, 0x024e, 0x06, 0x2200, 0x06, 0x024d,
	0x06, 0xdfff, 0x06, 0x014e, 0x06, 0xddff, 0x06, 0x01f8,
	0x06, 0xfafb, 0x06, 0xef79, 0x06, 0xbff8, 0x06, 0x22d8,
	0x06, 0x19d9, 0x06, 0x5884, 0x06, 0x9f09, 0x06, 0xbf82,
	0x06, 0x6dd6, 0x06, 0x8275, 0x06, 0x0201, 0x06, 0x4fef,
	0x06, 0x97ff, 0x06, 0xfefc, 0x06, 0x0517, 0x06, 0xfffe,
	0x06, 0x0117, 0x06, 0x0001, 0x06, 0x0200, 0x05, 0x83d8,
	0x06, 0x8000, 0x05, 0x83d6, 0x06, 0x824f, 0x02, 0x2010,
	0x03, 0xdc00, 0x1f, 0x0000, 0x0b, 0x0600, 0x1f, 0x0005,
	0x05, 0xfff6, 0x06, 0x00fc, 0x1f, 0x0000,
};

static struct gani_patch gani_8168_m11_patch[] = {
	/* 035 */
	0x1F, 0x0002, 0x10, 0x0008, 0x0D, 0x006C,

	0x1F, 0x0001, 0x17, 0x0CC0,

	0x1F, 0x0001, 0x0B, 0xA4D8, 0x09, 0x281C, 0x07, 0x2883,
	0x0A, 0x6B35, 0x1D, 0x3DA4, 0x1C, 0xEFFD, 0x14, 0x7F52,
	0x18, 0x7FC6, 0x08, 0x0601, 0x06, 0x4063, 0x10, 0xF074,
	0x1F, 0x0003, 0x13, 0x0789, 0x12, 0xF4BD, 0x1A, 0x04FD,
	0x14, 0x84B0, 0x1F, 0x0000, 0x00, 0x9200,

	0x1F, 0x0005, 0x01, 0x0340, 0x1F, 0x0001, 0x04, 0x4000,
	0x03, 0x1D21, 0x02, 0x0C32, 0x01, 0x0200, 0x00, 0x5554,
	0x04, 0x4800, 0x04, 0x4000, 0x04, 0xF000, 0x03, 0xDF01,
	0x02, 0xDF20, 0x01, 0x101A, 0x00, 0xA0FF, 0x04, 0xF800,
	0x04, 0xF000, 0x1F, 0x0000,

	0x1F, 0x0007, 0x1E, 0x0023, 0x16, 0x0000, 0x1F, 0x0000,
};

static struct gani_patch gani_8168_m14_patch0[] = {
	/* 035 */
	0x1f, 0x0000, 0x1f, 0x0007, 0x1e, 0x0023, 0x16, 0x0306,
	0x16, 0x0307, 0x15, 0x000e, 0x19, 0x000a, 0x15, 0x0010,
	0x19, 0x0008, 0x15, 0x0018, 0x19, 0x4801, 0x15, 0x0019,
	0x19, 0x6801, 0x15, 0x001a, 0x19, 0x66a1, 0x15, 0x001f,
	0x19, 0x0000, 0x15, 0x0020, 0x19, 0x0000, 0x15, 0x0021,
	0x19, 0x0000, 0x15, 0x0022, 0x19, 0x0000, 0x15, 0x0023,
	0x19, 0x0000, 0x15, 0x0024, 0x19, 0x0000, 0x15, 0x0025,
	0x19, 0x64a1, 0x15, 0x0026, 0x19, 0x40ea, 0x15, 0x0027,
	0x19, 0x4503, 0x15, 0x0028, 0x19, 0x9f00, 0x15, 0x0029,
	0x19, 0xa631, 0x15, 0x002a, 0x19, 0x9717, 0x15, 0x002b,
	0x19, 0x302c, 0x15, 0x002c, 0x19, 0x4802, 0x15, 0x002d,
	0x19, 0x58da, 0x15, 0x002e, 0x19, 0x400d, 0x15, 0x002f,
	0x19, 0x4488, 0x15, 0x0030, 0x19, 0x9e00, 0x15, 0x0031,
	0x19, 0x63c8, 0x15, 0x0032, 0x19, 0x6481, 0x15, 0x0033,
	0x19, 0x0000, 0x15, 0x0034, 0x19, 0x0000, 0x15, 0x0035,
	0x19, 0x0000, 0x15, 0x0036, 0x19, 0x0000, 0x15, 0x0037,
	0x19, 0x0000, 0x15, 0x0038, 0x19, 0x0000, 0x15, 0x0039,
	0x19, 0x0000, 0x15, 0x003a, 0x19, 0x0000, 0x15, 0x003b,
	0x19, 0x63e8, 0x15, 0x003c, 0x19, 0x7d00, 0x15, 0x003d,
	0x19, 0x59d4, 0x15, 0x003e, 0x19, 0x63f8, 0x15, 0x0040,
	0x19, 0x64a1, 0x15, 0x0041, 0x19, 0x30de, 0x15, 0x0044,
	0x19, 0x480f, 0x15, 0x0045, 0x19, 0x6800, 0x15, 0x0046,
	0x19, 0x6680, 0x15, 0x0047, 0x19, 0x7c10, 0x15, 0x0048,
	0x19, 0x63c8, 0x15, 0x0049, 0x19, 0x0000, 0x15, 0x004a,
	0x19, 0x0000, 0x15, 0x004b, 0x19, 0x0000, 0x15, 0x004c,
	0x19, 0x0000, 0x15, 0x004d, 0x19, 0x0000, 0x15, 0x004e,
	0x19, 0x0000, 0x15, 0x004f, 0x19, 0x40ea, 0x15, 0x0050,
	0x19, 0x4503, 0x15, 0x0051, 0x19, 0x58ca, 0x15, 0x0052,
	0x19, 0x63c8, 0x15, 0x0053, 0x19, 0x63d8, 0x15, 0x0054,
	0x19, 0x66a0, 0x15, 0x0055, 0x19, 0x9f00, 0x15, 0x0056,
	0x19, 0x3000, 0x15, 0x006E, 0x19, 0x9afa, 0x15, 0x00a1,
	0x19, 0x3044, 0x15, 0x00ab, 0x19, 0x5820, 0x15, 0x00ac,
	0x19, 0x5e04, 0x15, 0x00ad, 0x19, 0xb60c, 0x15, 0x00af,
	0x19, 0x000a, 0x15, 0x00b2, 0x19, 0x30b9, 0x15, 0x00b9,
	0x19, 0x4408, 0x15, 0x00ba, 0x19, 0x480b, 0x15, 0x00bb,
	0x19, 0x5e00, 0x15, 0x00bc, 0x19, 0x405f, 0x15, 0x00bd,
	0x19, 0x4448, 0x15, 0x00be, 0x19, 0x4020, 0x15, 0x00bf,
	0x19, 0x4468, 0x15, 0x00c0, 0x19, 0x9c02, 0x15, 0x00c1,
	0x19, 0x58a0, 0x15, 0x00c2, 0x19, 0xb605, 0x15, 0x00c3,
	0x19, 0xc0d3, 0x15, 0x00c4, 0x19, 0x00e6, 0x15, 0x00c5,
	0x19, 0xdaec, 0x15, 0x00c6, 0x19, 0x00fa, 0x15, 0x00c7,
	0x19, 0x9df9, 0x15, 0x00c8, 0x19, 0x307a, 0x15, 0x0112,
	0x19, 0x6421, 0x15, 0x0113, 0x19, 0x7c08, 0x15, 0x0114,
	0x19, 0x63f0, 0x15, 0x0115, 0x19, 0x4003, 0x15, 0x0116,
	0x19, 0x4418, 0x15, 0x0117, 0x19, 0x9b00, 0x15, 0x0118,
	0x19, 0x6461, 0x15, 0x0119, 0x19, 0x64e1, 0x15, 0x011a,
	0x19, 0x0000, 0x15, 0x0150, 0x19, 0x7c80, 0x15, 0x0151,
	0x19, 0x6461, 0x15, 0x0152, 0x19, 0x4003, 0x15, 0x0153,
	0x19, 0x4540, 0x15, 0x0154, 0x19, 0x9f00, 0x15, 0x0155,
	0x19, 0x9d00, 0x15, 0x0156, 0x19, 0x7c40, 0x15, 0x0157,
	0x19, 0x6421, 0x15, 0x0158, 0x19, 0x7c80, 0x15, 0x0159,
	0x19, 0x64a1, 0x15, 0x015a, 0x19, 0x30fe, 0x15, 0x021e,
	0x19, 0x5410, 0x15, 0x0225, 0x19, 0x5400, 0x15, 0x023D,
	0x19, 0x4050, 0x15, 0x0295, 0x19, 0x6c08, 0x15, 0x02bd,
	0x19, 0xa523, 0x15, 0x02be, 0x19, 0x32ca, 0x15, 0x02ca,
	0x19, 0x48b3, 0x15, 0x02cb, 0x19, 0x4020, 0x15, 0x02cc,
	0x19, 0x4823, 0x15, 0x02cd, 0x19, 0x4510, 0x15, 0x02ce,
	0x19, 0xb63a, 0x15, 0x02cf, 0x19, 0x7dc8, 0x15, 0x02d6,
	0x19, 0x9bf8, 0x15, 0x02d8, 0x19, 0x85f6, 0x15, 0x02d9,
	0x19, 0x32e0, 0x15, 0x02e0, 0x19, 0x4834, 0x15, 0x02e1,
	0x19, 0x6c08, 0x15, 0x02e2, 0x19, 0x4020, 0x15, 0x02e3,
	0x19, 0x4824, 0x15, 0x02e4, 0x19, 0x4520, 0x15, 0x02e5,
	0x19, 0x4008, 0x15, 0x02e6, 0x19, 0x4560, 0x15, 0x02e7,
	0x19, 0x9d04, 0x15, 0x02e8, 0x19, 0x48c4, 0x15, 0x02e9,
	0x19, 0x0000, 0x15, 0x02ea, 0x19, 0x4844, 0x15, 0x02eb,
	0x19, 0x7dc8, 0x15, 0x02f0, 0x19, 0x9cf7, 0x15, 0x02f1,
	0x19, 0xdf94, 0x15, 0x02f2, 0x19, 0x0002, 0x15, 0x02f3,
	0x19, 0x6810, 0x15, 0x02f4, 0x19, 0xb614, 0x15, 0x02f5,
	0x19, 0xc42b, 0x15, 0x02f6, 0x19, 0x00d4, 0x15, 0x02f7,
	0x19, 0xc455, 0x15, 0x02f8, 0x19, 0x0093, 0x15, 0x02f9,
	0x19, 0x92ee, 0x15, 0x02fa, 0x19, 0xefed, 0x15, 0x02fb,
	0x19, 0x3312, 0x15, 0x0312, 0x19, 0x49b5, 0x15, 0x0313,
	0x19, 0x7d00, 0x15, 0x0314, 0x19, 0x4d00, 0x15, 0x0315,
	0x19, 0x6810, 0x15, 0x031e, 0x19, 0x404f, 0x15, 0x031f,
	0x19, 0x44c8, 0x15, 0x0320, 0x19, 0xd64f, 0x15, 0x0321,
	0x19, 0x00e7, 0x15, 0x0322, 0x19, 0x7c08, 0x15, 0x0323,
	0x19, 0x8203, 0x15, 0x0324, 0x19, 0x4d48, 0x15, 0x0325,
	0x19, 0x3327, 0x15, 0x0326, 0x19, 0x4d40, 0x15, 0x0327,
	0x19, 0xc8d7, 0x15, 0x0328, 0x19, 0x0003, 0x15, 0x0329,
	0x19, 0x7c20, 0x15, 0x032a, 0x19, 0x4c20, 0x15, 0x032b,
	0x19, 0xc8ed, 0x15, 0x032c, 0x19, 0x00f4, 0x15, 0x032d,
	0x19, 0x82b3, 0x15, 0x032e, 0x19, 0xd11d, 0x15, 0x032f,
	0x19, 0x00b1, 0x15, 0x0330, 0x19, 0xde18, 0x15, 0x0331,
	0x19, 0x0008, 0x15, 0x0332, 0x19, 0x91ee, 0x15, 0x0333,
	0x19, 0x3339, 0x15, 0x033a, 0x19, 0x4064, 0x15, 0x0340,
	0x19, 0x9e06, 0x15, 0x0341, 0x19, 0x7c08, 0x15, 0x0342,
	0x19, 0x8203, 0x15, 0x0343, 0x19, 0x4d48, 0x15, 0x0344,
	0x19, 0x3346, 0x15, 0x0345, 0x19, 0x4d40, 0x15, 0x0346,
	0x19, 0xd11d, 0x15, 0x0347, 0x19, 0x0099, 0x15, 0x0348,
	0x19, 0xbb17, 0x15, 0x0349, 0x19, 0x8102, 0x15, 0x034a,
	0x19, 0x334d, 0x15, 0x034b, 0x19, 0xa22c, 0x15, 0x034c,
	0x19, 0x3397, 0x15, 0x034d, 0x19, 0x91f2, 0x15, 0x034e,
	0x19, 0xc218, 0x15, 0x034f, 0x19, 0x00f0, 0x15, 0x0350,
	0x19, 0x3397, 0x15, 0x0351, 0x19, 0x0000, 0x15, 0x0364,
	0x19, 0xbc05, 0x15, 0x0367, 0x19, 0xa1fc, 0x15, 0x0368,
	0x19, 0x3377, 0x15, 0x0369, 0x19, 0x328b, 0x15, 0x036a,
	0x19, 0x0000, 0x15, 0x0377, 0x19, 0x4b97, 0x15, 0x0378,
	0x19, 0x6818, 0x15, 0x0379, 0x19, 0x4b07, 0x15, 0x037a,
	0x19, 0x40ac, 0x15, 0x037b, 0x19, 0x4445, 0x15, 0x037c,
	0x19, 0x404e, 0x15, 0x037d, 0x19, 0x4461, 0x15, 0x037e,
	0x19, 0x9c09, 0x15, 0x037f, 0x19, 0x63da, 0x15, 0x0380,
	0x19, 0x5440, 0x15, 0x0381, 0x19, 0x4b98, 0x15, 0x0382,
	0x19, 0x7c60, 0x15, 0x0383, 0x19, 0x4c00, 0x15, 0x0384,
	0x19, 0x4b08, 0x15, 0x0385, 0x19, 0x63d8, 0x15, 0x0386,
	0x19, 0x338d, 0x15, 0x0387, 0x19, 0xd64f, 0x15, 0x0388,
	0x19, 0x0080, 0x15, 0x0389, 0x19, 0x820c, 0x15, 0x038a,
	0x19, 0xa10b, 0x15, 0x038b, 0x19, 0x9df3, 0x15, 0x038c,
	0x19, 0x3395, 0x15, 0x038d, 0x19, 0xd64f, 0x15, 0x038e,
	0x19, 0x00f9, 0x15, 0x038f, 0x19, 0xc017, 0x15, 0x0390,
	0x19, 0x0005, 0x15, 0x0391, 0x19, 0x6c0b, 0x15, 0x0392,
	0x19, 0xa103, 0x15, 0x0393, 0x19, 0x6c08, 0x15, 0x0394,
	0x19, 0x9df9, 0x15, 0x0395, 0x19, 0x6c08, 0x15, 0x0396,
	0x19, 0x3397, 0x15, 0x0399, 0x19, 0x6810, 0x15, 0x03a4,
	0x19, 0x7c08, 0x15, 0x03a5, 0x19, 0x8203, 0x15, 0x03a6,
	0x19, 0x4d08, 0x15, 0x03a7, 0x19, 0x33a9, 0x15, 0x03a8,
	0x19, 0x4d00, 0x15, 0x03a9, 0x19, 0x9bfa, 0x15, 0x03aa,
	0x19, 0x33b6, 0x15, 0x03bb, 0x19, 0x4056, 0x15, 0x03bc,
	0x19, 0x44e9, 0x15, 0x03bd, 0x19, 0x405e, 0x15, 0x03be,
	0x19, 0x44f8, 0x15, 0x03bf, 0x19, 0xd64f, 0x15, 0x03c0,
	0x19, 0x0037, 0x15, 0x03c1, 0x19, 0xbd37, 0x15, 0x03c2,
	0x19, 0x9cfd, 0x15, 0x03c3, 0x19, 0xc639, 0x15, 0x03c4,
	0x19, 0x0011, 0x15, 0x03c5, 0x19, 0x9b03, 0x15, 0x03c6,
	0x19, 0x7c01, 0x15, 0x03c7, 0x19, 0x4c01, 0x15, 0x03c8,
	0x19, 0x9e03, 0x15, 0x03c9, 0x19, 0x7c20, 0x15, 0x03ca,
	0x19, 0x4c20, 0x15, 0x03cb, 0x19, 0x9af4, 0x15, 0x03cc,
	0x19, 0x7c12, 0x15, 0x03cd, 0x19, 0x4c52, 0x15, 0x03ce,
	0x19, 0x4470, 0x15, 0x03cf, 0x19, 0x7c12, 0x15, 0x03d0,
	0x19, 0x4c40, 0x15, 0x03d1, 0x19, 0x33bf, 0x15, 0x03d6,
	0x19, 0x4047, 0x15, 0x03d7, 0x19, 0x4469, 0x15, 0x03d8,
	0x19, 0x492b, 0x15, 0x03d9, 0x19, 0x4479, 0x15, 0x03da,
	0x19, 0x7c09, 0x15, 0x03db, 0x19, 0x8203, 0x15, 0x03dc,
	0x19, 0x4d48, 0x15, 0x03dd, 0x19, 0x33df, 0x15, 0x03de,
	0x19, 0x4d40, 0x15, 0x03df, 0x19, 0xd64f, 0x15, 0x03e0,
	0x19, 0x0017, 0x15, 0x03e1, 0x19, 0xbd17, 0x15, 0x03e2,
	0x19, 0x9b03, 0x15, 0x03e3, 0x19, 0x7c20, 0x15, 0x03e4,
	0x19, 0x4c20, 0x15, 0x03e5, 0x19, 0x88f5, 0x15, 0x03e6,
	0x19, 0xc428, 0x15, 0x03e7, 0x19, 0x0008, 0x15, 0x03e8,
	0x19, 0x9af2, 0x15, 0x03e9, 0x19, 0x7c12, 0x15, 0x03ea,
	0x19, 0x4c52, 0x15, 0x03eb, 0x19, 0x4470, 0x15, 0x03ec,
	0x19, 0x7c12, 0x15, 0x03ed, 0x19, 0x4c40, 0x15, 0x03ee,
	0x19, 0x33da, 0x15, 0x03ef, 0x19, 0x3312, 0x16, 0x0306,
	0x16, 0x0300, 0x1f, 0x0000, 0x17, 0x2179, 0x1f, 0x0007,
	0x1e, 0x0040, 0x18, 0x0645, 0x19, 0xe200, 0x18, 0x0655,
	0x19, 0x9000, 0x18, 0x0d05, 0x19, 0xbe00, 0x18, 0x0d15,
	0x19, 0xd300, 0x18, 0x0d25, 0x19, 0xfe00, 0x18, 0x0d35,
	0x19, 0x4000, 0x18, 0x0d45, 0x19, 0x7f00, 0x18, 0x0d55,
	0x19, 0x1000, 0x18, 0x0d65, 0x19, 0x0000, 0x18, 0x0d75,
	0x19, 0x8200, 0x18, 0x0d85, 0x19, 0x0000, 0x18, 0x0d95,
	0x19, 0x7000, 0x18, 0x0da5, 0x19, 0x0f00, 0x18, 0x0db5,
	0x19, 0x0100, 0x18, 0x0dc5, 0x19, 0x9b00, 0x18, 0x0dd5,
	0x19, 0x7f00, 0x18, 0x0de5, 0x19, 0xe000, 0x18, 0x0df5,
	0x19, 0xef00, 0x18, 0x16d5, 0x19, 0xe200, 0x18, 0x16e5,
	0x19, 0xab00, 0x18, 0x2904, 0x19, 0x4000, 0x18, 0x2914,
	0x19, 0x7f00, 0x18, 0x2924, 0x19, 0x0100, 0x18, 0x2934,
	0x19, 0x2000, 0x18, 0x2944, 0x19, 0x0000, 0x18, 0x2954,
	0x19, 0x4600, 0x18, 0x2964, 0x19, 0xfc00, 0x18, 0x2974,
	0x19, 0x0000, 0x18, 0x2984, 0x19, 0x5000, 0x18, 0x2994,
	0x19, 0x9d00, 0x18, 0x29a4, 0x19, 0xff00, 0x18, 0x29b4,
	0x19, 0x4000, 0x18, 0x29c4, 0x19, 0x7f00, 0x18, 0x29d4,
	0x19, 0x0000, 0x18, 0x29e4, 0x19, 0x2000, 0x18, 0x29f4,
	0x19, 0x0000, 0x18, 0x2a04, 0x19, 0xe600, 0x18, 0x2a14,
	0x19, 0xff00, 0x18, 0x2a24, 0x19, 0x0000, 0x18, 0x2a34,
	0x19, 0x5000, 0x18, 0x2a44, 0x19, 0x8500, 0x18, 0x2a54,
	0x19, 0x7f00, 0x18, 0x2a64, 0x19, 0xac00, 0x18, 0x2a74,
	0x19, 0x0800, 0x18, 0x2a84, 0x19, 0xfc00, 0x18, 0x2a94,
	0x19, 0xe000, 0x18, 0x2aa4, 0x19, 0x7400, 0x18, 0x2ab4,
	0x19, 0x4000, 0x18, 0x2ac4, 0x19, 0x7f00, 0x18, 0x2ad4,
	0x19, 0x0100, 0x18, 0x2ae4, 0x19, 0xff00, 0x18, 0x2af4,
	0x19, 0x0000, 0x18, 0x2b04, 0x19, 0x4400, 0x18, 0x2b14,
	0x19, 0xfc00, 0x18, 0x2b24, 0x19, 0x0000, 0x18, 0x2b34,
	0x19, 0x4000, 0x18, 0x2b44, 0x19, 0x9d00, 0x18, 0x2b54,
	0x19, 0xff00, 0x18, 0x2b64, 0x19, 0x4000, 0x18, 0x2b74,
	0x19, 0x7f00, 0x18, 0x2b84, 0x19, 0x0000, 0x18, 0x2b94,
	0x19, 0xff00, 0x18, 0x2ba4, 0x19, 0x0000, 0x18, 0x2bb4,
	0x19, 0xfc00, 0x18, 0x2bc4, 0x19, 0xff00, 0x18, 0x2bd4,
	0x19, 0x0000, 0x18, 0x2be4, 0x19, 0x4000, 0x18, 0x2bf4,
	0x19, 0x8900, 0x18, 0x2c04, 0x19, 0x8300, 0x18, 0x2c14,
	0x19, 0xe000, 0x18, 0x2c24, 0x19, 0x0000, 0x18, 0x2c34,
	0x19, 0xac00, 0x18, 0x2c44, 0x19, 0x0800, 0x18, 0x2c54,
	0x19, 0xfa00, 0x18, 0x2c64, 0x19, 0xe100, 0x18, 0x2c74,
	0x19, 0x7f00, 0x18, 0x0001, 0x1f, 0x0000, 0x17, 0x2100,
	0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080, 0x05, 0x8b88,
	0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000,
	0x05, 0x8000, 0x06, 0xd480, 0x06, 0xc1e4, 0x06, 0x8b9a,
	0x06, 0xe58b, 0x06, 0x9bee, 0x06, 0x8b83, 0x06, 0x41bf,
	0x06, 0x8b88, 0x06, 0xec00, 0x06, 0x19a9, 0x06, 0x8b90,
	0x06, 0xf9ee, 0x06, 0xfff6, 0x06, 0x00ee, 0x06, 0xfff7,
	0x06, 0xffe0, 0x06, 0xe140, 0x06, 0xe1e1, 0x06, 0x41f7,
	0x06, 0x2ff6, 0x06, 0x28e4, 0x06, 0xe140, 0x06, 0xe5e1,
	0x06, 0x41f7, 0x06, 0x0002, 0x06, 0x020c, 0x06, 0x0202,
	0x06, 0x1d02, 0x06, 0x0230, 0x06, 0x0202, 0x06, 0x4002,
	0x06, 0x028b, 0x06, 0x0280, 0x06, 0x6c02, 0x06, 0x8085,
	0x06, 0xe08b, 0x06, 0x88e1, 0x06, 0x8b89, 0x06, 0x1e01,
	0x06, 0xe18b, 0x06, 0x8a1e, 0x06, 0x01e1, 0x06, 0x8b8b,
	0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8c1e, 0x06, 0x01e1,
	0x06, 0x8b8d, 0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8e1e,
	0x06, 0x01a0, 0x06, 0x00c7, 0x06, 0xaec3, 0x06, 0xf8e0,
	0x06, 0x8b8d, 0x06, 0xad20, 0x06, 0x10ee, 0x06, 0x8b8d,
	0x06, 0x0002, 0x06, 0x1310, 0x06, 0x021f, 0x06, 0x9d02,
	0x06, 0x1f0c, 0x06, 0x0227, 0x06, 0x49fc, 0x06, 0x04f8,
	0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x200b, 0x06, 0xf620,
	0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x830e, 0x06, 0x021b,
	0x06, 0x67ad, 0x06, 0x2211, 0x06, 0xf622, 0x06, 0xe48b,
	0x06, 0x8e02, 0x06, 0x2ba5, 0x06, 0x022a, 0x06, 0x2402,
	0x06, 0x80c6, 0x06, 0x022a, 0x06, 0xf0ad, 0x06, 0x2511,
	0x06, 0xf625, 0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x8226,
	0x06, 0x0204, 0x06, 0x0302, 0x06, 0x19cc, 0x06, 0x022b,
	0x06, 0x5bfc, 0x06, 0x04ee, 0x06, 0x8b8d, 0x06, 0x0105,
	0x06, 0xf8e0, 0x06, 0x8b83, 0x06, 0xad24, 0x06, 0x44e0,
	0x06, 0xe022, 0x06, 0xe1e0, 0x06, 0x23ad, 0x06, 0x223b,
	0x06, 0xe08a, 0x06, 0xbea0, 0x06, 0x0005, 0x06, 0x0228,
	0x06, 0xdeae, 0x06, 0x42a0, 0x06, 0x0105, 0x06, 0x0228,
	0x06, 0xf1ae, 0x06, 0x3aa0, 0x06, 0x0205, 0x06, 0x0281,
	0x06, 0x25ae, 0x06, 0x32a0, 0x06, 0x0305, 0x06, 0x0229,
	0x06, 0x9aae, 0x06, 0x2aa0, 0x06, 0x0405, 0x06, 0x0229,
	0x06, 0xaeae, 0x06, 0x22a0, 0x06, 0x0505, 0x06, 0x0229,
	0x06, 0xd7ae, 0x06, 0x1aa0, 0x06, 0x0605, 0x06, 0x0229,
	0x06, 0xfeae, 0x06, 0x12ee, 0x06, 0x8ac0, 0x06, 0x00ee,
	0x06, 0x8ac1, 0x06, 0x00ee, 0x06, 0x8ac6, 0x06, 0x00ee,
	0x06, 0x8abe, 0x06, 0x00ae, 0x06, 0x00fc, 0x06, 0x04f8,
	0x06, 0x022a, 0x06, 0x67e0, 0x06, 0xe022, 0x06, 0xe1e0,
	0x06, 0x230d, 0x06, 0x0658, 0x06, 0x03a0, 0x06, 0x0202,
	0x06, 0xae2d, 0x06, 0xa001, 0x06, 0x02ae, 0x06, 0x2da0,
	0x06, 0x004d, 0x06, 0xe0e2, 0x06, 0x00e1, 0x06, 0xe201,
	0x06, 0xad24, 0x06, 0x44e0, 0x06, 0x8ac2, 0x06, 0xe48a,
	0x06, 0xc4e0, 0x06, 0x8ac3, 0x06, 0xe48a, 0x06, 0xc5ee,
	0x06, 0x8abe, 0x06, 0x03e0, 0x06, 0x8b83, 0x06, 0xad25,
	0x06, 0x3aee, 0x06, 0x8abe, 0x06, 0x05ae, 0x06, 0x34e0,
	0x06, 0x8ace, 0x06, 0xae03, 0x06, 0xe08a, 0x06, 0xcfe1,
	0x06, 0x8ac2, 0x06, 0x4905, 0x06, 0xe58a, 0x06, 0xc4e1,
	0x06, 0x8ac3, 0x06, 0x4905, 0x06, 0xe58a, 0x06, 0xc5ee,
	0x06, 0x8abe, 0x06, 0x0502, 0x06, 0x2ab6, 0x06, 0xac20,
	0x06, 0x1202, 0x06, 0x819b, 0x06, 0xac20, 0x06, 0x0cee,
	0x06, 0x8ac1, 0x06, 0x00ee, 0x06, 0x8ac6, 0x06, 0x00ee,
	0x06, 0x8abe, 0x06, 0x02fc, 0x06, 0x04d0, 0x06, 0x0002,
	0x06, 0x81ad, 0x06, 0x590f, 0x06, 0x3902, 0x06, 0xaa04,
	0x06, 0xd001, 0x06, 0xae02, 0x06, 0xd000, 0x06, 0x04f9,
	0x06, 0xfae2, 0x06, 0xe2d2, 0x06, 0xe3e2, 0x06, 0xd3f9,
	0x06, 0x5af7, 0x06, 0xe6e2, 0x06, 0xd2e7, 0x06, 0xe2d3,
	0x06, 0xe2e0, 0x06, 0x2ce3, 0x06, 0xe02d, 0x06, 0xf95b,
	0x06, 0xe01e, 0x06, 0x30e6, 0x06, 0xe02c, 0x06, 0xe7e0,
	0x06, 0x2de2, 0x06, 0xe2cc, 0x06, 0xe3e2, 0x06, 0xcdf9,
	0x06, 0x5a0f, 0x06, 0x6a50, 0x06, 0xe6e2, 0x06, 0xcce7,
	0x06, 0xe2cd, 0x06, 0xe0e0, 0x06, 0x3ce1, 0x06, 0xe03d,
	0x06, 0xef64, 0x06, 0xfde0, 0x06, 0xe2cc, 0x06, 0xe1e2,
	0x06, 0xcd58, 0x06, 0x0f5a, 0x06, 0xf01e, 0x06, 0x02e4,
	0x06, 0xe2cc, 0x06, 0xe5e2, 0x06, 0xcdfd, 0x06, 0xe0e0,
	0x06, 0x2ce1, 0x06, 0xe02d, 0x06, 0x59e0, 0x06, 0x5b1f,
	0x06, 0x1e13, 0x06, 0xe4e0, 0x06, 0x2ce5, 0x06, 0xe02d,
	0x06, 0xfde0, 0x06, 0xe2d2, 0x06, 0xe1e2, 0x06, 0xd358,
	0x06, 0xf75a, 0x06, 0x081e, 0x06, 0x02e4, 0x06, 0xe2d2,
	0x06, 0xe5e2, 0x06, 0xd3ef, 0x06, 0x46fe, 0x06, 0xfd04,
	0x06, 0xf8f9, 0x06, 0xfaef, 0x06, 0x69e0, 0x06, 0xe022,
	0x06, 0xe1e0, 0x06, 0x2358, 0x06, 0xc4e1, 0x06, 0x8b6e,
	0x06, 0x1f10, 0x06, 0x9e58, 0x06, 0xe48b, 0x06, 0x6ead,
	0x06, 0x2222, 0x06, 0xac27, 0x06, 0x55ac, 0x06, 0x2602,
	0x06, 0xae1a, 0x06, 0xd106, 0x06, 0xbf3b, 0x06, 0xba02,
	0x06, 0x2dc1, 0x06, 0xd107, 0x06, 0xbf3b, 0x06, 0xbd02,
	0x06, 0x2dc1, 0x06, 0xd107, 0x06, 0xbf3b, 0x06, 0xc002,
	0x06, 0x2dc1, 0x06, 0xae30, 0x06, 0xd103, 0x06, 0xbf3b,
	0x06, 0xc302, 0x06, 0x2dc1, 0x06, 0xd100, 0x06, 0xbf3b,
	0x06, 0xc602, 0x06, 0x2dc1, 0x06, 0xd100, 0x06, 0xbf82,
	0x06, 0xca02, 0x06, 0x2dc1, 0x06, 0xd10f, 0x06, 0xbf3b,
	0x06, 0xba02, 0x06, 0x2dc1, 0x06, 0xd101, 0x06, 0xbf3b,
	0x06, 0xbd02, 0x06, 0x2dc1, 0x06, 0xd101, 0x06, 0xbf3b,
	0x06, 0xc002, 0x06, 0x2dc1, 0x06, 0xef96, 0x06, 0xfefd,
	0x06, 0xfc04, 0x06, 0xd100, 0x06, 0xbf3b, 0x06, 0xc302,
	0x06, 0x2dc1, 0x06, 0xd011, 0x06, 0x022b, 0x06, 0xfb59,
	0x06, 0x03ef, 0x06, 0x01d1, 0x06, 0x00a0, 0x06, 0x0002,
	0x06, 0xd101, 0x06, 0xbf3b, 0x06, 0xc602, 0x06, 0x2dc1,
	0x06, 0xd111, 0x06, 0xad20, 0x06, 0x020c, 0x06, 0x11ad,
	0x06, 0x2102, 0x06, 0x0c12, 0x06, 0xbf82, 0x06, 0xca02,
	0x06, 0x2dc1, 0x06, 0xaec8, 0x06, 0x70e4, 0x06, 0x2602,
	0x06, 0x82d1, 0x06, 0x05f8, 0x06, 0xfaef, 0x06, 0x69e0,
	0x06, 0xe2fe, 0x06, 0xe1e2, 0x06, 0xffad, 0x06, 0x2d1a,
	0x06, 0xe0e1, 0x06, 0x4ee1, 0x06, 0xe14f, 0x06, 0xac2d,
	0x06, 0x22f6, 0x06, 0x0302, 0x06, 0x033b, 0x06, 0xf703,
	0x06, 0xf706, 0x06, 0xbf84, 0x06, 0x4402, 0x06, 0x2d21,
	0x06, 0xae11, 0x06, 0xe0e1, 0x06, 0x4ee1, 0x06, 0xe14f,
	0x06, 0xad2d, 0x06, 0x08bf, 0x06, 0x844f, 0x06, 0x022d,
	0x06, 0x21f6, 0x06, 0x06ef, 0x06, 0x96fe, 0x06, 0xfc04,
	0x06, 0xf8fa, 0x06, 0xef69, 0x06, 0x0283, 0x06, 0x4502,
	0x06, 0x83a2, 0x06, 0xe0e0, 0x06, 0x00e1, 0x06, 0xe001,
	0x06, 0xad27, 0x06, 0x1fd1, 0x06, 0x01bf, 0x06, 0x843b,
	0x06, 0x022d, 0x06, 0xc1e0, 0x06, 0xe020, 0x06, 0xe1e0,
	0x06, 0x21ad, 0x06, 0x200e, 0x06, 0xd100, 0x06, 0xbf84,
	0x06, 0x3b02, 0x06, 0x2dc1, 0x06, 0xbf3b, 0x06, 0x9602,
	0x06, 0x2d21, 0x06, 0xef96, 0x06, 0xfefc, 0x06, 0x04f8,
	0x06, 0xf9fa, 0x06, 0xef69, 0x06, 0xe08b, 0x06, 0x87ad,
	0x06, 0x204c, 0x06, 0xd200, 0x06, 0xe0e2, 0x06, 0x0058,
	0x06, 0x010c, 0x06, 0x021e, 0x06, 0x20e0, 0x06, 0xe000,
	0x06, 0x5810, 0x06, 0x1e20, 0x06, 0xe0e0, 0x06, 0x3658,
	0x06, 0x031e, 0x06, 0x20e0, 0x06, 0xe022, 0x06, 0xe1e0,
	0x06, 0x2358, 0x06, 0xe01e, 0x06, 0x20e0, 0x06, 0x8b64,
	0x06, 0x1f02, 0x06, 0x9e22, 0x06, 0xe68b, 0x06, 0x64ad,
	0x06, 0x3214, 0x06, 0xad34, 0x06, 0x11ef, 0x06, 0x0258,
	0x06, 0x039e, 0x06, 0x07ad, 0x06, 0x3508, 0x06, 0x5ac0,
	0x06, 0x9f04, 0x06, 0xd101, 0x06, 0xae02, 0x06, 0xd100,
	0x06, 0xbf84, 0x06, 0x3e02, 0x06, 0x2dc1, 0x06, 0xef96,
	0x06, 0xfefd, 0x06, 0xfc04, 0x06, 0xf8f9, 0x06, 0xfbe0,
	0x06, 0x8b85, 0x06, 0xad25, 0x06, 0x22e0, 0x06, 0xe022,
	0x06, 0xe1e0, 0x06, 0x23e2, 0x06, 0xe036, 0x06, 0xe3e0,
	0x06, 0x375a, 0x06, 0xc40d, 0x06, 0x0158, 0x06, 0x021e,
	0x06, 0x20e3, 0x06, 0x8ae7, 0x06, 0xac31, 0x06, 0x60ac,
	0x06, 0x3a08, 0x06, 0xac3e, 0x06, 0x26ae, 0x06, 0x67af,
	0x06, 0x8437, 0x06, 0xad37, 0x06, 0x61e0, 0x06, 0x8ae8,
	0x06, 0x10e4, 0x06, 0x8ae8, 0x06, 0xe18a, 0x06, 0xe91b,
	0x06, 0x109e, 0x06, 0x02ae, 0x06, 0x51d1, 0x06, 0x00bf,
	0x06, 0x8441, 0x06, 0x022d, 0x06, 0xc1ee, 0x06, 0x8ae8,
	0x06, 0x00ae, 0x06, 0x43ad, 0x06, 0x3627, 0x06, 0xe08a,
	0x06, 0xeee1, 0x06, 0x8aef, 0x06, 0xef74, 0x06, 0xe08a,
	0x06, 0xeae1, 0x06, 0x8aeb, 0x06, 0x1b74, 0x06, 0x9e2e,
	0x06, 0x14e4, 0x06, 0x8aea, 0x06, 0xe58a, 0x06, 0xebef,
	0x06, 0x74e0, 0x06, 0x8aee, 0x06, 0xe18a, 0x06, 0xef1b,
	0x06, 0x479e, 0x06, 0x0fae, 0x06, 0x19ee, 0x06, 0x8aea,
	0x06, 0x00ee, 0x06, 0x8aeb, 0x06, 0x00ae, 0x06, 0x0fac,
	0x06, 0x390c, 0x06, 0xd101, 0x06, 0xbf84, 0x06, 0x4102,
	0x06, 0x2dc1, 0x06, 0xee8a, 0x06, 0xe800, 0x06, 0xe68a,
	0x06, 0xe7ff, 0x06, 0xfdfc, 0x06, 0x0400, 0x06, 0xe234,
	0x06, 0xcce2, 0x06, 0x0088, 0x06, 0xe200, 0x06, 0xa725,
	0x06, 0xe50a, 0x06, 0x1de5, 0x06, 0x0a2c, 0x06, 0xe50a,
	0x06, 0x6de5, 0x06, 0x0a1d, 0x06, 0xe50a, 0x06, 0x1ce5,
	0x06, 0x0a2d, 0x06, 0xa755, 0x05, 0x8b64, 0x06, 0x0000,
	0x05, 0x8b94, 0x06, 0x82cd, 0x05, 0x8b85, 0x06, 0x2000,
	0x05, 0x8aee, 0x06, 0x03b8, 0x05, 0x8ae8, 0x06, 0x0002,
};

static struct gani_patch gani_8168_m14_patch1[] = {
	0x1f, 0x0000, 0x1f, 0x0007, 0x1e, 0x0023, 0x16, 0x0306,
	0x16, 0x0307, 0x15, 0x000e, 0x19, 0x000a, 0x15, 0x0010,
	0x19, 0x0008, 0x15, 0x0018, 0x19, 0x4801, 0x15, 0x0019,
	0x19, 0x6801, 0x15, 0x001a, 0x19, 0x66a1, 0x15, 0x001f,
	0x19, 0x0000, 0x15, 0x0020, 0x19, 0x0000, 0x15, 0x0021,
	0x19, 0x0000, 0x15, 0x0022, 0x19, 0x0000, 0x15, 0x0023,
	0x19, 0x0000, 0x15, 0x0024, 0x19, 0x0000, 0x15, 0x0025,
	0x19, 0x64a1, 0x15, 0x0026, 0x19, 0x40ea, 0x15, 0x0027,
	0x19, 0x4503, 0x15, 0x0028, 0x19, 0x9f00, 0x15, 0x0029,
	0x19, 0xa631, 0x15, 0x002a, 0x19, 0x9717, 0x15, 0x002b,
	0x19, 0x302c, 0x15, 0x002c, 0x19, 0x4802, 0x15, 0x002d,
	0x19, 0x58da, 0x15, 0x002e, 0x19, 0x400d, 0x15, 0x002f,
	0x19, 0x4488, 0x15, 0x0030, 0x19, 0x9e00, 0x15, 0x0031,
	0x19, 0x63c8, 0x15, 0x0032, 0x19, 0x6481, 0x15, 0x0033,
	0x19, 0x0000, 0x15, 0x0034, 0x19, 0x0000, 0x15, 0x0035,
	0x19, 0x0000, 0x15, 0x0036, 0x19, 0x0000, 0x15, 0x0037,
	0x19, 0x0000, 0x15, 0x0038, 0x19, 0x0000, 0x15, 0x0039,
	0x19, 0x0000, 0x15, 0x003a, 0x19, 0x0000, 0x15, 0x003b,
	0x19, 0x63e8, 0x15, 0x003c, 0x19, 0x7d00, 0x15, 0x003d,
	0x19, 0x59d4, 0x15, 0x003e, 0x19, 0x63f8, 0x15, 0x0040,
	0x19, 0x64a1, 0x15, 0x0041, 0x19, 0x30de, 0x15, 0x0044,
	0x19, 0x480f, 0x15, 0x0045, 0x19, 0x6800, 0x15, 0x0046,
	0x19, 0x6680, 0x15, 0x0047, 0x19, 0x7c10, 0x15, 0x0048,
	0x19, 0x63c8, 0x15, 0x0049, 0x19, 0x0000, 0x15, 0x004a,
	0x19, 0x0000, 0x15, 0x004b, 0x19, 0x0000, 0x15, 0x004c,
	0x19, 0x0000, 0x15, 0x004d, 0x19, 0x0000, 0x15, 0x004e,
	0x19, 0x0000, 0x15, 0x004f, 0x19, 0x40ea, 0x15, 0x0050,
	0x19, 0x4503, 0x15, 0x0051, 0x19, 0x58ca, 0x15, 0x0052,
	0x19, 0x63c8, 0x15, 0x0053, 0x19, 0x63d8, 0x15, 0x0054,
	0x19, 0x66a0, 0x15, 0x0055, 0x19, 0x9f00, 0x15, 0x0056,
	0x19, 0x3000, 0x15, 0x00a1, 0x19, 0x3044, 0x15, 0x00ab,
	0x19, 0x5820, 0x15, 0x00ac, 0x19, 0x5e04, 0x15, 0x00ad,
	0x19, 0xb60c, 0x15, 0x00af, 0x19, 0x000a, 0x15, 0x00b2,
	0x19, 0x30b9, 0x15, 0x00b9, 0x19, 0x4408, 0x15, 0x00ba,
	0x19, 0x480b, 0x15, 0x00bb, 0x19, 0x5e00, 0x15, 0x00bc,
	0x19, 0x405f, 0x15, 0x00bd, 0x19, 0x4448, 0x15, 0x00be,
	0x19, 0x4020, 0x15, 0x00bf, 0x19, 0x4468, 0x15, 0x00c0,
	0x19, 0x9c02, 0x15, 0x00c1, 0x19, 0x58a0, 0x15, 0x00c2,
	0x19, 0xb605, 0x15, 0x00c3, 0x19, 0xc0d3, 0x15, 0x00c4,
	0x19, 0x00e6, 0x15, 0x00c5, 0x19, 0xdaec, 0x15, 0x00c6,
	0x19, 0x00fa, 0x15, 0x00c7, 0x19, 0x9df9, 0x15, 0x0112,
	0x19, 0x6421, 0x15, 0x0113, 0x19, 0x7c08, 0x15, 0x0114,
	0x19, 0x63f0, 0x15, 0x0115, 0x19, 0x4003, 0x15, 0x0116,
	0x19, 0x4418, 0x15, 0x0117, 0x19, 0x9b00, 0x15, 0x0118,
	0x19, 0x6461, 0x15, 0x0119, 0x19, 0x64e1, 0x15, 0x011a,
	0x19, 0x0000, 0x15, 0x0150, 0x19, 0x7c80, 0x15, 0x0151,
	0x19, 0x6461, 0x15, 0x0152, 0x19, 0x4003, 0x15, 0x0153,
	0x19, 0x4540, 0x15, 0x0154, 0x19, 0x9f00, 0x15, 0x0155,
	0x19, 0x9d00, 0x15, 0x0156, 0x19, 0x7c40, 0x15, 0x0157,
	0x19, 0x6421, 0x15, 0x0158, 0x19, 0x7c80, 0x15, 0x0159,
	0x19, 0x64a1, 0x15, 0x015a, 0x19, 0x30fe, 0x15, 0x02e7,
	0x19, 0x0000, 0x15, 0x0329, 0x19, 0x7c00, 0x15, 0x0382,
	0x19, 0x7c40, 0x15, 0x03bd, 0x19, 0x405e, 0x15, 0x03c9,
	0x19, 0x7c00, 0x15, 0x03e3, 0x19, 0x7c00, 0x16, 0x0306,
	0x16, 0x0300, 0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080,
	0x05, 0x8000, 0x06, 0x0280, 0x06, 0x48f7, 0x06, 0x00e0,
	0x06, 0xfff7, 0x06, 0xa080, 0x06, 0x02ae, 0x06, 0xf602,
	0x06, 0x0200, 0x06, 0x0280, 0x06, 0x8c02, 0x06, 0x0224,
	0x06, 0x0202, 0x06, 0x3402, 0x06, 0x027f, 0x06, 0x0280,
	0x06, 0xa202, 0x06, 0x80bb, 0x06, 0xe08b, 0x06, 0x88e1,
	0x06, 0x8b89, 0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8a1e,
	0x06, 0x01e1, 0x06, 0x8b8b, 0x06, 0x1e01, 0x06, 0xe18b,
	0x06, 0x8c1e, 0x06, 0x01e1, 0x06, 0x8b8d, 0x06, 0x1e01,
	0x06, 0xe18b, 0x06, 0x8e1e, 0x06, 0x01a0, 0x06, 0x00c7,
	0x06, 0xaebb, 0x06, 0xee8a, 0x06, 0xe600, 0x06, 0xee8a,
	0x06, 0xee03, 0x06, 0xee8a, 0x06, 0xefb8, 0x06, 0xee8a,
	0x06, 0xe902, 0x06, 0xee8b, 0x06, 0x8520, 0x06, 0xee8b,
	0x06, 0x8701, 0x06, 0xd481, 0x06, 0x31e4, 0x06, 0x8b94,
	0x06, 0xe58b, 0x06, 0x95bf, 0x06, 0x8b88, 0x06, 0xec00,
	0x06, 0x19a9, 0x06, 0x8b90, 0x06, 0xf9ee, 0x06, 0xfff6,
	0x06, 0x00ee, 0x06, 0xfff7, 0x06, 0xffe0, 0x06, 0xe140,
	0x06, 0xe1e1, 0x06, 0x41f7, 0x06, 0x2ff6, 0x06, 0x28e4,
	0x06, 0xe140, 0x06, 0xe5e1, 0x06, 0x4104, 0x06, 0xf8e0,
	0x06, 0x8b89, 0x06, 0xad20, 0x06, 0x0dee, 0x06, 0x8b89,
	0x06, 0x0002, 0x06, 0x82ed, 0x06, 0x021f, 0x06, 0x4102,
	0x06, 0x2812, 0x06, 0xfc04, 0x06, 0xf8e0, 0x06, 0x8b8d,
	0x06, 0xad20, 0x06, 0x10ee, 0x06, 0x8b8d, 0x06, 0x0002,
	0x06, 0x139d, 0x06, 0x0281, 0x06, 0xcf02, 0x06, 0x1f99,
	0x06, 0x0227, 0x06, 0xeafc, 0x06, 0x04f8, 0x06, 0xe08b,
	0x06, 0x8ead, 0x06, 0x2014, 0x06, 0xf620, 0x06, 0xe48b,
	0x06, 0x8e02, 0x06, 0x8100, 0x06, 0x021b, 0x06, 0xf402,
	0x06, 0x2c9c, 0x06, 0x0281, 0x06, 0x7202, 0x06, 0x843c,
	0x06, 0xad22, 0x06, 0x11f6, 0x06, 0x22e4, 0x06, 0x8b8e,
	0x06, 0x022c, 0x06, 0x4602, 0x06, 0x2ac5, 0x06, 0x0229,
	0x06, 0x2002, 0x06, 0x2b91, 0x06, 0xad25, 0x06, 0x11f6,
	0x06, 0x25e4, 0x06, 0x8b8e, 0x06, 0x0203, 0x06, 0x5a02,
	0x06, 0x043a, 0x06, 0x021a, 0x06, 0x5902, 0x06, 0x2bfc,
	0x06, 0xfc04, 0x06, 0xf8fa, 0x06, 0xef69, 0x06, 0xe0e0,
	0x06, 0x00e1, 0x06, 0xe001, 0x06, 0xad27, 0x06, 0x1fd1,
	0x06, 0x01bf, 0x06, 0x84eb, 0x06, 0x022f, 0x06, 0x50e0,
	0x06, 0xe020, 0x06, 0xe1e0, 0x06, 0x21ad, 0x06, 0x200e,
	0x06, 0xd100, 0x06, 0xbf84, 0x06, 0xeb02, 0x06, 0x2f50,
	0x06, 0xbf3d, 0x06, 0x3902, 0x06, 0x2eb0, 0x06, 0xef96,
	0x06, 0xfefc, 0x06, 0x0402, 0x06, 0x8135, 0x06, 0x05f8,
	0x06, 0xfaef, 0x06, 0x69e0, 0x06, 0xe2fe, 0x06, 0xe1e2,
	0x06, 0xffad, 0x06, 0x2d1a, 0x06, 0xe0e1, 0x06, 0x4ee1,
	0x06, 0xe14f, 0x06, 0xac2d, 0x06, 0x22f6, 0x06, 0x0302,
	0x06, 0x0336, 0x06, 0xf703, 0x06, 0xf706, 0x06, 0xbf84,
	0x06, 0xd502, 0x06, 0x2eb0, 0x06, 0xae11, 0x06, 0xe0e1,
	0x06, 0x4ee1, 0x06, 0xe14f, 0x06, 0xad2d, 0x06, 0x08bf,
	0x06, 0x84e0, 0x06, 0x022e, 0x06, 0xb0f6, 0x06, 0x06ef,
	0x06, 0x96fe, 0x06, 0xfc04, 0x06, 0xf8f9, 0x06, 0xfaef,
	0x06, 0x69e0, 0x06, 0x8b87, 0x06, 0xad20, 0x06, 0x4cd2,
	0x06, 0x00e0, 0x06, 0xe200, 0x06, 0x5801, 0x06, 0x0c02,
	0x06, 0x1e20, 0x06, 0xe0e0, 0x06, 0x0058, 0x06, 0x101e,
	0x06, 0x20e0, 0x06, 0xe036, 0x06, 0x5803, 0x06, 0x1e20,
	0x06, 0xe0e0, 0x06, 0x22e1, 0x06, 0xe023, 0x06, 0x58e0,
	0x06, 0x1e20, 0x06, 0xe08a, 0x06, 0xe61f, 0x06, 0x029e,
	0x06, 0x22e6, 0x06, 0x8ae6, 0x06, 0xad32, 0x06, 0x14ad,
	0x06, 0x3411, 0x06, 0xef02, 0x06, 0x5803, 0x06, 0x9e07,
	0x06, 0xad35, 0x06, 0x085a, 0x06, 0xc09f, 0x06, 0x04d1,
	0x06, 0x01ae, 0x06, 0x02d1, 0x06, 0x00bf, 0x06, 0x84f1,
	0x06, 0x022f, 0x06, 0x50ef, 0x06, 0x96fe, 0x06, 0xfdfc,
	0x06, 0x04f8, 0x06, 0xf9fa, 0x06, 0xe08b, 0x06, 0x81ac,
	0x06, 0x260e, 0x06, 0xe08b, 0x06, 0x81ac, 0x06, 0x2108,
	0x06, 0xe08b, 0x06, 0x87ac, 0x06, 0x2402, 0x06, 0xae6b,
	0x06, 0xeee0, 0x06, 0xea00, 0x06, 0xeee0, 0x06, 0xeb00,
	0x06, 0xe2e0, 0x06, 0x7ce3, 0x06, 0xe07d, 0x06, 0xa511,
	0x06, 0x1115, 0x06, 0xd260, 0x06, 0xd666, 0x06, 0x6602,
	0x06, 0x07f9, 0x06, 0xd2a0, 0x06, 0xd6aa, 0x06, 0xaa02,
	0x06, 0x07f9, 0x06, 0x0282, 0x06, 0x55ae, 0x06, 0x44a5,
	0x06, 0x6666, 0x06, 0x02ae, 0x06, 0x38a5, 0x06, 0xaaaa,
	0x06, 0x02ae, 0x06, 0x32ee, 0x06, 0xe0ea, 0x06, 0x04ee,
	0x06, 0xe0eb, 0x06, 0x06e2, 0x06, 0xe07c, 0x06, 0xe3e0,
	0x06, 0x7de0, 0x06, 0xe038, 0x06, 0xe1e0, 0x06, 0x39ad,
	0x06, 0x2e21, 0x06, 0xad3f, 0x06, 0x13e0, 0x06, 0xe414,
	0x06, 0xe1e4, 0x06, 0x1568, 0x06, 0x80e4, 0x06, 0xe414,
	0x06, 0xe5e4, 0x06, 0x1502, 0x06, 0x8255, 0x06, 0xae0b,
	0x06, 0xac3e, 0x06, 0x02ae, 0x06, 0x0602, 0x06, 0x827f,
	0x06, 0x0282, 0x06, 0xa9fe, 0x06, 0xfdfc, 0x06, 0x04f8,
	0x06, 0xe18b, 0x06, 0x2ee0, 0x06, 0x8b81, 0x06, 0xad26,
	0x06, 0x0502, 0x06, 0x21f3, 0x06, 0xf728, 0x06, 0xe08b,
	0x06, 0x81ad, 0x06, 0x2105, 0x06, 0x0222, 0x06, 0xf8f7,
	0x06, 0x29e0, 0x06, 0x8b87, 0x06, 0xad24, 0x06, 0x0502,
	0x06, 0x82e4, 0x06, 0xf72a, 0x06, 0xe58b, 0x06, 0x2efc,
	0x06, 0x04f8, 0x06, 0xe08b, 0x06, 0x81ad, 0x06, 0x2603,
	0x06, 0x0221, 0x06, 0x34e0, 0x06, 0x8b81, 0x06, 0xad21,
	0x06, 0x09e0, 0x06, 0x8b2e, 0x06, 0xac20, 0x06, 0x0302,
	0x06, 0x834b, 0x06, 0xe08b, 0x06, 0x87ad, 0x06, 0x2409,
	0x06, 0xe08b, 0x06, 0x2eac, 0x06, 0x2103, 0x06, 0x0283,
	0x06, 0x30fc, 0x06, 0x04f8, 0x06, 0xe18b, 0x06, 0x2ee0,
	0x06, 0x8b81, 0x06, 0xad26, 0x06, 0x08e0, 0x06, 0x85d2,
	0x06, 0xad25, 0x06, 0x02f6, 0x06, 0x28e0, 0x06, 0x8b81,
	0x06, 0xad21, 0x06, 0x0ae0, 0x06, 0x860a, 0x06, 0xf627,
	0x06, 0xa005, 0x06, 0x02f6, 0x06, 0x29e0, 0x06, 0x8b87,
	0x06, 0xad24, 0x06, 0x08e0, 0x06, 0x8aed, 0x06, 0xad20,
	0x06, 0x02f6, 0x06, 0x2ae5, 0x06, 0x8b2e, 0x06, 0xa100,
	0x06, 0x0302, 0x06, 0x2111, 0x06, 0xfc04, 0x06, 0xee8a,
	0x06, 0xed00, 0x06, 0xee8a, 0x06, 0xec00, 0x06, 0x04f8,
	0x06, 0xe08b, 0x06, 0x87ad, 0x06, 0x243a, 0x06, 0xe0e0,
	0x06, 0xeae1, 0x06, 0xe0eb, 0x06, 0x58f8, 0x06, 0xd101,
	0x06, 0xe4e0, 0x06, 0xeae5, 0x06, 0xe0eb, 0x06, 0xe0e0,
	0x06, 0x7ce1, 0x06, 0xe07d, 0x06, 0x5c00, 0x06, 0xff3c,
	0x06, 0x001e, 0x06, 0xab1c, 0x06, 0xe0e0, 0x06, 0x4ce1,
	0x06, 0xe04d, 0x06, 0x58c1, 0x06, 0xe4e0, 0x06, 0x4ce5,
	0x06, 0xe04d, 0x06, 0xe0e0, 0x06, 0xeee1, 0x06, 0xe0ef,
	0x06, 0x693c, 0x06, 0xe4e0, 0x06, 0xeee5, 0x06, 0xe0ef,
	0x06, 0xfc04, 0x06, 0xf8e0, 0x06, 0x8b87, 0x06, 0xad24,
	0x06, 0x12e0, 0x06, 0xe0ee, 0x06, 0xe1e0, 0x06, 0xef59,
	0x06, 0xc3e4, 0x06, 0xe0ee, 0x06, 0xe5e0, 0x06, 0xefee,
	0x06, 0x8aed, 0x06, 0x01fc, 0x06, 0x04f8, 0x06, 0xe08b,
	0x06, 0x81ac, 0x06, 0x2505, 0x06, 0x0283, 0x06, 0x5cae,
	0x06, 0x0302, 0x06, 0x2516, 0x06, 0xfc04, 0x06, 0xf8f9,
	0x06, 0xfaef, 0x06, 0x69fa, 0x06, 0xe086, 0x06, 0x0aa0,
	0x06, 0x0019, 0x06, 0xe086, 0x06, 0x0be1, 0x06, 0x8b33,
	0x06, 0x1b10, 0x06, 0x9e04, 0x06, 0xaa02, 0x06, 0xae06,
	0x06, 0xee86, 0x06, 0x0a01, 0x06, 0xaee6, 0x06, 0x0224,
	0x06, 0x1eae, 0x06, 0x14a0, 0x06, 0x0114, 0x06, 0x0224,
	0x06, 0x26bf, 0x06, 0x266d, 0x06, 0x022e, 0x06, 0xb0ee,
	0x06, 0x860b, 0x06, 0x00ee, 0x06, 0x860a, 0x06, 0x02af,
	0x06, 0x8435, 0x06, 0xa002, 0x06, 0x52ee, 0x06, 0x8604,
	0x06, 0x00ee, 0x06, 0x8605, 0x06, 0x00e0, 0x06, 0x860b,
	0x06, 0xe18b, 0x06, 0x321b, 0x06, 0x109e, 0x06, 0x04aa,
	0x06, 0x02ae, 0x06, 0xcbee, 0x06, 0x860b, 0x06, 0x0002,
	0x06, 0x243a, 0x06, 0xe286, 0x06, 0x04e3, 0x06, 0x8605,
	0x06, 0xef65, 0x06, 0xe286, 0x06, 0x06e3, 0x06, 0x8607,
	0x06, 0x1b56, 0x06, 0xaa0e, 0x06, 0xef56, 0x06, 0xe686,
	0x06, 0x06e7, 0x06, 0x8607, 0x06, 0xe286, 0x06, 0x09e6,
	0x06, 0x8608, 0x06, 0xe086, 0x06, 0x09a0, 0x06, 0x0007,
	0x06, 0xee86, 0x06, 0x0a03, 0x06, 0xaf83, 0x06, 0x6202,
	0x06, 0x248e, 0x06, 0x0224, 0x06, 0x26ae, 0x06, 0x48a0,
	0x06, 0x0321, 0x06, 0xe086, 0x06, 0x08e1, 0x06, 0x8609,
	0x06, 0x1b01, 0x06, 0x9e0c, 0x06, 0xaa05, 0x06, 0x0224,
	0x06, 0x9dae, 0x06, 0xe702, 0x06, 0x248e, 0x06, 0xaee2,
	0x06, 0xee86, 0x06, 0x0a04, 0x06, 0xee86, 0x06, 0x0b00,
	0x06, 0xaf83, 0x06, 0x62a0, 0x06, 0x0415, 0x06, 0xe086,
	0x06, 0x0be1, 0x06, 0x8b34, 0x06, 0x1b10, 0x06, 0x9e05,
	0x06, 0xaa03, 0x06, 0xaf83, 0x06, 0x7cee, 0x06, 0x860a,
	0x06, 0x05ae, 0x06, 0x0ca0, 0x06, 0x0502, 0x06, 0xae07,
	0x06, 0x0223, 0x06, 0x09ee, 0x06, 0x860a, 0x06, 0x00fe,
	0x06, 0xef96, 0x06, 0xfefd, 0x06, 0xfc04, 0x06, 0xf8f9,
	0x06, 0xfbe0, 0x06, 0x8b85, 0x06, 0xad25, 0x06, 0x22e0,
	0x06, 0xe022, 0x06, 0xe1e0, 0x06, 0x23e2, 0x06, 0xe036,
	0x06, 0xe3e0, 0x06, 0x375a, 0x06, 0xc40d, 0x06, 0x0158,
	0x06, 0x021e, 0x06, 0x20e3, 0x06, 0x8ae7, 0x06, 0xac31,
	0x06, 0x60ac, 0x06, 0x3a08, 0x06, 0xac3e, 0x06, 0x26ae,
	0x06, 0x67af, 0x06, 0x84d1, 0x06, 0xad37, 0x06, 0x61e0,
	0x06, 0x8ae8, 0x06, 0x10e4, 0x06, 0x8ae8, 0x06, 0xe18a,
	0x06, 0xe91b, 0x06, 0x109e, 0x06, 0x02ae, 0x06, 0x51d1,
	0x06, 0x00bf, 0x06, 0x84ee, 0x06, 0x022f, 0x06, 0x50ee,
	0x06, 0x8ae8, 0x06, 0x00ae, 0x06, 0x43ad, 0x06, 0x3627,
	0x06, 0xe08a, 0x06, 0xeee1, 0x06, 0x8aef, 0x06, 0xef74,
	0x06, 0xe08a, 0x06, 0xeae1, 0x06, 0x8aeb, 0x06, 0x1b74,
	0x06, 0x9e2e, 0x06, 0x14e4, 0x06, 0x8aea, 0x06, 0xe58a,
	0x06, 0xebef, 0x06, 0x74e0, 0x06, 0x8aee, 0x06, 0xe18a,
	0x06, 0xef1b, 0x06, 0x479e, 0x06, 0x0fae, 0x06, 0x19ee,
	0x06, 0x8aea, 0x06, 0x00ee, 0x06, 0x8aeb, 0x06, 0x00ae,
	0x06, 0x0fac, 0x06, 0x390c, 0x06, 0xd101, 0x06, 0xbf84,
	0x06, 0xee02, 0x06, 0x2f50, 0x06, 0xee8a, 0x06, 0xe800,
	0x06, 0xe68a, 0x06, 0xe7ff, 0x06, 0xfdfc, 0x06, 0x04a7,
	0x06, 0x25e5, 0x06, 0x0a1d, 0x06, 0xe50a, 0x06, 0x2ce5,
	0x06, 0x0a6d, 0x06, 0xe50a, 0x06, 0x1de5, 0x06, 0x0a1c,
	0x06, 0xe50a, 0x06, 0x2da7, 0x06, 0x5500, 0x06, 0xe234,
	0x06, 0x88e2, 0x06, 0x00cc, 0x06, 0xe200,
};

static struct gani_patch gani_8168_m16_patch[] = {
	0x1f, 0x0004, 0x1f, 0x0007, 0x1e, 0x0023, 0x16, 0x0306,
	0x16, 0x0307, 0x15, 0x0000, 0x19, 0x407d, 0x15, 0x0001,
	0x19, 0x440f, 0x15, 0x0002, 0x19, 0x7c03, 0x15, 0x0003,
	0x19, 0x6c03, 0x15, 0x0004, 0x19, 0xc4d5, 0x15, 0x0005,
	0x19, 0x00ff, 0x15, 0x0006, 0x19, 0x74f0, 0x15, 0x0007,
	0x19, 0x4880, 0x15, 0x0008, 0x19, 0x4c00, 0x15, 0x0009,
	0x19, 0x4800, 0x15, 0x000a, 0x19, 0x5000, 0x15, 0x000b,
	0x19, 0x4400, 0x15, 0x000c, 0x19, 0x7801, 0x15, 0x000d,
	0x19, 0x4000, 0x15, 0x000e, 0x19, 0x7800, 0x15, 0x000f,
	0x19, 0x7010, 0x15, 0x0010, 0x19, 0x6804, 0x15, 0x0011,
	0x19, 0x64a0, 0x15, 0x0012, 0x19, 0x63da, 0x15, 0x0013,
	0x19, 0x63d8, 0x15, 0x0014, 0x19, 0x6f05, 0x15, 0x0015,
	0x19, 0x5420, 0x15, 0x0016, 0x19, 0x58ce, 0x15, 0x0017,
	0x19, 0x5cf3, 0x15, 0x0018, 0x19, 0xb600, 0x15, 0x0019,
	0x19, 0xc659, 0x15, 0x001a, 0x19, 0x0018, 0x15, 0x001b,
	0x19, 0xc403, 0x15, 0x001c, 0x19, 0x0016, 0x15, 0x001d,
	0x19, 0xaa05, 0x15, 0x001e, 0x19, 0xc503, 0x15, 0x001f,
	0x19, 0x0003, 0x15, 0x0020, 0x19, 0x89f8, 0x15, 0x0021,
	0x19, 0x32ae, 0x15, 0x0022, 0x19, 0x7c03, 0x15, 0x0023,
	0x19, 0x6c03, 0x15, 0x0024, 0x19, 0x7c03, 0x15, 0x0025,
	0x19, 0x6801, 0x15, 0x0026, 0x19, 0x66a0, 0x15, 0x0027,
	0x19, 0xa300, 0x15, 0x0028, 0x19, 0x64a0, 0x15, 0x0029,
	0x19, 0x76f0, 0x15, 0x002a, 0x19, 0x7670, 0x15, 0x002b,
	0x19, 0x7630, 0x15, 0x002c, 0x19, 0x31a6, 0x15, 0x002d,
	0x19, 0x0000, 0x15, 0x002e, 0x19, 0x0000, 0x15, 0x002f,
	0x19, 0x0000, 0x15, 0x0030, 0x19, 0x0000, 0x15, 0x0031,
	0x19, 0x0000, 0x15, 0x0032, 0x19, 0x4801, 0x15, 0x0033,
	0x19, 0x6803, 0x15, 0x0034, 0x19, 0x66a1, 0x15, 0x0035,
	0x19, 0x7c03, 0x15, 0x0036, 0x19, 0x6c03, 0x15, 0x0037,
	0x19, 0xa300, 0x15, 0x0038, 0x19, 0x64a1, 0x15, 0x0039,
	0x19, 0x7c08, 0x15, 0x003a, 0x19, 0x74f8, 0x15, 0x003b,
	0x19, 0x63d0, 0x15, 0x003c, 0x19, 0x7ff0, 0x15, 0x003d,
	0x19, 0x77f0, 0x15, 0x003e, 0x19, 0x7ff0, 0x15, 0x003f,
	0x19, 0x7750, 0x15, 0x0040, 0x19, 0x63d8, 0x15, 0x0041,
	0x19, 0x7cf0, 0x15, 0x0042, 0x19, 0x7708, 0x15, 0x0043,
	0x19, 0xa654, 0x15, 0x0044, 0x19, 0x304a, 0x15, 0x0045,
	0x19, 0x0000, 0x15, 0x0046, 0x19, 0x0000, 0x15, 0x0047,
	0x19, 0x0000, 0x15, 0x0048, 0x19, 0x0000, 0x15, 0x0049,
	0x19, 0x0000, 0x15, 0x004a, 0x19, 0x4802, 0x15, 0x004b,
	0x19, 0x4003, 0x15, 0x004c, 0x19, 0x4440, 0x15, 0x004d,
	0x19, 0x63c8, 0x15, 0x004e, 0x19, 0x6481, 0x15, 0x004f,
	0x19, 0x9d00, 0x15, 0x0050, 0x19, 0x63e8, 0x15, 0x0051,
	0x19, 0x7d00, 0x15, 0x0052, 0x19, 0x5900, 0x15, 0x0053,
	0x19, 0x63f8, 0x15, 0x0054, 0x19, 0x64a1, 0x15, 0x0055,
	0x19, 0x3116, 0x15, 0x0056, 0x19, 0x0000, 0x15, 0x0057,
	0x19, 0x0000, 0x15, 0x0058, 0x19, 0x0000, 0x15, 0x0059,
	0x19, 0x0000, 0x15, 0x005a, 0x19, 0x7c03, 0x15, 0x005b,
	0x19, 0x6c03, 0x15, 0x005c, 0x19, 0x7c08, 0x15, 0x005d,
	0x19, 0x6000, 0x15, 0x005e, 0x19, 0x59ce, 0x15, 0x005f,
	0x19, 0x4400, 0x15, 0x0060, 0x19, 0x7d00, 0x15, 0x0061,
	0x19, 0x72b0, 0x15, 0x0062, 0x19, 0x400e, 0x15, 0x0063,
	0x19, 0x4440, 0x15, 0x0064, 0x19, 0x9d00, 0x15, 0x0065,
	0x19, 0x7f00, 0x15, 0x0066, 0x19, 0x70b0, 0x15, 0x0067,
	0x19, 0x7c08, 0x15, 0x0068, 0x19, 0x6008, 0x15, 0x0069,
	0x19, 0x7cf0, 0x15, 0x006a, 0x19, 0x7750, 0x15, 0x006b,
	0x19, 0x4007, 0x15, 0x006c, 0x19, 0x4500, 0x15, 0x006d,
	0x19, 0x4023, 0x15, 0x006e, 0x19, 0x4580, 0x15, 0x006f,
	0x19, 0x9f00, 0x15, 0x0070, 0x19, 0xcd78, 0x15, 0x0071,
	0x19, 0x0003, 0x15, 0x0072, 0x19, 0xbe02, 0x15, 0x0073,
	0x19, 0x3070, 0x15, 0x0074, 0x19, 0x7cf0, 0x15, 0x0075,
	0x19, 0x77f0, 0x15, 0x0076, 0x19, 0x4400, 0x15, 0x0077,
	0x19, 0x4007, 0x15, 0x0078, 0x19, 0x4500, 0x15, 0x0079,
	0x19, 0x4023, 0x15, 0x007a, 0x19, 0x4580, 0x15, 0x007b,
	0x19, 0x9f00, 0x15, 0x007c, 0x19, 0xce80, 0x15, 0x007d,
	0x19, 0x0004, 0x15, 0x007e, 0x19, 0xce80, 0x15, 0x007f,
	0x19, 0x0002, 0x15, 0x0080, 0x19, 0x307c, 0x15, 0x0081,
	0x19, 0x4400, 0x15, 0x0082, 0x19, 0x480f, 0x15, 0x0083,
	0x19, 0x6802, 0x15, 0x0084, 0x19, 0x6680, 0x15, 0x0085,
	0x19, 0x7c10, 0x15, 0x0086, 0x19, 0x6010, 0x15, 0x0087,
	0x19, 0x400a, 0x15, 0x0088, 0x19, 0x4580, 0x15, 0x0089,
	0x19, 0x9e00, 0x15, 0x008a, 0x19, 0x7d00, 0x15, 0x008b,
	0x19, 0x5800, 0x15, 0x008c, 0x19, 0x63c8, 0x15, 0x008d,
	0x19, 0x63d8, 0x15, 0x008e, 0x19, 0x66a0, 0x15, 0x008f,
	0x19, 0x8300, 0x15, 0x0090, 0x19, 0x7ff0, 0x15, 0x0091,
	0x19, 0x74f0, 0x15, 0x0092, 0x19, 0x3006, 0x15, 0x0093,
	0x19, 0x0000, 0x15, 0x0094, 0x19, 0x0000, 0x15, 0x0095,
	0x19, 0x0000, 0x15, 0x0096, 0x19, 0x0000, 0x15, 0x0097,
	0x19, 0x4803, 0x15, 0x0098, 0x19, 0x7c03, 0x15, 0x0099,
	0x19, 0x6c03, 0x15, 0x009a, 0x19, 0xa203, 0x15, 0x009b,
	0x19, 0x64b1, 0x15, 0x009c, 0x19, 0x309e, 0x15, 0x009d,
	0x19, 0x64b3, 0x15, 0x009e, 0x19, 0x4030, 0x15, 0x009f,
	0x19, 0x440e, 0x15, 0x00a0, 0x19, 0x4020, 0x15, 0x00a1,
	0x19, 0x4419, 0x15, 0x00a2, 0x19, 0x7801, 0x15, 0x00a3,
	0x19, 0xc520, 0x15, 0x00a4, 0x19, 0x000b, 0x15, 0x00a5,
	0x19, 0x4020, 0x15, 0x00a6, 0x19, 0x7800, 0x15, 0x00a7,
	0x19, 0x58a4, 0x15, 0x00a8, 0x19, 0x63da, 0x15, 0x00a9,
	0x19, 0x5cb0, 0x15, 0x00aa, 0x19, 0x7d00, 0x15, 0x00ab,
	0x19, 0x72b0, 0x15, 0x00ac, 0x19, 0x7f00, 0x15, 0x00ad,
	0x19, 0x70b0, 0x15, 0x00ae, 0x19, 0x30b8, 0x15, 0x00AF,
	0x19, 0x4060, 0x15, 0x00B0, 0x19, 0x7800, 0x15, 0x00B1,
	0x19, 0x7e00, 0x15, 0x00B2, 0x19, 0x72B0, 0x15, 0x00B3,
	0x19, 0x7F00, 0x15, 0x00B4, 0x19, 0x73B0, 0x15, 0x00b5,
	0x19, 0x58a0, 0x15, 0x00b6, 0x19, 0x63d2, 0x15, 0x00b7,
	0x19, 0x5c00, 0x15, 0x00b8, 0x19, 0x5780, 0x15, 0x00b9,
	0x19, 0xb60d, 0x15, 0x00ba, 0x19, 0x9bff, 0x15, 0x00bb,
	0x19, 0x7c03, 0x15, 0x00bc, 0x19, 0x6001, 0x15, 0x00bd,
	0x19, 0xc020, 0x15, 0x00be, 0x19, 0x002b, 0x15, 0x00bf,
	0x19, 0xc137, 0x15, 0x00c0, 0x19, 0x0006, 0x15, 0x00c1,
	0x19, 0x9af8, 0x15, 0x00c2, 0x19, 0x30c6, 0x15, 0x00c3,
	0x19, 0x0000, 0x15, 0x00c4, 0x19, 0x0000, 0x15, 0x00c5,
	0x19, 0x0000, 0x15, 0x00c6, 0x19, 0x7d00, 0x15, 0x00c7,
	0x19, 0x70b0, 0x15, 0x00c8, 0x19, 0x4400, 0x15, 0x00c9,
	0x19, 0x4804, 0x15, 0x00ca, 0x19, 0x7c80, 0x15, 0x00cb,
	0x19, 0x5c80, 0x15, 0x00cc, 0x19, 0x4010, 0x15, 0x00cd,
	0x19, 0x4415, 0x15, 0x00ce, 0x19, 0x9b00, 0x15, 0x00cf,
	0x19, 0x7f00, 0x15, 0x00d0, 0x19, 0x70b0, 0x15, 0x00d1,
	0x19, 0x3177, 0x15, 0x00d2, 0x19, 0x0000, 0x15, 0x00d3,
	0x19, 0x0000, 0x15, 0x00d4, 0x19, 0x0000, 0x15, 0x00d5,
	0x19, 0x4808, 0x15, 0x00d6, 0x19, 0x4007, 0x15, 0x00d7,
	0x19, 0x4420, 0x15, 0x00d8, 0x19, 0x63d8, 0x15, 0x00d9,
	0x19, 0xb608, 0x15, 0x00da, 0x19, 0xbcbd, 0x15, 0x00db,
	0x19, 0xc60b, 0x15, 0x00dc, 0x19, 0x00fd, 0x15, 0x00dd,
	0x19, 0x30e1, 0x15, 0x00de, 0x19, 0x0000, 0x15, 0x00df,
	0x19, 0x0000, 0x15, 0x00e0, 0x19, 0x0000, 0x15, 0x00e1,
	0x19, 0x4809, 0x15, 0x00e2, 0x19, 0x7e40, 0x15, 0x00e3,
	0x19, 0x5a40, 0x15, 0x00e4, 0x19, 0x305a, 0x15, 0x00e5,
	0x19, 0x0000, 0x15, 0x00e6, 0x19, 0x0000, 0x15, 0x00e7,
	0x19, 0x0000, 0x15, 0x00e8, 0x19, 0x0000, 0x15, 0x00e9,
	0x19, 0x480a, 0x15, 0x00ea, 0x19, 0x5820, 0x15, 0x00eb,
	0x19, 0x6c03, 0x15, 0x00ec, 0x19, 0xb60a, 0x15, 0x00ed,
	0x19, 0xda07, 0x15, 0x00ee, 0x19, 0x0008, 0x15, 0x00ef,
	0x19, 0xc60b, 0x15, 0x00f0, 0x19, 0x00fc, 0x15, 0x00f1,
	0x19, 0x30f6, 0x15, 0x00f2, 0x19, 0x0000, 0x15, 0x00f3,
	0x19, 0x0000, 0x15, 0x00f4, 0x19, 0x0000, 0x15, 0x00f5,
	0x19, 0x0000, 0x15, 0x00f6, 0x19, 0x4408, 0x15, 0x00f7,
	0x19, 0x480b, 0x15, 0x00f8, 0x19, 0x6f03, 0x15, 0x00f9,
	0x19, 0x405f, 0x15, 0x00fa, 0x19, 0x4448, 0x15, 0x00fb,
	0x19, 0x4020, 0x15, 0x00fc, 0x19, 0x4468, 0x15, 0x00fd,
	0x19, 0x9c03, 0x15, 0x00fe, 0x19, 0x6f07, 0x15, 0x00ff,
	0x19, 0x58a0, 0x15, 0x0100, 0x19, 0xd6d1, 0x15, 0x0101,
	0x19, 0x0004, 0x15, 0x0102, 0x19, 0xc137, 0x15, 0x0103,
	0x19, 0x0002, 0x15, 0x0104, 0x19, 0xa0e5, 0x15, 0x0105,
	0x19, 0x9df8, 0x15, 0x0106, 0x19, 0x30c6, 0x15, 0x0107,
	0x19, 0x0000, 0x15, 0x0108, 0x19, 0x0000, 0x15, 0x0109,
	0x19, 0x0000, 0x15, 0x010a, 0x19, 0x0000, 0x15, 0x010b,
	0x19, 0x4808, 0x15, 0x010c, 0x19, 0xc32d, 0x15, 0x010d,
	0x19, 0x0003, 0x15, 0x010e, 0x19, 0xc8b3, 0x15, 0x010f,
	0x19, 0x00fc, 0x15, 0x0110, 0x19, 0x4400, 0x15, 0x0111,
	0x19, 0x3116, 0x15, 0x0112, 0x19, 0x0000, 0x15, 0x0113,
	0x19, 0x0000, 0x15, 0x0114, 0x19, 0x0000, 0x15, 0x0115,
	0x19, 0x0000, 0x15, 0x0116, 0x19, 0x4803, 0x15, 0x0117,
	0x19, 0x7c03, 0x15, 0x0118, 0x19, 0x6c02, 0x15, 0x0119,
	0x19, 0x7c04, 0x15, 0x011a, 0x19, 0x6000, 0x15, 0x011b,
	0x19, 0x5cf7, 0x15, 0x011c, 0x19, 0x7c2a, 0x15, 0x011d,
	0x19, 0x5800, 0x15, 0x011e, 0x19, 0x5400, 0x15, 0x011f,
	0x19, 0x7c08, 0x15, 0x0120, 0x19, 0x74f0, 0x15, 0x0121,
	0x19, 0x4019, 0x15, 0x0122, 0x19, 0x440d, 0x15, 0x0123,
	0x19, 0xb6c1, 0x15, 0x0124, 0x19, 0xc05b, 0x15, 0x0125,
	0x19, 0x00bf, 0x15, 0x0126, 0x19, 0xc025, 0x15, 0x0127,
	0x19, 0x00bd, 0x15, 0x0128, 0x19, 0xc603, 0x15, 0x0129,
	0x19, 0x00bb, 0x15, 0x012a, 0x19, 0x8805, 0x15, 0x012b,
	0x19, 0x7801, 0x15, 0x012c, 0x19, 0x4001, 0x15, 0x012d,
	0x19, 0x7800, 0x15, 0x012e, 0x19, 0xa3dd, 0x15, 0x012f,
	0x19, 0x7c03, 0x15, 0x0130, 0x19, 0x6c03, 0x15, 0x0131,
	0x19, 0x8407, 0x15, 0x0132, 0x19, 0x7c03, 0x15, 0x0133,
	0x19, 0x6c02, 0x15, 0x0134, 0x19, 0xd9b8, 0x15, 0x0135,
	0x19, 0x0003, 0x15, 0x0136, 0x19, 0xc240, 0x15, 0x0137,
	0x19, 0x0015, 0x15, 0x0138, 0x19, 0x7c03, 0x15, 0x0139,
	0x19, 0x6c02, 0x15, 0x013a, 0x19, 0x9ae9, 0x15, 0x013b,
	0x19, 0x3140, 0x15, 0x013c, 0x19, 0x0000, 0x15, 0x013d,
	0x19, 0x0000, 0x15, 0x013e, 0x19, 0x0000, 0x15, 0x013f,
	0x19, 0x0000, 0x15, 0x0140, 0x19, 0x4807, 0x15, 0x0141,
	0x19, 0x4004, 0x15, 0x0142, 0x19, 0x4410, 0x15, 0x0143,
	0x19, 0x7c0c, 0x15, 0x0144, 0x19, 0x600c, 0x15, 0x0145,
	0x19, 0x9b00, 0x15, 0x0146, 0x19, 0xa68f, 0x15, 0x0147,
	0x19, 0x3116, 0x15, 0x0148, 0x19, 0x0000, 0x15, 0x0149,
	0x19, 0x0000, 0x15, 0x014a, 0x19, 0x0000, 0x15, 0x014b,
	0x19, 0x0000, 0x15, 0x014c, 0x19, 0x4804, 0x15, 0x014d,
	0x19, 0x54c0, 0x15, 0x014e, 0x19, 0xb703, 0x15, 0x014f,
	0x19, 0x5cff, 0x15, 0x0150, 0x19, 0x315f, 0x15, 0x0151,
	0x19, 0x7c08, 0x15, 0x0152, 0x19, 0x74f8, 0x15, 0x0153,
	0x19, 0x6421, 0x15, 0x0154, 0x19, 0x7c08, 0x15, 0x0155,
	0x19, 0x6000, 0x15, 0x0156, 0x19, 0x4003, 0x15, 0x0157,
	0x19, 0x4418, 0x15, 0x0158, 0x19, 0x9b00, 0x15, 0x0159,
	0x19, 0x6461, 0x15, 0x015a, 0x19, 0x64e1, 0x15, 0x015b,
	0x19, 0x7c20, 0x15, 0x015c, 0x19, 0x5820, 0x15, 0x015d,
	0x19, 0x5ccf, 0x15, 0x015e, 0x19, 0x7050, 0x15, 0x015f,
	0x19, 0xd9b8, 0x15, 0x0160, 0x19, 0x0008, 0x15, 0x0161,
	0x19, 0xdab1, 0x15, 0x0162, 0x19, 0x0015, 0x15, 0x0163,
	0x19, 0xc244, 0x15, 0x0164, 0x19, 0x0013, 0x15, 0x0165,
	0x19, 0xc021, 0x15, 0x0166, 0x19, 0x00f9, 0x15, 0x0167,
	0x19, 0x3177, 0x15, 0x0168, 0x19, 0x5cf7, 0x15, 0x0169,
	0x19, 0x4010, 0x15, 0x016a, 0x19, 0x4428, 0x15, 0x016b,
	0x19, 0x9c00, 0x15, 0x016c, 0x19, 0x7c08, 0x15, 0x016d,
	0x19, 0x6008, 0x15, 0x016e, 0x19, 0x7c08, 0x15, 0x016f,
	0x19, 0x74f0, 0x15, 0x0170, 0x19, 0x6461, 0x15, 0x0171,
	0x19, 0x6421, 0x15, 0x0172, 0x19, 0x64a1, 0x15, 0x0173,
	0x19, 0x3116, 0x15, 0x0174, 0x19, 0x0000, 0x15, 0x0175,
	0x19, 0x0000, 0x15, 0x0176, 0x19, 0x0000, 0x15, 0x0177,
	0x19, 0x4805, 0x15, 0x0178, 0x19, 0xa103, 0x15, 0x0179,
	0x19, 0x7c02, 0x15, 0x017a, 0x19, 0x6002, 0x15, 0x017b,
	0x19, 0x7e00, 0x15, 0x017c, 0x19, 0x5400, 0x15, 0x017d,
	0x19, 0x7c6b, 0x15, 0x017e, 0x19, 0x5c63, 0x15, 0x017f,
	0x19, 0x407d, 0x15, 0x0180, 0x19, 0xa602, 0x15, 0x0181,
	0x19, 0x4001, 0x15, 0x0182, 0x19, 0x4420, 0x15, 0x0183,
	0x19, 0x4020, 0x15, 0x0184, 0x19, 0x44a1, 0x15, 0x0185,
	0x19, 0xd6e0, 0x15, 0x0186, 0x19, 0x0009, 0x15, 0x0187,
	0x19, 0x9efe, 0x15, 0x0188, 0x19, 0x7c02, 0x15, 0x0189,
	0x19, 0x6000, 0x15, 0x018a, 0x19, 0x9c00, 0x15, 0x018b,
	0x19, 0x318f, 0x15, 0x018c, 0x19, 0x0000, 0x15, 0x018d,
	0x19, 0x0000, 0x15, 0x018e, 0x19, 0x0000, 0x15, 0x018f,
	0x19, 0x4806, 0x15, 0x0190, 0x19, 0x7c10, 0x15, 0x0191,
	0x19, 0x5c10, 0x15, 0x0192, 0x19, 0x40fa, 0x15, 0x0193,
	0x19, 0xa602, 0x15, 0x0194, 0x19, 0x4010, 0x15, 0x0195,
	0x19, 0x4440, 0x15, 0x0196, 0x19, 0x9d00, 0x15, 0x0197,
	0x19, 0x7c80, 0x15, 0x0198, 0x19, 0x6400, 0x15, 0x0199,
	0x19, 0x4003, 0x15, 0x019a, 0x19, 0x4540, 0x15, 0x019b,
	0x19, 0x7c08, 0x15, 0x019c, 0x19, 0x6008, 0x15, 0x019d,
	0x19, 0x9f00, 0x15, 0x019e, 0x19, 0x7c40, 0x15, 0x019f,
	0x19, 0x6400, 0x15, 0x01a0, 0x19, 0x7c80, 0x15, 0x01a1,
	0x19, 0x6480, 0x15, 0x01a2, 0x19, 0x3140, 0x15, 0x01a3,
	0x19, 0x0000, 0x15, 0x01a4, 0x19, 0x0000, 0x15, 0x01a5,
	0x19, 0x0000, 0x15, 0x01a6, 0x19, 0x4400, 0x15, 0x01a7,
	0x19, 0x7c0b, 0x15, 0x01a8, 0x19, 0x6c01, 0x15, 0x01a9,
	0x19, 0x64a8, 0x15, 0x01aa, 0x19, 0x6800, 0x15, 0x01ab,
	0x19, 0x5cf0, 0x15, 0x01ac, 0x19, 0x588f, 0x15, 0x01ad,
	0x19, 0xb628, 0x15, 0x01ae, 0x19, 0xc053, 0x15, 0x01af,
	0x19, 0x0026, 0x15, 0x01b0, 0x19, 0xc02d, 0x15, 0x01b1,
	0x19, 0x0024, 0x15, 0x01b2, 0x19, 0xc603, 0x15, 0x01b3,
	0x19, 0x0022, 0x15, 0x01b4, 0x19, 0x8cf9, 0x15, 0x01b5,
	0x19, 0x31ba, 0x15, 0x01b6, 0x19, 0x0000, 0x15, 0x01b7,
	0x19, 0x0000, 0x15, 0x01b8, 0x19, 0x0000, 0x15, 0x01b9,
	0x19, 0x0000, 0x15, 0x01ba, 0x19, 0x4400, 0x15, 0x01bb,
	0x19, 0x5420, 0x15, 0x01bc, 0x19, 0x4811, 0x15, 0x01bd,
	0x19, 0x5000, 0x15, 0x01be, 0x19, 0x4801, 0x15, 0x01bf,
	0x19, 0x6800, 0x15, 0x01c0, 0x19, 0x31f5, 0x15, 0x01c1,
	0x19, 0xb614, 0x15, 0x01c2, 0x19, 0x8ce4, 0x15, 0x01c3,
	0x19, 0xb30c, 0x15, 0x01c4, 0x19, 0x7c03, 0x15, 0x01c5,
	0x19, 0x6c02, 0x15, 0x01c6, 0x19, 0x8206, 0x15, 0x01c7,
	0x19, 0x7c03, 0x15, 0x01c8, 0x19, 0x6c00, 0x15, 0x01c9,
	0x19, 0x7c04, 0x15, 0x01ca, 0x19, 0x7404, 0x15, 0x01cb,
	0x19, 0x31c0, 0x15, 0x01cc, 0x19, 0x7c04, 0x15, 0x01cd,
	0x19, 0x7400, 0x15, 0x01ce, 0x19, 0x31c0, 0x15, 0x01cf,
	0x19, 0x8df1, 0x15, 0x01d0, 0x19, 0x3248, 0x15, 0x01d1,
	0x19, 0x0000, 0x15, 0x01d2, 0x19, 0x0000, 0x15, 0x01d3,
	0x19, 0x0000, 0x15, 0x01d4, 0x19, 0x0000, 0x15, 0x01d5,
	0x19, 0x4400, 0x15, 0x01d6, 0x19, 0x7c03, 0x15, 0x01d7,
	0x19, 0x6c03, 0x15, 0x01d8, 0x19, 0x7670, 0x15, 0x01d9,
	0x19, 0x4023, 0x15, 0x01da, 0x19, 0x4500, 0x15, 0x01db,
	0x19, 0x4069, 0x15, 0x01dc, 0x19, 0x4580, 0x15, 0x01dd,
	0x19, 0x9f00, 0x15, 0x01de, 0x19, 0xcff5, 0x15, 0x01df,
	0x19, 0x00ff, 0x15, 0x01e0, 0x19, 0x76f0, 0x15, 0x01e1,
	0x19, 0x4400, 0x15, 0x01e2, 0x19, 0x4023, 0x15, 0x01e3,
	0x19, 0x4500, 0x15, 0x01e4, 0x19, 0x4069, 0x15, 0x01e5,
	0x19, 0x4580, 0x15, 0x01e6, 0x19, 0x9f00, 0x15, 0x01e7,
	0x19, 0xd0f5, 0x15, 0x01e8, 0x19, 0x00ff, 0x15, 0x01e9,
	0x19, 0x4400, 0x15, 0x01ea, 0x19, 0x7c03, 0x15, 0x01eb,
	0x19, 0x6800, 0x15, 0x01ec, 0x19, 0x66a0, 0x15, 0x01ed,
	0x19, 0x8300, 0x15, 0x01ee, 0x19, 0x74f0, 0x15, 0x01ef,
	0x19, 0x3006, 0x15, 0x01f0, 0x19, 0x0000, 0x15, 0x01f1,
	0x19, 0x0000, 0x15, 0x01f2, 0x19, 0x0000, 0x15, 0x01f3,
	0x19, 0x0000, 0x15, 0x01f4, 0x19, 0x0000, 0x15, 0x01f5,
	0x19, 0x7c03, 0x15, 0x01f6, 0x19, 0x6c02, 0x15, 0x01f7,
	0x19, 0x409d, 0x15, 0x01f8, 0x19, 0x7c87, 0x15, 0x01f9,
	0x19, 0xae14, 0x15, 0x01fa, 0x19, 0x4400, 0x15, 0x01fb,
	0x19, 0x7c40, 0x15, 0x01fc, 0x19, 0x6800, 0x15, 0x01fd,
	0x19, 0x7801, 0x15, 0x01fe, 0x19, 0x980e, 0x15, 0x01ff,
	0x19, 0x930c, 0x15, 0x0200, 0x19, 0x9206, 0x15, 0x0201,
	0x19, 0x4002, 0x15, 0x0202, 0x19, 0x7800, 0x15, 0x0203,
	0x19, 0x588f, 0x15, 0x0204, 0x19, 0x5520, 0x15, 0x0205,
	0x19, 0x320c, 0x15, 0x0206, 0x19, 0x4000, 0x15, 0x0207,
	0x19, 0x7800, 0x15, 0x0208, 0x19, 0x588d, 0x15, 0x0209,
	0x19, 0x5500, 0x15, 0x020a, 0x19, 0x320c, 0x15, 0x020b,
	0x19, 0x4002, 0x15, 0x020c, 0x19, 0x3220, 0x15, 0x020d,
	0x19, 0x4480, 0x15, 0x020e, 0x19, 0x9e03, 0x15, 0x020f,
	0x19, 0x7c40, 0x15, 0x0210, 0x19, 0x6840, 0x15, 0x0211,
	0x19, 0x7801, 0x15, 0x0212, 0x19, 0x980e, 0x15, 0x0213,
	0x19, 0x930c, 0x15, 0x0214, 0x19, 0x9206, 0x15, 0x0215,
	0x19, 0x4000, 0x15, 0x0216, 0x19, 0x7800, 0x15, 0x0217,
	0x19, 0x588f, 0x15, 0x0218, 0x19, 0x5520, 0x15, 0x0219,
	0x19, 0x3220, 0x15, 0x021a, 0x19, 0x4002, 0x15, 0x021b,
	0x19, 0x7800, 0x15, 0x021c, 0x19, 0x588d, 0x15, 0x021d,
	0x19, 0x5540, 0x15, 0x021e, 0x19, 0x3220, 0x15, 0x021f,
	0x19, 0x4000, 0x15, 0x0220, 0x19, 0x7800, 0x15, 0x0221,
	0x19, 0x7c03, 0x15, 0x0222, 0x19, 0x6c00, 0x15, 0x0223,
	0x19, 0x3231, 0x15, 0x0224, 0x19, 0xab06, 0x15, 0x0225,
	0x19, 0xbf08, 0x15, 0x0226, 0x19, 0x4076, 0x15, 0x0227,
	0x19, 0x7d07, 0x15, 0x0228, 0x19, 0x4502, 0x15, 0x0229,
	0x19, 0x3231, 0x15, 0x022a, 0x19, 0x7d80, 0x15, 0x022b,
	0x19, 0x5180, 0x15, 0x022c, 0x19, 0x322f, 0x15, 0x022d,
	0x19, 0x7d80, 0x15, 0x022e, 0x19, 0x5000, 0x15, 0x022f,
	0x19, 0x7d07, 0x15, 0x0230, 0x19, 0x4402, 0x15, 0x0231,
	0x19, 0x7c03, 0x15, 0x0232, 0x19, 0x6c02, 0x15, 0x0233,
	0x19, 0x7c03, 0x15, 0x0234, 0x19, 0xb309, 0x15, 0x0235,
	0x19, 0xb204, 0x15, 0x0236, 0x19, 0xb105, 0x15, 0x0237,
	0x19, 0x6c00, 0x15, 0x0238, 0x19, 0x31c1, 0x15, 0x0239,
	0x19, 0x6c00, 0x15, 0x023a, 0x19, 0x3261, 0x15, 0x023b,
	0x19, 0x6c00, 0x15, 0x023c, 0x19, 0x3250, 0x15, 0x023d,
	0x19, 0xb203, 0x15, 0x023e, 0x19, 0x6c00, 0x15, 0x023f,
	0x19, 0x327a, 0x15, 0x0240, 0x19, 0x6c00, 0x15, 0x0241,
	0x19, 0x3293, 0x15, 0x0242, 0x19, 0x0000, 0x15, 0x0243,
	0x19, 0x0000, 0x15, 0x0244, 0x19, 0x0000, 0x15, 0x0245,
	0x19, 0x0000, 0x15, 0x0246, 0x19, 0x0000, 0x15, 0x0247,
	0x19, 0x32a3, 0x15, 0x0248, 0x19, 0x5520, 0x15, 0x0249,
	0x19, 0x403d, 0x15, 0x024a, 0x19, 0x440c, 0x15, 0x024b,
	0x19, 0x4812, 0x15, 0x024c, 0x19, 0x5001, 0x15, 0x024d,
	0x19, 0x4802, 0x15, 0x024e, 0x19, 0x6880, 0x15, 0x024f,
	0x19, 0x31f5, 0x15, 0x0250, 0x19, 0xb685, 0x15, 0x0251,
	0x19, 0x801c, 0x15, 0x0252, 0x19, 0xbaf5, 0x15, 0x0253,
	0x19, 0xc07c, 0x15, 0x0254, 0x19, 0x00fb, 0x15, 0x0255,
	0x19, 0x325a, 0x15, 0x0256, 0x19, 0x0000, 0x15, 0x0257,
	0x19, 0x0000, 0x15, 0x0258, 0x19, 0x0000, 0x15, 0x0259,
	0x19, 0x0000, 0x15, 0x025a, 0x19, 0x481a, 0x15, 0x025b,
	0x19, 0x5001, 0x15, 0x025c, 0x19, 0x401b, 0x15, 0x025d,
	0x19, 0x480a, 0x15, 0x025e, 0x19, 0x4418, 0x15, 0x025f,
	0x19, 0x6900, 0x15, 0x0260, 0x19, 0x31f5, 0x15, 0x0261,
	0x19, 0xb64b, 0x15, 0x0262, 0x19, 0xdb00, 0x15, 0x0263,
	0x19, 0x0048, 0x15, 0x0264, 0x19, 0xdb7d, 0x15, 0x0265,
	0x19, 0x0002, 0x15, 0x0266, 0x19, 0xa0fa, 0x15, 0x0267,
	0x19, 0x4408, 0x15, 0x0268, 0x19, 0x3248, 0x15, 0x0269,
	0x19, 0x0000, 0x15, 0x026a, 0x19, 0x0000, 0x15, 0x026b,
	0x19, 0x0000, 0x15, 0x026c, 0x19, 0x0000, 0x15, 0x026d,
	0x19, 0xb806, 0x15, 0x026e, 0x19, 0x588d, 0x15, 0x026f,
	0x19, 0x5500, 0x15, 0x0270, 0x19, 0x7801, 0x15, 0x0271,
	0x19, 0x4002, 0x15, 0x0272, 0x19, 0x7800, 0x15, 0x0273,
	0x19, 0x4814, 0x15, 0x0274, 0x19, 0x500b, 0x15, 0x0275,
	0x19, 0x4804, 0x15, 0x0276, 0x19, 0x40c4, 0x15, 0x0277,
	0x19, 0x4425, 0x15, 0x0278, 0x19, 0x6a00, 0x15, 0x0279,
	0x19, 0x31f5, 0x15, 0x027a, 0x19, 0xb632, 0x15, 0x027b,
	0x19, 0xdc03, 0x15, 0x027c, 0x19, 0x0027, 0x15, 0x027d,
	0x19, 0x80fc, 0x15, 0x027e, 0x19, 0x3283, 0x15, 0x027f,
	0x19, 0x0000, 0x15, 0x0280, 0x19, 0x0000, 0x15, 0x0281,
	0x19, 0x0000, 0x15, 0x0282, 0x19, 0x0000, 0x15, 0x0283,
	0x19, 0xb806, 0x15, 0x0284, 0x19, 0x588f, 0x15, 0x0285,
	0x19, 0x5520, 0x15, 0x0286, 0x19, 0x7801, 0x15, 0x0287,
	0x19, 0x4000, 0x15, 0x0288, 0x19, 0x7800, 0x15, 0x0289,
	0x19, 0x4818, 0x15, 0x028a, 0x19, 0x5051, 0x15, 0x028b,
	0x19, 0x4808, 0x15, 0x028c, 0x19, 0x4050, 0x15, 0x028d,
	0x19, 0x4462, 0x15, 0x028e, 0x19, 0x40c4, 0x15, 0x028f,
	0x19, 0x4473, 0x15, 0x0290, 0x19, 0x5041, 0x15, 0x0291,
	0x19, 0x6b00, 0x15, 0x0292, 0x19, 0x31f5, 0x15, 0x0293,
	0x19, 0xb619, 0x15, 0x0294, 0x19, 0x80d9, 0x15, 0x0295,
	0x19, 0xbd06, 0x15, 0x0296, 0x19, 0xbb0d, 0x15, 0x0297,
	0x19, 0xaf14, 0x15, 0x0298, 0x19, 0x8efa, 0x15, 0x0299,
	0x19, 0x5049, 0x15, 0x029a, 0x19, 0x3248, 0x15, 0x029b,
	0x19, 0x4c10, 0x15, 0x029c, 0x19, 0x44b0, 0x15, 0x029d,
	0x19, 0x4c00, 0x15, 0x029e, 0x19, 0x3292, 0x15, 0x029f,
	0x19, 0x0000, 0x15, 0x02a0, 0x19, 0x0000, 0x15, 0x02a1,
	0x19, 0x0000, 0x15, 0x02a2, 0x19, 0x0000, 0x15, 0x02a3,
	0x19, 0x481f, 0x15, 0x02a4, 0x19, 0x5005, 0x15, 0x02a5,
	0x19, 0x480f, 0x15, 0x02a6, 0x19, 0xac00, 0x15, 0x02a7,
	0x19, 0x31a6, 0x15, 0x02a8, 0x19, 0x0000, 0x15, 0x02a9,
	0x19, 0x0000, 0x15, 0x02aa, 0x19, 0x0000, 0x15, 0x02ab,
	0x19, 0x31ba, 0x15, 0x02ac, 0x19, 0x31d5, 0x15, 0x02ad,
	0x19, 0x0000, 0x15, 0x02ae, 0x19, 0x5cf0, 0x15, 0x02af,
	0x19, 0x588c, 0x15, 0x02b0, 0x19, 0x542f, 0x15, 0x02b1,
	0x19, 0x7ffb, 0x15, 0x02b2, 0x19, 0x6ff8, 0x15, 0x02b3,
	0x19, 0x64a4, 0x15, 0x02b4, 0x19, 0x64a0, 0x15, 0x02b5,
	0x19, 0x6800, 0x15, 0x02b6, 0x19, 0x4400, 0x15, 0x02b7,
	0x19, 0x4020, 0x15, 0x02b8, 0x19, 0x4480, 0x15, 0x02b9,
	0x19, 0x9e00, 0x15, 0x02ba, 0x19, 0x4891, 0x15, 0x02bb,
	0x19, 0x4cc0, 0x15, 0x02bc, 0x19, 0x4801, 0x15, 0x02bd,
	0x19, 0xa609, 0x15, 0x02be, 0x19, 0xd64f, 0x15, 0x02bf,
	0x19, 0x004e, 0x15, 0x02c0, 0x19, 0x87fe, 0x15, 0x02c1,
	0x19, 0x32c6, 0x15, 0x02c2, 0x19, 0x0000, 0x15, 0x02c3,
	0x19, 0x0000, 0x15, 0x02c4, 0x19, 0x0000, 0x15, 0x02c5,
	0x19, 0x0000, 0x15, 0x02c6, 0x19, 0x48b2, 0x15, 0x02c7,
	0x19, 0x4020, 0x15, 0x02c8, 0x19, 0x4822, 0x15, 0x02c9,
	0x19, 0x4488, 0x15, 0x02ca, 0x19, 0xd64f, 0x15, 0x02cb,
	0x19, 0x0042, 0x15, 0x02cc, 0x19, 0x8203, 0x15, 0x02cd,
	0x19, 0x4cc8, 0x15, 0x02ce, 0x19, 0x32d0, 0x15, 0x02cf,
	0x19, 0x4cc0, 0x15, 0x02d0, 0x19, 0xc4d4, 0x15, 0x02d1,
	0x19, 0x00f9, 0x15, 0x02d2, 0x19, 0xa51a, 0x15, 0x02d3,
	0x19, 0x32d9, 0x15, 0x02d4, 0x19, 0x0000, 0x15, 0x02d5,
	0x19, 0x0000, 0x15, 0x02d6, 0x19, 0x0000, 0x15, 0x02d7,
	0x19, 0x0000, 0x15, 0x02d8, 0x19, 0x0000, 0x15, 0x02d9,
	0x19, 0x48b3, 0x15, 0x02da, 0x19, 0x4020, 0x15, 0x02db,
	0x19, 0x4823, 0x15, 0x02dc, 0x19, 0x4410, 0x15, 0x02dd,
	0x19, 0xb630, 0x15, 0x02de, 0x19, 0x7dc8, 0x15, 0x02df,
	0x19, 0x8203, 0x15, 0x02e0, 0x19, 0x4c48, 0x15, 0x02e1,
	0x19, 0x32e3, 0x15, 0x02e2, 0x19, 0x4c40, 0x15, 0x02e3,
	0x19, 0x9bfa, 0x15, 0x02e4, 0x19, 0x84ca, 0x15, 0x02e5,
	0x19, 0x85f8, 0x15, 0x02e6, 0x19, 0x32ec, 0x15, 0x02e7,
	0x19, 0x0000, 0x15, 0x02e8, 0x19, 0x0000, 0x15, 0x02e9,
	0x19, 0x0000, 0x15, 0x02ea, 0x19, 0x0000, 0x15, 0x02eb,
	0x19, 0x0000, 0x15, 0x02ec, 0x19, 0x48d4, 0x15, 0x02ed,
	0x19, 0x4020, 0x15, 0x02ee, 0x19, 0x4844, 0x15, 0x02ef,
	0x19, 0x4420, 0x15, 0x02f0, 0x19, 0x6800, 0x15, 0x02f1,
	0x19, 0x7dc0, 0x15, 0x02f2, 0x19, 0x4c40, 0x15, 0x02f3,
	0x19, 0x7c0b, 0x15, 0x02f4, 0x19, 0x6c08, 0x15, 0x02f5,
	0x19, 0x3311, 0x15, 0x02f6, 0x19, 0x9cfd, 0x15, 0x02f7,
	0x19, 0xb616, 0x15, 0x02f8, 0x19, 0xc42b, 0x15, 0x02f9,
	0x19, 0x00e0, 0x15, 0x02fa, 0x19, 0xc455, 0x15, 0x02fb,
	0x19, 0x00b3, 0x15, 0x02fc, 0x19, 0xb20a, 0x15, 0x02fd,
	0x19, 0x7c03, 0x15, 0x02fe, 0x19, 0x6c02, 0x15, 0x02ff,
	0x19, 0x8204, 0x15, 0x0300, 0x19, 0x7c04, 0x15, 0x0301,
	0x19, 0x7404, 0x15, 0x0302, 0x19, 0x32f3, 0x15, 0x0303,
	0x19, 0x7c04, 0x15, 0x0304, 0x19, 0x7400, 0x15, 0x0305,
	0x19, 0x32f3, 0x15, 0x0306, 0x19, 0xefed, 0x15, 0x0307,
	0x19, 0x3342, 0x15, 0x0308, 0x19, 0x0000, 0x15, 0x0309,
	0x19, 0x0000, 0x15, 0x030a, 0x19, 0x0000, 0x15, 0x030b,
	0x19, 0x0000, 0x15, 0x030c, 0x19, 0x0000, 0x15, 0x030d,
	0x19, 0x3006, 0x15, 0x030e, 0x19, 0x0000, 0x15, 0x030f,
	0x19, 0x0000, 0x15, 0x0310, 0x19, 0x0000, 0x15, 0x0311,
	0x19, 0x7c08, 0x15, 0x0312, 0x19, 0xa207, 0x15, 0x0313,
	0x19, 0x4c00, 0x15, 0x0314, 0x19, 0x3322, 0x15, 0x0315,
	0x19, 0x4041, 0x15, 0x0316, 0x19, 0x7d07, 0x15, 0x0317,
	0x19, 0x4502, 0x15, 0x0318, 0x19, 0x3322, 0x15, 0x0319,
	0x19, 0x4c08, 0x15, 0x031a, 0x19, 0x3322, 0x15, 0x031b,
	0x19, 0x7d80, 0x15, 0x031c, 0x19, 0x5180, 0x15, 0x031d,
	0x19, 0x3320, 0x15, 0x031e, 0x19, 0x7d80, 0x15, 0x031f,
	0x19, 0x5000, 0x15, 0x0320, 0x19, 0x7d07, 0x15, 0x0321,
	0x19, 0x4402, 0x15, 0x0322, 0x19, 0x7c03, 0x15, 0x0323,
	0x19, 0x6c02, 0x15, 0x0324, 0x19, 0x7c03, 0x15, 0x0325,
	0x19, 0xb30c, 0x15, 0x0326, 0x19, 0xb206, 0x15, 0x0327,
	0x19, 0xb103, 0x15, 0x0328, 0x19, 0x6c00, 0x15, 0x0329,
	0x19, 0x32f6, 0x15, 0x032a, 0x19, 0x6c00, 0x15, 0x032b,
	0x19, 0x3352, 0x15, 0x032c, 0x19, 0xb103, 0x15, 0x032d,
	0x19, 0x6c00, 0x15, 0x032e, 0x19, 0x336a, 0x15, 0x032f,
	0x19, 0x6c00, 0x15, 0x0330, 0x19, 0x3382, 0x15, 0x0331,
	0x19, 0xb206, 0x15, 0x0332, 0x19, 0xb103, 0x15, 0x0333,
	0x19, 0x6c00, 0x15, 0x0334, 0x19, 0x3395, 0x15, 0x0335,
	0x19, 0x6c00, 0x15, 0x0336, 0x19, 0x33c6, 0x15, 0x0337,
	0x19, 0xb103, 0x15, 0x0338, 0x19, 0x6c00, 0x15, 0x0339,
	0x19, 0x33d7, 0x15, 0x033a, 0x19, 0x6c00, 0x15, 0x033b,
	0x19, 0x33f2, 0x15, 0x033c, 0x19, 0x0000, 0x15, 0x033d,
	0x19, 0x0000, 0x15, 0x033e, 0x19, 0x0000, 0x15, 0x033f,
	0x19, 0x0000, 0x15, 0x0340, 0x19, 0x0000, 0x15, 0x0341,
	0x19, 0x0000, 0x15, 0x0342, 0x19, 0x49b5, 0x15, 0x0343,
	0x19, 0x7d00, 0x15, 0x0344, 0x19, 0x4d00, 0x15, 0x0345,
	0x19, 0x6880, 0x15, 0x0346, 0x19, 0x7c08, 0x15, 0x0347,
	0x19, 0x6c08, 0x15, 0x0348, 0x19, 0x4925, 0x15, 0x0349,
	0x19, 0x403b, 0x15, 0x034a, 0x19, 0xa602, 0x15, 0x034b,
	0x19, 0x402f, 0x15, 0x034c, 0x19, 0x4484, 0x15, 0x034d,
	0x19, 0x40c8, 0x15, 0x034e, 0x19, 0x44c4, 0x15, 0x034f,
	0x19, 0xd64f, 0x15, 0x0350, 0x19, 0x00bd, 0x15, 0x0351,
	0x19, 0x3311, 0x15, 0x0352, 0x19, 0xc8ed, 0x15, 0x0353,
	0x19, 0x00fc, 0x15, 0x0354, 0x19, 0x8221, 0x15, 0x0355,
	0x19, 0xd11d, 0x15, 0x0356, 0x19, 0x001f, 0x15, 0x0357,
	0x19, 0xde18, 0x15, 0x0358, 0x19, 0x0008, 0x15, 0x0359,
	0x19, 0x91f6, 0x15, 0x035a, 0x19, 0x3360, 0x15, 0x035b,
	0x19, 0x0000, 0x15, 0x035c, 0x19, 0x0000, 0x15, 0x035d,
	0x19, 0x0000, 0x15, 0x035e, 0x19, 0x0000, 0x15, 0x035f,
	0x19, 0x0000, 0x15, 0x0360, 0x19, 0x4bb6, 0x15, 0x0361,
	0x19, 0x4064, 0x15, 0x0362, 0x19, 0x4b26, 0x15, 0x0363,
	0x19, 0x4410, 0x15, 0x0364, 0x19, 0x4006, 0x15, 0x0365,
	0x19, 0x4490, 0x15, 0x0366, 0x19, 0x6900, 0x15, 0x0367,
	0x19, 0xb6a6, 0x15, 0x0368, 0x19, 0x9e02, 0x15, 0x0369,
	0x19, 0x3311, 0x15, 0x036a, 0x19, 0xd11d, 0x15, 0x036b,
	0x19, 0x000a, 0x15, 0x036c, 0x19, 0xbb0f, 0x15, 0x036d,
	0x19, 0x8102, 0x15, 0x036e, 0x19, 0x3371, 0x15, 0x036f,
	0x19, 0xa21e, 0x15, 0x0370, 0x19, 0x33b6, 0x15, 0x0371,
	0x19, 0x91f6, 0x15, 0x0372, 0x19, 0xc218, 0x15, 0x0373,
	0x19, 0x00f4, 0x15, 0x0374, 0x19, 0x33b6, 0x15, 0x0375,
	0x19, 0x32ec, 0x15, 0x0376, 0x19, 0x0000, 0x15, 0x0377,
	0x19, 0x0000, 0x15, 0x0378, 0x19, 0x0000, 0x15, 0x0379,
	0x19, 0x0000, 0x15, 0x037a, 0x19, 0x0000, 0x15, 0x037b,
	0x19, 0x4b97, 0x15, 0x037c, 0x19, 0x402b, 0x15, 0x037d,
	0x19, 0x4b07, 0x15, 0x037e, 0x19, 0x4422, 0x15, 0x037f,
	0x19, 0x6980, 0x15, 0x0380, 0x19, 0xb608, 0x15, 0x0381,
	0x19, 0x3311, 0x15, 0x0382, 0x19, 0xbc05, 0x15, 0x0383,
	0x19, 0xc21c, 0x15, 0x0384, 0x19, 0x0032, 0x15, 0x0385,
	0x19, 0xa1fb, 0x15, 0x0386, 0x19, 0x338d, 0x15, 0x0387,
	0x19, 0x32ae, 0x15, 0x0388, 0x19, 0x330d, 0x15, 0x0389,
	0x19, 0x0000, 0x15, 0x038a, 0x19, 0x0000, 0x15, 0x038b,
	0x19, 0x0000, 0x15, 0x038c, 0x19, 0x0000, 0x15, 0x038d,
	0x19, 0x4b97, 0x15, 0x038e, 0x19, 0x6a08, 0x15, 0x038f,
	0x19, 0x4b07, 0x15, 0x0390, 0x19, 0x40ac, 0x15, 0x0391,
	0x19, 0x4445, 0x15, 0x0392, 0x19, 0x404e, 0x15, 0x0393,
	0x19, 0x4461, 0x15, 0x0394, 0x19, 0x3311, 0x15, 0x0395,
	0x19, 0x9c0a, 0x15, 0x0396, 0x19, 0x63da, 0x15, 0x0397,
	0x19, 0x6f0c, 0x15, 0x0398, 0x19, 0x5440, 0x15, 0x0399,
	0x19, 0x4b98, 0x15, 0x039a, 0x19, 0x7c40, 0x15, 0x039b,
	0x19, 0x4c00, 0x15, 0x039c, 0x19, 0x4b08, 0x15, 0x039d,
	0x19, 0x63d8, 0x15, 0x039e, 0x19, 0x33a5, 0x15, 0x039f,
	0x19, 0xd64f, 0x15, 0x03a0, 0x19, 0x00e8, 0x15, 0x03a1,
	0x19, 0x820e, 0x15, 0x03a2, 0x19, 0xa10d, 0x15, 0x03a3,
	0x19, 0x9df1, 0x15, 0x03a4, 0x19, 0x33af, 0x15, 0x03a5,
	0x19, 0xd64f, 0x15, 0x03a6, 0x19, 0x00f9, 0x15, 0x03a7,
	0x19, 0xc017, 0x15, 0x03a8, 0x19, 0x0007, 0x15, 0x03a9,
	0x19, 0x7c03, 0x15, 0x03aa, 0x19, 0x6c03, 0x15, 0x03ab,
	0x19, 0xa104, 0x15, 0x03ac, 0x19, 0x7c03, 0x15, 0x03ad,
	0x19, 0x6c00, 0x15, 0x03ae, 0x19, 0x9df7, 0x15, 0x03af,
	0x19, 0x7c03, 0x15, 0x03b0, 0x19, 0x6c08, 0x15, 0x03b1,
	0x19, 0x33b6, 0x15, 0x03b2, 0x19, 0x0000, 0x15, 0x03b3,
	0x19, 0x0000, 0x15, 0x03b4, 0x19, 0x0000, 0x15, 0x03b5,
	0x19, 0x0000, 0x15, 0x03b6, 0x19, 0x55af, 0x15, 0x03b7,
	0x19, 0x7ff0, 0x15, 0x03b8, 0x19, 0x6ff0, 0x15, 0x03b9,
	0x19, 0x4bb9, 0x15, 0x03ba, 0x19, 0x6a80, 0x15, 0x03bb,
	0x19, 0x4b29, 0x15, 0x03bc, 0x19, 0x4041, 0x15, 0x03bd,
	0x19, 0x440a, 0x15, 0x03be, 0x19, 0x4029, 0x15, 0x03bf,
	0x19, 0x4418, 0x15, 0x03c0, 0x19, 0x4090, 0x15, 0x03c1,
	0x19, 0x4438, 0x15, 0x03c2, 0x19, 0x40c4, 0x15, 0x03c3,
	0x19, 0x447b, 0x15, 0x03c4, 0x19, 0xb6c4, 0x15, 0x03c5,
	0x19, 0x3311, 0x15, 0x03c6, 0x19, 0x9bfe, 0x15, 0x03c7,
	0x19, 0x33cc, 0x15, 0x03c8, 0x19, 0x0000, 0x15, 0x03c9,
	0x19, 0x0000, 0x15, 0x03ca, 0x19, 0x0000, 0x15, 0x03cb,
	0x19, 0x0000, 0x15, 0x03cc, 0x19, 0x542f, 0x15, 0x03cd,
	0x19, 0x499a, 0x15, 0x03ce, 0x19, 0x7c40, 0x15, 0x03cf,
	0x19, 0x4c40, 0x15, 0x03d0, 0x19, 0x490a, 0x15, 0x03d1,
	0x19, 0x405e, 0x15, 0x03d2, 0x19, 0x44f8, 0x15, 0x03d3,
	0x19, 0x6b00, 0x15, 0x03d4, 0x19, 0xd64f, 0x15, 0x03d5,
	0x19, 0x0028, 0x15, 0x03d6, 0x19, 0x3311, 0x15, 0x03d7,
	0x19, 0xbd27, 0x15, 0x03d8, 0x19, 0x9cfc, 0x15, 0x03d9,
	0x19, 0xc639, 0x15, 0x03da, 0x19, 0x000f, 0x15, 0x03db,
	0x19, 0x9e03, 0x15, 0x03dc, 0x19, 0x7c01, 0x15, 0x03dd,
	0x19, 0x4c01, 0x15, 0x03de, 0x19, 0x9af6, 0x15, 0x03df,
	0x19, 0x7c12, 0x15, 0x03e0, 0x19, 0x4c52, 0x15, 0x03e1,
	0x19, 0x4470, 0x15, 0x03e2, 0x19, 0x7c12, 0x15, 0x03e3,
	0x19, 0x4c40, 0x15, 0x03e4, 0x19, 0x33d4, 0x15, 0x03e5,
	0x19, 0x0000, 0x15, 0x03e6, 0x19, 0x0000, 0x15, 0x03e7,
	0x19, 0x0000, 0x15, 0x03e8, 0x19, 0x0000, 0x15, 0x03e9,
	0x19, 0x49bb, 0x15, 0x03ea, 0x19, 0x4478, 0x15, 0x03eb,
	0x19, 0x492b, 0x15, 0x03ec, 0x19, 0x6b80, 0x15, 0x03ed,
	0x19, 0x7c01, 0x15, 0x03ee, 0x19, 0x4c00, 0x15, 0x03ef,
	0x19, 0xd64f, 0x15, 0x03f0, 0x19, 0x000d, 0x15, 0x03f1,
	0x19, 0x3311, 0x15, 0x03f2, 0x19, 0xbd0c, 0x15, 0x03f3,
	0x19, 0xc428, 0x15, 0x03f4, 0x19, 0x0008, 0x15, 0x03f5,
	0x19, 0x9afa, 0x15, 0x03f6, 0x19, 0x7c12, 0x15, 0x03f7,
	0x19, 0x4c52, 0x15, 0x03f8, 0x19, 0x4470, 0x15, 0x03f9,
	0x19, 0x7c12, 0x15, 0x03fa, 0x19, 0x4c40, 0x15, 0x03fb,
	0x19, 0x33ef, 0x15, 0x03fc, 0x19, 0x3342, 0x15, 0x03fd,
	0x19, 0x330d, 0x15, 0x03fe, 0x19, 0x32ae, 0x15, 0x0000,
	0x16, 0x0306, 0x16, 0x0300, 0x1f, 0x0002, 0x1f, 0x0000,
	0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080, 0x05, 0x8000,
	0x06, 0x0280, 0x06, 0x48f7, 0x06, 0x00e0, 0x06, 0xfff7,
	0x06, 0xa080, 0x06, 0x02ae, 0x06, 0xf602, 0x06, 0x0112,
	0x06, 0x0201, 0x06, 0x1f02, 0x06, 0x012c, 0x06, 0x0201,
	0x06, 0x3c02, 0x06, 0x0156, 0x06, 0x0201, 0x06, 0x6d02,
	0x06, 0x809d, 0x06, 0xe08b, 0x06, 0x88e1, 0x06, 0x8b89,
	0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8a1e, 0x06, 0x01e1,
	0x06, 0x8b8b, 0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8c1e,
	0x06, 0x01e1, 0x06, 0x8b8d, 0x06, 0x1e01, 0x06, 0xe18b,
	0x06, 0x8e1e, 0x06, 0x01a0, 0x06, 0x00c7, 0x06, 0xaebb,
	0x06, 0xd100, 0x06, 0xbf82, 0x06, 0xc702, 0x06, 0x320a,
	0x06, 0xd105, 0x06, 0xbf82, 0x06, 0xcd02, 0x06, 0x320a,
	0x06, 0xd100, 0x06, 0xbf82, 0x06, 0xca02, 0x06, 0x320a,
	0x06, 0xd105, 0x06, 0xbf82, 0x06, 0xd002, 0x06, 0x320a,
	0x06, 0xd481, 0x06, 0xc9e4, 0x06, 0x8b90, 0x06, 0xe58b,
	0x06, 0x91d4, 0x06, 0x81b8, 0x06, 0xe48b, 0x06, 0x92e5,
	0x06, 0x8b93, 0x06, 0xbf8b, 0x06, 0x88ec, 0x06, 0x0019,
	0x06, 0xa98b, 0x06, 0x90f9, 0x06, 0xeeff, 0x06, 0xf600,
	0x06, 0xeeff, 0x06, 0xf7fc, 0x06, 0xd100, 0x06, 0xbf82,
	0x06, 0xc102, 0x06, 0x320a, 0x06, 0xd101, 0x06, 0xbf82,
	0x06, 0xc402, 0x06, 0x320a, 0x06, 0x04f8, 0x06, 0xe08b,
	0x06, 0x8ead, 0x06, 0x201a, 0x06, 0xf620, 0x06, 0xe48b,
	0x06, 0x8e02, 0x06, 0x824b, 0x06, 0x0281, 0x06, 0x1902,
	0x06, 0x2c9d, 0x06, 0x0203, 0x06, 0x9602, 0x06, 0x0473,
	0x06, 0x022e, 0x06, 0x3902, 0x06, 0x044d, 0x06, 0xe08b,
	0x06, 0x8ead, 0x06, 0x210b, 0x06, 0xf621, 0x06, 0xe48b,
	0x06, 0x8e02, 0x06, 0x0416, 0x06, 0x021b, 0x06, 0xa4e0,
	0x06, 0x8b8e, 0x06, 0xad22, 0x06, 0x05f6, 0x06, 0x22e4,
	0x06, 0x8b8e, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x2305,
	0x06, 0xf623, 0x06, 0xe48b, 0x06, 0x8ee0, 0x06, 0x8b8e,
	0x06, 0xad24, 0x06, 0x05f6, 0x06, 0x24e4, 0x06, 0x8b8e,
	0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x2505, 0x06, 0xf625,
	0x06, 0xe48b, 0x06, 0x8ee0, 0x06, 0x8b8e, 0x06, 0xad26,
	0x06, 0x08f6, 0x06, 0x26e4, 0x06, 0x8b8e, 0x06, 0x0281,
	0x06, 0xdae0, 0x06, 0x8b8e, 0x06, 0xad27, 0x06, 0x05f6,
	0x06, 0x27e4, 0x06, 0x8b8e, 0x06, 0x0203, 0x06, 0x5cfc,
	0x06, 0x04f8, 0x06, 0xfaef, 0x06, 0x69e0, 0x06, 0x8b85,
	0x06, 0xad21, 0x06, 0x57e0, 0x06, 0xe022, 0x06, 0xe1e0,
	0x06, 0x2358, 0x06, 0xc059, 0x06, 0x021e, 0x06, 0x01e1,
	0x06, 0x8b3c, 0x06, 0x1f10, 0x06, 0x9e44, 0x06, 0xe48b,
	0x06, 0x3cad, 0x06, 0x211d, 0x06, 0xe18b, 0x06, 0x84f7,
	0x06, 0x29e5, 0x06, 0x8b84, 0x06, 0xac27, 0x06, 0x0dac,
	0x06, 0x2605, 0x06, 0x0281, 0x06, 0x7fae, 0x06, 0x2b02,
	0x06, 0x2c23, 0x06, 0xae26, 0x06, 0x022c, 0x06, 0x41ae,
	0x06, 0x21e0, 0x06, 0x8b87, 0x06, 0xad22, 0x06, 0x18e0,
	0x06, 0xfff7, 0x06, 0x58fc, 0x06, 0xe4ff, 0x06, 0xf7d1,
	0x06, 0x00bf, 0x06, 0x2eee, 0x06, 0x0232, 0x06, 0x0ad1,
	0x06, 0x00bf, 0x06, 0x82e8, 0x06, 0x0232, 0x06, 0x0a02,
	0x06, 0x2bdf, 0x06, 0xef96, 0x06, 0xfefc, 0x06, 0x04d0,
	0x06, 0x0202, 0x06, 0x1e97, 0x06, 0xe08b, 0x06, 0x87ad,
	0x06, 0x2228, 0x06, 0xd100, 0x06, 0xbf82, 0x06, 0xd302,
	0x06, 0x320a, 0x06, 0xd10c, 0x06, 0xbf82, 0x06, 0xd602,
	0x06, 0x320a, 0x06, 0xd104, 0x06, 0xbf82, 0x06, 0xd902,
	0x06, 0x320a, 0x06, 0xd101, 0x06, 0xbf82, 0x06, 0xe802,
	0x06, 0x320a, 0x06, 0xe0ff, 0x06, 0xf768, 0x06, 0x03e4,
	0x06, 0xfff7, 0x06, 0xd004, 0x06, 0x0228, 0x06, 0x7a04,
	0x06, 0xf8e0, 0x06, 0xe234, 0x06, 0xe1e2, 0x06, 0x35f6,
	0x06, 0x2be4, 0x06, 0xe234, 0x06, 0xe5e2, 0x06, 0x35fc,
	0x06, 0x05f8, 0x06, 0xe0e2, 0x06, 0x34e1, 0x06, 0xe235,
	0x06, 0xf72b, 0x06, 0xe4e2, 0x06, 0x34e5, 0x06, 0xe235,
	0x06, 0xfc05, 0x06, 0xf8f9, 0x06, 0xfaef, 0x06, 0x69ac,
	0x06, 0x1b4c, 0x06, 0xbf2e, 0x06, 0x3002, 0x06, 0x31dd,
	0x06, 0xef01, 0x06, 0xe28a, 0x06, 0x76e4, 0x06, 0x8a76,
	0x06, 0x1f12, 0x06, 0x9e3a, 0x06, 0xef12, 0x06, 0x5907,
	0x06, 0x9f12, 0x06, 0xf8e0, 0x06, 0x8b40, 0x06, 0xf721,
	0x06, 0xe48b, 0x06, 0x40d0, 0x06, 0x0302, 0x06, 0x287a,
	0x06, 0x0282, 0x06, 0x34fc, 0x06, 0xa000, 0x06, 0x1002,
	0x06, 0x2dc3, 0x06, 0x022e, 0x06, 0x21e0, 0x06, 0x8b40,
	0x06, 0xf621, 0x06, 0xe48b, 0x06, 0x40ae, 0x06, 0x0fbf,
	0x06, 0x3fa5, 0x06, 0x0231, 0x06, 0x6cbf, 0x06, 0x3fa2,
	0x06, 0x0231, 0x06, 0x6c02, 0x06, 0x2dc3, 0x06, 0xef96,
	0x06, 0xfefd, 0x06, 0xfc04, 0x06, 0xf8e0, 0x06, 0xe2f4,
	0x06, 0xe1e2, 0x06, 0xf5e4, 0x06, 0x8a78, 0x06, 0xe58a,
	0x06, 0x79ee, 0x06, 0xe2f4, 0x06, 0xd8ee, 0x06, 0xe2f5,
	0x06, 0x20fc, 0x06, 0x04f8, 0x06, 0xf9fa, 0x06, 0xef69,
	0x06, 0xe08b, 0x06, 0x87ad, 0x06, 0x2065, 0x06, 0xd200,
	0x06, 0xbf2e, 0x06, 0xe802, 0x06, 0x31dd, 0x06, 0x1e21,
	0x06, 0xbf82, 0x06, 0xdf02, 0x06, 0x31dd, 0x06, 0x0c11,
	0x06, 0x1e21, 0x06, 0xbf82, 0x06, 0xe202, 0x06, 0x31dd,
	0x06, 0x0c12, 0x06, 0x1e21, 0x06, 0xbf82, 0x06, 0xe502,
	0x06, 0x31dd, 0x06, 0x0c13, 0x06, 0x1e21, 0x06, 0xbf1f,
	0x06, 0x5302, 0x06, 0x31dd, 0x06, 0x0c14, 0x06, 0x1e21,
	0x06, 0xbf82, 0x06, 0xeb02, 0x06, 0x31dd, 0x06, 0x0c16,
	0x06, 0x1e21, 0x06, 0xe083, 0x06, 0xe01f, 0x06, 0x029e,
	0x06, 0x22e6, 0x06, 0x83e0, 0x06, 0xad31, 0x06, 0x14ad,
	0x06, 0x3011, 0x06, 0xef02, 0x06, 0x580c, 0x06, 0x9e07,
	0x06, 0xad36, 0x06, 0x085a, 0x06, 0x309f, 0x06, 0x04d1,
	0x06, 0x01ae, 0x06, 0x02d1, 0x06, 0x00bf, 0x06, 0x82dc,
	0x06, 0x0232, 0x06, 0x0aef, 0x06, 0x96fe, 0x06, 0xfdfc,
	0x06, 0x0400, 0x06, 0xe140, 0x06, 0x77e1, 0x06, 0x4010,
	0x06, 0xe150, 0x06, 0x32e1, 0x06, 0x5030, 0x06, 0xe144,
	0x06, 0x74e1, 0x06, 0x44bb, 0x06, 0xe2d2, 0x06, 0x40e0,
	0x06, 0x2cfc, 0x06, 0xe2cc, 0x06, 0xcce2, 0x06, 0x00cc,
	0x06, 0xe000, 0x06, 0x99e0, 0x06, 0x3688, 0x06, 0xe036,
	0x06, 0x99e1, 0x06, 0x40dd, 0x06, 0xe022, 0x05, 0xe142,
};

static struct gani_patch gani_8168_m17_patch0[] = {
	0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080, 0x1f, 0x0007,
	0x1e, 0x0023, 0x16, 0x0306, 0x16, 0x0307, 0x15, 0x00AF,
	0x19, 0x4060, 0x15, 0x00B0, 0x19, 0x7800, 0x15, 0x00B1,
	0x19, 0x7e00, 0x15, 0x00B2, 0x19, 0x72B0, 0x15, 0x00B3,
	0x19, 0x7F00, 0x15, 0x00B4, 0x19, 0x73B0, 0x15, 0x0101,
	0x19, 0x0005, 0x15, 0x0103, 0x19, 0x0003, 0x15, 0x0105,
	0x19, 0x30FD, 0x15, 0x0106, 0x19, 0x9DF7, 0x15, 0x0107,
	0x19, 0x30C6, 0x15, 0x0098, 0x19, 0x7c0b, 0x15, 0x0099,
	0x19, 0x6c0b, 0x15, 0x00eb, 0x19, 0x6c0b, 0x15, 0x00f8,
	0x19, 0x6f0b, 0x15, 0x00fe, 0x19, 0x6f0f, 0x15, 0x00db,
	0x19, 0x6f09, 0x15, 0x00dc, 0x19, 0xaefd, 0x15, 0x00dd,
	0x19, 0x6f0b, 0x15, 0x00de, 0x19, 0xc60b, 0x15, 0x00df,
	0x19, 0x00fa, 0x15, 0x00e0, 0x19, 0x30e1, 0x15, 0x020c,
	0x19, 0x3224, 0x15, 0x020e, 0x19, 0x9813, 0x15, 0x020f,
	0x19, 0x7801, 0x15, 0x0210, 0x19, 0x930f, 0x15, 0x0211,
	0x19, 0x9206, 0x15, 0x0212, 0x19, 0x4002, 0x15, 0x0213,
	0x19, 0x7800, 0x15, 0x0214, 0x19, 0x588f, 0x15, 0x0215,
	0x19, 0x5520, 0x15, 0x0216, 0x19, 0x3224, 0x15, 0x0217,
	0x19, 0x4002, 0x15, 0x0218, 0x19, 0x7800, 0x15, 0x0219,
	0x19, 0x588d, 0x15, 0x021a, 0x19, 0x5540, 0x15, 0x021b,
	0x19, 0x9e03, 0x15, 0x021c, 0x19, 0x7c40, 0x15, 0x021d,
	0x19, 0x6840, 0x15, 0x021e, 0x19, 0x3224, 0x15, 0x021f,
	0x19, 0x4002, 0x15, 0x0220, 0x19, 0x3224, 0x15, 0x0221,
	0x19, 0x9e03, 0x15, 0x0222, 0x19, 0x7c40, 0x15, 0x0223,
	0x19, 0x6840, 0x15, 0x0224, 0x19, 0x7800, 0x15, 0x0225,
	0x19, 0x3231, 0x15, 0x0000, 0x16, 0x0306, 0x16, 0x0300,
	0x1f, 0x0002, 0x1f, 0x0000, 0x1f, 0x0000, 0x17, 0x2160,
	0x1f, 0x0007, 0x1e, 0x0040, 0x18, 0x0004,
};

static struct gani_patch gani_8168_m17_patch1[] = {
	0x18, 0x09d4, 0x19, 0x4000, 0x18, 0x09e4, 0x19, 0x0800,
	0x18, 0x09f4, 0x19, 0xff00, 0x18, 0x0a04, 0x19, 0x4000,
	0x18, 0x0a14, 0x19, 0x0c00, 0x18, 0x0a24, 0x19, 0xff00,
	0x18, 0x0a74, 0x19, 0xf600, 0x18, 0x1a24, 0x19, 0x7d00,
	0x18, 0x1a64, 0x19, 0x0500, 0x18, 0x1a74, 0x19, 0x9500,
	0x18, 0x1a84, 0x19, 0x8000, 0x18, 0x1a94, 0x19, 0x7d00,
	0x18, 0x1aa4, 0x19, 0x9600, 0x18, 0x1ac4, 0x19, 0x4000,
	0x18, 0x1ad4, 0x19, 0x0800, 0x18, 0x1af4, 0x19, 0xc400,
	0x18, 0x1b04, 0x19, 0x4000, 0x18, 0x1b14, 0x19, 0x0800,
	0x18, 0x1b24, 0x19, 0xfd00, 0x18, 0x1b34, 0x19, 0x4000,
	0x18, 0x1b44, 0x19, 0x0400, 0x18, 0x1b94, 0x19, 0xf100,
	0x1f, 0x0000, 0x17, 0x2100, 0x1f, 0x0007, 0x1e, 0x0040,
	0x18, 0x0000, 0x1f, 0x0000, 0x1f, 0x0005, 0x05, 0xfff6,
	0x06, 0x0080, 0x05, 0x8000, 0x06, 0x0280, 0x06, 0x48f7,
	0x06, 0x00e0, 0x06, 0xfff7, 0x06, 0xa080, 0x06, 0x02ae,
	0x06, 0xf602, 0x06, 0x0115, 0x06, 0x0201, 0x06, 0x2202,
	0x06, 0x80a0, 0x06, 0x0201, 0x06, 0x3f02, 0x06, 0x0159,
	0x06, 0x0280, 0x06, 0xbd02, 0x06, 0x80da, 0x06, 0xe08b,
	0x06, 0x88e1, 0x06, 0x8b89, 0x06, 0x1e01, 0x06, 0xe18b,
	0x06, 0x8a1e, 0x06, 0x01e1, 0x06, 0x8b8b, 0x06, 0x1e01,
	0x06, 0xe18b, 0x06, 0x8c1e, 0x06, 0x01e1, 0x06, 0x8b8d,
	0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8e1e, 0x06, 0x01a0,
	0x06, 0x00c7, 0x06, 0xaebb, 0x06, 0xd481, 0x06, 0xd2e4,
	0x06, 0x8b92, 0x06, 0xe58b, 0x06, 0x93d1, 0x06, 0x03bf,
	0x06, 0x859e, 0x06, 0x0237, 0x06, 0x23d1, 0x06, 0x02bf,
	0x06, 0x85a1, 0x06, 0x0237, 0x06, 0x23ee, 0x06, 0x8608,
	0x06, 0x03ee, 0x06, 0x860a, 0x06, 0x60ee, 0x06, 0x8610,
	0x06, 0x00ee, 0x06, 0x8611, 0x06, 0x00ee, 0x06, 0x8abe,
	0x06, 0x07ee, 0x06, 0x8abf, 0x06, 0x73ee, 0x06, 0x8a95,
	0x06, 0x02bf, 0x06, 0x8b88, 0x06, 0xec00, 0x06, 0x19a9,
	0x06, 0x8b90, 0x06, 0xf9ee, 0x06, 0xfff6, 0x06, 0x00ee,
	0x06, 0xfff7, 0x06, 0xfed1, 0x06, 0x00bf, 0x06, 0x8595,
	0x06, 0x0237, 0x06, 0x23d1, 0x06, 0x01bf, 0x06, 0x8598,
	0x06, 0x0237, 0x06, 0x2304, 0x06, 0xf8e0, 0x06, 0x8b8a,
	0x06, 0xad20, 0x06, 0x14ee, 0x06, 0x8b8a, 0x06, 0x0002,
	0x06, 0x1f9a, 0x06, 0xe0e4, 0x06, 0x26e1, 0x06, 0xe427,
	0x06, 0xeee4, 0x06, 0x2623, 0x06, 0xe5e4, 0x06, 0x27fc,
	0x06, 0x04f8, 0x06, 0xe08b, 0x06, 0x8dad, 0x06, 0x2014,
	0x06, 0xee8b, 0x06, 0x8d00, 0x06, 0xe08a, 0x06, 0x5a78,
	0x06, 0x039e, 0x06, 0x0902, 0x06, 0x05db, 0x06, 0x0282,
	0x06, 0x7b02, 0x06, 0x3231, 0x06, 0xfc04, 0x06, 0xf8e0,
	0x06, 0x8b8e, 0x06, 0xad20, 0x06, 0x1df6, 0x06, 0x20e4,
	0x06, 0x8b8e, 0x06, 0x0281, 0x06, 0x5c02, 0x06, 0x2bcb,
	0x06, 0x022d, 0x06, 0x2902, 0x06, 0x03b4, 0x06, 0x0285,
	0x06, 0x6402, 0x06, 0x2eca, 0x06, 0x0284, 0x06, 0xcd02,
	0x06, 0x046f, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x210b,
	0x06, 0xf621, 0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x8520,
	0x06, 0x021b, 0x06, 0xe8e0, 0x06, 0x8b8e, 0x06, 0xad22,
	0x06, 0x05f6, 0x06, 0x22e4, 0x06, 0x8b8e, 0x06, 0xe08b,
	0x06, 0x8ead, 0x06, 0x2308, 0x06, 0xf623, 0x06, 0xe48b,
	0x06, 0x8e02, 0x06, 0x311c, 0x06, 0xe08b, 0x06, 0x8ead,
	0x06, 0x2405, 0x06, 0xf624, 0x06, 0xe48b, 0x06, 0x8ee0,
	0x06, 0x8b8e, 0x06, 0xad25, 0x06, 0x05f6, 0x06, 0x25e4,
	0x06, 0x8b8e, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x2608,
	0x06, 0xf626, 0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x2df5,
	0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x2705, 0x06, 0xf627,
	0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x037a, 0x06, 0xfc04,
	0x06, 0xf8f9, 0x06, 0xfaef, 0x06, 0x69e0, 0x06, 0x8b87,
	0x06, 0xad20, 0x06, 0x65d2, 0x06, 0x00bf, 0x06, 0x2fe9,
	0x06, 0x0236, 0x06, 0xf61e, 0x06, 0x21bf, 0x06, 0x2ff5,
	0x06, 0x0236, 0x06, 0xf60c, 0x06, 0x111e, 0x06, 0x21bf,
	0x06, 0x2ff8, 0x06, 0x0236, 0x06, 0xf60c, 0x06, 0x121e,
	0x06, 0x21bf, 0x06, 0x2ffb, 0x06, 0x0236, 0x06, 0xf60c,
	0x06, 0x131e, 0x06, 0x21bf, 0x06, 0x1f97, 0x06, 0x0236,
	0x06, 0xf60c, 0x06, 0x141e, 0x06, 0x21bf, 0x06, 0x859b,
	0x06, 0x0236, 0x06, 0xf60c, 0x06, 0x161e, 0x06, 0x21e0,
	0x06, 0x8a8c, 0x06, 0x1f02, 0x06, 0x9e22, 0x06, 0xe68a,
	0x06, 0x8cad, 0x06, 0x3114, 0x06, 0xad30, 0x06, 0x11ef,
	0x06, 0x0258, 0x06, 0x0c9e, 0x06, 0x07ad, 0x06, 0x3608,
	0x06, 0x5a30, 0x06, 0x9f04, 0x06, 0xd101, 0x06, 0xae02,
	0x06, 0xd100, 0x06, 0xbf2f, 0x06, 0xf202, 0x06, 0x3723,
	0x06, 0xef96, 0x06, 0xfefd, 0x06, 0xfc04, 0x06, 0xf8f9,
	0x06, 0xface, 0x06, 0xfaef, 0x06, 0x69fa, 0x06, 0xd401,
	0x06, 0x55b4, 0x06, 0xfebf, 0x06, 0x85a7, 0x06, 0x0236,
	0x06, 0xf6ac, 0x06, 0x280b, 0x06, 0xbf85, 0x06, 0xa402,
	0x06, 0x36f6, 0x06, 0xac28, 0x06, 0x49ae, 0x06, 0x64bf,
	0x06, 0x85a4, 0x06, 0x0236, 0x06, 0xf6ac, 0x06, 0x285b,
	0x06, 0xd000, 0x06, 0x0282, 0x06, 0x60ac, 0x06, 0x2105,
	0x06, 0xac22, 0x06, 0x02ae, 0x06, 0x4ebf, 0x06, 0xe0c4,
	0x06, 0xbe86, 0x06, 0x14d2, 0x06, 0x04d8, 0x06, 0x19d9,
	0x06, 0x1907, 0x06, 0xdc19, 0x06, 0xdd19, 0x06, 0x0789,
	0x06, 0x89ef, 0x06, 0x645e, 0x06, 0x07ff, 0x06, 0x0d65,
	0x06, 0x5cf8, 0x06, 0x001e, 0x06, 0x46dc, 0x06, 0x19dd,
	0x06, 0x19b2, 0x06, 0xe2d4, 0x06, 0x0001, 0x06, 0xbf85,
	0x06, 0xa402, 0x06, 0x3723, 0x06, 0xae1d, 0x06, 0xbee0,
	0x06, 0xc4bf, 0x06, 0x8614, 0x06, 0xd204, 0x06, 0xd819,
	0x06, 0xd919, 0x06, 0x07dc, 0x06, 0x19dd, 0x06, 0x1907,
	0x06, 0xb2f4, 0x06, 0xd400, 0x06, 0x00bf, 0x06, 0x85a4,
	0x06, 0x0237, 0x06, 0x23fe, 0x06, 0xef96, 0x06, 0xfec6,
	0x06, 0xfefd, 0x06, 0xfc05, 0x06, 0xf9e2, 0x06, 0xe0ea,
	0x06, 0xe3e0, 0x06, 0xeb5a, 0x06, 0x070c, 0x06, 0x031e,
	0x06, 0x20e6, 0x06, 0xe0ea, 0x06, 0xe7e0, 0x06, 0xebe0,
	0x06, 0xe0fc, 0x06, 0xe1e0, 0x06, 0xfdfd, 0x06, 0x04f8,
	0x06, 0xf9e0, 0x06, 0x8b81, 0x06, 0xac26, 0x06, 0x1ae0,
	0x06, 0x8b81, 0x06, 0xac21, 0x06, 0x14e0, 0x06, 0x8b85,
	0x06, 0xac20, 0x06, 0x0ee0, 0x06, 0x8b85, 0x06, 0xac23,
	0x06, 0x08e0, 0x06, 0x8b87, 0x06, 0xac24, 0x06, 0x02ae,
	0x06, 0x3802, 0x06, 0x1ab5, 0x06, 0xeee4, 0x06, 0x1c04,
	0x06, 0xeee4, 0x06, 0x1d04, 0x06, 0xe2e0, 0x06, 0x7ce3,
	0x06, 0xe07d, 0x06, 0xe0e0, 0x06, 0x38e1, 0x06, 0xe039,
	0x06, 0xad2e, 0x06, 0x1bad, 0x06, 0x390d, 0x06, 0xd101,
	0x06, 0xbf21, 0x06, 0xd502, 0x06, 0x3723, 0x06, 0x0282,
	0x06, 0xd8ae, 0x06, 0x0bac, 0x06, 0x3802, 0x06, 0xae06,
	0x06, 0x0283, 0x06, 0x1802, 0x06, 0x8360, 0x06, 0x021a,
	0x06, 0xc6fd, 0x06, 0xfc04, 0x06, 0xf8e1, 0x06, 0x8af4,
	0x06, 0xe08b, 0x06, 0x81ad, 0x06, 0x2605, 0x06, 0x0222,
	0x06, 0xa4f7, 0x06, 0x28e0, 0x06, 0x8b81, 0x06, 0xad21,
	0x06, 0x0502, 0x06, 0x23a9, 0x06, 0xf729, 0x06, 0xe08b,
	0x06, 0x85ad, 0x06, 0x2005, 0x06, 0x0214, 0x06, 0xabf7,
	0x06, 0x2ae0, 0x06, 0x8b85, 0x06, 0xad23, 0x06, 0x0502,
	0x06, 0x12e7, 0x06, 0xf72b, 0x06, 0xe08b, 0x06, 0x87ad,
	0x06, 0x2405, 0x06, 0x0283, 0x06, 0xbcf7, 0x06, 0x2ce5,
	0x06, 0x8af4, 0x06, 0xfc04, 0x06, 0xf8e0, 0x06, 0x8b81,
	0x06, 0xad26, 0x06, 0x0302, 0x06, 0x21e5, 0x06, 0xe08b,
	0x06, 0x81ad, 0x06, 0x2109, 0x06, 0xe08a, 0x06, 0xf4ac,
	0x06, 0x2003, 0x06, 0x0223, 0x06, 0x98e0, 0x06, 0x8b85,
	0x06, 0xad20, 0x06, 0x09e0, 0x06, 0x8af4, 0x06, 0xac21,
	0x06, 0x0302, 0x06, 0x13fb, 0x06, 0xe08b, 0x06, 0x85ad,
	0x06, 0x2309, 0x06, 0xe08a, 0x06, 0xf4ac, 0x06, 0x2203,
	0x06, 0x0212, 0x06, 0xfae0, 0x06, 0x8b87, 0x06, 0xad24,
	0x06, 0x09e0, 0x06, 0x8af4, 0x06, 0xac23, 0x06, 0x0302,
	0x06, 0x83c1, 0x06, 0xfc04, 0x06, 0xf8e1, 0x06, 0x8af4,
	0x06, 0xe08b, 0x06, 0x81ad, 0x06, 0x2608, 0x06, 0xe083,
	0x06, 0xd2ad, 0x06, 0x2502, 0x06, 0xf628, 0x06, 0xe08b,
	0x06, 0x81ad, 0x06, 0x210a, 0x06, 0xe084, 0x06, 0x0af6,
	0x06, 0x27a0, 0x06, 0x0502, 0x06, 0xf629, 0x06, 0xe08b,
	0x06, 0x85ad, 0x06, 0x2008, 0x06, 0xe08a, 0x06, 0xe8ad,
	0x06, 0x2102, 0x06, 0xf62a, 0x06, 0xe08b, 0x06, 0x85ad,
	0x06, 0x2308, 0x06, 0xe08b, 0x06, 0x20a0, 0x06, 0x0302,
	0x06, 0xf62b, 0x06, 0xe08b, 0x06, 0x87ad, 0x06, 0x2408,
	0x06, 0xe086, 0x06, 0x02a0, 0x06, 0x0302, 0x06, 0xf62c,
	0x06, 0xe58a, 0x06, 0xf4a1, 0x06, 0x0008, 0x06, 0xd100,
	0x06, 0xbf21, 0x06, 0xd502, 0x06, 0x3723, 0x06, 0xfc04,
	0x06, 0xee86, 0x06, 0x0200, 0x06, 0x04f8, 0x06, 0xe08b,
	0x06, 0x87ad, 0x06, 0x241e, 0x06, 0xe086, 0x06, 0x02a0,
	0x06, 0x0005, 0x06, 0x0283, 0x06, 0xe8ae, 0x06, 0xf5a0,
	0x06, 0x0105, 0x06, 0x0283, 0x06, 0xf8ae, 0x06, 0x0ba0,
	0x06, 0x0205, 0x06, 0x0284, 0x06, 0x14ae, 0x06, 0x03a0,
	0x06, 0x0300, 0x06, 0xfc04, 0x06, 0xf8fa, 0x06, 0xef69,
	0x06, 0x0284, 0x06, 0x2bee, 0x06, 0x8602, 0x06, 0x01ef,
	0x06, 0x96fe, 0x06, 0xfc04, 0x06, 0xf8ee, 0x06, 0x8609,
	0x06, 0x0002, 0x06, 0x8461, 0x06, 0xfc04, 0x06, 0xae10,
	0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000,
	0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000,
	0x06, 0xf8e0, 0x06, 0x8608, 0x06, 0xe186, 0x06, 0x091f,
	0x06, 0x019e, 0x06, 0x0611, 0x06, 0xe586, 0x06, 0x09ae,
	0x06, 0x04ee, 0x06, 0x8602, 0x06, 0x01fc, 0x06, 0x04f8,
	0x06, 0xf9fa, 0x06, 0xef69, 0x06, 0xfbbf, 0x06, 0x8604,
	0x06, 0xef79, 0x06, 0xd200, 0x06, 0xd400, 0x06, 0x221e,
	0x06, 0x02bf, 0x06, 0x2fec, 0x06, 0x0237, 0x06, 0x23bf,
	0x06, 0x13f2, 0x06, 0x0236, 0x06, 0xf60d, 0x06, 0x4559,
	0x06, 0x1fef, 0x06, 0x97dd, 0x06, 0xd308, 0x06, 0x1a93,
	0x06, 0xdd12, 0x06, 0x17a2, 0x06, 0x04de, 0x06, 0xffef,
	0x06, 0x96fe, 0x06, 0xfdfc, 0x06, 0x04f8, 0x06, 0xf9fa,
	0x06, 0xef69, 0x06, 0xfbee, 0x06, 0x8602, 0x06, 0x03d5,
	0x06, 0x0080, 0x06, 0xbf86, 0x06, 0x04ef, 0x06, 0x79ef,
	0x06, 0x45bf, 0x06, 0x2fec, 0x06, 0x0237, 0x06, 0x23bf,
	0x06, 0x13f2, 0x06, 0x0236, 0x06, 0xf6ad, 0x06, 0x2702,
	0x06, 0x78ff, 0x06, 0xe186, 0x06, 0x0a1b, 0x06, 0x01aa,
	0x06, 0x2eef, 0x06, 0x97d9, 0x06, 0x7900, 0x06, 0x9e2b,
	0x06, 0x81dd, 0x06, 0xbf85, 0x06, 0xad02, 0x06, 0x3723,
	0x06, 0xd101, 0x06, 0xef02, 0x06, 0x100c, 0x06, 0x11b0,
	0x06, 0xfc0d, 0x06, 0x11bf, 0x06, 0x85aa, 0x06, 0x0237,
	0x06, 0x23d1, 0x06, 0x00bf, 0x06, 0x85aa, 0x06, 0x0237,
	0x06, 0x23ee, 0x06, 0x8602, 0x06, 0x02ae, 0x06, 0x0413,
	0x06, 0xa38b, 0x06, 0xb4d3, 0x06, 0x8012, 0x06, 0x17a2,
	0x06, 0x04ad, 0x06, 0xffef, 0x06, 0x96fe, 0x06, 0xfdfc,
	0x06, 0x04f8, 0x06, 0xf9e0, 0x06, 0x8b85, 0x06, 0xad25,
	0x06, 0x48e0, 0x06, 0x8a96, 0x06, 0xe18a, 0x06, 0x977c,
	0x06, 0x0000, 0x06, 0x9e35, 0x06, 0xee8a, 0x06, 0x9600,
	0x06, 0xee8a, 0x06, 0x9700, 0x06, 0xe08a, 0x06, 0xbee1,
	0x06, 0x8abf, 0x06, 0xe286, 0x06, 0x10e3, 0x06, 0x8611,
	0x06, 0x0236, 0x06, 0x1aad, 0x06, 0x2012, 0x06, 0xee8a,
	0x06, 0x9603, 0x06, 0xee8a, 0x06, 0x97b7, 0x06, 0xee86,
	0x06, 0x1000, 0x06, 0xee86, 0x06, 0x1100, 0x06, 0xae11,
	0x06, 0x15e6, 0x06, 0x8610, 0x06, 0xe786, 0x06, 0x11ae,
	0x06, 0x08ee, 0x06, 0x8610, 0x06, 0x00ee, 0x06, 0x8611,
	0x06, 0x00fd, 0x06, 0xfc04, 0x06, 0xf8fa, 0x06, 0xef69,
	0x06, 0xe0e0, 0x06, 0x00e1, 0x06, 0xe001, 0x06, 0xad27,
	0x06, 0x32e0, 0x06, 0x8b40, 0x06, 0xf720, 0x06, 0xe48b,
	0x06, 0x40bf, 0x06, 0x31f5, 0x06, 0x0236, 0x06, 0xf6ad,
	0x06, 0x2821, 0x06, 0xe0e0, 0x06, 0x20e1, 0x06, 0xe021,
	0x06, 0xad20, 0x06, 0x18e0, 0x06, 0x8b40, 0x06, 0xf620,
	0x06, 0xe48b, 0x06, 0x40ee, 0x06, 0x8b3b, 0x06, 0xffe0,
	0x06, 0x8a8a, 0x06, 0xe18a, 0x06, 0x8be4, 0x06, 0xe000,
	0x06, 0xe5e0, 0x06, 0x01ef, 0x06, 0x96fe, 0x06, 0xfc04,
	0x06, 0xf8fa, 0x06, 0xef69, 0x06, 0xe08b, 0x06, 0x80ad,
	0x06, 0x2722, 0x06, 0xbf44, 0x06, 0xfc02, 0x06, 0x36f6,
	0x06, 0xe08b, 0x06, 0x441f, 0x06, 0x019e, 0x06, 0x15e5,
	0x06, 0x8b44, 0x06, 0xad29, 0x06, 0x07ac, 0x06, 0x2804,
	0x06, 0xd101, 0x06, 0xae02, 0x06, 0xd100, 0x06, 0xbf85,
	0x06, 0xb002, 0x06, 0x3723, 0x06, 0xef96, 0x06, 0xfefc,
	0x06, 0x0400, 0x06, 0xe140, 0x06, 0x77e1, 0x06, 0x40dd,
	0x06, 0xe022, 0x06, 0x32e1, 0x06, 0x5074, 0x06, 0xe144,
	0x06, 0xffe0, 0x06, 0xdaff, 0x06, 0xe0c0, 0x06, 0x52e0,
	0x06, 0xeed9, 0x06, 0xe04c, 0x06, 0xbbe0, 0x06, 0x2a00,
	0x05, 0xe142,
};

static struct gani_patch gani_8168_m18_patch[] = {
	0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080, 0x1f, 0x0007,
	0x1e, 0x0023, 0x16, 0x0306, 0x16, 0x0307, 0x15, 0x0194,
	0x19, 0x407D, 0x15, 0x0098, 0x19, 0x7c0b, 0x15, 0x0099,
	0x19, 0x6c0b, 0x15, 0x00eb, 0x19, 0x6c0b, 0x15, 0x00f8,
	0x19, 0x6f0b, 0x15, 0x00fe, 0x19, 0x6f0f, 0x15, 0x00db,
	0x19, 0x6f09, 0x15, 0x00dc, 0x19, 0xaefd, 0x15, 0x00dd,
	0x19, 0x6f0b, 0x15, 0x00de, 0x19, 0xc60b, 0x15, 0x00df,
	0x19, 0x00fa, 0x15, 0x00e0, 0x19, 0x30e1, 0x15, 0x0000,
	0x16, 0x0306, 0x16, 0x0300, 0x1f, 0x0000, 0x1f, 0x0005,
	0x05, 0xfff6, 0x06, 0x0080, 0x05, 0x8000, 0x06, 0x0280,
	0x06, 0x48f7, 0x06, 0x00e0, 0x06, 0xfff7, 0x06, 0xa080,
	0x06, 0x02ae, 0x06, 0xf602, 0x06, 0x0118, 0x06, 0x0201,
	0x06, 0x2502, 0x06, 0x8090, 0x06, 0x0201, 0x06, 0x4202,
	0x06, 0x015c, 0x06, 0x0280, 0x06, 0xad02, 0x06, 0x80ca,
	0x06, 0xe08b, 0x06, 0x88e1, 0x06, 0x8b89, 0x06, 0x1e01,
	0x06, 0xe18b, 0x06, 0x8a1e, 0x06, 0x01e1, 0x06, 0x8b8b,
	0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8c1e, 0x06, 0x01e1,
	0x06, 0x8b8d, 0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8e1e,
	0x06, 0x01a0, 0x06, 0x00c7, 0x06, 0xaebb, 0x06, 0xd484,
	0x06, 0x3ce4, 0x06, 0x8b92, 0x06, 0xe58b, 0x06, 0x93ee,
	0x06, 0x8ac8, 0x06, 0x03ee, 0x06, 0x8aca, 0x06, 0x60ee,
	0x06, 0x8ac0, 0x06, 0x00ee, 0x06, 0x8ac1, 0x06, 0x00ee,
	0x06, 0x8abe, 0x06, 0x07ee, 0x06, 0x8abf, 0x06, 0x73ee,
	0x06, 0x8a95, 0x06, 0x02bf, 0x06, 0x8b88, 0x06, 0xec00,
	0x06, 0x19a9, 0x06, 0x8b90, 0x06, 0xf9ee, 0x06, 0xfff6,
	0x06, 0x00ee, 0x06, 0xfff7, 0x06, 0xfed1, 0x06, 0x00bf,
	0x06, 0x85a4, 0x06, 0x0238, 0x06, 0x7dd1, 0x06, 0x01bf,
	0x06, 0x85a7, 0x06, 0x0238, 0x06, 0x7d04, 0x06, 0xf8e0,
	0x06, 0x8b8a, 0x06, 0xad20, 0x06, 0x14ee, 0x06, 0x8b8a,
	0x06, 0x0002, 0x06, 0x204b, 0x06, 0xe0e4, 0x06, 0x26e1,
	0x06, 0xe427, 0x06, 0xeee4, 0x06, 0x2623, 0x06, 0xe5e4,
	0x06, 0x27fc, 0x06, 0x04f8, 0x06, 0xe08b, 0x06, 0x8dad,
	0x06, 0x2014, 0x06, 0xee8b, 0x06, 0x8d00, 0x06, 0xe08a,
	0x06, 0x5a78, 0x06, 0x039e, 0x06, 0x0902, 0x06, 0x05e8,
	0x06, 0x0281, 0x06, 0x4f02, 0x06, 0x326c, 0x06, 0xfc04,
	0x06, 0xf8e0, 0x06, 0x8b8e, 0x06, 0xad20, 0x06, 0x1df6,
	0x06, 0x20e4, 0x06, 0x8b8e, 0x06, 0x022f, 0x06, 0x0902,
	0x06, 0x2ab0, 0x06, 0x0285, 0x06, 0x1602, 0x06, 0x03ba,
	0x06, 0x0284, 0x06, 0xe502, 0x06, 0x2df1, 0x06, 0x0283,
	0x06, 0x8302, 0x06, 0x0475, 0x06, 0xe08b, 0x06, 0x8ead,
	0x06, 0x210b, 0x06, 0xf621, 0x06, 0xe48b, 0x06, 0x8e02,
	0x06, 0x83f8, 0x06, 0x021c, 0x06, 0x99e0, 0x06, 0x8b8e,
	0x06, 0xad22, 0x06, 0x08f6, 0x06, 0x22e4, 0x06, 0x8b8e,
	0x06, 0x0235, 0x06, 0x63e0, 0x06, 0x8b8e, 0x06, 0xad23,
	0x06, 0x08f6, 0x06, 0x23e4, 0x06, 0x8b8e, 0x06, 0x0231,
	0x06, 0x57e0, 0x06, 0x8b8e, 0x06, 0xad24, 0x06, 0x05f6,
	0x06, 0x24e4, 0x06, 0x8b8e, 0x06, 0xe08b, 0x06, 0x8ead,
	0x06, 0x2505, 0x06, 0xf625, 0x06, 0xe48b, 0x06, 0x8ee0,
	0x06, 0x8b8e, 0x06, 0xad26, 0x06, 0x08f6, 0x06, 0x26e4,
	0x06, 0x8b8e, 0x06, 0x022d, 0x06, 0x1ce0, 0x06, 0x8b8e,
	0x06, 0xad27, 0x06, 0x05f6, 0x06, 0x27e4, 0x06, 0x8b8e,
	0x06, 0x0203, 0x06, 0x80fc, 0x06, 0x04f8, 0x06, 0xf9e0,
	0x06, 0x8b81, 0x06, 0xac26, 0x06, 0x1ae0, 0x06, 0x8b81,
	0x06, 0xac21, 0x06, 0x14e0, 0x06, 0x8b85, 0x06, 0xac20,
	0x06, 0x0ee0, 0x06, 0x8b85, 0x06, 0xac23, 0x06, 0x08e0,
	0x06, 0x8b87, 0x06, 0xac24, 0x06, 0x02ae, 0x06, 0x3802,
	0x06, 0x1ac2, 0x06, 0xeee4, 0x06, 0x1c04, 0x06, 0xeee4,
	0x06, 0x1d04, 0x06, 0xe2e0, 0x06, 0x7ce3, 0x06, 0xe07d,
	0x06, 0xe0e0, 0x06, 0x38e1, 0x06, 0xe039, 0x06, 0xad2e,
	0x06, 0x1bad, 0x06, 0x390d, 0x06, 0xd101, 0x06, 0xbf22,
	0x06, 0x7a02, 0x06, 0x387d, 0x06, 0x0281, 0x06, 0xacae,
	0x06, 0x0bac, 0x06, 0x3802, 0x06, 0xae06, 0x06, 0x0281,
	0x06, 0xe902, 0x06, 0x822e, 0x06, 0x021a, 0x06, 0xd3fd,
	0x06, 0xfc04, 0x06, 0xf8e1, 0x06, 0x8af4, 0x06, 0xe08b,
	0x06, 0x81ad, 0x06, 0x2602, 0x06, 0xf728, 0x06, 0xe08b,
	0x06, 0x81ad, 0x06, 0x2105, 0x06, 0x0222, 0x06, 0x8ef7,
	0x06, 0x29e0, 0x06, 0x8b85, 0x06, 0xad20, 0x06, 0x0502,
	0x06, 0x14b8, 0x06, 0xf72a, 0x06, 0xe08b, 0x06, 0x85ad,
	0x06, 0x2305, 0x06, 0x0212, 0x06, 0xf4f7, 0x06, 0x2be0,
	0x06, 0x8b87, 0x06, 0xad24, 0x06, 0x0502, 0x06, 0x8284,
	0x06, 0xf72c, 0x06, 0xe58a, 0x06, 0xf4fc, 0x06, 0x04f8,
	0x06, 0xe08b, 0x06, 0x81ad, 0x06, 0x2600, 0x06, 0xe08b,
	0x06, 0x81ad, 0x06, 0x2109, 0x06, 0xe08a, 0x06, 0xf4ac,
	0x06, 0x2003, 0x06, 0x0222, 0x06, 0x7de0, 0x06, 0x8b85,
	0x06, 0xad20, 0x06, 0x09e0, 0x06, 0x8af4, 0x06, 0xac21,
	0x06, 0x0302, 0x06, 0x1408, 0x06, 0xe08b, 0x06, 0x85ad,
	0x06, 0x2309, 0x06, 0xe08a, 0x06, 0xf4ac, 0x06, 0x2203,
	0x06, 0x0213, 0x06, 0x07e0, 0x06, 0x8b87, 0x06, 0xad24,
	0x06, 0x09e0, 0x06, 0x8af4, 0x06, 0xac23, 0x06, 0x0302,
	0x06, 0x8289, 0x06, 0xfc04, 0x06, 0xf8e1, 0x06, 0x8af4,
	0x06, 0xe08b, 0x06, 0x81ad, 0x06, 0x2602, 0x06, 0xf628,
	0x06, 0xe08b, 0x06, 0x81ad, 0x06, 0x210a, 0x06, 0xe083,
	0x06, 0xecf6, 0x06, 0x27a0, 0x06, 0x0502, 0x06, 0xf629,
	0x06, 0xe08b, 0x06, 0x85ad, 0x06, 0x2008, 0x06, 0xe08a,
	0x06, 0xe8ad, 0x06, 0x2102, 0x06, 0xf62a, 0x06, 0xe08b,
	0x06, 0x85ad, 0x06, 0x2308, 0x06, 0xe08b, 0x06, 0x20a0,
	0x06, 0x0302, 0x06, 0xf62b, 0x06, 0xe08b, 0x06, 0x87ad,
	0x06, 0x2408, 0x06, 0xe08a, 0x06, 0xc2a0, 0x06, 0x0302,
	0x06, 0xf62c, 0x06, 0xe58a, 0x06, 0xf4a1, 0x06, 0x0008,
	0x06, 0xd100, 0x06, 0xbf22, 0x06, 0x7a02, 0x06, 0x387d,
	0x06, 0xfc04, 0x06, 0xee8a, 0x06, 0xc200, 0x06, 0x04f8,
	0x06, 0xe08b, 0x06, 0x87ad, 0x06, 0x241e, 0x06, 0xe08a,
	0x06, 0xc2a0, 0x06, 0x0005, 0x06, 0x0282, 0x06, 0xb0ae,
	0x06, 0xf5a0, 0x06, 0x0105, 0x06, 0x0282, 0x06, 0xc0ae,
	0x06, 0x0ba0, 0x06, 0x0205, 0x06, 0x0282, 0x06, 0xcaae,
	0x06, 0x03a0, 0x06, 0x0300, 0x06, 0xfc04, 0x06, 0xf8fa,
	0x06, 0xef69, 0x06, 0x0282, 0x06, 0xe1ee, 0x06, 0x8ac2,
	0x06, 0x01ef, 0x06, 0x96fe, 0x06, 0xfc04, 0x06, 0xf8ee,
	0x06, 0x8ac9, 0x06, 0x0002, 0x06, 0x8317, 0x06, 0xfc04,
	0x06, 0xf8e0, 0x06, 0x8ac8, 0x06, 0xe18a, 0x06, 0xc91f,
	0x06, 0x019e, 0x06, 0x0611, 0x06, 0xe58a, 0x06, 0xc9ae,
	0x06, 0x04ee, 0x06, 0x8ac2, 0x06, 0x01fc, 0x06, 0x04f8,
	0x06, 0xf9fa, 0x06, 0xef69, 0x06, 0xfbbf, 0x06, 0x8ac4,
	0x06, 0xef79, 0x06, 0xd200, 0x06, 0xd400, 0x06, 0x221e,
	0x06, 0x02bf, 0x06, 0x3024, 0x06, 0x0238, 0x06, 0x7dbf,
	0x06, 0x13ff, 0x06, 0x0238, 0x06, 0x500d, 0x06, 0x4559,
	0x06, 0x1fef, 0x06, 0x97dd, 0x06, 0xd308, 0x06, 0x1a93,
	0x06, 0xdd12, 0x06, 0x17a2, 0x06, 0x04de, 0x06, 0xffef,
	0x06, 0x96fe, 0x06, 0xfdfc, 0x06, 0x04f8, 0x06, 0xf9fa,
	0x06, 0xef69, 0x06, 0xfbee, 0x06, 0x8ac2, 0x06, 0x03d5,
	0x06, 0x0080, 0x06, 0xbf8a, 0x06, 0xc4ef, 0x06, 0x79ef,
	0x06, 0x45bf, 0x06, 0x3024, 0x06, 0x0238, 0x06, 0x7dbf,
	0x06, 0x13ff, 0x06, 0x0238, 0x06, 0x50ad, 0x06, 0x2702,
	0x06, 0x78ff, 0x06, 0xe18a, 0x06, 0xca1b, 0x06, 0x01aa,
	0x06, 0x2eef, 0x06, 0x97d9, 0x06, 0x7900, 0x06, 0x9e2b,
	0x06, 0x81dd, 0x06, 0xbf85, 0x06, 0xad02, 0x06, 0x387d,
	0x06, 0xd101, 0x06, 0xef02, 0x06, 0x100c, 0x06, 0x11b0,
	0x06, 0xfc0d, 0x06, 0x11bf, 0x06, 0x85aa, 0x06, 0x0238,
	0x06, 0x7dd1, 0x06, 0x00bf, 0x06, 0x85aa, 0x06, 0x0238,
	0x06, 0x7dee, 0x06, 0x8ac2, 0x06, 0x02ae, 0x06, 0x0413,
	0x06, 0xa38b, 0x06, 0xb4d3, 0x06, 0x8012, 0x06, 0x17a2,
	0x06, 0x04ad, 0x06, 0xffef, 0x06, 0x96fe, 0x06, 0xfdfc,
	0x06, 0x04f8, 0x06, 0xf9e0, 0x06, 0x8b85, 0x06, 0xad25,
	0x06, 0x48e0, 0x06, 0x8a96, 0x06, 0xe18a, 0x06, 0x977c,
	0x06, 0x0000, 0x06, 0x9e35, 0x06, 0xee8a, 0x06, 0x9600,
	0x06, 0xee8a, 0x06, 0x9700, 0x06, 0xe08a, 0x06, 0xbee1,
	0x06, 0x8abf, 0x06, 0xe28a, 0x06, 0xc0e3, 0x06, 0x8ac1,
	0x06, 0x0237, 0x06, 0x74ad, 0x06, 0x2012, 0x06, 0xee8a,
	0x06, 0x9603, 0x06, 0xee8a, 0x06, 0x97b7, 0x06, 0xee8a,
	0x06, 0xc000, 0x06, 0xee8a, 0x06, 0xc100, 0x06, 0xae11,
	0x06, 0x15e6, 0x06, 0x8ac0, 0x06, 0xe78a, 0x06, 0xc1ae,
	0x06, 0x08ee, 0x06, 0x8ac0, 0x06, 0x00ee, 0x06, 0x8ac1,
	0x06, 0x00fd, 0x06, 0xfc04, 0x06, 0xae20, 0x06, 0x0000,
	0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000,
	0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000,
	0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000,
	0x06, 0x0000, 0x06, 0x0000, 0x06, 0x0000, 0x06, 0xf8fa,
	0x06, 0xef69, 0x06, 0xe0e0, 0x06, 0x00e1, 0x06, 0xe001,
	0x06, 0xad27, 0x06, 0x32e0, 0x06, 0x8b40, 0x06, 0xf720,
	0x06, 0xe48b, 0x06, 0x40bf, 0x06, 0x3230, 0x06, 0x0238,
	0x06, 0x50ad, 0x06, 0x2821, 0x06, 0xe0e0, 0x06, 0x20e1,
	0x06, 0xe021, 0x06, 0xad20, 0x06, 0x18e0, 0x06, 0x8b40,
	0x06, 0xf620, 0x06, 0xe48b, 0x06, 0x40ee, 0x06, 0x8b3b,
	0x06, 0xffe0, 0x06, 0x8a8a, 0x06, 0xe18a, 0x06, 0x8be4,
	0x06, 0xe000, 0x06, 0xe5e0, 0x06, 0x01ef, 0x06, 0x96fe,
	0x06, 0xfc04, 0x06, 0xf8f9, 0x06, 0xface, 0x06, 0xfaef,
	0x06, 0x69fa, 0x06, 0xd401, 0x06, 0x55b4, 0x06, 0xfebf,
	0x06, 0x1c1e, 0x06, 0x0238, 0x06, 0x50ac, 0x06, 0x280b,
	0x06, 0xbf1c, 0x06, 0x1b02, 0x06, 0x3850, 0x06, 0xac28,
	0x06, 0x49ae, 0x06, 0x64bf, 0x06, 0x1c1b, 0x06, 0x0238,
	0x06, 0x50ac, 0x06, 0x285b, 0x06, 0xd000, 0x06, 0x0284,
	0x06, 0xcaac, 0x06, 0x2105, 0x06, 0xac22, 0x06, 0x02ae,
	0x06, 0x4ebf, 0x06, 0xe0c4, 0x06, 0xbe85, 0x06, 0xf6d2,
	0x06, 0x04d8, 0x06, 0x19d9, 0x06, 0x1907, 0x06, 0xdc19,
	0x06, 0xdd19, 0x06, 0x0789, 0x06, 0x89ef, 0x06, 0x645e,
	0x06, 0x07ff, 0x06, 0x0d65, 0x06, 0x5cf8, 0x06, 0x001e,
	0x06, 0x46dc, 0x06, 0x19dd, 0x06, 0x19b2, 0x06, 0xe2d4,
	0x06, 0x0001, 0x06, 0xbf1c, 0x06, 0x1b02, 0x06, 0x387d,
	0x06, 0xae1d, 0x06, 0xbee0, 0x06, 0xc4bf, 0x06, 0x85f6,
	0x06, 0xd204, 0x06, 0xd819, 0x06, 0xd919, 0x06, 0x07dc,
	0x06, 0x19dd, 0x06, 0x1907, 0x06, 0xb2f4, 0x06, 0xd400,
	0x06, 0x00bf, 0x06, 0x1c1b, 0x06, 0x0238, 0x06, 0x7dfe,
	0x06, 0xef96, 0x06, 0xfec6, 0x06, 0xfefd, 0x06, 0xfc05,
	0x06, 0xf9e2, 0x06, 0xe0ea, 0x06, 0xe3e0, 0x06, 0xeb5a,
	0x06, 0x070c, 0x06, 0x031e, 0x06, 0x20e6, 0x06, 0xe0ea,
	0x06, 0xe7e0, 0x06, 0xebe0, 0x06, 0xe0fc, 0x06, 0xe1e0,
	0x06, 0xfdfd, 0x06, 0x04f8, 0x06, 0xfaef, 0x06, 0x69e0,
	0x06, 0x8b80, 0x06, 0xad27, 0x06, 0x22bf, 0x06, 0x4616,
	0x06, 0x0238, 0x06, 0x50e0, 0x06, 0x8b44, 0x06, 0x1f01,
	0x06, 0x9e15, 0x06, 0xe58b, 0x06, 0x44ad, 0x06, 0x2907,
	0x06, 0xac28, 0x06, 0x04d1, 0x06, 0x01ae, 0x06, 0x02d1,
	0x06, 0x00bf, 0x06, 0x85b0, 0x06, 0x0238, 0x06, 0x7def,
	0x06, 0x96fe, 0x06, 0xfc04, 0x06, 0xf8e0, 0x06, 0x8b85,
	0x06, 0xad26, 0x06, 0x30e0, 0x06, 0xe036, 0x06, 0xe1e0,
	0x06, 0x37e1, 0x06, 0x8b3f, 0x06, 0x1f10, 0x06, 0x9e23,
	0x06, 0xe48b, 0x06, 0x3fac, 0x06, 0x200b, 0x06, 0xac21,
	0x06, 0x0dac, 0x06, 0x250f, 0x06, 0xac27, 0x06, 0x11ae,
	0x06, 0x1202, 0x06, 0x2c47, 0x06, 0xae0d, 0x06, 0x0285,
	0x06, 0x4fae, 0x06, 0x0802, 0x06, 0x2c69, 0x06, 0xae03,
	0x06, 0x022c, 0x06, 0x7cfc, 0x06, 0x04f8, 0x06, 0xfaef,
	0x06, 0x6902, 0x06, 0x856c, 0x06, 0xe0e0, 0x06, 0x14e1,
	0x06, 0xe015, 0x06, 0xad26, 0x06, 0x08d1, 0x06, 0x1ebf,
	0x06, 0x2cd9, 0x06, 0x0238, 0x06, 0x7def, 0x06, 0x96fe,
	0x06, 0xfc04, 0x06, 0xf8e0, 0x06, 0x8b85, 0x06, 0xad27,
	0x06, 0x2fd0, 0x06, 0x0b02, 0x06, 0x3682, 0x06, 0x5882,
	0x06, 0x7882, 0x06, 0x9f24, 0x06, 0xe08b, 0x06, 0x32e1,
	0x06, 0x8b33, 0x06, 0x1f10, 0x06, 0x9e1a, 0x06, 0x10e4,
	0x06, 0x8b32, 0x06, 0xe0e0, 0x06, 0x28e1, 0x06, 0xe029,
	0x06, 0xf72c, 0x06, 0xe4e0, 0x06, 0x28e5, 0x06, 0xe029,
	0x06, 0xf62c, 0x06, 0xe4e0, 0x06, 0x28e5, 0x06, 0xe029,
	0x06, 0xfc04, 0x06, 0x00e1, 0x06, 0x4077, 0x06, 0xe140,
	0x06, 0x52e0, 0x06, 0xeed9, 0x06, 0xe04c, 0x06, 0xbbe0,
	0x06, 0x2a00, 0x05, 0xe142,
};

static struct gani_patch gani_8168_m19_patch[] = {
	/* 035 */
	0x1f, 0x0007, 0x1e, 0x0023, 0x16, 0x0306, 0x16, 0x0307,
	0x15, 0x0098, 0x19, 0x7c0b, 0x15, 0x0099, 0x19, 0x6c0b,
	0x15, 0x00eb, 0x19, 0x6c0b, 0x15, 0x00f8, 0x19, 0x6f0b,
	0x15, 0x00fe, 0x19, 0x6f0f, 0x15, 0x00db, 0x19, 0x6f09,
	0x15, 0x00dc, 0x19, 0xaefd, 0x15, 0x00dd, 0x19, 0x6f0b,
	0x15, 0x00de, 0x19, 0xc60b, 0x15, 0x00df, 0x19, 0x00fa,
	0x15, 0x00e0, 0x19, 0x30e1, 0x15, 0x0000, 0x16, 0x0306,
	0x16, 0x0300, 0x1f, 0x0000, 0x1f, 0x0005, 0x05, 0xfff6,
	0x06, 0x0080, 0x05, 0x8000, 0x06, 0x0280, 0x06, 0x48f7,
	0x06, 0x00e0, 0x06, 0xfff7, 0x06, 0xa080, 0x06, 0x02ae,
	0x06, 0xf602, 0x06, 0x011b, 0x06, 0x0201, 0x06, 0x2802,
	0x06, 0x0135, 0x06, 0x0201, 0x06, 0x4502, 0x06, 0x015f,
	0x06, 0x0280, 0x06, 0x6b02, 0x06, 0x80e5, 0x06, 0xe08b,
	0x06, 0x88e1, 0x06, 0x8b89, 0x06, 0x1e01, 0x06, 0xe18b,
	0x06, 0x8a1e, 0x06, 0x01e1, 0x06, 0x8b8b, 0x06, 0x1e01,
	0x06, 0xe18b, 0x06, 0x8c1e, 0x06, 0x01e1, 0x06, 0x8b8d,
	0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8e1e, 0x06, 0x01a0,
	0x06, 0x00c7, 0x06, 0xaebb, 0x06, 0xbf8b, 0x06, 0x88ec,
	0x06, 0x0019, 0x06, 0xa98b, 0x06, 0x90f9, 0x06, 0xeeff,
	0x06, 0xf600, 0x06, 0xeeff, 0x06, 0xf7fe, 0x06, 0xd100,
	0x06, 0xbf81, 0x06, 0x9802, 0x06, 0x39f3, 0x06, 0xd101,
	0x06, 0xbf81, 0x06, 0x9b02, 0x06, 0x39f3, 0x06, 0x04f8,
	0x06, 0xe08b, 0x06, 0x8dad, 0x06, 0x2014, 0x06, 0xee8b,
	0x06, 0x8d00, 0x06, 0xe08a, 0x06, 0x5a78, 0x06, 0x039e,
	0x06, 0x0902, 0x06, 0x05fc, 0x06, 0x0280, 0x06, 0x8802,
	0x06, 0x32dd, 0x06, 0xfc04, 0x06, 0xf8f9, 0x06, 0xe08b,
	0x06, 0x81ac, 0x06, 0x261a, 0x06, 0xe08b, 0x06, 0x81ac,
	0x06, 0x2114, 0x06, 0xe08b, 0x06, 0x85ac, 0x06, 0x200e,
	0x06, 0xe08b, 0x06, 0x85ac, 0x06, 0x2308, 0x06, 0xe08b,
	0x06, 0x87ac, 0x06, 0x2402, 0x06, 0xae38, 0x06, 0x021a,
	0x06, 0xd6ee, 0x06, 0xe41c, 0x06, 0x04ee, 0x06, 0xe41d,
	0x06, 0x04e2, 0x06, 0xe07c, 0x06, 0xe3e0, 0x06, 0x7de0,
	0x06, 0xe038, 0x06, 0xe1e0, 0x06, 0x39ad, 0x06, 0x2e1b,
	0x06, 0xad39, 0x06, 0x0dd1, 0x06, 0x01bf, 0x06, 0x22c8,
	0x06, 0x0239, 0x06, 0xf302, 0x06, 0x21f0, 0x06, 0xae0b,
	0x06, 0xac38, 0x06, 0x02ae, 0x06, 0x0602, 0x06, 0x222d,
	0x06, 0x0222, 0x06, 0x7202, 0x06, 0x1ae7, 0x06, 0xfdfc,
	0x06, 0x04f8, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x201a,
	0x06, 0xf620, 0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x2afe,
	0x06, 0x022c, 0x06, 0x5c02, 0x06, 0x03c5, 0x06, 0x0281,
	0x06, 0x6702, 0x06, 0x2e4f, 0x06, 0x0204, 0x06, 0x8902,
	0x06, 0x2f7a, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x210b,
	0x06, 0xf621, 0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x0445,
	0x06, 0x021c, 0x06, 0xb8e0, 0x06, 0x8b8e, 0x06, 0xad22,
	0x06, 0x08f6, 0x06, 0x22e4, 0x06, 0x8b8e, 0x06, 0x0235,
	0x06, 0xd4e0, 0x06, 0x8b8e, 0x06, 0xad23, 0x06, 0x08f6,
	0x06, 0x23e4, 0x06, 0x8b8e, 0x06, 0x0231, 0x06, 0xc8e0,
	0x06, 0x8b8e, 0x06, 0xad24, 0x06, 0x05f6, 0x06, 0x24e4,
	0x06, 0x8b8e, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x2505,
	0x06, 0xf625, 0x06, 0xe48b, 0x06, 0x8ee0, 0x06, 0x8b8e,
	0x06, 0xad26, 0x06, 0x08f6, 0x06, 0x26e4, 0x06, 0x8b8e,
	0x06, 0x022d, 0x06, 0x6ae0, 0x06, 0x8b8e, 0x06, 0xad27,
	0x06, 0x05f6, 0x06, 0x27e4, 0x06, 0x8b8e, 0x06, 0x0203,
	0x06, 0x8bfc, 0x06, 0x04f8, 0x06, 0xfaef, 0x06, 0x69e0,
	0x06, 0x8b80, 0x06, 0xad27, 0x06, 0x22bf, 0x06, 0x479a,
	0x06, 0x0239, 0x06, 0xc6e0, 0x06, 0x8b44, 0x06, 0x1f01,
	0x06, 0x9e15, 0x06, 0xe58b, 0x06, 0x44ad, 0x06, 0x2907,
	0x06, 0xac28, 0x06, 0x04d1, 0x06, 0x01ae, 0x06, 0x02d1,
	0x06, 0x00bf, 0x06, 0x819e, 0x06, 0x0239, 0x06, 0xf3ef,
	0x06, 0x96fe, 0x06, 0xfc04, 0x06, 0x00e1, 0x06, 0x4077,
	0x06, 0xe140, 0x06, 0xbbe0, 0x06, 0x2a00, 0x05, 0xe142,
};

static struct gani_patch gani_8168_m20_patch[] = {
	/* 035 */
	0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080, 0x1f, 0x0007,
	0x1e, 0x0023, 0x16, 0x0306, 0x16, 0x0307, 0x15, 0x0098,
	0x19, 0x7c0b, 0x15, 0x0099, 0x19, 0x6c0b, 0x15, 0x00eb,
	0x19, 0x6c0b, 0x15, 0x00f8, 0x19, 0x6f0b, 0x15, 0x00fe,
	0x19, 0x6f0f, 0x15, 0x00db, 0x19, 0x6f09, 0x15, 0x00dc,
	0x19, 0xaefd, 0x15, 0x00dd, 0x19, 0x6f0b, 0x15, 0x00de,
	0x19, 0xc60b, 0x15, 0x00df, 0x19, 0x00fa, 0x15, 0x00e0,
	0x19, 0x30e1, 0x15, 0x0000, 0x16, 0x0306, 0x16, 0x0300,
	0x1f, 0x0000, 0x1f, 0x0005, 0x05, 0xfff6, 0x06, 0x0080,
	0x05, 0x8000, 0x06, 0x0280, 0x06, 0x48f7, 0x06, 0x00e0,
	0x06, 0xfff7, 0x06, 0xa080, 0x06, 0x02ae, 0x06, 0xf602,
	0x06, 0x011e, 0x06, 0x0201, 0x06, 0x2b02, 0x06, 0x8077,
	0x06, 0x0201, 0x06, 0x4802, 0x06, 0x0162, 0x06, 0x0280,
	0x06, 0x9402, 0x06, 0x810e, 0x06, 0xe08b, 0x06, 0x88e1,
	0x06, 0x8b89, 0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8a1e,
	0x06, 0x01e1, 0x06, 0x8b8b, 0x06, 0x1e01, 0x06, 0xe18b,
	0x06, 0x8c1e, 0x06, 0x01e1, 0x06, 0x8b8d, 0x06, 0x1e01,
	0x06, 0xe18b, 0x06, 0x8e1e, 0x06, 0x01a0, 0x06, 0x00c7,
	0x06, 0xaebb, 0x06, 0xd481, 0x06, 0xd4e4, 0x06, 0x8b92,
	0x06, 0xe58b, 0x06, 0x9302, 0x06, 0x2e5a, 0x06, 0xbf8b,
	0x06, 0x88ec, 0x06, 0x0019, 0x06, 0xa98b, 0x06, 0x90f9,
	0x06, 0xeeff, 0x06, 0xf600, 0x06, 0xeeff, 0x06, 0xf7fc,
	0x06, 0xd100, 0x06, 0xbf83, 0x06, 0x3c02, 0x06, 0x3a21,
	0x06, 0xd101, 0x06, 0xbf83, 0x06, 0x3f02, 0x06, 0x3a21,
	0x06, 0x04f8, 0x06, 0xe08b, 0x06, 0x8aad, 0x06, 0x2014,
	0x06, 0xee8b, 0x06, 0x8a00, 0x06, 0x0220, 0x06, 0x8be0,
	0x06, 0xe426, 0x06, 0xe1e4, 0x06, 0x27ee, 0x06, 0xe426,
	0x06, 0x23e5, 0x06, 0xe427, 0x06, 0xfc04, 0x06, 0xf8e0,
	0x06, 0x8b8d, 0x06, 0xad20, 0x06, 0x14ee, 0x06, 0x8b8d,
	0x06, 0x00e0, 0x06, 0x8a5a, 0x06, 0x7803, 0x06, 0x9e09,
	0x06, 0x0206, 0x06, 0x2802, 0x06, 0x80b1, 0x06, 0x0232,
	0x06, 0xfdfc, 0x06, 0x04f8, 0x06, 0xf9e0, 0x06, 0x8b81,
	0x06, 0xac26, 0x06, 0x1ae0, 0x06, 0x8b81, 0x06, 0xac21,
	0x06, 0x14e0, 0x06, 0x8b85, 0x06, 0xac20, 0x06, 0x0ee0,
	0x06, 0x8b85, 0x06, 0xac23, 0x06, 0x08e0, 0x06, 0x8b87,
	0x06, 0xac24, 0x06, 0x02ae, 0x06, 0x3802, 0x06, 0x1b02,
	0x06, 0xeee4, 0x06, 0x1c04, 0x06, 0xeee4, 0x06, 0x1d04,
	0x06, 0xe2e0, 0x06, 0x7ce3, 0x06, 0xe07d, 0x06, 0xe0e0,
	0x06, 0x38e1, 0x06, 0xe039, 0x06, 0xad2e, 0x06, 0x1bad,
	0x06, 0x390d, 0x06, 0xd101, 0x06, 0xbf22, 0x06, 0xe802,
	0x06, 0x3a21, 0x06, 0x0222, 0x06, 0x10ae, 0x06, 0x0bac,
	0x06, 0x3802, 0x06, 0xae06, 0x06, 0x0222, 0x06, 0x4d02,
	0x06, 0x2292, 0x06, 0x021b, 0x06, 0x13fd, 0x06, 0xfc04,
	0x06, 0xf8e0, 0x06, 0x8b8e, 0x06, 0xad20, 0x06, 0x1af6,
	0x06, 0x20e4, 0x06, 0x8b8e, 0x06, 0x022b, 0x06, 0x1e02,
	0x06, 0x82ae, 0x06, 0x0203, 0x06, 0xc002, 0x06, 0x827d,
	0x06, 0x022e, 0x06, 0x6f02, 0x06, 0x047b, 0x06, 0x022f,
	0x06, 0x9ae0, 0x06, 0x8b8e, 0x06, 0xad21, 0x06, 0x0bf6,
	0x06, 0x21e4, 0x06, 0x8b8e, 0x06, 0x0281, 0x06, 0x9002,
	0x06, 0x1cd9, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x2208,
	0x06, 0xf622, 0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x35f4,
	0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x2308, 0x06, 0xf623,
	0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x31e8, 0x06, 0xe08b,
	0x06, 0x8ead, 0x06, 0x2405, 0x06, 0xf624, 0x06, 0xe48b,
	0x06, 0x8ee0, 0x06, 0x8b8e, 0x06, 0xad25, 0x06, 0x05f6,
	0x06, 0x25e4, 0x06, 0x8b8e, 0x06, 0xe08b, 0x06, 0x8ead,
	0x06, 0x2608, 0x06, 0xf626, 0x06, 0xe48b, 0x06, 0x8e02,
	0x06, 0x2d8a, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x2705,
	0x06, 0xf627, 0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x0386,
	0x06, 0xfc04, 0x06, 0xf8fa, 0x06, 0xef69, 0x06, 0xe0e0,
	0x06, 0x00e1, 0x06, 0xe001, 0x06, 0xad27, 0x06, 0x32e0,
	0x06, 0x8b40, 0x06, 0xf720, 0x06, 0xe48b, 0x06, 0x40bf,
	0x06, 0x32c1, 0x06, 0x0239, 0x06, 0xf4ad, 0x06, 0x2821,
	0x06, 0xe0e0, 0x06, 0x20e1, 0x06, 0xe021, 0x06, 0xad20,
	0x06, 0x18e0, 0x06, 0x8b40, 0x06, 0xf620, 0x06, 0xe48b,
	0x06, 0x40ee, 0x06, 0x8b3b, 0x06, 0xffe0, 0x06, 0x8a8a,
	0x06, 0xe18a, 0x06, 0x8be4, 0x06, 0xe000, 0x06, 0xe5e0,
	0x06, 0x01ef, 0x06, 0x96fe, 0x06, 0xfc04, 0x06, 0xf8f9,
	0x06, 0xface, 0x06, 0xfaef, 0x06, 0x69fa, 0x06, 0xd401,
	0x06, 0x55b4, 0x06, 0xfebf, 0x06, 0x1c5e, 0x06, 0x0239,
	0x06, 0xf4ac, 0x06, 0x280b, 0x06, 0xbf1c, 0x06, 0x5b02,
	0x06, 0x39f4, 0x06, 0xac28, 0x06, 0x49ae, 0x06, 0x64bf,
	0x06, 0x1c5b, 0x06, 0x0239, 0x06, 0xf4ac, 0x06, 0x285b,
	0x06, 0xd000, 0x06, 0x0282, 0x06, 0x62ac, 0x06, 0x2105,
	0x06, 0xac22, 0x06, 0x02ae, 0x06, 0x4ebf, 0x06, 0xe0c4,
	0x06, 0xbe85, 0x06, 0xecd2, 0x06, 0x04d8, 0x06, 0x19d9,
	0x06, 0x1907, 0x06, 0xdc19, 0x06, 0xdd19, 0x06, 0x0789,
	0x06, 0x89ef, 0x06, 0x645e, 0x06, 0x07ff, 0x06, 0x0d65,
	0x06, 0x5cf8, 0x06, 0x001e, 0x06, 0x46dc, 0x06, 0x19dd,
	0x06, 0x19b2, 0x06, 0xe2d4, 0x06, 0x0001, 0x06, 0xbf1c,
	0x06, 0x5b02, 0x06, 0x3a21, 0x06, 0xae1d, 0x06, 0xbee0,
	0x06, 0xc4bf, 0x06, 0x85ec, 0x06, 0xd204, 0x06, 0xd819,
	0x06, 0xd919, 0x06, 0x07dc, 0x06, 0x19dd, 0x06, 0x1907,
	0x06, 0xb2f4, 0x06, 0xd400, 0x06, 0x00bf, 0x06, 0x1c5b,
	0x06, 0x023a, 0x06, 0x21fe, 0x06, 0xef96, 0x06, 0xfec6,
	0x06, 0xfefd, 0x06, 0xfc05, 0x06, 0xf9e2, 0x06, 0xe0ea,
	0x06, 0xe3e0, 0x06, 0xeb5a, 0x06, 0x070c, 0x06, 0x031e,
	0x06, 0x20e6, 0x06, 0xe0ea, 0x06, 0xe7e0, 0x06, 0xebe0,
	0x06, 0xe0fc, 0x06, 0xe1e0, 0x06, 0xfdfd, 0x06, 0x04f8,
	0x06, 0xfaef, 0x06, 0x69e0, 0x06, 0x8b80, 0x06, 0xad27,
	0x06, 0x22bf, 0x06, 0x47ba, 0x06, 0x0239, 0x06, 0xf4e0,
	0x06, 0x8b44, 0x06, 0x1f01, 0x06, 0x9e15, 0x06, 0xe58b,
	0x06, 0x44ad, 0x06, 0x2907, 0x06, 0xac28, 0x06, 0x04d1,
	0x06, 0x01ae, 0x06, 0x02d1, 0x06, 0x00bf, 0x06, 0x8342,
	0x06, 0x023a, 0x06, 0x21ef, 0x06, 0x96fe, 0x06, 0xfc04,
	0x06, 0xf8e0, 0x06, 0x8b85, 0x06, 0xad26, 0x06, 0x30e0,
	0x06, 0xe036, 0x06, 0xe1e0, 0x06, 0x37e1, 0x06, 0x8b3f,
	0x06, 0x1f10, 0x06, 0x9e23, 0x06, 0xe48b, 0x06, 0x3fac,
	0x06, 0x200b, 0x06, 0xac21, 0x06, 0x0dac, 0x06, 0x250f,
	0x06, 0xac27, 0x06, 0x11ae, 0x06, 0x1202, 0x06, 0x2cb5,
	0x06, 0xae0d, 0x06, 0x0282, 0x06, 0xe7ae, 0x06, 0x0802,
	0x06, 0x2cd7, 0x06, 0xae03, 0x06, 0x022c, 0x06, 0xeafc,
	0x06, 0x04f8, 0x06, 0xfaef, 0x06, 0x6902, 0x06, 0x8304,
	0x06, 0xe0e0, 0x06, 0x14e1, 0x06, 0xe015, 0x06, 0xad26,
	0x06, 0x08d1, 0x06, 0x1ebf, 0x06, 0x2d47, 0x06, 0x023a,
	0x06, 0x21ef, 0x06, 0x96fe, 0x06, 0xfc04, 0x06, 0xf8e0,
	0x06, 0x8b85, 0x06, 0xad27, 0x06, 0x2fd0, 0x06, 0x0b02,
	0x06, 0x3826, 0x06, 0x5882, 0x06, 0x7882, 0x06, 0x9f24,
	0x06, 0xe08b, 0x06, 0x32e1, 0x06, 0x8b33, 0x06, 0x1f10,
	0x06, 0x9e1a, 0x06, 0x10e4, 0x06, 0x8b32, 0x06, 0xe0e0,
	0x06, 0x28e1, 0x06, 0xe029, 0x06, 0xf72c, 0x06, 0xe4e0,
	0x06, 0x28e5, 0x06, 0xe029, 0x06, 0xf62c, 0x06, 0xe4e0,
	0x06, 0x28e5, 0x06, 0xe029, 0x06, 0xfc04, 0x06, 0x00e1,
	0x06, 0x4077, 0x06, 0xe140, 0x06, 0xbbe0, 0x06, 0x2a00,
	0x05, 0xe142,
};

static struct gani_patch gani_8168_m21_patch[] = {
	/* 035 */
	0x1f, 0x0A43, 0x13, 0x8146, 0x14, 0x2300, 0x13, 0xB820,
	0x14, 0x0210, 0x1F, 0x0A43, 0x13, 0xB820, 0x14, 0x0290,
	0x13, 0xA012, 0x14, 0x0000, 0x13, 0xA014, 0x14, 0x2c04,
	0x14, 0x2c1b, 0x14, 0x2c65, 0x14, 0x2d06, 0x14, 0xd71e,
	0x14, 0x4092, 0x14, 0xba04, 0x14, 0x3084, 0x14, 0x1cf6,
	0x14, 0x1ccf, 0x14, 0x1cda, 0x14, 0xaeff, 0x14, 0xaf02,
	0x14, 0x8f02, 0x14, 0x8eff, 0x14, 0xce01, 0x14, 0xe070,
	0x14, 0x0f00, 0x14, 0xaf01, 0x14, 0x8f01, 0x14, 0xd712,
	0x14, 0x5fe8, 0x14, 0xaf02, 0x14, 0x8f02, 0x14, 0x8e01,
	0x14, 0x1ce4, 0x14, 0x27f2, 0x14, 0xd05a, 0x14, 0xd19a,
	0x14, 0xd709, 0x14, 0x608f, 0x14, 0xd06b, 0x14, 0xd18a,
	0x14, 0x2c25, 0x14, 0xd0be, 0x14, 0xd188, 0x14, 0x2c25,
	0x14, 0xd708, 0x14, 0x4072, 0x14, 0xc104, 0x14, 0x2c37,
	0x14, 0x4076, 0x14, 0xc110, 0x14, 0x2c37, 0x14, 0x4071,
	0x14, 0xc102, 0x14, 0x2c37, 0x14, 0x4070, 0x14, 0xc101,
	0x14, 0x2c37, 0x14, 0x175b, 0x14, 0xd709, 0x14, 0x3390,
	0x14, 0x5c32, 0x14, 0x2c47, 0x14, 0x175b, 0x14, 0xd708,
	0x14, 0x6193, 0x14, 0xd709, 0x14, 0x5f9d, 0x14, 0x408b,
	0x14, 0xd71e, 0x14, 0x6042, 0x14, 0xb401, 0x14, 0x175b,
	0x14, 0xd708, 0x14, 0x6073, 0x14, 0x5fbc, 0x14, 0x2c46,
	0x14, 0x26ed, 0x14, 0xb280, 0x14, 0xa841, 0x14, 0x9420,
	0x14, 0x8710, 0x14, 0xd709, 0x14, 0x42ec, 0x14, 0x606d,
	0x14, 0xd207, 0x14, 0x2c50, 0x14, 0xd203, 0x14, 0x33ff,
	0x14, 0x563b, 0x14, 0x3275, 0x14, 0x7c57, 0x14, 0xb240,
	0x14, 0xb402, 0x14, 0x263b, 0x14, 0x6096, 0x14, 0xb240,
	0x14, 0xb406, 0x14, 0x263b, 0x14, 0x31d7, 0x14, 0x7c60,
	0x14, 0xb240, 0x14, 0xb40e, 0x14, 0x263b, 0x14, 0xb410,
	0x14, 0x8802, 0x14, 0xb240, 0x14, 0x940e, 0x14, 0x263b,
	0x14, 0xba04, 0x14, 0x1ccf, 0x14, 0xa902, 0x14, 0xd711,
	0x14, 0x4045, 0x14, 0xa980, 0x14, 0x3003, 0x14, 0x59b1,
	0x14, 0xa540, 0x14, 0xa601, 0x14, 0xd710, 0x14, 0x4043,
	0x14, 0xa910, 0x14, 0xd711, 0x14, 0x60a0, 0x14, 0xca33,
	0x14, 0xcb33, 0x14, 0xa941, 0x14, 0x2c7b, 0x14, 0xcaff,
	0x14, 0xcbff, 0x14, 0xa921, 0x14, 0xce02, 0x14, 0xe070,
	0x14, 0x0f10, 0x14, 0xaf01, 0x14, 0x8f01, 0x14, 0x1766,
	0x14, 0x8e02, 0x14, 0x1787, 0x14, 0xd710, 0x14, 0x609c,
	0x14, 0xd71e, 0x14, 0x7fa4, 0x14, 0x2ccd, 0x14, 0x1ce2,
	0x14, 0xce04, 0x14, 0xe070, 0x14, 0x0f20, 0x14, 0xaf01,
	0x14, 0x8f01, 0x14, 0x1766, 0x14, 0x8e04, 0x14, 0x6044,
	0x14, 0x2ccd, 0x14, 0xa520, 0x14, 0xd710, 0x14, 0x4043,
	0x14, 0x2cba, 0x14, 0xe00f, 0x14, 0x0501, 0x14, 0x1ce8,
	0x14, 0xb801, 0x14, 0xd71e, 0x14, 0x4060, 0x14, 0x7fc4,
	0x14, 0x2ccd, 0x14, 0x1cee, 0x14, 0xe00f, 0x14, 0x0502,
	0x14, 0x1ce8, 0x14, 0xb802, 0x14, 0xd71e, 0x14, 0x4061,
	0x14, 0x7fc4, 0x14, 0x2ccd, 0x14, 0x1cee, 0x14, 0xe00f,
	0x14, 0x0504, 0x14, 0xd710, 0x14, 0x6099, 0x14, 0xd71e,
	0x14, 0x7fa4, 0x14, 0x2ccd, 0x14, 0xc17f, 0x14, 0xc200,
	0x14, 0xc43f, 0x14, 0xcc03, 0x14, 0xa701, 0x14, 0xa510,
	0x14, 0xd710, 0x14, 0x4018, 0x14, 0x9910, 0x14, 0x8510,
	0x14, 0x2860, 0x14, 0xe00f, 0x14, 0x0504, 0x14, 0xd710,
	0x14, 0x6099, 0x14, 0xd71e, 0x14, 0x7fa4, 0x14, 0x2ccd,
	0x14, 0xa608, 0x14, 0xc17d, 0x14, 0xc200, 0x14, 0xc43f,
	0x14, 0xcc03, 0x14, 0xa701, 0x14, 0xa510, 0x14, 0xd710,
	0x14, 0x4018, 0x14, 0x9910, 0x14, 0x8510, 0x14, 0x2926,
	0x14, 0x1792, 0x14, 0x27db, 0x14, 0xc000, 0x14, 0xc100,
	0x14, 0xc200, 0x14, 0xc300, 0x14, 0xc400, 0x14, 0xc500,
	0x14, 0xc600, 0x14, 0xc7c1, 0x14, 0xc800, 0x14, 0xcc00,
	0x14, 0x0800, 0x14, 0xca0f, 0x14, 0xcbff, 0x14, 0xa901,
	0x14, 0x8902, 0x14, 0xc900, 0x14, 0xca00, 0x14, 0xcb00,
	0x14, 0x0800, 0x14, 0xb804, 0x14, 0x0800, 0x14, 0xd71e,
	0x14, 0x6044, 0x14, 0x9804, 0x14, 0x0800, 0x14, 0xd710,
	0x14, 0x6099, 0x14, 0xd71e, 0x14, 0x7fa4, 0x14, 0x2ccd,
	0x14, 0x0800, 0x14, 0xa510, 0x14, 0xd710, 0x14, 0x6098,
	0x14, 0xd71e, 0x14, 0x7fa4, 0x14, 0x2ccd, 0x14, 0x8510,
	0x14, 0x0800, 0x14, 0xd711, 0x14, 0x3003, 0x14, 0x1cfa,
	0x14, 0x2d04, 0x14, 0xd710, 0x14, 0x60be, 0x14, 0xe060,
	0x14, 0x0920, 0x14, 0x1ccf, 0x14, 0x2c82, 0x14, 0xd71e,
	0x14, 0x3063, 0x14, 0x1948, 0x14, 0x288a, 0x14, 0x1ccf,
	0x14, 0x29bd, 0x14, 0xa802, 0x14, 0xa303, 0x14, 0x843f,
	0x14, 0x81ff, 0x14, 0x8208, 0x14, 0xa201, 0x14, 0xc001,
	0x14, 0xd710, 0x14, 0x30a0, 0x14, 0x0d15, 0x14, 0x30a0,
	0x14, 0x3d0c, 0x14, 0xd71e, 0x14, 0x7f4c, 0x14, 0x2ab6,
	0x14, 0xe003, 0x14, 0x0202, 0x14, 0xd710, 0x14, 0x6090,
	0x14, 0xd71e, 0x14, 0x7fac, 0x14, 0x2ab6, 0x14, 0xa20c,
	0x14, 0xd710, 0x14, 0x6091, 0x14, 0xd71e, 0x14, 0x7fac,
	0x14, 0x2ab6, 0x14, 0x820e, 0x14, 0xa3e0, 0x14, 0xa520,
	0x14, 0xd710, 0x14, 0x609d, 0x14, 0xd71e, 0x14, 0x7fac,
	0x14, 0x2ab6, 0x14, 0x8520, 0x14, 0x6703, 0x14, 0x2d2d,
	0x14, 0xa13e, 0x14, 0xc001, 0x14, 0xd710, 0x14, 0x4000,
	0x14, 0x6046, 0x14, 0x2d06, 0x14, 0xa43f, 0x14, 0xa101,
	0x14, 0xc020, 0x14, 0xd710, 0x14, 0x3121, 0x14, 0x0d3e,
	0x14, 0x30c0, 0x14, 0x3d06, 0x14, 0xd71e, 0x14, 0x7f4c,
	0x14, 0x2ab6, 0x14, 0xa540, 0x14, 0xc001, 0x14, 0xd710,
	0x14, 0x4001, 0x14, 0xe00f, 0x14, 0x0501, 0x14, 0x1da5,
	0x14, 0xc1c4, 0x14, 0xa268, 0x14, 0xa303, 0x14, 0x8420,
	0x14, 0xe00f, 0x14, 0x0502, 0x14, 0x1da5, 0x14, 0xc002,
	0x14, 0xd710, 0x14, 0x4000, 0x14, 0x8208, 0x14, 0x8410,
	0x14, 0xa121, 0x14, 0xc002, 0x14, 0xd710, 0x14, 0x4000,
	0x14, 0x8120, 0x14, 0x8180, 0x14, 0x1d90, 0x14, 0xa180,
	0x14, 0xa13a, 0x14, 0x8240, 0x14, 0xa430, 0x14, 0xc010,
	0x14, 0xd710, 0x14, 0x30e1, 0x14, 0x0abc, 0x14, 0xd71e,
	0x14, 0x7f8c, 0x14, 0x2ab6, 0x14, 0xa480, 0x14, 0xa230,
	0x14, 0xa303, 0x14, 0xc001, 0x14, 0xd70c, 0x14, 0x4124,
	0x14, 0xd710, 0x14, 0x6120, 0x14, 0xd711, 0x14, 0x3128,
	0x14, 0x3d6f, 0x14, 0x2d69, 0x14, 0xa801, 0x14, 0x2d65,
	0x14, 0xd710, 0x14, 0x4000, 0x14, 0xe018, 0x14, 0x0208,
	0x14, 0xa1f8, 0x14, 0x8480, 0x14, 0xc004, 0x14, 0xd710,
	0x14, 0x4000, 0x14, 0x6046, 0x14, 0x2d06, 0x14, 0xa43f,
	0x14, 0xa105, 0x14, 0x8228, 0x14, 0xc004, 0x14, 0xd710,
	0x14, 0x4000, 0x14, 0x81bc, 0x14, 0xa220, 0x14, 0x1d90,
	0x14, 0x8220, 0x14, 0xa1bc, 0x14, 0xc040, 0x14, 0xd710,
	0x14, 0x30e1, 0x14, 0x0abc, 0x14, 0x30e1, 0x14, 0x3d06,
	0x14, 0xd71e, 0x14, 0x7f4c, 0x14, 0x2ab6, 0x14, 0xa802,
	0x14, 0xd70c, 0x14, 0x4244, 0x14, 0xa301, 0x14, 0xc004,
	0x14, 0xd711, 0x14, 0x3128, 0x14, 0x3d9e, 0x14, 0xd710,
	0x14, 0x5f80, 0x14, 0xd711, 0x14, 0x3109, 0x14, 0x3da0,
	0x14, 0x2da4, 0x14, 0xa801, 0x14, 0x2d93, 0x14, 0xa802,
	0x14, 0xc004, 0x14, 0xd710, 0x14, 0x4000, 0x14, 0x0800,
	0x14, 0xa510, 0x14, 0xd710, 0x14, 0x609a, 0x14, 0xd71e,
	0x14, 0x7fac, 0x14, 0x2ab6, 0x14, 0x8510, 0x14, 0x0800,
	0x13, 0xA01A, 0x14, 0x0000, 0x13, 0xA006, 0x14, 0x0ad6,
	0x13, 0xA004, 0x14, 0x07f5, 0x13, 0xA002, 0x14, 0x06cc,
	0x13, 0xA000, 0x14, 0xf7db, 0x13, 0xB820, 0x14, 0x0210,
	0x1F, 0x0A43, 0x13, 0x83a0, 0x14, 0xaf83, 0x14, 0xacaf,
	0x14, 0x83b8, 0x14, 0xaf83, 0x14, 0xcdaf, 0x14, 0x83d3,
	0x14, 0x0204, 0x14, 0x9a02, 0x14, 0x09a9, 0x14, 0x0284,
	0x14, 0x61af, 0x14, 0x02fc, 0x14, 0xad20, 0x14, 0x0302,
	0x14, 0x867c, 0x14, 0xad21, 0x14, 0x0302, 0x14, 0x85c9,
	0x14, 0xad22, 0x14, 0x0302, 0x14, 0x1bc0, 0x14, 0xaf17,
	0x14, 0xe302, 0x14, 0x8703, 0x14, 0xaf18, 0x14, 0x6201,
	0x14, 0x06e0, 0x14, 0x8148, 0x14, 0xaf3c, 0x14, 0x69f8,
	0x14, 0xf9fa, 0x14, 0xef69, 0x14, 0xee80, 0x14, 0x10f7,
	0x14, 0xee80, 0x14, 0x131f, 0x14, 0xd104, 0x14, 0xbf87,
	0x14, 0xf302, 0x14, 0x4259, 0x14, 0x0287, 0x14, 0x88bf,
	0x14, 0x87cf, 0x14, 0xd7b8, 0x14, 0x22d0, 0x14, 0x0c02,
	0x14, 0x4252, 0x14, 0xee80, 0x14, 0xcda0, 0x14, 0xee80,
	0x14, 0xce8b, 0x14, 0xee80, 0x14, 0xd1f5, 0x14, 0xee80,
	0x14, 0xd2a9, 0x14, 0xee80, 0x14, 0xd30a, 0x14, 0xee80,
	0x14, 0xf010, 0x14, 0xee80, 0x14, 0xf38f, 0x14, 0xee81,
	0x14, 0x011e, 0x14, 0xee81, 0x14, 0x0b4a, 0x14, 0xee81,
	0x14, 0x0c7c, 0x14, 0xee81, 0x14, 0x127f, 0x14, 0xd100,
	0x14, 0x0210, 0x14, 0xb5ee, 0x14, 0x8088, 0x14, 0xa4ee,
	0x14, 0x8089, 0x14, 0x44ee, 0x14, 0x809a, 0x14, 0xa4ee,
	0x14, 0x809b, 0x14, 0x44ee, 0x14, 0x809c, 0x14, 0xa7ee,
	0x14, 0x80a5, 0x14, 0xa7d2, 0x14, 0x0002, 0x14, 0x0e66,
	0x14, 0x0285, 0x14, 0xc0ee, 0x14, 0x87fc, 0x14, 0x00e0,
	0x14, 0x8245, 0x14, 0xf622, 0x14, 0xe482, 0x14, 0x45ef,
	0x14, 0x96fe, 0x14, 0xfdfc, 0x14, 0x0402, 0x14, 0x847a,
	0x14, 0x0284, 0x14, 0xb302, 0x14, 0x0cab, 0x14, 0x020c,
	0x14, 0xc402, 0x14, 0x0cef, 0x14, 0x020d, 0x14, 0x0802,
	0x14, 0x0d33, 0x14, 0x020c, 0x14, 0x3d04, 0x14, 0xf8fa,
	0x14, 0xef69, 0x14, 0xe182, 0x14, 0x2fac, 0x14, 0x291a,
	0x14, 0xe082, 0x14, 0x24ac, 0x14, 0x2102, 0x14, 0xae22,
	0x14, 0x0210, 0x14, 0x57f6, 0x14, 0x21e4, 0x14, 0x8224,
	0x14, 0xd101, 0x14, 0xbf44, 0x14, 0xd202, 0x14, 0x4259,
	0x14, 0xae10, 0x14, 0x0212, 0x14, 0x4cf6, 0x14, 0x29e5,
	0x14, 0x822f, 0x14, 0xe082, 0x14, 0x24f6, 0x14, 0x21e4,
	0x14, 0x8224, 0x14, 0xef96, 0x14, 0xfefc, 0x14, 0x04f8,
	0x14, 0xe182, 0x14, 0x2fac, 0x14, 0x2a18, 0x14, 0xe082,
	0x14, 0x24ac, 0x14, 0x2202, 0x14, 0xae26, 0x14, 0x0284,
	0x14, 0xf802, 0x14, 0x8565, 0x14, 0xd101, 0x14, 0xbf44,
	0x14, 0xd502, 0x14, 0x4259, 0x14, 0xae0e, 0x14, 0x0284,
	0x14, 0xea02, 0x14, 0x85a9, 0x14, 0xe182, 0x14, 0x2ff6,
	0x14, 0x2ae5, 0x14, 0x822f, 0x14, 0xe082, 0x14, 0x24f6,
	0x14, 0x22e4, 0x14, 0x8224, 0x14, 0xfc04, 0x14, 0xf9e2,
	0x14, 0x8011, 0x14, 0xad31, 0x14, 0x05d2, 0x14, 0x0002,
	0x14, 0x0e66, 0x14, 0xfd04, 0x14, 0xf8f9, 0x14, 0xfaef,
	0x14, 0x69e0, 0x14, 0x8011, 0x14, 0xad21, 0x14, 0x5cbf,
	0x14, 0x43be, 0x14, 0x0242, 0x14, 0x97ac, 0x14, 0x281b,
	0x14, 0xbf43, 0x14, 0xc102, 0x14, 0x4297, 0x14, 0xac28,
	0x14, 0x12bf, 0x14, 0x43c7, 0x14, 0x0242, 0x14, 0x97ac,
	0x14, 0x2804, 0x14, 0xd300, 0x14, 0xae07, 0x14, 0xd306,
	0x14, 0xaf85, 0x14, 0x56d3, 0x14, 0x03e0, 0x14, 0x8011,
	0x14, 0xad26, 0x14, 0x25bf, 0x14, 0x4559, 0x14, 0x0242,
	0x14, 0x97e2, 0x14, 0x8073, 0x14, 0x0d21, 0x14, 0xf637,
	0x14, 0x0d11, 0x14, 0xf62f, 0x14, 0x1b21, 0x14, 0xaa02,
	0x14, 0xae10, 0x14, 0xe280, 0x14, 0x740d, 0x14, 0x21f6,
	0x14, 0x371b, 0x14, 0x21aa, 0x14, 0x0313, 0x14, 0xae02,
	0x14, 0x2b02, 0x14, 0x020e, 0x14, 0x5102, 0x14, 0x0e66,
	0x14, 0x020f, 0x14, 0xa3ef, 0x14, 0x96fe, 0x14, 0xfdfc,
	0x14, 0x04f8, 0x14, 0xf9fa, 0x14, 0xef69, 0x14, 0xe080,
	0x14, 0x12ad, 0x14, 0x2733, 0x14, 0xbf43, 0x14, 0xbe02,
	0x14, 0x4297, 0x14, 0xac28, 0x14, 0x09bf, 0x14, 0x43c1,
	0x14, 0x0242, 0x14, 0x97ad, 0x14, 0x2821, 0x14, 0xbf45,
	0x14, 0x5902, 0x14, 0x4297, 0x14, 0xe387, 0x14, 0xffd2,
	0x14, 0x001b, 0x14, 0x45ac, 0x14, 0x2711, 0x14, 0xe187,
	0x14, 0xfebf, 0x14, 0x87e4, 0x14, 0x0242, 0x14, 0x590d,
	0x14, 0x11bf, 0x14, 0x87e7, 0x14, 0x0242, 0x14, 0x59ef,
	0x14, 0x96fe, 0x14, 0xfdfc, 0x14, 0x04f8, 0x14, 0xfaef,
	0x14, 0x69d1, 0x14, 0x00bf, 0x14, 0x87e4, 0x14, 0x0242,
	0x14, 0x59bf, 0x14, 0x87e7, 0x14, 0x0242, 0x14, 0x59ef,
	0x14, 0x96fe, 0x14, 0xfc04, 0x14, 0xee87, 0x14, 0xff46,
	0x14, 0xee87, 0x14, 0xfe01, 0x14, 0x04f8, 0x14, 0xfaef,
	0x14, 0x69e0, 0x14, 0x8241, 0x14, 0xa000, 0x14, 0x0502,
	0x14, 0x85eb, 0x14, 0xae0e, 0x14, 0xa001, 0x14, 0x0502,
	0x14, 0x1a5a, 0x14, 0xae06, 0x14, 0xa002, 0x14, 0x0302,
	0x14, 0x1ae6, 0x14, 0xef96, 0x14, 0xfefc, 0x14, 0x04f8,
	0x14, 0xf9fa, 0x14, 0xef69, 0x14, 0xe082, 0x14, 0x29f6,
	0x14, 0x21e4, 0x14, 0x8229, 0x14, 0xe080, 0x14, 0x10ac,
	0x14, 0x2202, 0x14, 0xae76, 0x14, 0xe082, 0x14, 0x27f7,
	0x14, 0x21e4, 0x14, 0x8227, 0x14, 0xbf43, 0x14, 0x1302,
	0x14, 0x4297, 0x14, 0xef21, 0x14, 0xbf43, 0x14, 0x1602,
	0x14, 0x4297, 0x14, 0x0c11, 0x14, 0x1e21, 0x14, 0xbf43,
	0x14, 0x1902, 0x14, 0x4297, 0x14, 0x0c12, 0x14, 0x1e21,
	0x14, 0xe682, 0x14, 0x43a2, 0x14, 0x000a, 0x14, 0xe182,
	0x14, 0x27f6, 0x14, 0x29e5, 0x14, 0x8227, 0x14, 0xae42,
	0x14, 0xe082, 0x14, 0x44f7, 0x14, 0x21e4, 0x14, 0x8244,
	0x14, 0x0246, 0x14, 0xaebf, 0x14, 0x4325, 0x14, 0x0242,
	0x14, 0x97ef, 0x14, 0x21bf, 0x14, 0x431c, 0x14, 0x0242,
	0x14, 0x970c, 0x14, 0x121e, 0x14, 0x21bf, 0x14, 0x431f,
	0x14, 0x0242, 0x14, 0x970c, 0x14, 0x131e, 0x14, 0x21bf,
	0x14, 0x4328, 0x14, 0x0242, 0x14, 0x970c, 0x14, 0x141e,
	0x14, 0x21bf, 0x14, 0x44b1, 0x14, 0x0242, 0x14, 0x970c,
	0x14, 0x161e, 0x14, 0x21e6, 0x14, 0x8242, 0x14, 0xee82,
	0x14, 0x4101, 0x14, 0xef96, 0x14, 0xfefd, 0x14, 0xfc04,
	0x14, 0xf8fa, 0x14, 0xef69, 0x14, 0xe082, 0x14, 0x46a0,
	0x14, 0x0005, 0x14, 0x0286, 0x14, 0x96ae, 0x14, 0x06a0,
	0x14, 0x0103, 0x14, 0x0219, 0x14, 0x19ef, 0x14, 0x96fe,
	0x14, 0xfc04, 0x14, 0xf8fa, 0x14, 0xef69, 0x14, 0xe082,
	0x14, 0x29f6, 0x14, 0x20e4, 0x14, 0x8229, 0x14, 0xe080,
	0x14, 0x10ac, 0x14, 0x2102, 0x14, 0xae54, 0x14, 0xe082,
	0x14, 0x27f7, 0x14, 0x20e4, 0x14, 0x8227, 0x14, 0xbf42,
	0x14, 0xe602, 0x14, 0x4297, 0x14, 0xac28, 0x14, 0x22bf,
	0x14, 0x430d, 0x14, 0x0242, 0x14, 0x97e5, 0x14, 0x8247,
	0x14, 0xac28, 0x14, 0x20d1, 0x14, 0x03bf, 0x14, 0x4307,
	0x14, 0x0242, 0x14, 0x59ee, 0x14, 0x8246, 0x14, 0x00e1,
	0x14, 0x8227, 0x14, 0xf628, 0x14, 0xe582, 0x14, 0x27ae,
	0x14, 0x21d1, 0x14, 0x04bf, 0x14, 0x4307, 0x14, 0x0242,
	0x14, 0x59ae, 0x14, 0x08d1, 0x14, 0x05bf, 0x14, 0x4307,
	0x14, 0x0242, 0x14, 0x59e0, 0x14, 0x8244, 0x14, 0xf720,
	0x14, 0xe482, 0x14, 0x4402, 0x14, 0x46ae, 0x14, 0xee82,
	0x14, 0x4601, 0x14, 0xef96, 0x14, 0xfefc, 0x14, 0x04f8,
	0x14, 0xfaef, 0x14, 0x69e0, 0x14, 0x8013, 0x14, 0xad24,
	0x14, 0x1cbf, 0x14, 0x87f0, 0x14, 0x0242, 0x14, 0x97ad,
	0x14, 0x2813, 0x14, 0xe087, 0x14, 0xfca0, 0x14, 0x0005,
	0x14, 0x0287, 0x14, 0x36ae, 0x14, 0x10a0, 0x14, 0x0105,
	0x14, 0x0287, 0x14, 0x48ae, 0x14, 0x08e0, 0x14, 0x8230,
	0x14, 0xf626, 0x14, 0xe482, 0x14, 0x30ef, 0x14, 0x96fe,
	0x14, 0xfc04, 0x14, 0xf8e0, 0x14, 0x8245, 0x14, 0xf722,
	0x14, 0xe482, 0x14, 0x4502, 0x14, 0x46ae, 0x14, 0xee87,
	0x14, 0xfc01, 0x14, 0xfc04, 0x14, 0xf8fa, 0x14, 0xef69,
	0x14, 0xfb02, 0x14, 0x46d3, 0x14, 0xad50, 0x14, 0x2fbf,
	0x14, 0x87ed, 0x14, 0xd101, 0x14, 0x0242, 0x14, 0x59bf,
	0x14, 0x87ed, 0x14, 0xd100, 0x14, 0x0242, 0x14, 0x59e0,
	0x14, 0x8245, 0x14, 0xf622, 0x14, 0xe482, 0x14, 0x4502,
	0x14, 0x46ae, 0x14, 0xd100, 0x14, 0xbf87, 0x14, 0xf002,
	0x14, 0x4259, 0x14, 0xee87, 0x14, 0xfc00, 0x14, 0xe082,
	0x14, 0x30f6, 0x14, 0x26e4, 0x14, 0x8230, 0x14, 0xffef,
	0x14, 0x96fe, 0x14, 0xfc04, 0x14, 0xf8f9, 0x14, 0xface,
	0x14, 0xfaef, 0x14, 0x69fb, 0x14, 0xbf87, 0x14, 0xb3d7,
	0x14, 0x001c, 0x14, 0xd819, 0x14, 0xd919, 0x14, 0xda19,
	0x14, 0xdb19, 0x14, 0x07ef, 0x14, 0x9502, 0x14, 0x4259,
	0x14, 0x073f, 0x14, 0x0004, 0x14, 0x9fec, 0x14, 0xffef,
	0x14, 0x96fe, 0x14, 0xc6fe, 0x14, 0xfdfc, 0x14, 0x0400,
	0x14, 0x0145, 0x14, 0x7d00, 0x14, 0x0345, 0x14, 0x5c00,
	0x14, 0x0143, 0x14, 0x4f00, 0x14, 0x0387, 0x14, 0xdb00,
	0x14, 0x0987, 0x14, 0xde00, 0x14, 0x0987, 0x14, 0xe100,
	0x14, 0x0087, 0x14, 0xeaa4, 0x14, 0x00b8, 0x14, 0x20c4,
	0x14, 0x1600, 0x14, 0x000f, 0x14, 0xf800, 0x14, 0x7098,
	0x14, 0xa58a, 0x14, 0xb6a8, 0x14, 0x3e50, 0x14, 0xa83e,
	0x14, 0x33bc, 0x14, 0xc622, 0x14, 0xbcc6, 0x14, 0xaaa4,
	0x14, 0x42ff, 0x14, 0xc408, 0x14, 0x00c4, 0x14, 0x16a8,
	0x14, 0xbcc0, 0x13, 0xb818, 0x14, 0x02f3, 0x13, 0xb81a,
	0x14, 0x17d1, 0x13, 0xb81c, 0x14, 0x185a, 0x13, 0xb81e,
	0x14, 0x3c66, 0x13, 0xb820, 0x14, 0x021f, 0x13, 0xc416,
	0x14, 0x0500, 0x13, 0xb82e, 0x14, 0xfffc, 0x1F, 0x0A43,
	0x13, 0x0000, 0x14, 0x0000, 0x1f, 0x0B82
};

static struct gani_patch gani_8101_m10_patch0[] = {
	0x1f, 0x0000, 0x1f, 0x0007, 0x1e, 0x0023, 0x16, 0x0306,
	0x16, 0x0307, 0x15, 0x000e, 0x19, 0x000a, 0x15, 0x0010,
	0x19, 0x0008, 0x15, 0x0018, 0x19, 0x4801, 0x15, 0x0019,
	0x19, 0x6801, 0x15, 0x001a, 0x19, 0x66a1, 0x15, 0x001f,
	0x19, 0x0000, 0x15, 0x0020, 0x19, 0x0000, 0x15, 0x0021,
	0x19, 0x0000, 0x15, 0x0022, 0x19, 0x0000, 0x15, 0x0023,
	0x19, 0x0000, 0x15, 0x0024, 0x19, 0x0000, 0x15, 0x0025,
	0x19, 0x64a1, 0x15, 0x0026, 0x19, 0x40ea, 0x15, 0x0027,
	0x19, 0x4503, 0x15, 0x0028, 0x19, 0x9f00, 0x15, 0x0029,
	0x19, 0xa631, 0x15, 0x002a, 0x19, 0x9717, 0x15, 0x002b,
	0x19, 0x302c, 0x15, 0x002c, 0x19, 0x4802, 0x15, 0x002d,
	0x19, 0x58da, 0x15, 0x002e, 0x19, 0x400d, 0x15, 0x002f,
	0x19, 0x4488, 0x15, 0x0030, 0x19, 0x9e00, 0x15, 0x0031,
	0x19, 0x63c8, 0x15, 0x0032, 0x19, 0x6481, 0x15, 0x0033,
	0x19, 0x0000, 0x15, 0x0034, 0x19, 0x0000, 0x15, 0x0035,
	0x19, 0x0000, 0x15, 0x0036, 0x19, 0x0000, 0x15, 0x0037,
	0x19, 0x0000, 0x15, 0x0038, 0x19, 0x0000, 0x15, 0x0039,
	0x19, 0x0000, 0x15, 0x003a, 0x19, 0x0000, 0x15, 0x003b,
	0x19, 0x63e8, 0x15, 0x003c, 0x19, 0x7d00, 0x15, 0x003d,
	0x19, 0x59d4, 0x15, 0x003e, 0x19, 0x63f8, 0x15, 0x0040,
	0x19, 0x64a1, 0x15, 0x0041, 0x19, 0x30de, 0x15, 0x0044,
	0x19, 0x480f, 0x15, 0x0045, 0x19, 0x6800, 0x15, 0x0046,
	0x19, 0x6680, 0x15, 0x0047, 0x19, 0x7c10, 0x15, 0x0048,
	0x19, 0x63c8, 0x15, 0x0049, 0x19, 0x0000, 0x15, 0x004a,
	0x19, 0x0000, 0x15, 0x004b, 0x19, 0x0000, 0x15, 0x004c,
	0x19, 0x0000, 0x15, 0x004d, 0x19, 0x0000, 0x15, 0x004e,
	0x19, 0x0000, 0x15, 0x004f, 0x19, 0x40ea, 0x15, 0x0050,
	0x19, 0x4503, 0x15, 0x0051, 0x19, 0x58ca, 0x15, 0x0052,
	0x19, 0x63c8, 0x15, 0x0053, 0x19, 0x63d8, 0x15, 0x0054,
	0x19, 0x66a0, 0x15, 0x0055, 0x19, 0x9f00, 0x15, 0x0056,
	0x19, 0x3000, 0x15, 0x006E, 0x19, 0x9afa, 0x15, 0x00a1,
	0x19, 0x3044, 0x15, 0x00ab, 0x19, 0x5820, 0x15, 0x00ac,
	0x19, 0x5e04, 0x15, 0x00ad, 0x19, 0xb60c, 0x15, 0x00af,
	0x19, 0x000a, 0x15, 0x00b2, 0x19, 0x30b9, 0x15, 0x00b9,
	0x19, 0x4408, 0x15, 0x00ba, 0x19, 0x480b, 0x15, 0x00bb,
	0x19, 0x5e00, 0x15, 0x00bc, 0x19, 0x405f, 0x15, 0x00bd,
	0x19, 0x4448, 0x15, 0x00be, 0x19, 0x4020, 0x15, 0x00bf,
	0x19, 0x4468, 0x15, 0x00c0, 0x19, 0x9c02, 0x15, 0x00c1,
	0x19, 0x58a0, 0x15, 0x00c2, 0x19, 0xb605, 0x15, 0x00c3,
	0x19, 0xc0d3, 0x15, 0x00c4, 0x19, 0x00e6, 0x15, 0x00c5,
	0x19, 0xdaec, 0x15, 0x00c6, 0x19, 0x00fa, 0x15, 0x00c7,
	0x19, 0x9df9, 0x15, 0x00c8, 0x19, 0x307a, 0x15, 0x0112,
	0x19, 0x6421, 0x15, 0x0113, 0x19, 0x7c08, 0x15, 0x0114,
	0x19, 0x63f0, 0x15, 0x0115, 0x19, 0x4003, 0x15, 0x0116,
	0x19, 0x4418, 0x15, 0x0117, 0x19, 0x9b00, 0x15, 0x0118,
	0x19, 0x6461, 0x15, 0x0119, 0x19, 0x64e1, 0x15, 0x011a,
	0x19, 0x0000, 0x15, 0x0150, 0x19, 0x6461, 0x15, 0x0151,
	0x19, 0x4003, 0x15, 0x0152, 0x19, 0x4540, 0x15, 0x0153,
	0x19, 0x9f00, 0x15, 0x0155, 0x19, 0x6421, 0x15, 0x0156,
	0x19, 0x64a1, 0x15, 0x021e, 0x19, 0x5410, 0x15, 0x0225,
	0x19, 0x5400, 0x15, 0x023D, 0x19, 0x4050, 0x15, 0x0295,
	0x19, 0x6c08, 0x15, 0x02bd, 0x19, 0xa523, 0x15, 0x02be,
	0x19, 0x32ca, 0x15, 0x02ca, 0x19, 0x48b3, 0x15, 0x02cb,
	0x19, 0x4020, 0x15, 0x02cc, 0x19, 0x4823, 0x15, 0x02cd,
	0x19, 0x4510, 0x15, 0x02ce, 0x19, 0xb63a, 0x15, 0x02cf,
	0x19, 0x7dc8, 0x15, 0x02d6, 0x19, 0x9bf8, 0x15, 0x02d8,
	0x19, 0x85f6, 0x15, 0x02d9, 0x19, 0x32e0, 0x15, 0x02e0,
	0x19, 0x4834, 0x15, 0x02e1, 0x19, 0x6c08, 0x15, 0x02e2,
	0x19, 0x4020, 0x15, 0x02e3, 0x19, 0x4824, 0x15, 0x02e4,
	0x19, 0x4520, 0x15, 0x02e5, 0x19, 0x4008, 0x15, 0x02e6,
	0x19, 0x4560, 0x15, 0x02e7, 0x19, 0x9d04, 0x15, 0x02e8,
	0x19, 0x48c4, 0x15, 0x02e9, 0x19, 0x0000, 0x15, 0x02ea,
	0x19, 0x4844, 0x15, 0x02eb, 0x19, 0x7dc8, 0x15, 0x02f0,
	0x19, 0x9cf7, 0x15, 0x02f1, 0x19, 0xdf94, 0x15, 0x02f2,
	0x19, 0x0002, 0x15, 0x02f3, 0x19, 0x6810, 0x15, 0x02f4,
	0x19, 0xb614, 0x15, 0x02f5, 0x19, 0xc42b, 0x15, 0x02f6,
	0x19, 0x00d4, 0x15, 0x02f7, 0x19, 0xc455, 0x15, 0x02f8,
	0x19, 0x0093, 0x15, 0x02f9, 0x19, 0x92ee, 0x15, 0x02fa,
	0x19, 0xefed, 0x15, 0x02fb, 0x19, 0x3312, 0x15, 0x0312,
	0x19, 0x49b5, 0x15, 0x0313, 0x19, 0x7d00, 0x15, 0x0314,
	0x19, 0x4d00, 0x15, 0x0315, 0x19, 0x6810, 0x15, 0x031e,
	0x19, 0x404f, 0x15, 0x031f, 0x19, 0x44c8, 0x15, 0x0320,
	0x19, 0xd64f, 0x15, 0x0321, 0x19, 0x00e7, 0x15, 0x0322,
	0x19, 0x7c08, 0x15, 0x0323, 0x19, 0x8203, 0x15, 0x0324,
	0x19, 0x4d48, 0x15, 0x0325, 0x19, 0x3327, 0x15, 0x0326,
	0x19, 0x4d40, 0x15, 0x0327, 0x19, 0xc8d7, 0x15, 0x0328,
	0x19, 0x0003, 0x15, 0x0329, 0x19, 0x7c20, 0x15, 0x032a,
	0x19, 0x4c20, 0x15, 0x032b, 0x19, 0xc8ed, 0x15, 0x032c,
	0x19, 0x00f4, 0x15, 0x032d, 0x19, 0x82b3, 0x15, 0x032e,
	0x19, 0xd11d, 0x15, 0x032f, 0x19, 0x00b1, 0x15, 0x0330,
	0x19, 0xde18, 0x15, 0x0331, 0x19, 0x0008, 0x15, 0x0332,
	0x19, 0x91ee, 0x15, 0x0333, 0x19, 0x3339, 0x15, 0x033a,
	0x19, 0x4064, 0x15, 0x0340, 0x19, 0x9e06, 0x15, 0x0341,
	0x19, 0x7c08, 0x15, 0x0342, 0x19, 0x8203, 0x15, 0x0343,
	0x19, 0x4d48, 0x15, 0x0344, 0x19, 0x3346, 0x15, 0x0345,
	0x19, 0x4d40, 0x15, 0x0346, 0x19, 0xd11d, 0x15, 0x0347,
	0x19, 0x0099, 0x15, 0x0348, 0x19, 0xbb17, 0x15, 0x0349,
	0x19, 0x8102, 0x15, 0x034a, 0x19, 0x334d, 0x15, 0x034b,
	0x19, 0xa22c, 0x15, 0x034c, 0x19, 0x3397, 0x15, 0x034d,
	0x19, 0x91f2, 0x15, 0x034e, 0x19, 0xc218, 0x15, 0x034f,
	0x19, 0x00f0, 0x15, 0x0350, 0x19, 0x3397, 0x15, 0x0351,
	0x19, 0x0000, 0x15, 0x0364, 0x19, 0xbc05, 0x15, 0x0367,
	0x19, 0xa1fc, 0x15, 0x0368, 0x19, 0x3377, 0x15, 0x0369,
	0x19, 0x328b, 0x15, 0x036a, 0x19, 0x0000, 0x15, 0x0377,
	0x19, 0x4b97, 0x15, 0x0378, 0x19, 0x6818, 0x15, 0x0379,
	0x19, 0x4b07, 0x15, 0x037a, 0x19, 0x40ac, 0x15, 0x037b,
	0x19, 0x4445, 0x15, 0x037c, 0x19, 0x404e, 0x15, 0x037d,
	0x19, 0x4461, 0x15, 0x037e, 0x19, 0x9c09, 0x15, 0x037f,
	0x19, 0x63da, 0x15, 0x0380, 0x19, 0x5440, 0x15, 0x0381,
	0x19, 0x4b98, 0x15, 0x0382, 0x19, 0x7c60, 0x15, 0x0383,
	0x19, 0x4c00, 0x15, 0x0384, 0x19, 0x4b08, 0x15, 0x0385,
	0x19, 0x63d8, 0x15, 0x0386, 0x19, 0x338d, 0x15, 0x0387,
	0x19, 0xd64f, 0x15, 0x0388, 0x19, 0x0080, 0x15, 0x0389,
	0x19, 0x820c, 0x15, 0x038a, 0x19, 0xa10b, 0x15, 0x038b,
	0x19, 0x9df3, 0x15, 0x038c, 0x19, 0x3395, 0x15, 0x038d,
	0x19, 0xd64f, 0x15, 0x038e, 0x19, 0x00f9, 0x15, 0x038f,
	0x19, 0xc017, 0x15, 0x0390, 0x19, 0x0005, 0x15, 0x0391,
	0x19, 0x6c0b, 0x15, 0x0392, 0x19, 0xa103, 0x15, 0x0393,
	0x19, 0x6c08, 0x15, 0x0394, 0x19, 0x9df9, 0x15, 0x0395,
	0x19, 0x6c08, 0x15, 0x0396, 0x19, 0x3397, 0x15, 0x0399,
	0x19, 0x6810, 0x15, 0x03a4, 0x19, 0x7c08, 0x15, 0x03a5,
	0x19, 0x8203, 0x15, 0x03a6, 0x19, 0x4d08, 0x15, 0x03a7,
	0x19, 0x33a9, 0x15, 0x03a8, 0x19, 0x4d00, 0x15, 0x03a9,
	0x19, 0x9bfa, 0x15, 0x03aa, 0x19, 0x33b6, 0x15, 0x03bb,
	0x19, 0x4056, 0x15, 0x03bc, 0x19, 0x44e9, 0x15, 0x03bd,
	0x19, 0x4054, 0x15, 0x03be, 0x19, 0x44f8, 0x15, 0x03bf,
	0x19, 0xd64f, 0x15, 0x03c0, 0x19, 0x0037, 0x15, 0x03c1,
	0x19, 0xbd37, 0x15, 0x03c2, 0x19, 0x9cfd, 0x15, 0x03c3,
	0x19, 0xc639, 0x15, 0x03c4, 0x19, 0x0011, 0x15, 0x03c5,
	0x19, 0x9b03, 0x15, 0x03c6, 0x19, 0x7c01, 0x15, 0x03c7,
	0x19, 0x4c01, 0x15, 0x03c8, 0x19, 0x9e03, 0x15, 0x03c9,
	0x19, 0x7c20, 0x15, 0x03ca, 0x19, 0x4c20, 0x15, 0x03cb,
	0x19, 0x9af4, 0x15, 0x03cc, 0x19, 0x7c12, 0x15, 0x03cd,
	0x19, 0x4c52, 0x15, 0x03ce, 0x19, 0x4470, 0x15, 0x03cf,
	0x19, 0x7c12, 0x15, 0x03d0, 0x19, 0x4c40, 0x15, 0x03d1,
	0x19, 0x33bf, 0x15, 0x03d6, 0x19, 0x4047, 0x15, 0x03d7,
	0x19, 0x4469, 0x15, 0x03d8, 0x19, 0x492b, 0x15, 0x03d9,
	0x19, 0x4479, 0x15, 0x03da, 0x19, 0x7c09, 0x15, 0x03db,
	0x19, 0x8203, 0x15, 0x03dc, 0x19, 0x4d48, 0x15, 0x03dd,
	0x19, 0x33df, 0x15, 0x03de, 0x19, 0x4d40, 0x15, 0x03df,
	0x19, 0xd64f, 0x15, 0x03e0, 0x19, 0x0017, 0x15, 0x03e1,
	0x19, 0xbd17, 0x15, 0x03e2, 0x19, 0x9b03, 0x15, 0x03e3,
	0x19, 0x7c20, 0x15, 0x03e4, 0x19, 0x4c20, 0x15, 0x03e5,
	0x19, 0x88f5, 0x15, 0x03e6, 0x19, 0xc428, 0x15, 0x03e7,
	0x19, 0x0008, 0x15, 0x03e8, 0x19, 0x9af2, 0x15, 0x03e9,
	0x19, 0x7c12, 0x15, 0x03ea, 0x19, 0x4c52, 0x15, 0x03eb,
	0x19, 0x4470, 0x15, 0x03ec, 0x19, 0x7c12, 0x15, 0x03ed,
	0x19, 0x4c40, 0x15, 0x03ee, 0x19, 0x33da, 0x15, 0x03ef,
	0x19, 0x3312, 0x16, 0x0306, 0x16, 0x0300, 0x1f, 0x0000,
	0x17, 0x2179, 0x1f, 0x0007, 0x1e, 0x0040, 0x18, 0x0645,
	0x19, 0xe200, 0x18, 0x0655, 0x19, 0x9000, 0x18, 0x0d05,
	0x19, 0xbe00, 0x18, 0x0d15, 0x19, 0xd300, 0x18, 0x0d25,
	0x19, 0xfe00, 0x18, 0x0d35, 0x19, 0x4000, 0x18, 0x0d45,
	0x19, 0x7f00, 0x18, 0x0d55, 0x19, 0x1000, 0x18, 0x0d65,
	0x19, 0x0000, 0x18, 0x0d75, 0x19, 0x8200, 0x18, 0x0d85,
	0x19, 0x0000, 0x18, 0x0d95, 0x19, 0x7000, 0x18, 0x0da5,
	0x19, 0x0f00, 0x18, 0x0db5, 0x19, 0x0100, 0x18, 0x0dc5,
	0x19, 0x9b00, 0x18, 0x0dd5, 0x19, 0x7f00, 0x18, 0x0de5,
	0x19, 0xe000, 0x18, 0x0df5, 0x19, 0xef00, 0x18, 0x16d5,
	0x19, 0xe200, 0x18, 0x16e5, 0x19, 0xab00, 0x18, 0x2904,
	0x19, 0x4000, 0x18, 0x2914, 0x19, 0x7f00, 0x18, 0x2924,
	0x19, 0x0100, 0x18, 0x2934, 0x19, 0x2000, 0x18, 0x2944,
	0x19, 0x0000, 0x18, 0x2954, 0x19, 0x4600, 0x18, 0x2964,
	0x19, 0xfc00, 0x18, 0x2974, 0x19, 0x0000, 0x18, 0x2984,
	0x19, 0x5000, 0x18, 0x2994, 0x19, 0x9d00, 0x18, 0x29a4,
	0x19, 0xff00, 0x18, 0x29b4, 0x19, 0x4000, 0x18, 0x29c4,
	0x19, 0x7f00, 0x18, 0x29d4, 0x19, 0x0000, 0x18, 0x29e4,
	0x19, 0x2000, 0x18, 0x29f4, 0x19, 0x0000, 0x18, 0x2a04,
	0x19, 0xe600, 0x18, 0x2a14, 0x19, 0xff00, 0x18, 0x2a24,
	0x19, 0x0000, 0x18, 0x2a34, 0x19, 0x5000, 0x18, 0x2a44,
	0x19, 0x8500, 0x18, 0x2a54, 0x19, 0x7f00, 0x18, 0x2a64,
	0x19, 0xac00, 0x18, 0x2a74, 0x19, 0x0800, 0x18, 0x2a84,
	0x19, 0xfc00, 0x18, 0x2a94, 0x19, 0xe000, 0x18, 0x2aa4,
	0x19, 0x7400, 0x18, 0x2ab4, 0x19, 0x4000, 0x18, 0x2ac4,
	0x19, 0x7f00, 0x18, 0x2ad4, 0x19, 0x0100, 0x18, 0x2ae4,
	0x19, 0xff00, 0x18, 0x2af4, 0x19, 0x0000, 0x18, 0x2b04,
	0x19, 0x4400, 0x18, 0x2b14, 0x19, 0xfc00, 0x18, 0x2b24,
	0x19, 0x0000, 0x18, 0x2b34, 0x19, 0x4000, 0x18, 0x2b44,
	0x19, 0x9d00, 0x18, 0x2b54, 0x19, 0xff00, 0x18, 0x2b64,
	0x19, 0x4000, 0x18, 0x2b74, 0x19, 0x7f00, 0x18, 0x2b84,
	0x19, 0x0000, 0x18, 0x2b94, 0x19, 0xff00, 0x18, 0x2ba4,
	0x19, 0x0000, 0x18, 0x2bb4, 0x19, 0xfc00, 0x18, 0x2bc4,
	0x19, 0xff00, 0x18, 0x2bd4, 0x19, 0x0000, 0x18, 0x2be4,
	0x19, 0x4000, 0x18, 0x2bf4, 0x19, 0x8900, 0x18, 0x2c04,
	0x19, 0x8300, 0x18, 0x2c14, 0x19, 0xe000, 0x18, 0x2c24,
	0x19, 0x0000, 0x18, 0x2c34, 0x19, 0xac00, 0x18, 0x2c44,
	0x19, 0x0800, 0x18, 0x2c54, 0x19, 0xfa00, 0x18, 0x2c64,
	0x19, 0xe100, 0x18, 0x2c74, 0x19, 0x7f00, 0x18, 0x0001,
	0x1f, 0x0000, 0x17, 0x2100, 0x1f, 0x0005, 0x05, 0xfff6,
	0x06, 0x0080, 0x05, 0x8000, 0x06, 0xd480, 0x06, 0xc1e4,
	0x06, 0x8b9a, 0x06, 0xe58b, 0x06, 0x9bee, 0x06, 0x8b83,
	0x06, 0x41bf, 0x06, 0x8b88, 0x06, 0xec00, 0x06, 0x19a9,
	0x06, 0x8b90, 0x06, 0xf9ee, 0x06, 0xfff6, 0x06, 0x00ee,
	0x06, 0xfff7, 0x06, 0xffe0, 0x06, 0xe140, 0x06, 0xe1e1,
	0x06, 0x41f7, 0x06, 0x2ff6, 0x06, 0x28e4, 0x06, 0xe140,
	0x06, 0xe5e1, 0x06, 0x41f7, 0x06, 0x0002, 0x06, 0x020c,
	0x06, 0x0202, 0x06, 0x1d02, 0x06, 0x0230, 0x06, 0x0202,
	0x06, 0x4002, 0x06, 0x028b, 0x06, 0x0280, 0x06, 0x6c02,
	0x06, 0x8085, 0x06, 0xe08b, 0x06, 0x88e1, 0x06, 0x8b89,
	0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8a1e, 0x06, 0x01e1,
	0x06, 0x8b8b, 0x06, 0x1e01, 0x06, 0xe18b, 0x06, 0x8c1e,
	0x06, 0x01e1, 0x06, 0x8b8d, 0x06, 0x1e01, 0x06, 0xe18b,
	0x06, 0x8e1e, 0x06, 0x01a0, 0x06, 0x00c7, 0x06, 0xaec3,
	0x06, 0xf8e0, 0x06, 0x8b8d, 0x06, 0xad20, 0x06, 0x10ee,
	0x06, 0x8b8d, 0x06, 0x0002, 0x06, 0x1310, 0x06, 0x0280,
	0x06, 0xc602, 0x06, 0x1f0c, 0x06, 0x0227, 0x06, 0x49fc,
	0x06, 0x04f8, 0x06, 0xe08b, 0x06, 0x8ead, 0x06, 0x200b,
	0x06, 0xf620, 0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x852d,
	0x06, 0x021b, 0x06, 0x67ad, 0x06, 0x2211, 0x06, 0xf622,
	0x06, 0xe48b, 0x06, 0x8e02, 0x06, 0x2ba5, 0x06, 0x022a,
	0x06, 0x2402, 0x06, 0x82e5, 0x06, 0x022a, 0x06, 0xf0ad,
	0x06, 0x2511, 0x06, 0xf625, 0x06, 0xe48b, 0x06, 0x8e02,
	0x06, 0x8445, 0x06, 0x0204, 0x06, 0x0302, 0x06, 0x19cc,
	0x06, 0x022b, 0x06, 0x5bfc, 0x06, 0x04ee, 0x06, 0x8b8d,
	0x06, 0x0105, 0x06, 0xf8f9, 0x06, 0xfae0, 0x06, 0x8b81,
	0x06, 0xac26, 0x06, 0x08e0, 0x06, 0x8b81, 0x06, 0xac21,
	0x06, 0x02ae, 0x06, 0x6bee, 0x06, 0xe0ea, 0x06, 0x00ee,
	0x06, 0xe0eb, 0x06, 0x00e2, 0x06, 0xe07c, 0x06, 0xe3e0,
	0x06, 0x7da5, 0x06, 0x1111, 0x06, 0x15d2, 0x06, 0x60d6,
	0x06, 0x6666, 0x06, 0x0207, 0x06, 0x6cd2, 0x06, 0xa0d6,
	0x06, 0xaaaa, 0x06, 0x0207, 0x06, 0x6c02, 0x06, 0x201d,
	0x06, 0xae44, 0x06, 0xa566, 0x06, 0x6602, 0x06, 0xae38,
	0x06, 0xa5aa, 0x06, 0xaa02, 0x06, 0xae32, 0x06, 0xeee0,
	0x06, 0xea04, 0x06, 0xeee0, 0x06, 0xeb06, 0x06, 0xe2e0,
	0x06, 0x7ce3, 0x06, 0xe07d, 0x06, 0xe0e0, 0x06, 0x38e1,
	0x06, 0xe039, 0x06, 0xad2e, 0x06, 0x21ad, 0x06, 0x3f13,
	0x06, 0xe0e4, 0x06, 0x14e1, 0x06, 0xe415, 0x06, 0x6880,
	0x06, 0xe4e4, 0x06, 0x14e5, 0x06, 0xe415, 0x06, 0x0220,
	0x06, 0x1dae, 0x06, 0x0bac, 0x06, 0x3e02, 0x06, 0xae06,
	0x06, 0x0281, 0x06, 0x4602, 0x06, 0x2057, 0x06, 0xfefd,
	0x06, 0xfc04, 0x06, 0xf8e0, 0x06, 0x8b81, 0x06, 0xad26,
	0x06, 0x0302, 0x06, 0x20a7, 0x06, 0xe08b, 0x06, 0x81ad,
	0x06, 0x2109, 0x06, 0xe08b, 0x06, 0x2eac, 0x06, 0x2003,
	0x06, 0x0281, 0x06, 0x61fc, 0x06, 0x04f8, 0x06, 0xe08b,
	0x06, 0x81ac, 0x06, 0x2505, 0x06, 0x0222, 0x06, 0xaeae,
	0x06, 0x0302, 0x06, 0x8172, 0x06, 0xfc04, 0x06, 0xf8f9,
	0x06, 0xfaef, 0x06, 0x69fa, 0x06, 0xe086, 0x06, 0x20a0,
	0x06, 0x8016, 0x06, 0xe086, 0x06, 0x21e1, 0x06, 0x8b33,
	0x06, 0x1b10, 0x06, 0x9e06, 0x06, 0x0223, 0x06, 0x91af,
	0x06, 0x8252, 0x06, 0xee86, 0x06, 0x2081, 0x06, 0xaee4,
	0x06, 0xa081, 0x06, 0x1402, 0x06, 0x2399, 0x06, 0xbf25,
	0x06, 0xcc02, 0x06, 0x2d21, 0x06, 0xee86, 0x06, 0x2100,
	0x06, 0xee86, 0x06, 0x2082, 0x06, 0xaf82, 0x06, 0x52a0,
	0x06, 0x8232, 0x06, 0xe086, 0x06, 0x21e1, 0x06, 0x8b32,
	0x06, 0x1b10, 0x06, 0x9e06, 0x06, 0x0223, 0x06, 0x91af,
	0x06, 0x8252, 0x06, 0xee86, 0x06, 0x2100, 0x06, 0xd000,
	0x06, 0x0282, 0x06, 0x5910, 0x06, 0xa004, 0x06, 0xf9e0,
	0x06, 0x861f, 0x06, 0xa000, 0x06, 0x07ee, 0x06, 0x8620,
	0x06, 0x83af, 0x06, 0x8178, 0x06, 0x0224, 0x06, 0x0102,
	0x06, 0x2399, 0x06, 0xae72, 0x06, 0xa083, 0x06, 0x4b1f,
	0x06, 0x55d0, 0x06, 0x04bf, 0x06, 0x8615, 0x06, 0x1a90,
	0x06, 0x0c54, 0x06, 0xd91e, 0x06, 0x31b0, 0x06, 0xf4e0,
	0x06, 0xe022, 0x06, 0xe1e0, 0x06, 0x23ad, 0x06, 0x2e0c,
	0x06, 0xef02, 0x06, 0xef12, 0x06, 0x0e44, 0x06, 0xef23,
	0x06, 0x0e54, 0x06, 0xef21, 0x06, 0xe6e4, 0x06, 0x2ae7,
	0x06, 0xe42b, 0x06, 0xe2e4, 0x06, 0x28e3, 0x06, 0xe429,
	0x06, 0x6d20, 0x06, 0x00e6, 0x06, 0xe428, 0x06, 0xe7e4,
	0x06, 0x29bf, 0x06, 0x25ca, 0x06, 0x022d, 0x06, 0x21ee,
	0x06, 0x8620, 0x06, 0x84ee, 0x06, 0x8621, 0x06, 0x00af,
	0x06, 0x8178, 0x06, 0xa084, 0x06, 0x19e0, 0x06, 0x8621,
	0x06, 0xe18b, 0x06, 0x341b, 0x06, 0x109e, 0x06, 0x0602,
	0x06, 0x2391, 0x06, 0xaf82, 0x06, 0x5202, 0x06, 0x241f,
	0x06, 0xee86, 0x06, 0x2085, 0x06, 0xae08, 0x06, 0xa085,
	0x06, 0x02ae, 0x06, 0x0302, 0x06, 0x2442, 0x06, 0xfeef,
	0x06, 0x96fe, 0x06, 0xfdfc, 0x06, 0x04f8, 0x06, 0xf9fa,
	0x06, 0xef69, 0x06, 0xfad1, 0x06, 0x801f, 0x06, 0x66e2,
	0x06, 0xe0ea, 0x06, 0xe3e0, 0x06, 0xeb5a, 0x06, 0xf81e,
	0x06, 0x20e6, 0x06, 0xe0ea, 0x06, 0xe5e0, 0x06, 0xebd3,
	0x06, 0x05b3, 0x06, 0xfee2, 0x06, 0xe07c, 0x06, 0xe3e0,
	0x06, 0x7dad, 0x06, 0x3703, 0x06, 0x7dff, 0x06, 0xff0d,
	0x06, 0x581c, 0x06, 0x55f8, 0x06, 0xef46, 0x06, 0x0282,
	0x06, 0xc7ef, 0x06, 0x65ef, 0x06, 0x54fc, 0x06, 0xac30,
	0x06, 0x2b11, 0x06, 0xa188, 0x06, 0xcabf, 0x06, 0x860e,
	0x06, 0xef10, 0x06, 0x0c11, 0x06, 0x1a91, 0x06, 0xda19,
	0x06, 0xdbf8, 0x06, 0xef46, 0x06, 0x021e, 0x06, 0x17ef,
	0x06, 0x54fc, 0x06, 0xad30, 0x06, 0x0fef, 0x06, 0x5689,
	0x06, 0xde19, 0x06, 0xdfe2, 0x06, 0x861f, 0x06, 0xbf86,
	0x06, 0x161a, 0x06, 0x90de, 0x06, 0xfeef, 0x06, 0x96fe,
	0x06, 0xfdfc, 0x06, 0x04ac, 0x06, 0x2707, 0x06, 0xac37,
	0x06, 0x071a, 0x06, 0x54ae, 0x06, 0x11ac, 0x06, 0x3707,
	0x06, 0xae00, 0x06, 0x1a54, 0x06, 0xac37, 0x06, 0x07d0,
	0x06, 0x01d5, 0x06, 0xffff, 0x06, 0xae02, 0x06, 0xd000,
	0x06, 0x04f8, 0x06, 0xe08b, 0x06, 0x83ad, 0x06, 0x2444,
	0x06, 0xe0e0, 0x06, 0x22e1, 0x06, 0xe023, 0x06, 0xad22,
	0x06, 0x3be0, 0x06, 0x8abe, 0x06, 0xa000, 0x06, 0x0502,
	0x06, 0x28de, 0x06, 0xae42, 0x06, 0xa001, 0x06, 0x0502,
	0x06, 0x28f1, 0x06, 0xae3a, 0x06, 0xa002, 0x06, 0x0502,
	0x06, 0x8344, 0x06, 0xae32, 0x06, 0xa003, 0x06, 0x0502,
	0x06, 0x299a, 0x06, 0xae2a, 0x06, 0xa004, 0x06, 0x0502,
	0x06, 0x29ae, 0x06, 0xae22, 0x06, 0xa005, 0x06, 0x0502,
	0x06, 0x29d7, 0x06, 0xae1a, 0x06, 0xa006, 0x06, 0x0502,
	0x06, 0x29fe, 0x06, 0xae12, 0x06, 0xee8a, 0x06, 0xc000,
	0x06, 0xee8a, 0x06, 0xc100, 0x06, 0xee8a, 0x06, 0xc600,
	0x06, 0xee8a, 0x06, 0xbe00, 0x06, 0xae00, 0x06, 0xfc04,
	0x06, 0xf802, 0x06, 0x2a67, 0x06, 0xe0e0, 0x06, 0x22e1,
	0x06, 0xe023, 0x06, 0x0d06, 0x06, 0x5803, 0x06, 0xa002,
	0x06, 0x02ae, 0x06, 0x2da0, 0x06, 0x0102, 0x06, 0xae2d,
	0x06, 0xa000, 0x06, 0x4de0, 0x06, 0xe200, 0x06, 0xe1e2,
	0x06, 0x01ad, 0x06, 0x2444, 0x06, 0xe08a, 0x06, 0xc2e4,
	0x06, 0x8ac4, 0x06, 0xe08a, 0x06, 0xc3e4, 0x06, 0x8ac5,
	0x06, 0xee8a, 0x06, 0xbe03, 0x06, 0xe08b, 0x06, 0x83ad,
	0x06, 0x253a, 0x06, 0xee8a, 0x06, 0xbe05, 0x06, 0xae34,
	0x06, 0xe08a, 0x06, 0xceae, 0x06, 0x03e0, 0x06, 0x8acf,
	0x06, 0xe18a, 0x06, 0xc249, 0x06, 0x05e5, 0x06, 0x8ac4,
	0x06, 0xe18a, 0x06, 0xc349, 0x06, 0x05e5, 0x06, 0x8ac5,
	0x06, 0xee8a, 0x06, 0xbe05, 0x06, 0x022a, 0x06, 0xb6ac,
	0x06, 0x2012, 0x06, 0x0283, 0x06, 0xbaac, 0x06, 0x200c,
	0x06, 0xee8a, 0x06, 0xc100, 0x06, 0xee8a, 0x06, 0xc600,
	0x06, 0xee8a, 0x06, 0xbe02, 0x06, 0xfc04, 0x06, 0xd000,
	0x06, 0x0283, 0x06, 0xcc59, 0x06, 0x0f39, 0x06, 0x02aa,
	0x06, 0x04d0, 0x06, 0x01ae, 0x06, 0x02d0, 0x06, 0x0004,
	0x06, 0xf9fa, 0x06, 0xe2e2, 0x06, 0xd2e3, 0x06, 0xe2d3,
	0x06, 0xf95a, 0x06, 0xf7e6, 0x06, 0xe2d2, 0x06, 0xe7e2,
	0x06, 0xd3e2, 0x06, 0xe02c, 0x06, 0xe3e0, 0x06, 0x2df9,
	0x06, 0x5be0, 0x06, 0x1e30, 0x06, 0xe6e0, 0x06, 0x2ce7,
	0x06, 0xe02d, 0x06, 0xe2e2, 0x06, 0xcce3, 0x06, 0xe2cd,
	0x06, 0xf95a, 0x06, 0x0f6a, 0x06, 0x50e6, 0x06, 0xe2cc,
	0x06, 0xe7e2, 0x06, 0xcde0, 0x06, 0xe03c, 0x06, 0xe1e0,
	0x06, 0x3def, 0x06, 0x64fd, 0x06, 0xe0e2, 0x06, 0xcce1,
	0x06, 0xe2cd, 0x06, 0x580f, 0x06, 0x5af0, 0x06, 0x1e02,
	0x06, 0xe4e2, 0x06, 0xcce5, 0x06, 0xe2cd, 0x06, 0xfde0,
	0x06, 0xe02c, 0x06, 0xe1e0, 0x06, 0x2d59, 0x06, 0xe05b,
	0x06, 0x1f1e, 0x06, 0x13e4, 0x06, 0xe02c, 0x06, 0xe5e0,
	0x06, 0x2dfd, 0x06, 0xe0e2, 0x06, 0xd2e1, 0x06, 0xe2d3,
	0x06, 0x58f7, 0x06, 0x5a08, 0x06, 0x1e02, 0x06, 0xe4e2,
	0x06, 0xd2e5, 0x06, 0xe2d3, 0x06, 0xef46, 0x06, 0xfefd,
	0x06, 0x04f8, 0x06, 0xf9fa, 0x06, 0xef69, 0x06, 0xe0e0,
	0x06, 0x22e1, 0x06, 0xe023, 0x06, 0x58c4, 0x06, 0xe18b,
	0x06, 0x6e1f, 0x06, 0x109e, 0x06, 0x58e4, 0x06, 0x8b6e,
	0x06, 0xad22, 0x06, 0x22ac, 0x06, 0x2755, 0x06, 0xac26,
	0x06, 0x02ae, 0x06, 0x1ad1, 0x06, 0x06bf, 0x06, 0x3bba,
	0x06, 0x022d, 0x06, 0xc1d1, 0x06, 0x07bf, 0x06, 0x3bbd,
	0x06, 0x022d, 0x06, 0xc1d1, 0x06, 0x07bf, 0x06, 0x3bc0,
	0x06, 0x022d, 0x06, 0xc1ae, 0x06, 0x30d1, 0x06, 0x03bf,
	0x06, 0x3bc3, 0x06, 0x022d, 0x06, 0xc1d1, 0x06, 0x00bf,
	0x06, 0x3bc6, 0x06, 0x022d, 0x06, 0xc1d1, 0x06, 0x00bf,
	0x06, 0x84e9, 0x06, 0x022d, 0x06, 0xc1d1, 0x06, 0x0fbf,
	0x06, 0x3bba, 0x06, 0x022d, 0x06, 0xc1d1, 0x06, 0x01bf,
	0x06, 0x3bbd, 0x06, 0x022d, 0x06, 0xc1d1, 0x06, 0x01bf,
	0x06, 0x3bc0, 0x06, 0x022d, 0x06, 0xc1ef, 0x06, 0x96fe,
	0x06, 0xfdfc, 0x06, 0x04d1, 0x06, 0x00bf, 0x06, 0x3bc3,
	0x06, 0x022d, 0x06, 0xc1d0, 0x06, 0x1102, 0x06, 0x2bfb,
	0x06, 0x5903, 0x06, 0xef01, 0x06, 0xd100, 0x06, 0xa000,
	0x06, 0x02d1, 0x06, 0x01bf, 0x06, 0x3bc6, 0x06, 0x022d,
	0x06, 0xc1d1, 0x06, 0x11ad, 0x06, 0x2002, 0x06, 0x0c11,
	0x06, 0xad21, 0x06, 0x020c, 0x06, 0x12bf, 0x06, 0x84e9,
	0x06, 0x022d, 0x06, 0xc1ae, 0x06, 0xc870, 0x06, 0xe426,
	0x06, 0x0284, 0x06, 0xf005, 0x06, 0xf8fa, 0x06, 0xef69,
	0x06, 0xe0e2, 0x06, 0xfee1, 0x06, 0xe2ff, 0x06, 0xad2d,
	0x06, 0x1ae0, 0x06, 0xe14e, 0x06, 0xe1e1, 0x06, 0x4fac,
	0x06, 0x2d22, 0x06, 0xf603, 0x06, 0x0203, 0x06, 0x3bf7,
	0x06, 0x03f7, 0x06, 0x06bf, 0x06, 0x8561, 0x06, 0x022d,
	0x06, 0x21ae, 0x06, 0x11e0, 0x06, 0xe14e, 0x06, 0xe1e1,
	0x06, 0x4fad, 0x06, 0x2d08, 0x06, 0xbf85, 0x06, 0x6c02,
	0x06, 0x2d21, 0x06, 0xf606, 0x06, 0xef96, 0x06, 0xfefc,
	0x06, 0x04f8, 0x06, 0xfaef, 0x06, 0x69e0, 0x06, 0xe000,
	0x06, 0xe1e0, 0x06, 0x01ad, 0x06, 0x271f, 0x06, 0xd101,
	0x06, 0xbf85, 0x06, 0x5e02, 0x06, 0x2dc1, 0x06, 0xe0e0,
	0x06, 0x20e1, 0x06, 0xe021, 0x06, 0xad20, 0x06, 0x0ed1,
	0x06, 0x00bf, 0x06, 0x855e, 0x06, 0x022d, 0x06, 0xc1bf,
	0x06, 0x3b96, 0x06, 0x022d, 0x06, 0x21ef, 0x06, 0x96fe,
	0x06, 0xfc04, 0x06, 0x00e2, 0x06, 0x34a7, 0x06, 0x25e5,
	0x06, 0x0a1d, 0x06, 0xe50a, 0x06, 0x2ce5, 0x06, 0x0a6d,
	0x06, 0xe50a, 0x06, 0x1de5, 0x06, 0x0a1c, 0x06, 0xe50a,
	0x06, 0x2da7, 0x06, 0x5500, 0x05, 0x8b94, 0x06, 0x84ec,
};

static struct gani_patch gani_8101_m10_patch1[] = {
	0x1f, 0x0007, 0x1e, 0x0023, 0x17, 0x0116, 0x1f, 0x0007,
	0x1e, 0x0028, 0x15, 0x0010, 0x1f, 0x0007, 0x1e, 0x0020,
	0x15, 0x0100, 0x1f, 0x0007, 0x1e, 0x0041, 0x15, 0x0802,
	0x16, 0x2185, 0x1f, 0x0000, 0x1F, 0x0007,

	0x1E, 0x0023, 0x17, 0x0116, 0x1F, 0x0000,

	0x1f, 0x0005, 0x05, 0x8b80, 0x06, 0xc896, 0x1f, 0x0000,
	0x1F, 0x0001, 0x0B, 0x8C60, 0x07, 0x2872,

	0x1C, 0xEFFF, 0x1F, 0x0003, 0x14, 0x94B0, 0x1F, 0x0000,

	0x1F, 0x0002,
};

static struct gani_patch gani_8101_m10_patch2[] = {
	0x1F, 0x0002, 0x00, 0x080B, 0x0B, 0x09D7, 0x1f, 0x0000,
	0x15, 0x1006,

	0x1F, 0x0003, 0x19, 0x7F46, 0x1F, 0x0005, 0x05, 0x8AD2,
	0x06, 0x6810, 0x05, 0x8AD4, 0x06, 0x8002, 0x05, 0x8ADE,
	0x06, 0x8025, 0x1F, 0x0000,
};

static struct gani_patch gani_8101_m11_patch[] = {
	0x1f, 0x0004, 0x1f, 0x0004, 0x19, 0x7070, 0x1c, 0x0600,
	0x1d, 0x9700, 0x1d, 0x7d00, 0x1d, 0x6900, 0x1d, 0x7d00,
	0x1d, 0x6800, 0x1d, 0x4899, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x8000, 0x1d, 0x7fe0,
	0x1d, 0x4c00, 0x1d, 0x4007, 0x1d, 0x4400, 0x1d, 0x4800,
	0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x5310, 0x1d, 0x6000,
	0x1d, 0x6800, 0x1d, 0x6736, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0x571f, 0x1d, 0x5ffb, 0x1d, 0xaa03, 0x1d, 0x5b58,
	0x1d, 0x301e, 0x1d, 0x5b64, 0x1d, 0xa6fc, 0x1d, 0xdcdb,
	0x1d, 0x0014, 0x1d, 0xd9a9, 0x1d, 0x0013, 0x1d, 0xd16b,
	0x1d, 0x0011, 0x1d, 0xb40e, 0x1d, 0xd06b, 0x1d, 0x000c,
	0x1d, 0xb206, 0x1d, 0x7c01, 0x1d, 0x5800, 0x1d, 0x7c04,
	0x1d, 0x5c00, 0x1d, 0x301a, 0x1d, 0x7c01, 0x1d, 0x5801,
	0x1d, 0x7c04, 0x1d, 0x5c04, 0x1d, 0x301e, 0x1d, 0x314d,
	0x1d, 0x31f0, 0x1d, 0x7fe0, 0x1d, 0x4c20, 0x1d, 0x6004,
	0x1d, 0x5310, 0x1d, 0x4833, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c08, 0x1d, 0x8300, 0x1d, 0x6800,
	0x1d, 0x6600, 0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0xb90c,
	0x1d, 0x30d3, 0x1d, 0x7fe0, 0x1d, 0x4de0, 0x1d, 0x7c04,
	0x1d, 0x6000, 0x1d, 0x6800, 0x1d, 0x6736, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x5310, 0x1d, 0x300b, 0x1d, 0x7fe0,
	0x1d, 0x4c60, 0x1d, 0x6803, 0x1d, 0x6520, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0xaf03, 0x1d, 0x6015, 0x1d, 0x3059,
	0x1d, 0x6017, 0x1d, 0x57e0, 0x1d, 0x580c, 0x1d, 0x588c,
	0x1d, 0x7ffc, 0x1d, 0x5fa3, 0x1d, 0x4827, 0x1d, 0x7c1f,
	0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c10, 0x1d, 0x8400,
	0x1d, 0x7c30, 0x1d, 0x6020, 0x1d, 0x48bf, 0x1d, 0x7c1f,
	0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0xad09,
	0x1d, 0x7c03, 0x1d, 0x5c03, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x4400,
	0x1d, 0xad2c, 0x1d, 0xd6cf, 0x1d, 0x0002, 0x1d, 0x80f4,
	0x1d, 0x7fe0, 0x1d, 0x4c80, 0x1d, 0x7c20, 0x1d, 0x5c20,
	0x1d, 0x481e, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c02, 0x1d, 0xad0a, 0x1d, 0x7c03, 0x1d, 0x5c03,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x4400, 0x1d, 0x5310, 0x1d, 0x8d02,
	0x1d, 0x4401, 0x1d, 0x81f4, 0x1d, 0x3114, 0x1d, 0x7fe0,
	0x1d, 0x4d00, 0x1d, 0x4832, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c10, 0x1d, 0x7c08, 0x1d, 0x6000,
	0x1d, 0xa4b7, 0x1d, 0xd9b3, 0x1d, 0xfffe, 0x1d, 0x7fe0,
	0x1d, 0x4d20, 0x1d, 0x7e00, 0x1d, 0x6200, 0x1d, 0x3045,
	0x1d, 0x7fe0, 0x1d, 0x4d40, 0x1d, 0x7c40, 0x1d, 0x6000,
	0x1d, 0x4401, 0x1d, 0x5210, 0x1d, 0x4833, 0x1d, 0x7c08,
	0x1d, 0x4c00, 0x1d, 0x7c08, 0x1d, 0x4c08, 0x1d, 0x8300,
	0x1d, 0x5f80, 0x1d, 0x55e0, 0x1d, 0xc06f, 0x1d, 0x0005,
	0x1d, 0xd9b3, 0x1d, 0xfffd, 0x1d, 0x7c40, 0x1d, 0x6040,
	0x1d, 0x7fe0, 0x1d, 0x4d60, 0x1d, 0x57e0, 0x1d, 0x4814,
	0x1d, 0x7c04, 0x1d, 0x4c00, 0x1d, 0x7c04, 0x1d, 0x4c04,
	0x1d, 0x8200, 0x1d, 0x7c03, 0x1d, 0x5c03, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0xad02, 0x1d, 0x4400, 0x1d, 0xc0e9, 0x1d, 0x0003,
	0x1d, 0xadd8, 0x1d, 0x30c6, 0x1d, 0x3078, 0x1d, 0x7fe0,
	0x1d, 0x4dc0, 0x1d, 0x6730, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0xd09d, 0x1d, 0x0002, 0x1d, 0xb4fe, 0x1d, 0x7fe0,
	0x1d, 0x4d80, 0x1d, 0x6802, 0x1d, 0x6600, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x7c08, 0x1d, 0x6000, 0x1d, 0x486c,
	0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c01,
	0x1d, 0x9503, 0x1d, 0x7e00, 0x1d, 0x6200, 0x1d, 0x571f,
	0x1d, 0x5fbb, 0x1d, 0xaa03, 0x1d, 0x5b58, 0x1d, 0x30e9,
	0x1d, 0x5b64, 0x1d, 0xcdab, 0x1d, 0xff5b, 0x1d, 0xcd8d,
	0x1d, 0xff59, 0x1d, 0xd96b, 0x1d, 0xff57, 0x1d, 0xd0a0,
	0x1d, 0xffdb, 0x1d, 0xcba0, 0x1d, 0x0003, 0x1d, 0x80f0,
	0x1d, 0x30f6, 0x1d, 0x3109, 0x1d, 0x7fe0, 0x1d, 0x4ce0,
	0x1d, 0x7d30, 0x1d, 0x6530, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0x7ce0, 0x1d, 0x5400, 0x1d, 0x4832, 0x1d, 0x7c1f,
	0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c08, 0x1d, 0x7c08,
	0x1d, 0x6008, 0x1d, 0x8300, 0x1d, 0xb902, 0x1d, 0x30d3,
	0x1d, 0x308f, 0x1d, 0x7fe0, 0x1d, 0x4da0, 0x1d, 0x57a0,
	0x1d, 0x590c, 0x1d, 0x5fa2, 0x1d, 0xcba4, 0x1d, 0x0005,
	0x1d, 0xcd8d, 0x1d, 0x0003, 0x1d, 0x80fc, 0x1d, 0x0000,
	0x1d, 0x7fe0, 0x1d, 0x4ca0, 0x1d, 0xb603, 0x1d, 0x7c10,
	0x1d, 0x6010, 0x1d, 0x7c1f, 0x1d, 0x541f, 0x1d, 0x7ffc,
	0x1d, 0x5fb3, 0x1d, 0x9403, 0x1d, 0x7c03, 0x1d, 0x5c03,
	0x1d, 0xaa05, 0x1d, 0x7c80, 0x1d, 0x5800, 0x1d, 0x5b58,
	0x1d, 0x3128, 0x1d, 0x7c80, 0x1d, 0x5800, 0x1d, 0x5b64,
	0x1d, 0x4827, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c10, 0x1d, 0x8400, 0x1d, 0x7c10, 0x1d, 0x6000,
	0x1d, 0x4824, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c04, 0x1d, 0x8200, 0x1d, 0x7fe0, 0x1d, 0x4cc0,
	0x1d, 0x7d00, 0x1d, 0x6400, 0x1d, 0x7ffc, 0x1d, 0x5fbb,
	0x1d, 0x4824, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c04, 0x1d, 0x8200, 0x1d, 0x7e00, 0x1d, 0x6a00,
	0x1d, 0x4824, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c04, 0x1d, 0x8200, 0x1d, 0x7e00, 0x1d, 0x6800,
	0x1d, 0x30f6, 0x1d, 0x7fe0, 0x1d, 0x4e00, 0x1d, 0x4007,
	0x1d, 0x4400, 0x1d, 0x5310, 0x1d, 0x6800, 0x1d, 0x6736,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x570f, 0x1d, 0x5fff,
	0x1d, 0xaa03, 0x1d, 0x585b, 0x1d, 0x315c, 0x1d, 0x5867,
	0x1d, 0x9402, 0x1d, 0x6200, 0x1d, 0xcda3, 0x1d, 0x009d,
	0x1d, 0xcd85, 0x1d, 0x009b, 0x1d, 0xd96b, 0x1d, 0x0099,
	0x1d, 0x96e9, 0x1d, 0x6800, 0x1d, 0x6736, 0x1d, 0x7fe0,
	0x1d, 0x4e20, 0x1d, 0x96e4, 0x1d, 0x8b04, 0x1d, 0x7c08,
	0x1d, 0x5008, 0x1d, 0xab03, 0x1d, 0x7c08, 0x1d, 0x5000,
	0x1d, 0x6801, 0x1d, 0x6776, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0xdb7c, 0x1d, 0xfff0, 0x1d, 0x0000, 0x1d, 0x7fe1,
	0x1d, 0x4e40, 0x1d, 0x4837, 0x1d, 0x4418, 0x1d, 0x41c7,
	0x1d, 0x7fe0, 0x1d, 0x4e40, 0x1d, 0x7c40, 0x1d, 0x5400,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x7c1f, 0x1d, 0x4c01,
	0x1d, 0x8fc9, 0x1d, 0xd2a0, 0x1d, 0x004a, 0x1d, 0x9203,
	0x1d, 0xa041, 0x1d, 0x3184, 0x1d, 0x7fe1, 0x1d, 0x4e60,
	0x1d, 0x489c, 0x1d, 0x4628, 0x1d, 0x7fe0, 0x1d, 0x4e60,
	0x1d, 0x7e28, 0x1d, 0x4628, 0x1d, 0x7c40, 0x1d, 0x5400,
	0x1d, 0x7c01, 0x1d, 0x5800, 0x1d, 0x7c04, 0x1d, 0x5c00,
	0x1d, 0x41e8, 0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x7c1f,
	0x1d, 0x4c01, 0x1d, 0x8fb0, 0x1d, 0xb241, 0x1d, 0xa02a,
	0x1d, 0x319d, 0x1d, 0x7fe0, 0x1d, 0x4ea0, 0x1d, 0x7c02,
	0x1d, 0x4402, 0x1d, 0x4448, 0x1d, 0x4894, 0x1d, 0x7c1f,
	0x1d, 0x4c01, 0x1d, 0x7c1f, 0x1d, 0x4c03, 0x1d, 0x4824,
	0x1d, 0x7c1f, 0x1d, 0x4c07, 0x1d, 0x41ef, 0x1d, 0x41ff,
	0x1d, 0x4891, 0x1d, 0x7c1f, 0x1d, 0x4c07, 0x1d, 0x7c1f,
	0x1d, 0x4c17, 0x1d, 0x8400, 0x1d, 0x8ef8, 0x1d, 0x41c7,
	0x1d, 0x8f95, 0x1d, 0x92d5, 0x1d, 0xa10f, 0x1d, 0xd480,
	0x1d, 0x0008, 0x1d, 0xd580, 0x1d, 0xffb9, 0x1d, 0xa202,
	0x1d, 0x31b8, 0x1d, 0x7c04, 0x1d, 0x4404, 0x1d, 0x31b8,
	0x1d, 0xd484, 0x1d, 0xfff3, 0x1d, 0xd484, 0x1d, 0xfff1,
	0x1d, 0x314d, 0x1d, 0x7fe0, 0x1d, 0x4ee0, 0x1d, 0x7c40,
	0x1d, 0x5400, 0x1d, 0x4488, 0x1d, 0x41cf, 0x1d, 0x314d,
	0x1d, 0x7fe0, 0x1d, 0x4ec0, 0x1d, 0x48f3, 0x1d, 0x7c1f,
	0x1d, 0x4c01, 0x1d, 0x7c1f, 0x1d, 0x4c09, 0x1d, 0x4508,
	0x1d, 0x41c7, 0x1d, 0x8f24, 0x1d, 0xd218, 0x1d, 0x0022,
	0x1d, 0xd2a4, 0x1d, 0xff9f, 0x1d, 0x31d9, 0x1d, 0x7fe0,
	0x1d, 0x4e80, 0x1d, 0x4832, 0x1d, 0x7c1f, 0x1d, 0x4c01,
	0x1d, 0x7c1f, 0x1d, 0x4c11, 0x1d, 0x4428, 0x1d, 0x7c40,
	0x1d, 0x5440, 0x1d, 0x7c01, 0x1d, 0x5801, 0x1d, 0x7c04,
	0x1d, 0x5c04, 0x1d, 0x41e8, 0x1d, 0xa4b3, 0x1d, 0x31ee,
	0x1d, 0x6800, 0x1d, 0x6736, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0x570f, 0x1d, 0x5fff, 0x1d, 0xaa03, 0x1d, 0x585b,
	0x1d, 0x31fa, 0x1d, 0x5867, 0x1d, 0xbcf6, 0x1d, 0x300b,
	0x1d, 0x300b, 0x1d, 0x314d, 0x1f, 0x0004, 0x1c, 0x0200,
	0x19, 0x7030, 0x1f, 0x0000
};

static struct gani_patch gani_8101_m14_patch[] = {
	0x1f, 0x0004, 0x1f, 0x0004, 0x19, 0x7070, 0x1c, 0x0600,
	0x1d, 0x9700, 0x1d, 0x7d00, 0x1d, 0x6900, 0x1d, 0x7d00,
	0x1d, 0x6800, 0x1d, 0x4899, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x8000, 0x1d, 0x7fe0,
	0x1d, 0x4c00, 0x1d, 0x4007, 0x1d, 0x4400, 0x1d, 0x4800,
	0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x5310, 0x1d, 0x6000,
	0x1d, 0x6800, 0x1d, 0x6736, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0x571f, 0x1d, 0x5ffb, 0x1d, 0xaa03, 0x1d, 0x5b58,
	0x1d, 0x301e, 0x1d, 0x5b64, 0x1d, 0xa6fc, 0x1d, 0xdcdb,
	0x1d, 0x0015, 0x1d, 0xb915, 0x1d, 0xb511, 0x1d, 0xd16b,
	0x1d, 0x000f, 0x1d, 0xb40f, 0x1d, 0xd06b, 0x1d, 0x000d,
	0x1d, 0xb206, 0x1d, 0x7c01, 0x1d, 0x5800, 0x1d, 0x7c04,
	0x1d, 0x5c00, 0x1d, 0x301a, 0x1d, 0x7c01, 0x1d, 0x5801,
	0x1d, 0x7c04, 0x1d, 0x5c04, 0x1d, 0x301e, 0x1d, 0x3079,
	0x1d, 0x30f1, 0x1d, 0x3199, 0x1d, 0x7fe0, 0x1d, 0x4c60,
	0x1d, 0x6803, 0x1d, 0x6420, 0x1d, 0x0000, 0x1d, 0x0000,
	0x1d, 0xaf03, 0x1d, 0x6015, 0x1d, 0x3040, 0x1d, 0x6017,
	0x1d, 0x57e0, 0x1d, 0x580c, 0x1d, 0x588c, 0x1d, 0x5fa3,
	0x1d, 0x0000, 0x1d, 0x4827, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c10, 0x1d, 0x8400, 0x1d, 0x7c30,
	0x1d, 0x6020, 0x1d, 0x48bf, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0xd6cf, 0x1d, 0x0002,
	0x1d, 0x80fe, 0x1d, 0x7fe0, 0x1d, 0x4c80, 0x1d, 0x7c20,
	0x1d, 0x5c20, 0x1d, 0x481e, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c02, 0x1d, 0x5310, 0x1d, 0x81ff,
	0x1d, 0x30ba, 0x1d, 0x7fe0, 0x1d, 0x4d00, 0x1d, 0x4832,
	0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c10,
	0x1d, 0x7c08, 0x1d, 0x6000, 0x1d, 0xa4cc, 0x1d, 0xd9b3,
	0x1d, 0xfffe, 0x1d, 0x7fe0, 0x1d, 0x4d20, 0x1d, 0x7e00,
	0x1d, 0x6200, 0x1d, 0x300b, 0x1d, 0x7fe0, 0x1d, 0x4dc0,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0xd09d, 0x1d, 0x0002,
	0x1d, 0xb4fe, 0x1d, 0x7fe0, 0x1d, 0x4d80, 0x1d, 0x7c04,
	0x1d, 0x6004, 0x1d, 0x5310, 0x1d, 0x6802, 0x1d, 0x6720,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x7c08, 0x1d, 0x6000,
	0x1d, 0x486c, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c01, 0x1d, 0x9503, 0x1d, 0x7e00, 0x1d, 0x6200,
	0x1d, 0x571f, 0x1d, 0x5fbb, 0x1d, 0xaa03, 0x1d, 0x5b58,
	0x1d, 0x3092, 0x1d, 0x5b64, 0x1d, 0xcdab, 0x1d, 0xff78,
	0x1d, 0xcd8d, 0x1d, 0xff76, 0x1d, 0xd96b, 0x1d, 0xff74,
	0x1d, 0xd0a0, 0x1d, 0xffd9, 0x1d, 0xcba0, 0x1d, 0x0003,
	0x1d, 0x80f0, 0x1d, 0x309f, 0x1d, 0x30ac, 0x1d, 0x7fe0,
	0x1d, 0x4ce0, 0x1d, 0x4832, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c08, 0x1d, 0x7c08, 0x1d, 0x6008,
	0x1d, 0x8300, 0x1d, 0xb902, 0x1d, 0x3079, 0x1d, 0x3061,
	0x1d, 0x7fe0, 0x1d, 0x4da0, 0x1d, 0x6400, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x57a0, 0x1d, 0x590c, 0x1d, 0x5fa3,
	0x1d, 0x0000, 0x1d, 0xcba4, 0x1d, 0x0004, 0x1d, 0xcd8d,
	0x1d, 0x0002, 0x1d, 0x80fc, 0x1d, 0x7fe0, 0x1d, 0x4ca0,
	0x1d, 0xb603, 0x1d, 0x7c10, 0x1d, 0x6010, 0x1d, 0x7c1f,
	0x1d, 0x541f, 0x1d, 0x5fb3, 0x1d, 0xaa05, 0x1d, 0x7c80,
	0x1d, 0x5800, 0x1d, 0x5b58, 0x1d, 0x30ca, 0x1d, 0x7c80,
	0x1d, 0x5800, 0x1d, 0x5b64, 0x1d, 0x4824, 0x1d, 0x7c1f,
	0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c04, 0x1d, 0x8200,
	0x1d, 0x4827, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c10, 0x1d, 0x8400, 0x1d, 0x7c10, 0x1d, 0x6000,
	0x1d, 0x7fe0, 0x1d, 0x4cc0, 0x1d, 0x5fbb, 0x1d, 0x4824,
	0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c04,
	0x1d, 0x8200, 0x1d, 0x7ce0, 0x1d, 0x5400, 0x1d, 0x6720,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x7e00, 0x1d, 0x6a00,
	0x1d, 0x4824, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c04, 0x1d, 0x8200, 0x1d, 0x7e00, 0x1d, 0x6800,
	0x1d, 0x309f, 0x1d, 0x7fe0, 0x1d, 0x4e00, 0x1d, 0x4007,
	0x1d, 0x4400, 0x1d, 0x5310, 0x1d, 0x6800, 0x1d, 0x6736,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x570f, 0x1d, 0x5fff,
	0x1d, 0xaa03, 0x1d, 0x585b, 0x1d, 0x3100, 0x1d, 0x5867,
	0x1d, 0x9403, 0x1d, 0x7e00, 0x1d, 0x6200, 0x1d, 0xcda3,
	0x1d, 0x002d, 0x1d, 0xcd85, 0x1d, 0x002b, 0x1d, 0xd96b,
	0x1d, 0x0029, 0x1d, 0x9629, 0x1d, 0x6800, 0x1d, 0x6736,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x9624, 0x1d, 0x7fe0,
	0x1d, 0x4e20, 0x1d, 0x8b04, 0x1d, 0x7c08, 0x1d, 0x5008,
	0x1d, 0xab03, 0x1d, 0x7c08, 0x1d, 0x5000, 0x1d, 0x6801,
	0x1d, 0x6776, 0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0xdb7c,
	0x1d, 0xffee, 0x1d, 0x0000, 0x1d, 0x7fe1, 0x1d, 0x4e40,
	0x1d, 0x4837, 0x1d, 0x4418, 0x1d, 0x41c7, 0x1d, 0x7fe0,
	0x1d, 0x4e40, 0x1d, 0x7c40, 0x1d, 0x5400, 0x1d, 0x7c1f,
	0x1d, 0x4c01, 0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x8f07,
	0x1d, 0xd2a0, 0x1d, 0x004c, 0x1d, 0x9205, 0x1d, 0xa043,
	0x1d, 0x312b, 0x1d, 0x300b, 0x1d, 0x30f1, 0x1d, 0x7fe1,
	0x1d, 0x4e60, 0x1d, 0x489c, 0x1d, 0x4628, 0x1d, 0x7fe0,
	0x1d, 0x4e60, 0x1d, 0x7e28, 0x1d, 0x4628, 0x1d, 0x7c40,
	0x1d, 0x5400, 0x1d, 0x7c01, 0x1d, 0x5800, 0x1d, 0x7c04,
	0x1d, 0x5c00, 0x1d, 0x41e8, 0x1d, 0x7c1f, 0x1d, 0x4c01,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x8fec, 0x1d, 0xb241,
	0x1d, 0xa02a, 0x1d, 0x3146, 0x1d, 0x7fe0, 0x1d, 0x4ea0,
	0x1d, 0x7c02, 0x1d, 0x4402, 0x1d, 0x4448, 0x1d, 0x4894,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x7c1f, 0x1d, 0x4c03,
	0x1d, 0x4824, 0x1d, 0x7c1f, 0x1d, 0x4c07, 0x1d, 0x41ef,
	0x1d, 0x41ff, 0x1d, 0x4891, 0x1d, 0x7c1f, 0x1d, 0x4c07,
	0x1d, 0x7c1f, 0x1d, 0x4c17, 0x1d, 0x8400, 0x1d, 0x8ef8,
	0x1d, 0x41c7, 0x1d, 0x8fd1, 0x1d, 0x92d5, 0x1d, 0xa10f,
	0x1d, 0xd480, 0x1d, 0x0008, 0x1d, 0xd580, 0x1d, 0xffb7,
	0x1d, 0xa202, 0x1d, 0x3161, 0x1d, 0x7c04, 0x1d, 0x4404,
	0x1d, 0x3161, 0x1d, 0xd484, 0x1d, 0xfff3, 0x1d, 0xd484,
	0x1d, 0xfff1, 0x1d, 0x30f1, 0x1d, 0x7fe0, 0x1d, 0x4ee0,
	0x1d, 0x7c40, 0x1d, 0x5400, 0x1d, 0x4488, 0x1d, 0x41cf,
	0x1d, 0x30f1, 0x1d, 0x7fe0, 0x1d, 0x4ec0, 0x1d, 0x48f3,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x7c1f, 0x1d, 0x4c09,
	0x1d, 0x4508, 0x1d, 0x41c7, 0x1d, 0x8fb0, 0x1d, 0xd218,
	0x1d, 0xffae, 0x1d, 0xd2a4, 0x1d, 0xff9d, 0x1d, 0x3182,
	0x1d, 0x7fe0, 0x1d, 0x4e80, 0x1d, 0x4832, 0x1d, 0x7c1f,
	0x1d, 0x4c01, 0x1d, 0x7c1f, 0x1d, 0x4c11, 0x1d, 0x4428,
	0x1d, 0x7c40, 0x1d, 0x5440, 0x1d, 0x7c01, 0x1d, 0x5801,
	0x1d, 0x7c04, 0x1d, 0x5c04, 0x1d, 0x41e8, 0x1d, 0xa4b3,
	0x1d, 0x3197, 0x1d, 0x7fe0, 0x1d, 0x4f20, 0x1d, 0x6800,
	0x1d, 0x6736, 0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x570f,
	0x1d, 0x5fff, 0x1d, 0xaa03, 0x1d, 0x585b, 0x1d, 0x31a5,
	0x1d, 0x5867, 0x1d, 0xbcf4, 0x1d, 0x300b, 0x1f, 0x0004,
	0x1c, 0x0200, 0x19, 0x7030, 0x1f, 0x0000
};

static struct gani_patch gani_8101_m16_patch[] = {
	0x1f, 0x0004, 0x1f, 0x0004, 0x19, 0x7070, 0x1c, 0x0600,
	0x1d, 0x9700, 0x1d, 0x7fe0, 0x1d, 0x4c00, 0x1d, 0x4007,
	0x1d, 0x4400, 0x1d, 0x4800, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x5310, 0x1d, 0x6000, 0x1d, 0x6800, 0x1d, 0x673e,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x571f, 0x1d, 0x5ffb,
	0x1d, 0xaa04, 0x1d, 0x5b58, 0x1d, 0x6100, 0x1d, 0x3016,
	0x1d, 0x5b64, 0x1d, 0x6080, 0x1d, 0xa6fa, 0x1d, 0xdcdb,
	0x1d, 0x0015, 0x1d, 0xb915, 0x1d, 0xb511, 0x1d, 0xd16b,
	0x1d, 0x000f, 0x1d, 0xb40f, 0x1d, 0xd06b, 0x1d, 0x000d,
	0x1d, 0xb206, 0x1d, 0x7c01, 0x1d, 0x5800, 0x1d, 0x7c04,
	0x1d, 0x5c00, 0x1d, 0x3010, 0x1d, 0x7c01, 0x1d, 0x5801,
	0x1d, 0x7c04, 0x1d, 0x5c04, 0x1d, 0x3016, 0x1d, 0x307e,
	0x1d, 0x30f4, 0x1d, 0x319f, 0x1d, 0x7fe0, 0x1d, 0x4c60,
	0x1d, 0x6803, 0x1d, 0x7d00, 0x1d, 0x6900, 0x1d, 0x6520,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0xaf03, 0x1d, 0x6115,
	0x1d, 0x303a, 0x1d, 0x6097, 0x1d, 0x57e0, 0x1d, 0x580c,
	0x1d, 0x588c, 0x1d, 0x5f80, 0x1d, 0x4827, 0x1d, 0x7c1f,
	0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c10, 0x1d, 0x8400,
	0x1d, 0x7c30, 0x1d, 0x6020, 0x1d, 0x48bf, 0x1d, 0x7c1f,
	0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0xb802,
	0x1d, 0x3053, 0x1d, 0x7c08, 0x1d, 0x6808, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x7c10, 0x1d, 0x6810, 0x1d, 0xd6cf,
	0x1d, 0x0002, 0x1d, 0x80fe, 0x1d, 0x7fe0, 0x1d, 0x4c80,
	0x1d, 0x7c10, 0x1d, 0x6800, 0x1d, 0x7c08, 0x1d, 0x6800,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x7c23, 0x1d, 0x5c23,
	0x1d, 0x481e, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c02, 0x1d, 0x5310, 0x1d, 0x81ff, 0x1d, 0x30c1,
	0x1d, 0x7fe0, 0x1d, 0x4d00, 0x1d, 0x4832, 0x1d, 0x7c1f,
	0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c10, 0x1d, 0x7c08,
	0x1d, 0x6000, 0x1d, 0xa4bd, 0x1d, 0xd9b3, 0x1d, 0x00fe,
	0x1d, 0x7fe0, 0x1d, 0x4d20, 0x1d, 0x7e00, 0x1d, 0x6200,
	0x1d, 0x3001, 0x1d, 0x7fe0, 0x1d, 0x4dc0, 0x1d, 0xd09d,
	0x1d, 0x0002, 0x1d, 0xb4fe, 0x1d, 0x7fe0, 0x1d, 0x4d80,
	0x1d, 0x7c04, 0x1d, 0x6004, 0x1d, 0x6802, 0x1d, 0x6728,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x7c08, 0x1d, 0x6000,
	0x1d, 0x486c, 0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f,
	0x1d, 0x4c01, 0x1d, 0x9503, 0x1d, 0x7e00, 0x1d, 0x6200,
	0x1d, 0x571f, 0x1d, 0x5fbb, 0x1d, 0xaa05, 0x1d, 0x5b58,
	0x1d, 0x7d80, 0x1d, 0x6100, 0x1d, 0x309a, 0x1d, 0x5b64,
	0x1d, 0x7d80, 0x1d, 0x6080, 0x1d, 0xcdab, 0x1d, 0x0058,
	0x1d, 0xcd8d, 0x1d, 0x0056, 0x1d, 0xd96b, 0x1d, 0x0054,
	0x1d, 0xd0a0, 0x1d, 0x00d8, 0x1d, 0xcba0, 0x1d, 0x0003,
	0x1d, 0x80ec, 0x1d, 0x30a7, 0x1d, 0x30b4, 0x1d, 0x7fe0,
	0x1d, 0x4ce0, 0x1d, 0x4832, 0x1d, 0x7c1f, 0x1d, 0x4c00,
	0x1d, 0x7c1f, 0x1d, 0x4c08, 0x1d, 0x7c08, 0x1d, 0x6008,
	0x1d, 0x8300, 0x1d, 0xb902, 0x1d, 0x307e, 0x1d, 0x3068,
	0x1d, 0x7fe0, 0x1d, 0x4da0, 0x1d, 0x6628, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x56a0, 0x1d, 0x590c, 0x1d, 0x5fa0,
	0x1d, 0xcba4, 0x1d, 0x0004, 0x1d, 0xcd8d, 0x1d, 0x0002,
	0x1d, 0x80fc, 0x1d, 0x7fe0, 0x1d, 0x4ca0, 0x1d, 0x7c08,
	0x1d, 0x6408, 0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x7d00,
	0x1d, 0x6800, 0x1d, 0xb603, 0x1d, 0x7c10, 0x1d, 0x6010,
	0x1d, 0x7d1f, 0x1d, 0x551f, 0x1d, 0x5fb3, 0x1d, 0xaa05,
	0x1d, 0x7c80, 0x1d, 0x5800, 0x1d, 0x5b58, 0x1d, 0x30d7,
	0x1d, 0x7c80, 0x1d, 0x5800, 0x1d, 0x5b64, 0x1d, 0x4827,
	0x1d, 0x7c1f, 0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c10,
	0x1d, 0x8400, 0x1d, 0x7c10, 0x1d, 0x6000, 0x1d, 0x7fe0,
	0x1d, 0x4cc0, 0x1d, 0x7d00, 0x1d, 0x6400, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x5fbb, 0x1d, 0x4824, 0x1d, 0x7c1f,
	0x1d, 0x4c00, 0x1d, 0x7c1f, 0x1d, 0x4c04, 0x1d, 0x8200,
	0x1d, 0x7ce0, 0x1d, 0x5400, 0x1d, 0x7d00, 0x1d, 0x6500,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0x30a7, 0x1d, 0x3001,
	0x1d, 0x7fe0, 0x1d, 0x4e00, 0x1d, 0x4007, 0x1d, 0x4400,
	0x1d, 0x5310, 0x1d, 0x6800, 0x1d, 0x673e, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x570f, 0x1d, 0x5fff, 0x1d, 0xaa05,
	0x1d, 0x585b, 0x1d, 0x7d80, 0x1d, 0x6100, 0x1d, 0x3107,
	0x1d, 0x5867, 0x1d, 0x7d80, 0x1d, 0x6080, 0x1d, 0x9403,
	0x1d, 0x7e00, 0x1d, 0x6200, 0x1d, 0xcda3, 0x1d, 0x00e8,
	0x1d, 0xcd85, 0x1d, 0x00e6, 0x1d, 0xd96b, 0x1d, 0x00e4,
	0x1d, 0x96e4, 0x1d, 0x6800, 0x1d, 0x673e, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x7fe0, 0x1d, 0x4e20, 0x1d, 0x96dd,
	0x1d, 0x8b04, 0x1d, 0x7c08, 0x1d, 0x5008, 0x1d, 0xab03,
	0x1d, 0x7c08, 0x1d, 0x5000, 0x1d, 0x6801, 0x1d, 0x677e,
	0x1d, 0x0000, 0x1d, 0x0000, 0x1d, 0xdb7c, 0x1d, 0x00ee,
	0x1d, 0x0000, 0x1d, 0x7fe1, 0x1d, 0x4e40, 0x1d, 0x4837,
	0x1d, 0x4418, 0x1d, 0x41c7, 0x1d, 0x7fe0, 0x1d, 0x4e40,
	0x1d, 0x7c40, 0x1d, 0x5400, 0x1d, 0x7c1f, 0x1d, 0x4c01,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x8fc2, 0x1d, 0xd2a0,
	0x1d, 0x004b, 0x1d, 0x9204, 0x1d, 0xa042, 0x1d, 0x3132,
	0x1d, 0x30f4, 0x1d, 0x7fe1, 0x1d, 0x4e60, 0x1d, 0x489c,
	0x1d, 0x4628, 0x1d, 0x7fe0, 0x1d, 0x4e60, 0x1d, 0x7e28,
	0x1d, 0x4628, 0x1d, 0x7c40, 0x1d, 0x5400, 0x1d, 0x7c01,
	0x1d, 0x5800, 0x1d, 0x7c04, 0x1d, 0x5c00, 0x1d, 0x41e8,
	0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x7c1f, 0x1d, 0x4c01,
	0x1d, 0x8fa8, 0x1d, 0xb241, 0x1d, 0xa02a, 0x1d, 0x314c,
	0x1d, 0x7fe0, 0x1d, 0x4ea0, 0x1d, 0x7c02, 0x1d, 0x4402,
	0x1d, 0x4448, 0x1d, 0x4894, 0x1d, 0x7c1f, 0x1d, 0x4c01,
	0x1d, 0x7c1f, 0x1d, 0x4c03, 0x1d, 0x4824, 0x1d, 0x7c1f,
	0x1d, 0x4c07, 0x1d, 0x41ef, 0x1d, 0x41ff, 0x1d, 0x4891,
	0x1d, 0x7c1f, 0x1d, 0x4c07, 0x1d, 0x7c1f, 0x1d, 0x4c17,
	0x1d, 0x8400, 0x1d, 0x8ef8, 0x1d, 0x41c7, 0x1d, 0x8f8d,
	0x1d, 0x92d5, 0x1d, 0xa10f, 0x1d, 0xd480, 0x1d, 0x0008,
	0x1d, 0xd580, 0x1d, 0x00b8, 0x1d, 0xa202, 0x1d, 0x3167,
	0x1d, 0x7c04, 0x1d, 0x4404, 0x1d, 0x3167, 0x1d, 0xd484,
	0x1d, 0x00f3, 0x1d, 0xd484, 0x1d, 0x00f1, 0x1d, 0x30f4,
	0x1d, 0x7fe0, 0x1d, 0x4ee0, 0x1d, 0x7c40, 0x1d, 0x5400,
	0x1d, 0x4488, 0x1d, 0x41cf, 0x1d, 0x30f4, 0x1d, 0x7fe0,
	0x1d, 0x4ec0, 0x1d, 0x48f3, 0x1d, 0x7c1f, 0x1d, 0x4c01,
	0x1d, 0x7c1f, 0x1d, 0x4c09, 0x1d, 0x4508, 0x1d, 0x41c7,
	0x1d, 0x8fb0, 0x1d, 0xd218, 0x1d, 0x00ae, 0x1d, 0xd2a4,
	0x1d, 0x009e, 0x1d, 0x3188, 0x1d, 0x7fe0, 0x1d, 0x4e80,
	0x1d, 0x4832, 0x1d, 0x7c1f, 0x1d, 0x4c01, 0x1d, 0x7c1f,
	0x1d, 0x4c11, 0x1d, 0x4428, 0x1d, 0x7c40, 0x1d, 0x5440,
	0x1d, 0x7c01, 0x1d, 0x5801, 0x1d, 0x7c04, 0x1d, 0x5c04,
	0x1d, 0x41e8, 0x1d, 0xa4b3, 0x1d, 0x319d, 0x1d, 0x7fe0,
	0x1d, 0x4f20, 0x1d, 0x6800, 0x1d, 0x673e, 0x1d, 0x0000,
	0x1d, 0x0000, 0x1d, 0x570f, 0x1d, 0x5fff, 0x1d, 0xaa04,
	0x1d, 0x585b, 0x1d, 0x6100, 0x1d, 0x31ad, 0x1d, 0x5867,
	0x1d, 0x6080, 0x1d, 0xbcf2, 0x1d, 0x3001, 0x1f, 0x0004,
	0x1c, 0x0200, 0x19, 0x7030, 0x1f, 0x0000,

	0x1F, 0x0000, 0x18, 0x8310,
};

#define	GANI_PHY_PATCH(t)	{ \
	int	i;	\
	for (i = 0; i < sizeof (t) / sizeof ((t)[0]); i++) {	\
		gani_mii_write_raw(dp, (t)[i].addr, (t)[i].val);	\
	}	\
}

static void
gani_patch_phy(struct gem_dev *dp)
{
	int		i;
	uint32_t	val;
	boolean_t	aspm = B_FALSE;
	struct gani_dev	*lp = dp->private;

	switch (lp->chip->type) {
	case R8169_2: /* RTL8169S rev.D */
	case R8169_3: /* RTL8169S rev.E */
		GANI_PHY_PATCH(gani_8169_m2_patch);
		break;

	case R8169_4: /* RTL8169SB */
		GANI_PHY_PATCH(gani_8169_m4_patch);
		break;

	case R8169_5: /* RTL8169SC rev.D */
		GANI_PHY_PATCH(gani_8169_m5_patch0);
		if (lp->svid == 0x1458 && lp->ssid == 0xe000) {
			GANI_PHY_PATCH(gani_8169_m5_patch1);
		}
		break;

	case R8169_6: /* RTL8169SC rev.E */
		GANI_PHY_PATCH(gani_8169_m6_patch);
		break;

	case R8168_1: /* RTL8168B rev.B */
		GANI_PHY_PATCH(gani_8168_m1_patch);
		break;

	case R8168_2: /* RTL8168B rev.E */
		GANI_PHY_PATCH(gani_8168_m2_patch);
		break;

	case R8168_3: /* RTL8168B rev.F */
		GANI_PHY_PATCH(gani_8168_m3_patch);
		break;

	case R8168_4: /* RTL8168C */
		/* 035 */
		GANI_PHY_PATCH(gani_8168_m4_patch);

		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) | BIT_5);
		gani_mii_write_raw(dp, 0x0D,
		    gani_mii_read_raw(dp, 0x0D) & ~BIT_5);
		break;

	case R8168_5: /* RTL8168C rev.B */
		/* 035 */
		GANI_PHY_PATCH(gani_8168_m5_patch0);

		gani_mii_write_raw(dp, 0x16,
		    gani_mii_read_raw(dp, 0x16) | BIT_0);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) | BIT_5);
		gani_mii_write_raw(dp, 0x0D,
		    gani_mii_read_raw(dp, 0x0D) & ~BIT_5);

		GANI_PHY_PATCH(gani_8168_m5_patch1);
		break;

	case R8168_6: /* RTL8168C rev.C */
		GANI_PHY_PATCH(gani_8168_m6_patch0);

		gani_mii_write_raw(dp, 0x16,
		    gani_mii_read_raw(dp, 0x16) | BIT_0);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) | BIT_5);
		gani_mii_write_raw(dp, 0x0D,
		    gani_mii_read_raw(dp, 0x0D) & ~BIT_5);

		GANI_PHY_PATCH(gani_8168_m6_patch1);
		break;

	case R8168_7: /* RTL8168CP rev.B */
		/* 035 */
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) | BIT_5);
		gani_mii_write_raw(dp, 0x0D,
		    gani_mii_read_raw(dp, 0x0D) & ~BIT_5);

		GANI_PHY_PATCH(gani_8168_m7_patch);
		break;

	case R8168_8: /* RTL8168CP rev.C */
		/* 035 */
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) | BIT_5);
		gani_mii_write_raw(dp, 0x0D,
		    gani_mii_read_raw(dp, 0x0D) & ~BIT_5);

		GANI_PHY_PATCH(gani_8168_m8_patch);

		gani_mii_write_raw(dp, 0x16,
		    gani_mii_read_raw(dp, 0x16) | BIT_0);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		break;

	case R8168_9: /* RTL8168D */
		/* 035 */
		GANI_PHY_PATCH(gani_8168_m9_patch0);

		gani_mii_write_raw(dp, 0x1F, 0x0002);
		val = gani_mii_read_raw(dp, 0x0B) & 0xFF00;
		val |= 0x10;
		gani_mii_write_raw(dp, 0x0B, val);
		val = gani_mii_read_raw(dp, 0x0C) & 0x00FF;
		val |= 0xA200;
		gani_mii_write_raw(dp, 0x0C, val);

		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x06, 0x5561);
		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8332);
		gani_mii_write_raw(dp, 0x06, 0x5561);

		if (gani_efuse_read(dp, 0x01) == 0xb1) {
			gani_mii_write_raw(dp, 0x1F, 0x0002);
			gani_mii_write_raw(dp, 0x05, 0x669A);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8330);
			gani_mii_write_raw(dp, 0x06, 0x669A);

			gani_mii_write_raw(dp, 0x1F, 0x0002);
			val = gani_mii_read_raw(dp, 0x0D);
			if ((val & 0x00FF) != 0x006C) {
				/* 035 */
				val &= 0xFF00;
				gani_mii_write_raw(dp, 0x1F, 0x0002);
				gani_mii_write_raw(dp, 0x0D, val | 0x0065);
				gani_mii_write_raw(dp, 0x0D, val | 0x0066);
				gani_mii_write_raw(dp, 0x0D, val | 0x0067);
				gani_mii_write_raw(dp, 0x0D, val | 0x0068);
				gani_mii_write_raw(dp, 0x0D, val | 0x0069);
				gani_mii_write_raw(dp, 0x0D, val | 0x006A);
				gani_mii_write_raw(dp, 0x0D, val | 0x006B);
				gani_mii_write_raw(dp, 0x0D, val | 0x006C);
			}
		} else {
			gani_mii_write_raw(dp, 0x1F, 0x0002);
			gani_mii_write_raw(dp, 0x05, 0x6662);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8330);
			gani_mii_write_raw(dp, 0x06, 0x6662);
		}

		gani_mii_write_raw(dp, 0x1F, 0x0002);
		val = gani_mii_read_raw(dp, 0x0D);
		val |= BIT_9 | BIT_8;
		gani_mii_write_raw(dp, 0x0D, val);

		val = gani_mii_read_raw(dp, 0x0F);
		val |= BIT_4;
		gani_mii_write_raw(dp, 0x0F, val);

		gani_mii_write_raw(dp, 0x1F, 0x0002);
		val = gani_mii_read_raw(dp, 0x02);
		val &= ~(BIT_10 | BIT_9);
		val |= BIT_8;
		gani_mii_write_raw(dp, 0x02, val);

		val = gani_mii_read_raw(dp, 0x03);
		val &= ~(BIT_15 | BIT_14 | BIT_13);
		gani_mii_write_raw(dp, 0x03, val);

		gani_mii_write_raw(dp, 0x1F, 0x0001);
		gani_mii_write_raw(dp, 0x17, 0x0CC0);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x001B);

		if (gani_mii_read_raw(dp, 0x06) == 0xBF00) {
			GANI_PHY_PATCH(gani_8168_m9_patch1);
		}

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0xF880);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		break;

	case R8168_10: /* RTL8168D rev.C */
		GANI_PHY_PATCH(gani_8168_m10_patch0);

		if (gani_efuse_read(dp, 0x01) == 0xb1) {
			/* 035 */
			gani_mii_write_raw(dp, 0x1F, 0x0002);
			gani_mii_write_raw(dp, 0x05, 0x669A);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8330);
			gani_mii_write_raw(dp, 0x06, 0x669A);

			gani_mii_write_raw(dp, 0x1F, 0x0002);
			val = gani_mii_read_raw(dp, 0x0D);
			if ((val & 0x00FF) != 0x006C) {
				/* 035 */
				val &= 0xFF00;
				gani_mii_write_raw(dp, 0x1F, 0x0002);
				gani_mii_write_raw(dp, 0x0D, val | 0x0065);
				gani_mii_write_raw(dp, 0x0D, val | 0x0066);
				gani_mii_write_raw(dp, 0x0D, val | 0x0067);
				gani_mii_write_raw(dp, 0x0D, val | 0x0068);
				gani_mii_write_raw(dp, 0x0D, val | 0x0069);
				gani_mii_write_raw(dp, 0x0D, val | 0x006A);
				gani_mii_write_raw(dp, 0x0D, val | 0x006B);
				gani_mii_write_raw(dp, 0x0D, val | 0x006C);
			}
		} else {
			/* 035 */
			gani_mii_write_raw(dp, 0x1F, 0x0002);
			gani_mii_write_raw(dp, 0x05, 0x2642);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8330);
			gani_mii_write_raw(dp, 0x06, 0x2642);
		}

		/* 035 */
		if (gani_efuse_read(dp, 0x30) == 0x98) {
			gani_mii_write_raw(dp, 0x1F, 0x0000);
			gani_mii_write_raw(dp, 0x11,
			    gani_mii_read_raw(dp, 0x11) & ~BIT_1);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x01,
			    gani_mii_read_raw(dp, 0x01) | BIT_9);
		} else if (gani_efuse_read(dp, 0x30) == 0x90) {
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x01,
			    gani_mii_read_raw(dp, 0x01) & ~BIT_9);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
			gani_mii_write_raw(dp, 0x16, 0x5101);
		}

		/* 035 */
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		val = gani_mii_read_raw(dp, 0x02);
		val &= ~(BIT_10 | BIT_9);
		val |= BIT_8;
		gani_mii_write_raw(dp, 0x02, val);
		val = gani_mii_read_raw(dp, 0x03);
		val &= ~(BIT_15 | BIT_14 | BIT_13);
		gani_mii_write_raw(dp, 0x03, val);

		gani_mii_write_raw(dp, 0x1F, 0x0001);
		gani_mii_write_raw(dp, 0x17, 0x0CC0);

		gani_mii_write_raw(dp, 0x1F, 0x0002);
		val = gani_mii_read_raw(dp, 0x0F);
		val |= BIT_4 | BIT_2 | BIT_1 | BIT_0;
		gani_mii_write_raw(dp, 0x0F, val);

		/* 035 */
		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x001B);
		if (gani_mii_read_raw(dp, 0x06) == 0xB300) {
			GANI_PHY_PATCH(gani_8168_m10_patch1);
		}

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x0D, 0xF880);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		break;

	case R8168_11: /* RTL8168DP */
		GANI_PHY_PATCH(gani_8168_m11_patch);
		break;

	case R8168_12:
	case R8168_13:
		/* 035 */
		gani_mii_write_raw(dp, 0x1F, 0x0001);
		gani_mii_write_raw(dp, 0x17, 0x0CC0);

		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x002D);
		gani_mii_write_raw(dp, 0x18, 0x0040);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		val = gani_mii_read_raw(dp, 0x0D);
		val |= BIT_5;
		gani_mii_write_raw(dp, 0x0D, val);

		gani_mii_write_raw(dp, 0x1F, 0x0002);
		val = gani_mii_read_raw(dp, 0x0C);
		val |= BIT_10;
		gani_mii_write_raw(dp, 0x0C, val);
		break;

	case R8168_14:
	case R8168_15:
		OUTB(dp, 0xF3, INB(dp, 0xF3) | BIT_2);

		if (lp->chip->type == R8168_14) {
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x00, 0x1800);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x0023);
			gani_mii_write_raw(dp, 0x17, 0x0117);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1E, 0x002C);
			gani_mii_write_raw(dp, 0x1B, 0x5000);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x16, 0x4104);
			for (i = 0; i < 200; i++) {
				drv_usecwait(99);
				val = gani_mii_read_raw(dp, 0x1E);
				val &= 0x03FF;
				if (val == 0x000C) {
					break;
				}
			}
			gani_mii_write_raw(dp, 0x1f, 0x0005);
			for (i = 0; i < 200; i++) {
				drv_usecwait(100);
				val = gani_mii_read_raw(dp, 0x07);
				if ((val & BIT_5) == 0) {
					break;
				}
			}
			val = gani_mii_read_raw(dp, 0x07);
			if (val & BIT_5) {
				gani_mii_write_raw(dp, 0x1f, 0x0007);
				gani_mii_write_raw(dp, 0x1e, 0x00a1);
				gani_mii_write_raw(dp, 0x17, 0x1000);
				gani_mii_write_raw(dp, 0x17, 0x0000);
				gani_mii_write_raw(dp, 0x17, 0x2000);
				gani_mii_write_raw(dp, 0x1e, 0x002f);
				gani_mii_write_raw(dp, 0x18, 0x9bfb);
				gani_mii_write_raw(dp, 0x1f, 0x0005);
				gani_mii_write_raw(dp, 0x07, 0x0000);
				gani_mii_write_raw(dp, 0x1f, 0x0000);
			}
			gani_mii_write_raw(dp, 0x1f, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0xfff6);
			gani_mii_write_raw(dp, 0x06, 0x0080);
			val = gani_mii_read_raw(dp, 0x00);
			val &= ~(BIT_7);
			gani_mii_write_raw(dp, 0x00, val);
			gani_mii_write_raw(dp, 0x1f, 0x0002);
			val = gani_mii_read_raw(dp, 0x08);
			val &= ~(BIT_7);
			gani_mii_write_raw(dp, 0x08, val);

			GANI_PHY_PATCH(gani_8168_m14_patch0);

			val = gani_mii_read_raw(dp, 0x01);
			val |= BIT_0;
			gani_mii_write_raw(dp, 0x01, val);
			val = gani_mii_read_raw(dp, 0x00);
			val |= BIT_0;
			gani_mii_write_raw(dp, 0x00, val);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x1f, 0x0005);
			for (i = 0; i < 200; i++) {
				drv_usecwait(100);
				val = gani_mii_read_raw(dp, 0x00);
				if (val & BIT_7) {
					break;
				}
			}
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x0023);
			val = gani_mii_read_raw(dp, 0x17);
			val &= ~BIT_0;

			if ((lp->svid == 0x144d && lp->ssid == 0xc098) ||
			    (lp->svid == 0x144d && lp->ssid == 0xc0b1)) {
				val &= ~BIT_2;
			}

			gani_mii_write_raw(dp, 0x17, val);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x0028);
			gani_mii_write_raw(dp, 0x15, 0x0010);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x0041);
			gani_mii_write_raw(dp, 0x15, 0x0802);
			gani_mii_write_raw(dp, 0x16, 0x2185);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
		} else {
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x00, 0x1800);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x0023);
			gani_mii_write_raw(dp, 0x17, 0x0117);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1E, 0x002C);
			gani_mii_write_raw(dp, 0x1B, 0x5000);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x16, 0x4104);
			for (i = 0; i < 200; i++) {
				drv_usecwait(100);
				val = gani_mii_read_raw(dp, 0x1E);
				val &= 0x03FF;
				if (val == 0x000C) {
					break;
				}
			}
			gani_mii_write_raw(dp, 0x1f, 0x0005);
			for (i = 0; i < 200; i++) {
				drv_usecwait(100);
				val = gani_mii_read_raw(dp, 0x07);
				if ((val & BIT_5) == 0)
					break;
			}
			val = gani_mii_read_raw(dp, 0x07);
			if (val & BIT_5) {
				gani_mii_write_raw(dp, 0x1f, 0x0007);
				gani_mii_write_raw(dp, 0x1e, 0x00a1);
				gani_mii_write_raw(dp, 0x17, 0x1000);
				gani_mii_write_raw(dp, 0x17, 0x0000);
				gani_mii_write_raw(dp, 0x17, 0x2000);
				gani_mii_write_raw(dp, 0x1e, 0x002f);
				gani_mii_write_raw(dp, 0x18, 0x9bfb);
				gani_mii_write_raw(dp, 0x1f, 0x0005);
				gani_mii_write_raw(dp, 0x07, 0x0000);
				gani_mii_write_raw(dp, 0x1f, 0x0000);
			}
			gani_mii_write_raw(dp, 0x1f, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0xfff6);
			gani_mii_write_raw(dp, 0x06, 0x0080);
			val = gani_mii_read_raw(dp, 0x00);
			val &= ~(BIT_7);
			gani_mii_write_raw(dp, 0x00, val);
			gani_mii_write_raw(dp, 0x1f, 0x0002);
			val = gani_mii_read_raw(dp, 0x08);
			val &= ~(BIT_7);
			gani_mii_write_raw(dp, 0x08, val);

			GANI_PHY_PATCH(gani_8168_m14_patch1);

			val = gani_mii_read_raw(dp, 0x01);
			val |= BIT_0;
			gani_mii_write_raw(dp, 0x01, val);
			val = gani_mii_read_raw(dp, 0x00);
			val |= BIT_0;
			gani_mii_write_raw(dp, 0x00, val);

			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x17, 0x2179);
			gani_mii_write_raw(dp, 0x1f, 0x0001);
			gani_mii_write_raw(dp, 0x10, 0xf274);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x0042);
			gani_mii_write_raw(dp, 0x15, 0x0f00);
			gani_mii_write_raw(dp, 0x15, 0x0f00);
			gani_mii_write_raw(dp, 0x16, 0x7408);
			gani_mii_write_raw(dp, 0x15, 0x0e00);
			gani_mii_write_raw(dp, 0x15, 0x0f00);
			gani_mii_write_raw(dp, 0x15, 0x0f01);
			gani_mii_write_raw(dp, 0x16, 0x4000);
			gani_mii_write_raw(dp, 0x15, 0x0e01);
			gani_mii_write_raw(dp, 0x15, 0x0f01);
			gani_mii_write_raw(dp, 0x15, 0x0f02);
			gani_mii_write_raw(dp, 0x16, 0x9400);
			gani_mii_write_raw(dp, 0x15, 0x0e02);
			gani_mii_write_raw(dp, 0x15, 0x0f02);
			gani_mii_write_raw(dp, 0x15, 0x0f03);
			gani_mii_write_raw(dp, 0x16, 0x7408);
			gani_mii_write_raw(dp, 0x15, 0x0e03);
			gani_mii_write_raw(dp, 0x15, 0x0f03);
			gani_mii_write_raw(dp, 0x15, 0x0f04);
			gani_mii_write_raw(dp, 0x16, 0x4008);
			gani_mii_write_raw(dp, 0x15, 0x0e04);
			gani_mii_write_raw(dp, 0x15, 0x0f04);
			gani_mii_write_raw(dp, 0x15, 0x0f05);
			gani_mii_write_raw(dp, 0x16, 0x9400);
			gani_mii_write_raw(dp, 0x15, 0x0e05);
			gani_mii_write_raw(dp, 0x15, 0x0f05);
			gani_mii_write_raw(dp, 0x15, 0x0f06);
			gani_mii_write_raw(dp, 0x16, 0x0803);
			gani_mii_write_raw(dp, 0x15, 0x0e06);
			gani_mii_write_raw(dp, 0x15, 0x0f06);
			gani_mii_write_raw(dp, 0x15, 0x0d00);
			gani_mii_write_raw(dp, 0x15, 0x0100);
			gani_mii_write_raw(dp, 0x1f, 0x0001);
			gani_mii_write_raw(dp, 0x10, 0xf074);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x17, 0x2149);
			gani_mii_write_raw(dp, 0x1f, 0x0005);

			for (i = 0; i < 200; i++) {
				drv_usecwait(100);
				val = gani_mii_read_raw(dp, 0x00);
				if (val & BIT_7)
					break;
			}
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x0023);
			val = gani_mii_read_raw(dp, 0x17);
			val &= ~(BIT_0 | BIT_2);
			gani_mii_write_raw(dp, 0x17, val);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x0023);
			val = gani_mii_read_raw(dp, 0x17);
			val |= BIT_14;
			gani_mii_write_raw(dp, 0x17, val);
			gani_mii_write_raw(dp, 0x1e, 0x0020);
			val = gani_mii_read_raw(dp, 0x1b);
			val |= BIT_7;
			gani_mii_write_raw(dp, 0x1b, val);
			gani_mii_write_raw(dp, 0x1e, 0x0041);
			gani_mii_write_raw(dp, 0x15, 0x0e02);
			gani_mii_write_raw(dp, 0x1e, 0x0028);
			val = gani_mii_read_raw(dp, 0x19);
			val |= BIT_15;
			gani_mii_write_raw(dp, 0x19, val);
			gani_mii_write_raw(dp, 0x1f, 0x0000);

			if (lp->svid == 0x104d && lp->ssid == 0x907b) {
				gani_mii_write_raw(dp, 0x1F, 0x0005);
				gani_mii_write_raw(dp, 0x05, 0x8B86);
				val = gani_mii_read_raw(dp, 0x06) | BIT_4;
				gani_mii_write_raw(dp, 0x06, val);
				gani_mii_write_raw(dp, 0x1f, 0x0000);
			}
		}

		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x0023);
		val = gani_mii_read_raw(dp, 0x17)|0x02;
		gani_mii_write_raw(dp, 0x17, val);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8b80);
		gani_mii_write_raw(dp, 0x06, 0xc896);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0001);
		gani_mii_write_raw(dp, 0x0B, 0x6C20);
		gani_mii_write_raw(dp, 0x07, 0x2872);
		gani_mii_write_raw(dp, 0x1C, 0xEFFF);
		gani_mii_write_raw(dp, 0x1F, 0x0003);
		gani_mii_write_raw(dp, 0x14, 0x6420);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0002);
		val = gani_mii_read_raw(dp, 0x08) & 0x00FF;
		gani_mii_write_raw(dp, 0x08, val | 0x8000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x002D);
		val = gani_mii_read_raw(dp, 0x18);
		gani_mii_write_raw(dp, 0x18, val | 0x0050);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		val = gani_mii_read_raw(dp, 0x14);
		gani_mii_write_raw(dp, 0x14, val | 0x8000);

		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x00, 0x080B);
		gani_mii_write_raw(dp, 0x0B, 0x09D7);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x15, 0x1006);

		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x002F);
		gani_mii_write_raw(dp, 0x15, 0x1919);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B86);
		val = gani_mii_read_raw(dp, 0x06);
		gani_mii_write_raw(dp, 0x06, val | 0x0001);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x00AC);
		gani_mii_write_raw(dp, 0x18, 0x0006);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0003);
		gani_mii_write_raw(dp, 0x19, 0x7F46);
		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8AD2);
		gani_mii_write_raw(dp, 0x06, 0x6810);
		gani_mii_write_raw(dp, 0x05, 0x8AD4);
		gani_mii_write_raw(dp, 0x06, 0x8002);
		gani_mii_write_raw(dp, 0x05, 0x8ADE);
		gani_mii_write_raw(dp, 0x06, 0x8025);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		break;

	case R8168_16:
		if (aspm) {
			OUTB(dp, 0x6E, INB(dp, 0x6E) | BIT_6);
			gani_eri_write(dp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		val = gani_mii_read_raw(dp, 0x15);
		val &= ~(BIT_12);
		gani_mii_write_raw(dp, 0x15, val);
		delay(drv_usectohz(20000));
		gani_mii_write_raw(dp, 0x1f, 0x0004);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		val = gani_mii_read_raw(dp, 0x17);
		if ((val & BIT_11) == 0x0000) {
			val |= BIT_0;
			gani_mii_write_raw(dp, 0x17, val);
			for (i = 0; i < 200; i++) {
				drv_usecwait(100);
				val = gani_mii_read_raw(dp, 0x17);
				if (val & BIT_11)
					break;
			}
		}
		val = gani_mii_read_raw(dp, 0x17);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x17, val);
		gani_mii_write_raw(dp, 0x1f, 0x0004);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x002C);
		gani_mii_write_raw(dp, 0x1B, 0x5000);
		gani_mii_write_raw(dp, 0x1E, 0x002d);
		gani_mii_write_raw(dp, 0x19, 0x0004);
		gani_mii_write_raw(dp, 0x1f, 0x0002);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x1E);
			if ((val & 0x03FF) == 0x0014)
				break;
		}
		gani_mii_write_raw(dp, 0x1f, 0x0005);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x07);
			if ((val & BIT_5) == 0)
				break;
		}
		val = gani_mii_read_raw(dp, 0x07);
		if (val & BIT_5) {
			gani_mii_write_raw(dp, 0x1f, 0x0004);
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x00a1);
			gani_mii_write_raw(dp, 0x17, 0x1000);
			gani_mii_write_raw(dp, 0x17, 0x0000);
			gani_mii_write_raw(dp, 0x17, 0x2000);
			gani_mii_write_raw(dp, 0x1e, 0x002f);
			gani_mii_write_raw(dp, 0x18, 0x9bfb);
			gani_mii_write_raw(dp, 0x1f, 0x0005);
			gani_mii_write_raw(dp, 0x07, 0x0000);
			gani_mii_write_raw(dp, 0x1f, 0x0002);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
		}
		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0xfff6);
		gani_mii_write_raw(dp, 0x06, 0x0080);
		val = gani_mii_read_raw(dp, 0x00);
		val &= ~(BIT_7);
		gani_mii_write_raw(dp, 0x00, val);

		GANI_PHY_PATCH(gani_8168_m16_patch);

		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x05, 0xe140);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x1f, 0x0005);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x00);
			if (val & BIT_7) {
				break;
			}
		}
		gani_mii_write_raw(dp, 0x1f, 0x0004);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		val = gani_mii_read_raw(dp, 0x17);
		val &= ~(BIT_0);
		val |= BIT_2;
		gani_mii_write_raw(dp, 0x17, val);
		gani_mii_write_raw(dp, 0x1f, 0x0002);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B80);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_2 | BIT_1;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0004);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002D);
		val = gani_mii_read_raw(dp, 0x18);
		val |= BIT_4;
		gani_mii_write_raw(dp, 0x18, val);
		gani_mii_write_raw(dp, 0x1f, 0x0002);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		val = gani_mii_read_raw(dp, 0x14);
		val |= BIT_15;
		gani_mii_write_raw(dp, 0x14, val);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x15, 0x1006);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B86);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0001);
		gani_mii_write_raw(dp, 0x0B, 0x6C14);
		gani_mii_write_raw(dp, 0x14, 0x7F3D);
		gani_mii_write_raw(dp, 0x1C, 0xFAFE);
		gani_mii_write_raw(dp, 0x08, 0x07C5);
		gani_mii_write_raw(dp, 0x10, 0xF090);
		gani_mii_write_raw(dp, 0x1F, 0x0003);
		gani_mii_write_raw(dp, 0x14, 0x641A);
		gani_mii_write_raw(dp, 0x1A, 0x0606);
		gani_mii_write_raw(dp, 0x12, 0xF480);
		gani_mii_write_raw(dp, 0x13, 0x0747);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x0078);
		gani_mii_write_raw(dp, 0x15, 0xA408);
		gani_mii_write_raw(dp, 0x17, 0x5100);
		gani_mii_write_raw(dp, 0x19, 0x0008);
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0003);
		gani_mii_write_raw(dp, 0x0D, 0x0207);
		gani_mii_write_raw(dp, 0x02, 0x5FD0);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x00A1);
		val = gani_mii_read_raw(dp, 0x1A);
		val &= ~BIT_2;
		gani_mii_write_raw(dp, 0x1A, val);
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x002D);
		val = gani_mii_read_raw(dp, 0x16);
		val |= BIT_5;
		gani_mii_write_raw(dp, 0x16, val);
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x00AC);
		gani_mii_write_raw(dp, 0x18, 0x0006);
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0003);
		gani_mii_write_raw(dp, 0x09, 0xA20F);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B5B);
		gani_mii_write_raw(dp, 0x06, 0x9222);
		gani_mii_write_raw(dp, 0x05, 0x8B6D);
		gani_mii_write_raw(dp, 0x06, 0x8000);
		gani_mii_write_raw(dp, 0x05, 0x8B76);
		gani_mii_write_raw(dp, 0x06, 0x8000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		if (lp->svid == 0x1043 && lp->ssid == 0x13F7) {

			static const uint16_t evl_phy_value[] = {
				0x8B56, 0x8B5F, 0x8B68, 0x8B71,
				0x8B7A, 0x8A7B, 0x8A7E, 0x8A81,
				0x8A84, 0x8A87
			}
			;

			gani_mii_write_raw(dp, 0x1F, 0x0005);
			for (i = 0; i < ARRAY_SIZE(evl_phy_value); i++) {
				gani_mii_write_raw(dp,
				    0x05, evl_phy_value[i]);
				val = (0xAA << 8)
				    | (gani_mii_read_raw(dp, 0x06) & 0xFF);
				gani_mii_write_raw(dp, 0x06, val);
			}
			gani_mii_write_raw(dp, 0x1F, 0x0007);
			gani_mii_write_raw(dp, 0x1E, 0x0078);
			gani_mii_write_raw(dp, 0x17, 0x51AA);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B54);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8B5D);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8A7C);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A7F);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) | BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A82);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A85);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A88);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B85);
		val = gani_mii_read_raw(dp, 0x06) | BIT_15;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		break;

	case R8168_17:
		if (aspm) {
			OUTB(dp, 0x6E, INB(dp, 0x6E) | BIT_6);
			gani_eri_write(dp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		val = gani_mii_read_raw(dp, 0x15);
		val &= ~(BIT_12);
		gani_mii_write_raw(dp, 0x15, val);
		gani_mii_write_raw(dp, 0x00, 0x4800);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002f);
		for (i = 0; i < 1000; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x1c);
			if ((val & 0x0080) == 0x0080)
				break;
		}
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x17);
			if (!(val & 0x0001))
				break;
		}

		GANI_PHY_PATCH(gani_8168_m17_patch0);

		if (lp->svid == 0x144d && lp->ssid == 0xc0a6) {
			gani_mii_write_raw(dp, 0x18, 0x0724);
			gani_mii_write_raw(dp, 0x19, 0xfe00);
			gani_mii_write_raw(dp, 0x18, 0x0734);
			gani_mii_write_raw(dp, 0x19, 0xfd00);
			gani_mii_write_raw(dp, 0x18, 0x1824);
			gani_mii_write_raw(dp, 0x19, 0xfc00);
			gani_mii_write_raw(dp, 0x18, 0x1834);
			gani_mii_write_raw(dp, 0x19, 0xfd00);
		}

		GANI_PHY_PATCH(gani_8168_m17_patch1);

		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x05, 0xe140);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x1f, 0x0005);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x00);
			if (val & BIT_7) {
				break;
			}
		}
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0042);
		gani_mii_write_raw(dp, 0x18, 0x2300);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);

		if ((lp->svid == 0x144d && lp->ssid == 0xc098) ||
		    (lp->svid == 0x144d && lp->ssid == 0xc0b1)) {
			val = gani_mii_read_raw(dp, 0x17);
			val &= ~BIT_2;
			gani_mii_write_raw(dp, 0x17, val);
		}
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x9200);

		if (lp->svid == 0x144d && lp->ssid == 0xc0a6) {
			gani_mii_write_raw(dp, 0x1F, 0x0001);
			gani_mii_write_raw(dp, 0x0e, 0x6b7f);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8B86);
			val = gani_mii_read_raw(dp, 0x06);
			val |= BIT_4;
			gani_mii_write_raw(dp, 0x06, val);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
		} else {
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8B80);
			val = gani_mii_read_raw(dp, 0x06);
			val |= BIT_2 | BIT_1;
			gani_mii_write_raw(dp, 0x06, val);
			gani_mii_write_raw(dp, 0x1f, 0x0000);

			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x05, 0x8B86);
			val = gani_mii_read_raw(dp, 0x06);
			val &= ~BIT_4;
			gani_mii_write_raw(dp, 0x06, val);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0004);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002D);
		val = gani_mii_read_raw(dp, 0x18);
		val |= BIT_4;
		gani_mii_write_raw(dp, 0x18, val);
		gani_mii_write_raw(dp, 0x1f, 0x0002);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		val = gani_mii_read_raw(dp, 0x14);
		val |= BIT_15;
		gani_mii_write_raw(dp, 0x14, val);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B86);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0004);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x00AC);
		gani_mii_write_raw(dp, 0x18, 0x0006);
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0003);
		gani_mii_write_raw(dp, 0x09, 0xA20F);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B85);
		val = gani_mii_read_raw(dp, 0x06) | BIT_14 | BIT_15;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B5B);
		gani_mii_write_raw(dp, 0x06, 0x9222);
		gani_mii_write_raw(dp, 0x05, 0x8B6D);
		gani_mii_write_raw(dp, 0x06, 0x8000);
		gani_mii_write_raw(dp, 0x05, 0x8B76);
		gani_mii_write_raw(dp, 0x06, 0x8000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		if (lp->svid == 0x1043 && lp->ssid == 0x13F7) {

			static const uint16_t evl_phy_value[] = {
				0x8B56, 0x8B5F, 0x8B68, 0x8B71,
				0x8B7A, 0x8A7B, 0x8A7E, 0x8A81,
				0x8A84, 0x8A87 }
			;

			gani_mii_write_raw(dp, 0x1F, 0x0005);
			for (i = 0; i < ARRAY_SIZE(evl_phy_value); i++) {
				gani_mii_write_raw(dp, 0x05, evl_phy_value[i]);
				val = (0xAA << 8)
				    | (gani_mii_read_raw(dp, 0x06) & 0xFF);
				gani_mii_write_raw(dp, 0x06, val);
			}
			gani_mii_write_raw(dp, 0x1F, 0x0007);
			gani_mii_write_raw(dp, 0x1E, 0x0078);
			gani_mii_write_raw(dp, 0x17, 0x51AA);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B54);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8B5D);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8A7C);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A7F);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) | BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A82);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A85);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A88);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		if (aspm) {
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			val = gani_mii_read_raw(dp, 0x15);
			val |= BIT_12;
			gani_mii_write_raw(dp, 0x15, val);
		}
		break;

	case R8168_18:
		if (aspm) {
			OUTB(dp, 0x6E, INB(dp, 0x6E) | BIT_6);
			gani_eri_write(dp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		val = gani_mii_read_raw(dp, 0x15);
		val &= ~(BIT_12);
		gani_mii_write_raw(dp, 0x15, val);
		gani_mii_write_raw(dp, 0x00, 0x4800);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002f);
		for (i = 0; i < 1000; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x1c);
			if (val & 0x0080)
				break;
		}
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x18);
			if (!(val & 0x0001))
				break;
		}

		GANI_PHY_PATCH(gani_8168_m18_patch);

		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x05, 0xe140);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x1f, 0x0005);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x00);
			if (val & BIT_7)
				break;
		}
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		val = gani_mii_read_raw(dp, 0x17);
		val |= BIT_1;
		gani_mii_write_raw(dp, 0x17, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x9200);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8b80);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_2 | BIT_1;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002D);
		val = gani_mii_read_raw(dp, 0x18);
		val |= BIT_4;
		gani_mii_write_raw(dp, 0x18, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		val = gani_mii_read_raw(dp, 0x14);
		val |= BIT_15;
		gani_mii_write_raw(dp, 0x14, val);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B86);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B85);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_14;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0003);
		gani_mii_write_raw(dp, 0x09, 0xA20F);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B55);
		gani_mii_write_raw(dp, 0x06, 0x0000);
		gani_mii_write_raw(dp, 0x05, 0x8B5E);
		gani_mii_write_raw(dp, 0x06, 0x0000);
		gani_mii_write_raw(dp, 0x05, 0x8B67);
		gani_mii_write_raw(dp, 0x06, 0x0000);
		gani_mii_write_raw(dp, 0x05, 0x8B70);
		gani_mii_write_raw(dp, 0x06, 0x0000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x0078);
		gani_mii_write_raw(dp, 0x17, 0x0000);
		gani_mii_write_raw(dp, 0x19, 0x00FB);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B79);
		gani_mii_write_raw(dp, 0x06, 0xAA00);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0003);
		gani_mii_write_raw(dp, 0x01, 0x328A);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B54);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8B5D);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8A7C);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A7F);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) | BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A82);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A85);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A88);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8b85);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) | BIT_15);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		if (aspm) {
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			val = gani_mii_read_raw(dp, 0x15);
			val |= BIT_12;
			gani_mii_write_raw(dp, 0x15, val);
		}
		break;

	case R8168_19:
		if (aspm) {
			OUTB(dp, 0x6E, INB(dp, 0x6E) | BIT_6);
			gani_eri_write(dp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		val = gani_mii_read_raw(dp, 0x15);
		val &= ~(BIT_12);
		gani_mii_write_raw(dp, 0x15, val);
		gani_mii_write_raw(dp, 0x00, 0x9800);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002f);
		for (i = 0; i < 1000; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x1c);
			if (val & 0x0080)
				break;
		}

		GANI_PHY_PATCH(gani_8168_m19_patch);

		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x05, 0xe140);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		val = gani_mii_read_raw(dp, 0x17);
		val |= BIT_1;
		gani_mii_write_raw(dp, 0x17, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x9200);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8b80);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_2 | BIT_1;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002D);
		val = gani_mii_read_raw(dp, 0x18);
		val |= BIT_4;
		gani_mii_write_raw(dp, 0x18, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		val = gani_mii_read_raw(dp, 0x14);
		val |= BIT_15;
		gani_mii_write_raw(dp, 0x14, val);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B86);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B54);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8B5D);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8A7C);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A7F);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) | BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A82);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A85);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A88);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8b85);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) | BIT_15);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		if (aspm) {
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			val = gani_mii_read_raw(dp, 0x15);
			val |= BIT_12;
			gani_mii_write_raw(dp, 0x15, val);
		}
		break;

	case R8168_20:
		if (aspm) {
			OUTB(dp, 0x6E, INB(dp, 0x6E) | BIT_6);
			gani_eri_write(dp, 0x1AE, 2, 0x0403, ERIAR_ExGMAC);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		val = gani_mii_read_raw(dp, 0x15);
		val &= ~(BIT_12);
		gani_mii_write_raw(dp, 0x15, val);
		gani_mii_write_raw(dp, 0x00, 0x4800);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002f);
		for (i = 0; i < 1000; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x1c);
			if (val & 0x0080)
				break;
		}
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x18);
			if (!(val & 0x0001))
				break;
		}

		GANI_PHY_PATCH(gani_8168_m20_patch);

		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x05, 0xe140);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x1f, 0x0005);
		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x00);
			if (val & BIT_7)
				break;
		}
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		val = gani_mii_read_raw(dp, 0x17);
		val |= BIT_1;
		gani_mii_write_raw(dp, 0x17, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x9200);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8b80);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_2 | BIT_1;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x002D);
		val = gani_mii_read_raw(dp, 0x18);
		val |= BIT_4;
		gani_mii_write_raw(dp, 0x18, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		val = gani_mii_read_raw(dp, 0x14);
		val |= BIT_15;
		gani_mii_write_raw(dp, 0x14, val);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B86);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B85);
		val = gani_mii_read_raw(dp, 0x06);
		val |= BIT_14;
		gani_mii_write_raw(dp, 0x06, val);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0003);
		gani_mii_write_raw(dp, 0x09, 0xA20F);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B55);
		gani_mii_write_raw(dp, 0x06, 0x0000);
		gani_mii_write_raw(dp, 0x05, 0x8B5E);
		gani_mii_write_raw(dp, 0x06, 0x0000);
		gani_mii_write_raw(dp, 0x05, 0x8B67);
		gani_mii_write_raw(dp, 0x06, 0x0000);
		gani_mii_write_raw(dp, 0x05, 0x8B70);
		gani_mii_write_raw(dp, 0x06, 0x0000);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x0078);
		gani_mii_write_raw(dp, 0x17, 0x0000);
		gani_mii_write_raw(dp, 0x19, 0x00aa);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B79);
		gani_mii_write_raw(dp, 0x06, 0xAA00);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8B54);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8B5D);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_11);
		gani_mii_write_raw(dp, 0x05, 0x8A7C);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A7F);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) | BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A82);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A85);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x05, 0x8A88);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) & ~BIT_8);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0x8b85);
		gani_mii_write_raw(dp, 0x06,
		    gani_mii_read_raw(dp, 0x06) | BIT_15);
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		if (aspm) {
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			val = gani_mii_read_raw(dp, 0x15);
			val |= BIT_12;
			gani_mii_write_raw(dp, 0x15, val);
		}
		break;

	case R8168_21: {
		uint16_t rtl8111g_phy_value[] = {
			0xE008, 0xE01B, 0xE01D, 0xE01F, 0xE022,
			0xE025, 0xE031, 0xE04D, 0x49D2, 0xF10D,
			0x766C, 0x49E2, 0xF00A, 0x1EC0, 0x8EE1,
			0xC60A, 0x77C0, 0x4870, 0x9FC0, 0x1EA0,
			0xC707, 0x8EE1, 0x9D6C, 0xC603, 0xBE00,
			0xB416, 0x0076, 0xE86C, 0xC602, 0xBE00,
			0xA000, 0xC602, 0xBE00, 0x0000, 0x1B76,
			0xC202, 0xBA00, 0x059C, 0x1B76, 0xC602,
			0xBE00, 0x065A, 0x74E6, 0x1B78, 0x46DC,
			0x1300, 0xF005, 0x74F8, 0x48C3, 0x48C4,
			0x8CF8, 0x64E7, 0xC302, 0xBB00, 0x06A0,
			0x74E4, 0x49C5, 0xF106, 0x49C6, 0xF107,
			0x48C8, 0x48C9, 0xE011, 0x48C9, 0x4848,
			0xE00E, 0x4848, 0x49C7, 0xF00A, 0x48C9,
			0xC60D, 0x1D1F, 0x8DC2, 0x1D00, 0x8DC3,
			0x1D11, 0x8DC0, 0xE002, 0x4849, 0x94E5,
			0xC602, 0xBE00, 0x01F0, 0xE434, 0x49D9,
			0xF01B, 0xC31E, 0x7464, 0x49C4, 0xF114,
			0xC31B, 0x6460, 0x14FA, 0xFA02, 0xE00F,
			0xC317, 0x7460, 0x49C0, 0xF10B, 0xC311,
			0x7462, 0x48C1, 0x9C62, 0x4841, 0x9C62,
			0xC30A, 0x1C04, 0x8C60, 0xE004, 0x1C15,
			0xC305, 0x8C60, 0xC602, 0xBE00, 0x0384,
			0xE434, 0xE030, 0xE61C, 0xE906 }
		;

		gani_mac_ocp_write(dp, 0xFC28, 0x0000);
		gani_mac_ocp_write(dp, 0xFC2A, 0x0000);
		gani_mac_ocp_write(dp, 0xFC2C, 0x0000);
		gani_mac_ocp_write(dp, 0xFC2E, 0x0000);
		gani_mac_ocp_write(dp, 0xFC30, 0x0000);
		gani_mac_ocp_write(dp, 0xFC32, 0x0000);
		gani_mac_ocp_write(dp, 0xFC34, 0x0000);
		gani_mac_ocp_write(dp, 0xFC36, 0x0000);
		drv_usecwait(3000);
		gani_mac_ocp_write(dp, 0xFC26, 0x0000);
		for (i = 0; i < ARRAY_SIZE(rtl8111g_phy_value); i++) {
			gani_mac_ocp_write(dp,
			    0xF800 + i * 2, rtl8111g_phy_value[i]);
		}
		gani_mac_ocp_write(dp, 0xFC26, 0x8000);
		gani_mac_ocp_write(dp, 0xFC28, 0x0075);
		gani_mac_ocp_write(dp, 0xFC2E, 0x059B);
		gani_mac_ocp_write(dp, 0xFC30, 0x0659);
		gani_mac_ocp_write(dp, 0xFC32, 0x069F);
		gani_mac_ocp_write(dp, 0xFC34, 0x01CD);
		gani_mac_ocp_write(dp, 0xFC36, 0x0303);

		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_6 | BIT_7);
		OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_6);
		gani_eri_write(dp, 0x5F0, 2, 0x4f87, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0xD4, 2, ERIAR_ExGMAC);
		val |= (0x3F << 7);
		gani_eri_write(dp, 0xD4, 2, val, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x1D0, 1, ERIAR_ExGMAC);
		val |= BIT_1;
		gani_eri_write(dp, 0x1D0, 1, val, ERIAR_ExGMAC);

		gani_mii_write_raw(dp, 0x1f, 0x0B82);
		val = gani_mii_read_raw(dp, 0x10);
		val |= BIT_4;
		gani_mii_write_raw(dp, 0x10, val);
		gani_mii_write_raw(dp, 0x1f, 0x0B80);
		for (i = 0; i < 10; i++) {
			if (gani_mii_read_raw(dp, 0x10) & 0x0040)
				break;
			delay(drv_usectohz(10000));
		}

		GANI_PHY_PATCH(gani_8168_m21_patch);

		val = gani_mii_read_raw(dp, 0x10);
		val &= ~(BIT_4);
		gani_mii_write_raw(dp, 0x10, val);

		gani_mii_write_raw(dp, 0x1F, 0x0A46);
		val = gani_mii_read_raw(dp, 0x10);
		gani_mii_write_raw(dp, 0x1F, 0x0BCC);
		if (val & BIT_8)
			val = gani_mii_read_raw(dp, 0x12) & ~BIT_15;
		else
			val = gani_mii_read_raw(dp, 0x12) | BIT_15;
		gani_mii_write_raw(dp, 0x1F, 0x0A46);
		val = gani_mii_read_raw(dp, 0x13);
		gani_mii_write_raw(dp, 0x1F, 0x0C41);
		if (val & BIT_8)
			val = gani_mii_read_raw(dp, 0x15) | BIT_1;
		else
			val = gani_mii_read_raw(dp, 0x15) & ~BIT_1;

		gani_mii_write_raw(dp, 0x1F, 0x0A44);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | BIT_2 | BIT_3);

		gani_mii_write_raw(dp, 0x1F, 0x0BCC);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) & ~BIT_8);
		gani_mii_write_raw(dp, 0x1F, 0x0A44);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | BIT_7);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | BIT_6);
		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		gani_mii_write_raw(dp, 0x13, 0x8084);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) & ~(BIT_14 | BIT_13));
		gani_mii_write_raw(dp, 0x10,
		    gani_mii_read_raw(dp, 0x10) | BIT_12);
		gani_mii_write_raw(dp, 0x10,
		    gani_mii_read_raw(dp, 0x10) | BIT_1);
		gani_mii_write_raw(dp, 0x10,
		    gani_mii_read_raw(dp, 0x10) | BIT_0);

		gani_mii_write_raw(dp, 0x1F, 0x0A4B);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | BIT_2);

		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		gani_mii_write_raw(dp, 0x13, 0x8012);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) | BIT_15);

		gani_mii_write_raw(dp, 0x1F, 0x0C42);
		val = gani_mii_read_raw(dp, 0x11);
		val |= BIT_14;
		val &= ~BIT_13;
		gani_mii_write_raw(dp, 0x11, val);

		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		gani_mii_write_raw(dp, 0x13, 0x809A);
		gani_mii_write_raw(dp, 0x14, 0x8022);
		gani_mii_write_raw(dp, 0x13, 0x80A0);
		val = gani_mii_read_raw(dp, 0x14) & 0x00FF;
		val |= 0x1000;
		gani_mii_write_raw(dp, 0x14, val);
		gani_mii_write_raw(dp, 0x13, 0x8088);
		gani_mii_write_raw(dp, 0x14, 0x9222);

		gani_mii_write_raw(dp, 0x1F, 0x0BCD);
		gani_mii_write_raw(dp, 0x14, 0x5065);
		gani_mii_write_raw(dp, 0x14, 0xD065);
		gani_mii_write_raw(dp, 0x1F, 0x0BC8);
		gani_mii_write_raw(dp, 0x11, 0x5655);
		gani_mii_write_raw(dp, 0x1F, 0x0BCD);
		gani_mii_write_raw(dp, 0x14, 0x1065);
		gani_mii_write_raw(dp, 0x14, 0x9065);
		gani_mii_write_raw(dp, 0x14, 0x1065);

		if (aspm) {
			gani_mii_write_raw(dp, 0x1F, 0x0A43);
			gani_mii_write_raw(dp, 0x10,
			    gani_mii_read_raw(dp, 0x10) | BIT_2);
		}

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		break;
	}
	case R8168_22:
		/* 035 */
		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_6 | BIT_7);
		OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_6);
		gani_eri_write(dp, 0x5F0, 2, 0x4f87, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x1D0, 1, ERIAR_ExGMAC);
		val |= BIT_1;
		gani_eri_write(dp, 0x1D0, 1, val, ERIAR_ExGMAC);

		gani_mii_write_raw(dp, 0x1F, 0x0BCD);
		gani_mii_write_raw(dp, 0x14, 0x5065);
		gani_mii_write_raw(dp, 0x14, 0xD065);
		gani_mii_write_raw(dp, 0x1F, 0x0BC8);
		gani_mii_write_raw(dp, 0x11, 0x5655);
		gani_mii_write_raw(dp, 0x1F, 0x0BCD);
		gani_mii_write_raw(dp, 0x14, 0x1065);
		gani_mii_write_raw(dp, 0x14, 0x9065);
		gani_mii_write_raw(dp, 0x14, 0x1065);
		break;

	case R8168_23:
		/* 035 */
		gani_mii_write_raw(dp, 0x1F, 0x0BCC);
		val = gani_mii_read_raw(dp, 0x14);
		val |= BIT_8;
		gani_mii_write_raw(dp, 0x14, val);
		gani_mii_write_raw(dp, 0x1F, 0x0A44);
		val = gani_mii_read_raw(dp, 0x11);
		val |= BIT_7 | BIT_6;
		gani_mii_write_raw(dp, 0x11, val);
		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		gani_mii_write_raw(dp, 0x13, 0x8084);
		val = gani_mii_read_raw(dp, 0x14);
		val &= ~(BIT_14 | BIT_13);
		gani_mii_write_raw(dp, 0x14, val);
		val = gani_mii_read_raw(dp, 0x10);
		val |= BIT_12;
		gani_mii_write_raw(dp, 0x10, val);
		val = gani_mii_read_raw(dp, 0x10);
		val |= BIT_2 | BIT_1 | BIT_0;
		gani_mii_write_raw(dp, 0x10, val);

		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		gani_mii_write_raw(dp, 0x13, 0x809C);
		gani_mii_write_raw(dp, 0x14, 0xA700);
		gani_mii_write_raw(dp, 0x13, 0x80A5);
		gani_mii_write_raw(dp, 0x14, 0xA700);
		break;

	case R8168_24: {
		uint16_t rtl8111g_phy_value[] = {
			0xE008, 0xE011, 0xE015, 0xE018, 0xE01B,
			0xE027, 0xE043, 0xE065, 0x49E2, 0xF005,
			0x49EA, 0xF003, 0xC404, 0xBC00, 0xC403,
			0xBC00, 0x0496, 0x051A, 0x1D01, 0x8DE8,
			0xC602, 0xBE00, 0x0206, 0x1B76, 0xC202,
			0xBA00, 0x058A, 0x1B76, 0xC602, 0xBE00,
			0x0648, 0x74E6, 0x1B78, 0x46DC, 0x1300,
			0xF005, 0x74F8, 0x48C3, 0x48C4, 0x8CF8,
			0x64E7, 0xC302, 0xBB00, 0x068E, 0x74E4,
			0x49C5, 0xF106, 0x49C6, 0xF107, 0x48C8,
			0x48C9, 0xE011, 0x48C9, 0x4848, 0xE00E,
			0x4848, 0x49C7, 0xF00A, 0x48C9, 0xC60D,
			0x1D1F, 0x8DC2, 0x1D00, 0x8DC3, 0x1D11,
			0x8DC0, 0xE002, 0x4849, 0x94E5, 0xC602,
			0xBE00, 0x0238, 0xE434, 0x49D9, 0xF01B,
			0xC31E, 0x7464, 0x49C4, 0xF114, 0xC31B,
			0x6460, 0x14FA, 0xFA02, 0xE00F, 0xC317,
			0x7460, 0x49C0, 0xF10B, 0xC311, 0x7462,
			0x48C1, 0x9C62, 0x4841, 0x9C62, 0xC30A,
			0x1C04, 0x8C60, 0xE004, 0x1C15, 0xC305,
			0x8C60, 0xC602, 0xBE00, 0x0374, 0xE434,
			0xE030, 0xE61C, 0xE906, 0xC602, 0xBE00,
			0x0000 }
		;

		gani_mac_ocp_write(dp, 0xFC28, 0x0000);
		gani_mac_ocp_write(dp, 0xFC2A, 0x0000);
		gani_mac_ocp_write(dp, 0xFC2C, 0x0000);
		gani_mac_ocp_write(dp, 0xFC2E, 0x0000);
		gani_mac_ocp_write(dp, 0xFC30, 0x0000);
		gani_mac_ocp_write(dp, 0xFC32, 0x0000);
		gani_mac_ocp_write(dp, 0xFC34, 0x0000);
		gani_mac_ocp_write(dp, 0xFC36, 0x0000);
		drv_usectohz(3000);
		gani_mac_ocp_write(dp, 0xFC26, 0x0000);
		for (i = 0; i < ARRAY_SIZE(rtl8111g_phy_value); i++) {
			gani_mac_ocp_write(dp,
			    0xF800 + i * 2, rtl8111g_phy_value[i]);
		}
		gani_mac_ocp_write(dp, 0xFC26, 0x8000);
		gani_mac_ocp_write(dp, 0xFC28, 0x0493);
		gani_mac_ocp_write(dp, 0xFC2A, 0x0205);
		gani_mac_ocp_write(dp, 0xFC2C, 0x0589);
		gani_mac_ocp_write(dp, 0xFC2E, 0x0647);
		gani_mac_ocp_write(dp, 0xFC30, 0x068D);
		gani_mac_ocp_write(dp, 0xFC32, 0x0215);
		gani_mac_ocp_write(dp, 0xFC34, 0x0285);

		val = gani_eri_read(dp, 0x3F2, 2, ERIAR_ExGMAC);
		val &= ~(BIT_8 | BIT_11 | BIT_12);
		val |= (BIT_9 | BIT_10 | BIT_13 | BIT_14 | BIT_15);
		gani_eri_write(dp, 0x3F2, 2, val, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x3F5, 1, ERIAR_ExGMAC);
		val |= BIT_6 | BIT_7;
		gani_eri_write(dp, 0x3F5, 1, val, ERIAR_ExGMAC);
		gani_mac_ocp_write(dp, 0xE02C, 0x1880);
		gani_mac_ocp_write(dp, 0xE02E, 0x4880);
		gani_eri_write(dp, 0x2E8, 2, 0x9003, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x2EA, 2, 0x9003, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x2EC, 2, 0x9003, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x2E2, 2, 0x883C, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x2E4, 2, 0x8C12, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x2E6, 2, 0x9003, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x3F2, 2, ERIAR_ExGMAC);
		val |= BIT_0 | BIT_1;
		gani_eri_write(dp, 0x3F2, 2, val, ERIAR_ExGMAC);
		gani_eri_write(dp, 0x1E2, 2, 0x04, ERIAR_ExGMAC);

		OUTB(dp, 0xD0, INB(dp, 0xD0) | BIT_6 | BIT_7);
		OUTB(dp, 0xF2, INB(dp, 0xF2) | BIT_6);
		gani_eri_write(dp, 0x5F0, 2, 0x4f87, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0xD4, 2, ERIAR_ExGMAC);
		val |= (0x3F << 7);
		gani_eri_write(dp, 0xD4, 2, val, ERIAR_ExGMAC);
		val = gani_eri_read(dp, 0x1D0, 1, ERIAR_ExGMAC);
		val |= BIT_1;
		gani_eri_write(dp, 0x1D0, 1, val, ERIAR_ExGMAC);

		gani_mii_write_raw(dp, 0x1F, 0x0BCC);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) & ~BIT_8);
		gani_mii_write_raw(dp, 0x1F, 0x0A44);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | BIT_7);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | BIT_6);
		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		gani_mii_write_raw(dp, 0x13, 0x8084);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) & ~(BIT_14 | BIT_13));
		gani_mii_write_raw(dp, 0x10,
		    gani_mii_read_raw(dp, 0x10) | BIT_12);
		gani_mii_write_raw(dp, 0x10,
		    gani_mii_read_raw(dp, 0x10) | BIT_1);
		gani_mii_write_raw(dp, 0x10,
		    gani_mii_read_raw(dp, 0x10) | BIT_0);
		gani_mii_write_raw(dp, 0x1F, 0x0A43);
		gani_mii_write_raw(dp, 0x13, 0x8012);
		gani_mii_write_raw(dp, 0x14,
		    gani_mii_read_raw(dp, 0x14) | BIT_15);
		gani_mii_write_raw(dp, 0x1F, 0x0C42);
		val = gani_mii_read_raw(dp, 0x11);
		val |= BIT_14;
		val &= ~BIT_13;
		gani_mii_write_raw(dp, 0x11, val);
		if (aspm) {
			gani_mii_write_raw(dp, 0x1F, 0x0A43);
			gani_mii_write_raw(dp, 0x10,
			    gani_mii_read_raw(dp, 0x10) | BIT_2);
		}
		break;
	}

	case R8101_4: /* RTL8102E */
	case R8101_5: /* RTL8102E rev.B */
		/* 023 */
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | 0x1000);
		gani_mii_write_raw(dp, 0x19,
		    gani_mii_read_raw(dp, 0x19) | 0x2000);
		gani_mii_write_raw(dp, 0x10,
		    gani_mii_read_raw(dp, 0x10) | 0x8000);

		gani_mii_write_raw(dp, 0x1f, 0x0003);
		gani_mii_write_raw(dp, 0x08, 0x441D);
		gani_mii_write_raw(dp, 0x01, 0x9100);

		gani_mii_write_raw(dp, 0x1f, 0x0000);
		break;

	case R8101_6: /* RTL8103E */
		/* 015 */
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | 0x1000);
		gani_mii_write_raw(dp, 0x19,
		    gani_mii_read_raw(dp, 0x19) | 0x2000);
		gani_mii_write_raw(dp, 0x10,
		    gani_mii_read_raw(dp, 0x10) | 0x8000);

		gani_mii_write_raw(dp, 0x1f, 0x0003);
		gani_mii_write_raw(dp, 0x08, 0x441D);

		gani_mii_write_raw(dp, 0x1f, 0x0000);
		break;

	case R8101_7:
	case R8101_8:
		/* EMPTY */
		break;

	case R8101_9:
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x11,
		    gani_mii_read_raw(dp, 0x11) | BIT_12);
		gani_mii_write_raw(dp, 0x1F, 0x0002);
		gani_mii_write_raw(dp, 0x0F,
		    gani_mii_read_raw(dp, 0x0F) | BIT_0 | BIT_1);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_phyio_write(dp, 0x0E, 0x0068);
		gani_phyio_write(dp, 0x0E, 0x0069);
		gani_phyio_write(dp, 0x0E, 0x006A);
		gani_phyio_write(dp, 0x0E, 0x006B);
		gani_phyio_write(dp, 0x0E, 0x006C);
		break;

	case R8101_10:
		OUTB(dp, 0xF3, INB(dp, 0xF3) | BIT_2);

		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x00, 0x1800);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1e, 0x0023);
		gani_mii_write_raw(dp, 0x17, 0x0117);
		gani_mii_write_raw(dp, 0x1f, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x002C);
		gani_mii_write_raw(dp, 0x1B, 0x5000);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x16, 0x4104);

		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x1E);
			val &= 0x03FF;
			if (val == 0x000C) {
				break;
			}
		}

		gani_mii_write_raw(dp, 0x1f, 0x0005);

		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x07);
			if ((val & BIT_5) == 0) {
				break;
			}
		}

		val = gani_mii_read_raw(dp, 0x07);
		if (val & BIT_5) {
			gani_mii_write_raw(dp, 0x1f, 0x0007);
			gani_mii_write_raw(dp, 0x1e, 0x00a1);
			gani_mii_write_raw(dp, 0x17, 0x1000);
			gani_mii_write_raw(dp, 0x17, 0x0000);
			gani_mii_write_raw(dp, 0x17, 0x2000);
			gani_mii_write_raw(dp, 0x1e, 0x002f);
			gani_mii_write_raw(dp, 0x18, 0x9bfb);
			gani_mii_write_raw(dp, 0x1f, 0x0005);
			gani_mii_write_raw(dp, 0x07, 0x0000);
			gani_mii_write_raw(dp, 0x1f, 0x0000);
		}

		gani_mii_write_raw(dp, 0x1f, 0x0005);
		gani_mii_write_raw(dp, 0x05, 0xfff6);
		gani_mii_write_raw(dp, 0x06, 0x0080);

		val = gani_mii_read_raw(dp, 0x00);
		val &= ~(BIT_7);
		gani_mii_write_raw(dp, 0x00, val);

		gani_mii_write_raw(dp, 0x1f, 0x0002);

		val = gani_mii_read_raw(dp, 0x08);
		val &= ~(BIT_7);
		gani_mii_write_raw(dp, 0x08, val);

		GANI_PHY_PATCH(gani_8101_m10_patch0);

		val = gani_mii_read_raw(dp, 0x01);
		val |= BIT_0;
		gani_mii_write_raw(dp, 0x01, val);

		gani_mii_write_raw(dp, 0x00, 0x0005);
		gani_mii_write_raw(dp, 0x1f, 0x0000);
		gani_mii_write_raw(dp, 0x1f, 0x0005);

		for (i = 0; i < 200; i++) {
			drv_usecwait(100);
			val = gani_mii_read_raw(dp, 0x00);
			if (val & BIT_7) {
				break;
			}
		}

		GANI_PHY_PATCH(gani_8101_m10_patch1);

		val = gani_mii_read_raw(dp, 0x08) & 0x00FF;
		gani_mii_write_raw(dp, 0x08, val | 0x8000);

		gani_mii_write_raw(dp, 0x1F, 0x0007);
		gani_mii_write_raw(dp, 0x1E, 0x002D);
		val = gani_mii_read_raw(dp, 0x18);
		gani_mii_write_raw(dp, 0x18, val | 0x0010);
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		val = gani_mii_read_raw(dp, 0x14);
		gani_mii_write_raw(dp, 0x14, val | 0x8000);

		GANI_PHY_PATCH(gani_8101_m10_patch2);

		break;

	case R8101_11:
	case R8101_12:
	case R8101_13:
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x18, 0x0310);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		delay(drv_usectohz(100000));

		if (lp->chip->type == R8101_13 &&
		    (INB(dp, 0xEF) & BIT_2)) {
			gani_mii_write_raw(dp, 0x1F, 0x0001);
			val = gani_mii_read_raw(dp, 0x1B) | BIT_2;
			gani_mii_write_raw(dp, 0x1B, val);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		}

		GANI_PHY_PATCH(gani_8101_m11_patch);

		if (INB(dp, 0xEF) & 0x08) {
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x1A, 0x0004);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		} else {
			gani_mii_write_raw(dp, 0x1F, 0x0005);
			gani_mii_write_raw(dp, 0x1A, 0x0000);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		}

		if (INB(dp, 0xEF) & 0x010) {
			gani_mii_write_raw(dp, 0x1F, 0x0004);
			gani_mii_write_raw(dp, 0x1C, 0x0000);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		} else {
			gani_mii_write_raw(dp, 0x1F, 0x0004);
			gani_mii_write_raw(dp, 0x1C, 0x0200);
			gani_mii_write_raw(dp, 0x1F, 0x0000);
		}

		gani_mii_write_raw(dp, 0x1F, 0x0001);
		gani_mii_write_raw(dp, 0x15, 0x7701);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x18, 0x8310);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		break;

	case R8101_14:
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x18, 0x0310);
		gani_mii_write_raw(dp, 0x1F, 0x0000);

		delay(drv_usectohz(100000));

		GANI_PHY_PATCH(gani_8101_m14_patch);

		if (INL(dp, 0x8c) & BIT_28) {
			gani_mii_write_raw(dp, 0x1F, 0x0001);
			val = gani_mii_read_raw(dp, 0x13);
			val |= 0x8000;
			gani_mii_write_raw(dp, 0x13, val);
		}

		break;

	case R8101_15:
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x18, 0x0310);

		delay(drv_usectohz(20000));

		gani_mii_write_raw(dp, 0x1F, 0x0001);
		gani_mii_write_raw(dp, 0x11, 0x83BA);

		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x18, 0x8310);

		break;

	case R8101_16:
		gani_mii_write_raw(dp, 0x1F, 0x0000);
		gani_mii_write_raw(dp, 0x18, 0x0310);

		delay(drv_usectohz(20000));

		GANI_PHY_PATCH(gani_8101_m16_patch);

		break;
	}

	if (GANI_PCIE_GIGA(lp)) {
		if (gani_eee_enable) {
			gani_enable_EEE(dp);
		} else {
			gani_disable_EEE(dp);
		}
	}
}

static int
gani_mii_probe(struct gem_dev *dp)
{
	uint_t	val;
	int	ret = GEM_SUCCESS;
	struct gani_dev	*lp = dp->private;

	sema_p(&dp->hal_op_lock);

	/* select page 0 */
	gani_mii_write_raw(dp, 0x1f, 0x0000);

	dp->mii_status = gani_mii_read_raw(dp, MII_STATUS);
	if (dp->mii_status == 0xffff || dp->mii_status == 0) {
		cmn_err(CE_NOTE,
		    "!%s: failed to probe default internal and/or non-MII PHY",
		    dp->name);
		ret = GEM_FAILURE;
		goto x;
	}

	/* ensure phy is powered up */
	val = gani_mii_read_raw(dp, MII_CONTROL);
	gani_mii_write_raw(dp, MII_CONTROL,
	    val & (MII_CONTROL_SPEED | MII_CONTROL_ANE));

	dp->mii_status_ro = ~dp->mii_status;
	dp->mii_phy_id  = (gani_mii_read_raw(dp, MII_PHYIDH) << 16) |
	    gani_mii_read_raw(dp, MII_PHYIDL);

	cmn_err(CE_CONT, "!%s: using internal/non-MII PHY(0x%08x)",
	    dp->name, dp->mii_phy_id);

	cmn_err(CE_CONT,
	    "!%s: PHY control:%b, status:%b, advert:%b, lpar:%b, exp:%b",
	    dp->name,
	    gani_mii_read_raw(dp, MII_CONTROL), MII_CONTROL_BITS,
	    dp->mii_status, MII_STATUS_BITS,
	    gani_mii_read_raw(dp, MII_AN_ADVERT), MII_ABILITY_BITS,
	    gani_mii_read_raw(dp, MII_AN_LPABLE), MII_ABILITY_BITS,
	    gani_mii_read_raw(dp, MII_AN_EXPANSION), MII_AN_EXP_BITS);

	dp->mii_xstatus = 0;
	if (dp->mii_status & MII_STATUS_XSTATUS) {
		dp->mii_xstatus = gani_mii_read_raw(dp, MII_XSTATUS);

		cmn_err(CE_CONT, "!%s: xstatus:%b",
		    dp->name, dp->mii_xstatus, MII_XSTATUS_BITS);

		dp->mii_xstatus &=
		    ~ (MII_XSTATUS_1000BASET | MII_XSTATUS_1000BASEX);
	}
	dp->mii_xstatus_ro = ~dp->mii_xstatus;
x:
	sema_v(&dp->hal_op_lock);
	return (GEM_SUCCESS);
}

static void
gani_phy_power_up(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	/* select page 0 */
	gani_mii_write_raw(dp, 0x1F, 0x0000);

	if (GANI_PCI(lp)) {
		gani_mii_write_raw(dp, 0x0E, 0x0000);
	} else {
		switch (lp->chip->type) {
		case R8168_1:
		case R8168_2:
		case R8168_3:
		case R8168_4:
		case R8168_5:
		case R8168_6:
		case R8168_7:
		case R8168_8:
		case R8168_9:
		case R8168_10:
		case R8168_11:
		case R8168_12:
		case R8168_13:
			gani_mii_write_raw(dp, 0x0E, 0x0000);
			break;

		default:
			/* all 8101 chipset */
			/* EMPTY */
			break;
		}
		gani_mii_write_raw(dp, MII_CONTROL, MII_CONTROL_ANE);
	}
}

static int
gani_phy_power_down(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	/* select page 0 */
	gani_mii_write_raw(dp, 0x1F, 0x0000);

	switch (lp->chip->type) {
	case R8168_1:
	case R8168_2:
	case R8168_3:
	case R8168_4:
	case R8168_5:
	case R8168_6:
	case R8168_7:
	case R8168_8:
	case R8168_9:
	case R8168_10:
	case R8168_11:
	case R8168_12:
	case R8168_13:
		gani_mii_write_raw(dp, 0x0e, 0x0200);
		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_PWRDN);
		break;

	case R8168_14:
	case R8168_15:
	case R8101_10:
		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_ANE | MII_CONTROL_PWRDN);
		break;

	default:
		/* including all 8169 chipset */
		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_PWRDN);
		break;
	}
}

static void
gani_powerup_pll(struct gem_dev *dp)
{
	struct gani_dev	*lp = dp->private;

	if (GANI_PCI(lp)) {
		/* EMPTY */
	}

	if (GANI_PCIE_GIGA(lp)) {
		switch (lp->chip->type) {
		case R8168_11:
		case R8168_12:
		case R8168_13:
		case R8168_23:
			gani_driver_start(dp);
			/* FALLTHRU */
		case R8168_9:
		case R8168_10:
		case R8168_14:
		case R8168_15:
			OUTB(dp, PMCH, INB(dp, PMCH) | BIT_7);
			break;
		case R8168_17:
		case R8168_18:
		case R8168_19:
			OUTB(dp, PMCH, INB(dp, PMCH) | BIT_7 | BIT_6);
			break;
		case R8168_21:
		case R8168_22:
		case R8168_24:
			OUTB(dp, PMCH, INB(dp, PMCH) | BIT_6);
			break;
		}
	}

	if (GANI_PCIE_FAST(lp)) {
		/* power up pll */
		switch (lp->chip->type) {
		case R8101_6:
		case R8101_9:
			OUTB(dp, PMCH, INB(dp, PMCH) | BIT_7);
			OUTB(dp, DBGREG, INB(dp, DBGREG) & ~BIT_3);
			break;
		case R8101_7:
		case R8101_8:
		case R8101_10:
		case R8101_11:
		case R8101_12:
		case R8101_13:
		case R8101_14:
		case R8101_15:
		case R8101_16:
			OUTB(dp, PMCH, INB(dp, PMCH) | BIT_7);
			break;
		}
	}

	/* XXX - Doesn't it cause link down? */
	gani_phy_power_up(dp);
}

static void
gani_powerdown_pll(struct gem_dev *dp)
{
	uint16_t	auto_nego;
	uint16_t	giga_ctrl;
	uint16_t	val;
	uint_t		csi_tmp;
	struct gani_dev	*lp = dp->private;

	if ((lp->chip->type == R8168_11 || lp->chip->type == R8168_12 ||
	    lp->chip->type == R8168_13 || lp->chip->type == R8168_23)
	    && gani_check_dash(dp)) {
		gani_driver_stop(dp);
		return;
	}

	if (((lp->chip->type == R8168_7) || (lp->chip->type == R8168_8))
	    && (INW(dp, CpCR) & CpCR_ASF)) {
		return;
	}

	if (lp->chip->type == R8168_16 || lp->chip->type == R8168_17) {
		OUTB(dp, CFG2, INB(dp, CFG2) | BIT_5);
	}

	if ((lp->chip->type == R8101_11 || lp->chip->type == R8101_12 ||
	     lp->chip->type == R8101_13) && (gani_eee_enable == 1)) {
		gani_disable_EEE(dp);
	}

	if (lp->chip->type == R8101_13) {
		if ((INB(dp, 0x8c) & BIT_28) && !(INB(dp, 0xEF) & BIT_2)) {
			uint32_t gphy_val;

			gani_mii_write_raw(dp, 0x1F, 0x0000);
			gani_mii_write_raw(dp, 0x04, 0x0061);
			gani_mii_write_raw(dp, 0x00, 0x1200);
			gani_mii_write_raw(dp, 0x18, 0x0310);
			delay(drv_usectohz(20000));
			gani_mii_write_raw(dp, 0x1F, 0x0005);

			gphy_val = gani_mii_read_raw(dp, 0x1a);
			gphy_val |= BIT_8 | BIT_0;
			gani_mii_write_raw(dp, 0x1a, gphy_val);

			delay(drv_usectohz(30000));

			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp, 0x18, 0x8310);
		}
	}

	if (lp->wol_enabled) {

		gani_mii_write_raw(dp, 0x1F, 0x0000);

		auto_nego = gani_mii_read_raw(dp, MII_AN_ADVERT) &
		    ~(MII_ABILITY_100BASE_TX_FD
		    | MII_ABILITY_100BASE_TX
		    | MII_ABILITY_10BASE_T
		    | MII_ABILITY_10BASE_T_FD);

		val = gani_mii_read_raw(dp, MII_AN_LPABLE);

#ifdef CONFIG_DOWN_SPEED_100
		auto_nego |= (MII_ABILITY_100BASE_TX_FD
		    | MII_ABILITY_100BASE_TX
		    | MII_ABILITY_10BASE_T
		    | MII_ABILITY_10BASE_T_FD);

#else
		if (val & (MII_ABILITY_10BASE_T | MII_ABILITY_10BASE_T_FD)) {
			auto_nego |= (MII_ABILITY_10BASE_T | MII_ABILITY_10BASE_T_FD);
		} else {
			auto_nego |= (MII_ABILITY_100BASE_TX_FD
			    | MII_ABILITY_100BASE_TX
			    | MII_ABILITY_10BASE_T
			    | MII_ABILITY_10BASE_T_FD);
		}
#endif
		/* don't advertize giga speed capabilities */
		giga_ctrl = gani_mii_read_raw(dp, MII_1000TC)
		    & ~(MII_1000TC_ADV_HALF | MII_1000TC_ADV_FULL);

		gani_mii_write_raw(dp, MII_AN_ADVERT, auto_nego);
		gani_mii_write_raw(dp, MII_1000TC, giga_ctrl);
		gani_mii_write_raw(dp, MII_CONTROL,
		    MII_CONTROL_RESET | MII_CONTROL_ANE | MII_CONTROL_RSAN);

		switch (lp->chip->type) {
		case R8101_1:
		case R8101_2:
		case R8101_3:
		case R8101_4:
		case R8101_5:
		case R8101_6:
		case R8101_7:
		case R8101_8:
		case R8101_9:
			break;
		default:
			OUTL(dp, RCR,
			    INL(dp, RCR) | RCR_AB | RCR_AM | RCR_APM);
			break;
		}
		return;
	}
#if 0
	gani_phy_power_down(dp);
#endif
	if (GANI_PCIE(lp)) {
		switch (lp->chip->type) {
		case R8168_9:
		case R8168_10:
		case R8168_11:
		case R8168_12:
		case R8168_13:
		case R8168_14:
		case R8168_15:
			OUTB(dp, PMCH, INB(dp, PMCH) & ~BIT_7);
			break;

		case R8168_17:
		case R8168_18:
		case R8168_19:
			OUTB(dp, PMCH, INB(dp, PMCH) & ~(BIT_7 | BIT_6));
			break;

		case R8168_21:
		case R8168_22:
			OUTB(dp, PMCH, INB(dp, PMCH) & ~BIT_6);
			break;

		case R8168_24:
			gani_eri_write(dp, 0x3FC, 2, 0x0, ERIAR_ExGMAC);
			csi_tmp = gani_eri_read(dp, 0x3F8, 1, ERIAR_ExGMAC);
			csi_tmp |= BIT_0;
			gani_eri_write(dp, 0x3F8, 1, csi_tmp, ERIAR_ExGMAC);
			break;

		case R8101_6:
		case R8101_9:
			OUTB(dp, DBGREG, INB(dp, DBGREG) | BIT_3);
			OUTB(dp, PMCH, INB(dp, PMCH) & ~BIT_7);
			break;

		case R8101_8:
#ifdef notyet
			pci_conf_put8(conf_ha, 0x81, 0);
#endif
			OUTB(dp, PMCH, INB(dp, PMCH) & ~BIT_7);
			break;

		case R8101_7:
		case R8101_10:
		case R8101_11:
		case R8101_12:
		case R8101_13:
		case R8101_14:
		case R8101_15:
		case R8101_16:
			OUTB(dp, PMCH, INB(dp, PMCH) & ~BIT_7);
			break;

		default:
			break;
		}
	}
}

static int
gani_mii_init(struct gem_dev *dp)
{
	struct gani_dev		*lp = dp->private;

	/* install latest patch for phy */
	sema_p(&dp->hal_op_lock);

	if (GANI_PCI(lp)) {
		OUTB(dp, CR9346, CR9346_EEM_WE);
		OUTB(dp, CFG1, INB(dp, CFG1) | CFG1_PMEn);
		OUTB(dp, CFG5, INB(dp, CFG5) & CFG5_PME_STS);
		OUTB(dp, CR9346, CR9346_EEM_NORMAL);

		if (lp->busclk == BUSCLK_PCI66) {
			if (lp->chip->type == R8169_5) {
				OUTL(dp, 0x7c, 0x000fffff);
			} else if (lp->chip->type == R8169_6) {
				OUTL(dp, 0x7c, 0x003fffff);
			}
		} else if (lp->busclk == BUSCLK_PCI33) {
			if (lp->chip->type == R8169_5) {
				OUTL(dp, 0x7c, 0x000fff00);
			} else if (lp->chip->type == R8169_6) {
				OUTL(dp, 0x7c, 0x003fff00);
			}
		}
	}

	switch (lp->chip->type) {
	case R8168_11:
	case R8168_12:
	case R8168_13:
	case R8168_23:
		gani_driver_start(dp);
	}

	gani_phy_power_up(dp);

	gani_patch_phy(dp);

	if (GANI_PCI(lp)) {
		OUTB(dp, 0x82, 1);
		/* XXX - set pci latency timer to 0x40 */
		if (lp->chip->type == R8169_2 /* RTL8169S rev.D */) {
			OUTB(dp, 0x82, 1);
			gani_mii_write_raw(dp, 0x0b, 0x0000);
		}
	}

	sema_v(&dp->hal_op_lock);

	return (GEM_SUCCESS);
}

static int
gani_mii_config(struct gem_dev *dp)
{
	int		ret;
	uint16_t	val;
	uint_t		anadv_1000hdx_org;
	struct gani_dev	*lp = dp->private;

	/*
	 * rtl816x nics seem to have a problem in forced mode. The link
	 * never go up again after once it become down from up.
	 */
	if ((!dp->anadv_autoneg) && dp->speed == GEM_SPD_1000) {
		cmn_err(CE_NOTE,
		    "!%s: rtl816x series does not support fixed mode for "
		    "1Gbps connection. "
		    "limited auto negotiation capability is used "
		    "instead of given forced mode.",
		    dp->name);
		dp->anadv_1000fdx = B_TRUE;
		dp->anadv_1000hdx = B_FALSE;
		dp->anadv_100fdx = B_TRUE;
		dp->anadv_100hdx = B_TRUE;
		dp->anadv_10fdx = B_TRUE;
		dp->anadv_10hdx = B_TRUE;
#ifdef SANITY
		dp->full_duplex = B_TRUE;
#endif
		dp->anadv_autoneg = B_TRUE;
	}

	/*
	 * the datasheet says rtl816x doesn't support 1Gbps half duplex
	 * mode, but the phy unit in rtl8168 seem to require 1000hdx bit
	 * in 1000TC register set for 1G bps negotiation.
	 */
	anadv_1000hdx_org = dp->anadv_1000hdx;
	dp->anadv_1000hdx = dp->anadv_1000fdx;

	if (GANI_PCI(lp) &&
	    dp->anadv_autoneg && dp->anadv_1000fdx &&
	    (dp->anadv_100fdx | dp->anadv_100hdx |
	    dp->anadv_10fdx | dp->anadv_10hdx) == B_FALSE) {
		/*
		 * At least one capability in 100Mbps or 10Mbps should be
		 * enabled. Otherwise autonegotion will timeout.
		 */
		dp->anadv_100fdx = B_TRUE;
		dp->anadv_100hdx = B_TRUE;
		dp->anadv_10fdx = B_TRUE;
		dp->anadv_10hdx = B_TRUE;
	}

	sema_p(&dp->hal_op_lock);

	if (GANI_PCIE_GIGA(lp) && dp->anadv_autoneg) {
#ifdef notdef
		/* select page 0 */
		gani_mii_write_raw(dp, 0x1f, 0x0000);

		/* power up phy */
		switch (lp->chip->type) {
		case R8168_14:
		case R8168_15:
			break;
		default:
			gani_mii_write_raw(dp, 0x0e, 0x0000);
			break;
		}
		gani_mii_write_raw(dp, MII_CONTROL, MII_CONTROL_ANE);
#endif

	} else if (GANI_PCIE_FAST(lp) && dp->anadv_autoneg) {
		switch (lp->chip->type) {
		case R8101_1: /* RTL8101E rev.B */
		case R8101_2: /* RTL8101E rev.E */
		case R8101_3: /* RTL8101E rev.F */
			if (dp->anadv_100fdx || dp->anadv_100hdx) {
				break;
			}
			/* FALL THROU */

		case R8101_4: /* RTL8102E */
		case R8101_5: /* RTL8102E rev.B */
			gani_mii_write_raw(dp, 0x1f, 0x0000);
			gani_mii_write_raw(dp,
			    MII_CONTROL, MII_CONTROL_RESET);
			drv_usecwait(100);
			gani_patch_phy(dp);
			break;
		}
	}
	sema_v(&dp->hal_op_lock);
	ret = gem_mii_config_default(dp);
	sema_p(&dp->hal_op_lock);

	dp->anadv_1000hdx = anadv_1000hdx_org;

	if (lp->chip->type == R8169_2 /* RTL8169S rev.D */ ||
	    lp->chip->type == R8169_3 /* RTL8169S rev.E */) {
		if ((!dp->anadv_autoneg) && dp->speed == GEM_SPD_100) {
			gani_mii_write_raw(dp, 0x17, 0x2138);
			gani_mii_write_raw(dp, 0x0e, 0x0260);
		} else {
			gani_mii_write_raw(dp, 0x17, 0x2108);
			gani_mii_write_raw(dp, 0x0e, 0x0000);
		}
	}
	sema_v(&dp->hal_op_lock);
	return (ret);
}

/* ======================================================== */
/*
 * OS depend (device driver kernel interface) routine
 */
/* ======================================================== */
#define	gani_eeprom_delay(dp)	\
	{ (void) INB(dp, CR9346); drv_usecwait(3); }

static uint16_t
gani_read_eeprom(struct gem_dev *dp, int addr)
{
	int		i;
	int		addr_bits;
	uint_t		cmd;
	uint8_t		chip_select;
	uint8_t		di;
	uint16_t	ret;

	addr_bits = (INL(dp, RCR) & RCR_9356SEL) ? 8 : 6;

	DPRINTF(4, (CE_CONT, "!%s: %s: called: addr_bits:%d",
	    dp->name, __func__, addr_bits));

	/* make command bits */
	cmd = (6 << addr_bits) | addr;

	/* enable eeprom interface register */
	chip_select = INB(dp, CR9346);
	chip_select &= ~(CR9346_EEDI | CR9346_EEDO | CR9346_EEM | CR9346_EESK);
	chip_select |= CR9346_EEM_PROGRAM | CR9346_EECS;
	OUTB(dp, CR9346, chip_select);
	gani_eeprom_delay(dp);

	/* output eeprom command */
	for (i = 3 + addr_bits - 1; i >= 0; i--) {
		di = ((cmd >> i) << CR9346_EEDI_SHIFT) & CR9346_EEDI;
		OUTB(dp, CR9346, chip_select | di);
		gani_eeprom_delay(dp);

		OUTB(dp, CR9346, chip_select | di | CR9346_EESK);
		gani_eeprom_delay(dp);

		OUTB(dp, CR9346, chip_select | di);
		gani_eeprom_delay(dp);
	}

	/* get returned value */
#ifdef lint
	ret = ret;
#endif
	for (i = 16; i > 0; i--) {
		/* get 1 bit */
		OUTB(dp, CR9346, chip_select | CR9346_EESK);
		gani_eeprom_delay(dp);

		ret = (ret << 1) |
		    ((INB(dp, CR9346) >> CR9346_EEDO_SHIFT) & 1);

		OUTB(dp, CR9346, chip_select);
		gani_eeprom_delay(dp);
	}

	/* Terminate the EEPROM access. */
	chip_select &= ~CR9346_EECS;
	OUTB(dp, CR9346, chip_select | CR9346_EESK);
	gani_eeprom_delay(dp);
	OUTB(dp, CR9346, chip_select);
	gani_eeprom_delay(dp);

	chip_select &= ~CR9346_EEM;	/* i.e. CR9346_EEM_NORMAL */
	OUTB(dp, CR9346, chip_select);

	DPRINTF(4, (CE_CONT, "!%s: %s: returned 0x%x",
	    dp->name, __func__, ret));

	return (ret);
}

#if DEBUG_LEVEL > 0
static void
gani_eeprom_dump(struct gem_dev *dp, int size)
{
	int		i;
	uint16_t	x0, x1, x2, x3;

	cmn_err(CE_CONT, "!%s: eeprom dump:", dp->name);
	for (i = 0; i < size; i += 4) {
		x0 = gani_read_eeprom(dp, i + 0);
		x1 = gani_read_eeprom(dp, i + 1);
		x2 = gani_read_eeprom(dp, i + 2);
		x3 = gani_read_eeprom(dp, i + 3);
		cmn_err(CE_CONT,
		    "!0x%02x: %02x %02x %02x %02x %02x %02x %02x %02x",
		    i*2,
		    (uint8_t)x0, (uint8_t)(x0 >> 8),
		    (uint8_t)x1, (uint8_t)(x1 >> 8),
		    (uint8_t)x2, (uint8_t)(x2 >> 8),
		    (uint8_t)x3, (uint8_t)(x3 >> 8));
	}
}
#endif

static void
gani_fixup_params(struct gem_dev *dp)
{
	int	rxmaxsize;
	struct gani_dev	*lp = dp->private;

	/*
	 * Fix rx buffer length:
	 * It must have additional space depend on chip model,
	 * and its size must be multiple of 8.
	 */
	rxmaxsize = GANI_PCI(lp) ? RMS_PCI(dp->mtu)
	    : GANI_PCIE_GIGA(lp) ? RMS_PCIE_GIGA(dp->mtu)
	    : RMS_PCIE_FAST(lp->chip->type);
	dp->rx_buf_len = ROUNDUP2(rxmaxsize, 8);

#ifdef GEM_CONFIG_VLAN_HW
	/*
	 * RTL816x series will corrupt vlan-tagged outgoing packets
	 * when hw checksum is enabled and hw vlan tagging is disabled.
	 */
	dp->misc_flag |= GEM_VLAN_HARD;
#ifdef CONFIG_CKSUM_OFFLOAD
	lp->new_cksum = !(GANI_PCI(lp) ||
	    lp->chip->type == R8168_1 /* RTL8168B rev.B */ ||
	    lp->chip->type == R8168_2 /* RTL8168B rev.E */ ||
	    lp->chip->type == R8168_3 /* RTL8168B rev.F */ ||
	    lp->chip->type == R8101_1 /* RTL8101E rev.B */ ||
	    lp->chip->type == R8101_2 /* RTL8101E rev.E */ ||
	    lp->chip->type == R8101_3 /* RTL8191E rev.h */);

	if (GANI_PCIE_GIGA(lp) && dp->mtu > ETHERMTU) {
		/*
		 * checksum offload doesn't work for jumbo packets.
		 */
		dp->misc_flag &=
		    ~(GEM_CKSUM_FULL_IPv4 | GEM_CKSUM_HEADER_IPv4 | GEM_LSO);
	} else if (lp->chip->type == R8168_16 || lp->chip->type == R8168_17) {
		/*
		 * they have problems for short packets < 60 byte
		 */
		dp->misc_flag &=
		    ~(GEM_CKSUM_FULL_IPv4 | GEM_CKSUM_HEADER_IPv4 | GEM_LSO);
	} else {
		/* XXX - ok for 8169 and 8101 */
		dp->misc_flag |= GEM_CKSUM_FULL_IPv4 | GEM_CKSUM_HEADER_IPv4;
#ifdef CONFIG_LSO
		dp->misc_flag |= GEM_LSO;
#endif
	}
#endif /* CONFIG_CKSUM_OFFLOAD */
#else
	dp->misc_flag |= GEM_VLAN_SOFT;
#endif /* GEM_CONFIG_VLAN_HW */
}

static int
gani_attach_chip(struct gem_dev *dp)
{
	int		i;
	uint32_t	val;
	uint8_t		*m;
	struct gani_dev	*lp = dp->private;

	DPRINTF(0, (CE_CONT, "!%s: %s: called", dp->name, __func__));

	m = &dp->dev_addr.ether_addr_octet[0];
	if (GANI_PCIE_GIGA(lp) &&
	    (gani_read_eeprom(dp, 0) == 0x8128 ||
	    gani_read_eeprom(dp, 0) == 0x8129)) {
		/*
		 * read factory mac address from EEPROM
		 */
#if DEBUG_LEVEL > 10
		gani_eeprom_dump(dp, 0x40);
#endif
		for (i = 0; i < ETHERADDRL/2; i++) {
			val = gani_read_eeprom(dp, 7 + i);
			m[i*2 + 0] = (uint8_t)val;
			m[i*2 + 1] = (uint8_t)(val >> 8);
		}
	} else {
		/*
		 * no eeprom
		 * read factory mac address from IDR register
		 */
		for (i = 0; i < ETHERADDRL; i++) {
			m[i] = INB(dp, IDR + i);
		}
	}
	/* invalidate soft copy of mac registers */
	bzero(lp->mac, ETHERADDRL);

	/* no need to scan phy */
	dp->mii_phy_addr = -1;

#ifndef GEM_CONFIG_JUMBO_FRAME
	gani_fixup_params(dp);
#endif
	/* RTL816x series support only store & forword for tx and rx */
	dp->txthr = INT32_MAX;
	dp->rxthr = INT32_MAX;

	/*
	 * XXX - workaround for SiS965L
	 * txmaxdma and rxmaxdma must be unlimited otherwise the system
	 * will hang.
	 */
	if (GANI_PCIE_FAST(lp)) {
		/* force to use MXDMA_1024, i.e. 6 */
		dp->txmaxdma = max(dp->mtu, 1024);
	} else {
		/* force to use MXDMA_UNLIMIT, i.e. 7 */
		dp->txmaxdma = max(dp->mtu, 1024+1);
	}
	if (lp->chip->type == R8168_1 /* RTL8168B rev.B */) {
		dp->txmaxdma = min(dp->txmaxdma,  512);
	}

	dp->rxmaxdma = dp->mtu;

	/* Receive control register: write an initial value */
	val = 0;
	if (dp->rxmaxdma <= 64) {
		val |= RCR_MXDMA_64;
	} else if (dp->rxmaxdma <= 128) {
		val |= RCR_MXDMA_128;
	} else if (dp->rxmaxdma <= 256) {
		val |= RCR_MXDMA_256;
	} else if (dp->rxmaxdma <= 512) {
		val |= RCR_MXDMA_512;
	} else if (dp->rxmaxdma <= 1024) {
		val |= RCR_MXDMA_1024;
	} else {
		val |= RCR_MXDMA_UNLIMITED;
	}

	if (GANI_PCIE_FAST(lp)) {
		val |= RCR_RXFTH_SF;	/* reserved */
	} else if (GANI_PCIE_GIGA(lp)) {
		switch (lp->chip->type) {
		case R8168_1: /* RTL8168B rev.B  */
		case R8168_2: /* RTL8168B rev.E */
		case R8168_3: /* RTL8168B rev.F */
			val |= 7 << 13;	/* reserved */
			break;

		case R8168_4: /* RTL8168C */
		case R8168_5: /* RTL8168C rev.B */
		case R8168_6: /* RTL8168C rev.C */
		case R8168_7: /* RTL8168CP rev.B */
		case R8168_8: /* RTL8168CP rev.C */
			val |= RCR_128_INT | RCR_FET_MULTI;
			break;

		case R8168_9: /* RTL8168D */
		case R8168_10: /* RTL8168D rev.B */
		case R8168_11: /* RTL8168D rev.C */
		case R8168_12:
		case R8168_13:
		case R8168_14:
		case R8168_15:
		case R8168_16:
		case R8168_17:
		case R8168_18:
		case R8168_19:
		case R8168_20:
			val |= RCR_128_INT;
			break;

		case R8168_21:
		case R8168_22:
		case R8168_24:
			val |= RCR_128_INT | RCR_EARLY_OFF;
			break;

		case R8168_23:
			val |= 0x8f00;
			break;

		default:
			val |= RCR_128_INT;
			break;
		}
	} else if (dp->rxthr <= 64) {
		val |= RCR_RXFTH_64;
	} else if (dp->rxthr <= 128) {
		val |= RCR_RXFTH_128;
	} else if (dp->rxthr <= 256) {
		val |= RCR_RXFTH_256;
	} else if (dp->rxthr <= 512) {
		val |= RCR_RXFTH_512;
	} else if (dp->rxthr <= 1024) {
		val |= RCR_RXFTH_1024;
	} else {
		val |= RCR_RXFTH_SF;	/* store and forward */
	}

	/* keep undocumented bits in RCR  */
	lp->rcr = val | (INL(dp, RCR) & ~lp->chip->rcr_mask);
	DPRINTF(10, (CE_CONT, "!%s: %s: RCR_MASK:0x%x",
	    dp->name, __func__, ~lp->chip->rcr_mask));

	/* Transmit configration register : */
	val = 0;
	if (dp->txmaxdma <= 16) {
		val |= TCR_MXDMA_16;
	} else if (dp->txmaxdma <= 32) {
		val |= TCR_MXDMA_32;
	} else if (dp->txmaxdma <= 64) {
		val |= TCR_MXDMA_64;
	} else if (dp->txmaxdma <= 128) {
		val |= TCR_MXDMA_128;
	} else if (dp->txmaxdma <= 256) {
		val |= TCR_MXDMA_256;
	} else if (dp->txmaxdma <= 512) {
		val |= TCR_MXDMA_512;
	} else if (dp->txmaxdma <= 1024) {
		/* vendor choice for 8101 */
		val |= TCR_MXDMA_1024;
	} else {
		val |= TCR_MXDMA_UNLIMITED;
	}
	lp->tcr |= TCR_IFG_802_3 | TCR_LBK_NORMAL | val;

	mutex_init(&lp->stat_lock, NULL, MUTEX_DRIVER, dp->iblock_cookie);
	lp->stat_to_id = (timeout_id_t)0;

	if (GANI_PCIE_GIGA(lp)) {
		/* disable flow control by default for pci-e GbE */
		if (!ddi_prop_exists(DDI_DEV_T_ANY, dp->dip,
		    DDI_PROP_DONTPASS, "adv_pause")) {
			dp->anadv_pause = 0;
		}
		if (!ddi_prop_exists(DDI_DEV_T_ANY, dp->dip,
		    DDI_PROP_DONTPASS, "adv_asmpause")) {
			dp->anadv_asmpause = 0;
		}
	}

	return (GEM_SUCCESS);	/* currently return code is not used. */
}

static int
ganiattach(dev_info_t *dip, ddi_attach_cmd_t cmd)
{
	int			i;
	int			n;
	ddi_acc_handle_t	conf_ha;
	int			revid;
	int			unit;
	struct chip_info	*p;
	uint32_t		tcr;
	const char		*drv_name;
	struct gem_dev		*dp;
	caddr_t			base;
	ddi_acc_handle_t	regs_ha;
	struct gem_conf		*gcp;
	struct gani_dev		*lp;
	uint32_t		ilr;
	uint_t			pcie_cap;
	uint_t			msi_cap;
	uint16_t		vid;
	uint16_t		svid;
	uint16_t		did;
	uint16_t		ssid;

	unit = ddi_get_instance(dip);
	drv_name = ddi_driver_name(dip);
	DPRINTF(1, (CE_CONT, "!%s%d: %s: called", drv_name, unit, __func__));

	/*
	 * Common routine after power-on
	 */

	/* fix config registers */
	if (pci_config_setup(dip, &conf_ha) != DDI_SUCCESS) {
		cmn_err(CE_WARN, "!%s%d: pci_config_setup failed",
		    drv_name, unit);
		goto err;
	}

	pci_config_put16(conf_ha, PCI_CONF_COMM,
	    pci_config_get16(conf_ha, PCI_CONF_COMM)
	    | PCI_COMM_IO | PCI_COMM_MAE | PCI_COMM_ME);

	/* ensure D0 mode */
	(void) gem_pci_set_power_state(dip, conf_ha, PCI_PMCSR_D0);

	revid = pci_config_get8(conf_ha, PCI_CONF_REVID);
	ilr = pci_config_get32(conf_ha, PCI_CONF_ILINE);
	DPRINTF(0, (CE_CONT, "!%s%d: ilr 0x%08x", drv_name, unit, ilr));

	vid = pci_config_get16(conf_ha, PCI_CONF_VENID);
	did = pci_config_get16(conf_ha, PCI_CONF_DEVID);
	svid = pci_config_get16(conf_ha, PCI_CONF_SUBVENID);
	ssid = pci_config_get16(conf_ha, PCI_CONF_SUBSYSID);

	pcie_cap = gem_search_pci_cap(dip, conf_ha, PCI_CAP_ID_PCI_E);
	msi_cap = gem_search_pci_cap(dip, conf_ha, PCI_CAP_ID_MSI);
	pci_config_teardown(&conf_ha);

	switch (cmd) {
	case DDI_RESUME:
		dp = GEM_GET_DEV(dip);

		if ((i = gem_resume(dip)) == DDI_SUCCESS) {
			gani_stat_timeout(dp);
		}
		return (i);

	case DDI_ATTACH:
		/*
		 * Map in the device registers. Try to map register
		 * in pci memory space first.
		 *  rnumber 1: i/o
		 *  rnumber 2: mem32(pci) or mem64(pcie)
		 */
#ifdef MAP_MEM
		if (ddi_regs_map_setup(dip, 2, &base, 0, 0,
		    &gani_dev_attr, &regs_ha) != DDI_SUCCESS) {
#endif
			if (ddi_regs_map_setup(dip, 1, &base, 0, 0,
			    &gani_dev_attr, &regs_ha) != DDI_SUCCESS) {
				goto err;
			}
#ifdef MAP_MEM
		}
#endif

		/*
		 * Check hardware revision
		 */
		tcr = ddi_get32(regs_ha, (void *)(base + TCR));

		if (did == 0x8169 || did == 0x8167) {
			p = chiptbl_8169;
			n = ARRAY_SIZE(chiptbl_8169);
		} else if (did == 0x8168 || did == 0x4300 /* DLINK 4300 */) {
			p = chiptbl_8168;
			n = ARRAY_SIZE(chiptbl_8168);
		} else {
			p = chiptbl_8101;
			n = ARRAY_SIZE(chiptbl_8101);
		}
		for (i = 0; i < n; p++, i++) {
			if (((p->tcr_val ^ tcr) & p->tcr_mask) == 0) {
				/* found */
				goto chip_found;
			}
		}

		cmn_err(CE_WARN,
		    "!%s%d: attach: unknown mac version: tcr:0x%08x",
		    drv_name, unit, tcr);

		/* choose the latest chipset */
		p -= 1;

chip_found:
		cmn_err(CE_CONT,
		    "!%s%d: chip is %s rev:0x%02x tcr:0x%08x "
		    "pcie_cap:0x%02x msi_cap:0x%02x",
		    drv_name, unit, p->name, revid, tcr, pcie_cap, msi_cap);

		lp = kmem_zalloc(sizeof (struct gani_dev), KM_SLEEP);
		lp->chip = p;
		lp->pcie_cap = pcie_cap;
		lp->msi = msi_cap != 0;
		lp->isr_pended = 0;
#ifdef notdef
		lp->initialized = B_FALSE;
#endif
		lp->tcr = tcr & p->tcr_mask;
		lp->svid = svid;
		lp->ssid = ssid;

		/* read config2 register */
		if (GANI_PCIE(lp)) {
			lp->busclk = BUSCLK_PCIE; /* PCI-E (125MHz) */
		} else {
			lp->busclk = ddi_get8(regs_ha,
			    (uint8_t *)(base + CFG2)) & CFG2_PCICLKF
			    ? BUSCLK_PCI66 : BUSCLK_PCI33;
		}
		DPRINTF(0, (CE_CONT, "!%s%d: busclk:%d, cfg2:0x%x",
		    drv_name, unit, lp->busclk,
		    ddi_get8(regs_ha, (uint8_t *)(base + CFG2))));

		/*
		 * construct gem configration
		 */
		gcp = kmem_zalloc(sizeof (*gcp), KM_SLEEP);

		/* name */
		(void) sprintf(gcp->gc_name, "%s%d", drv_name, unit);

		gcp->gc_tx_buf_align = sizeof (uint8_t) - 1;
		gcp->gc_tx_max_frags = TX_MAX_FRAGS;
		gcp->gc_tx_max_descs_per_pkt = gcp->gc_tx_max_frags;
		gcp->gc_tx_desc_unit_shift = 4;
		gcp->gc_tx_buf_size = TX_BUF_SIZE;
		gcp->gc_tx_buf_limit = gcp->gc_tx_buf_size;
		gcp->gc_tx_ring_size = TX_RING_SIZE;
		gcp->gc_tx_ring_limit = gcp->gc_tx_ring_size;
		gcp->gc_tx_auto_pad = B_TRUE;
		gcp->gc_tx_copy_thresh = gani_tx_copy_thresh;

		gcp->gc_rx_buf_align = (GANI_PCIE(lp) ? 8 : 1) - 1;
		gcp->gc_rx_max_frags = 1;
		gcp->gc_rx_desc_unit_shift = 4;
		gcp->gc_rx_ring_size = RX_RING_SIZE;
		gcp->gc_rx_buf_max = RX_BUF_SIZE;
		gcp->gc_rx_copy_thresh = gani_rx_copy_thresh;
		gcp->gc_io_area_size = TC_SIZE;

		/* map attributes */
		gcp->gc_dev_attr = gani_dev_attr;
		gcp->gc_buf_attr = gani_buf_attr;
		gcp->gc_desc_attr = gani_buf_attr;

		/* dma attributes */
		gcp->gc_dma_attr_desc = gani_dma_attr_desc;
		if (gani_64bit_addr) {
			gcp->gc_dma_attr_txbuf = gani_dma_attr_buf64;
		} else {
			gcp->gc_dma_attr_txbuf = gani_dma_attr_buf32;
		}
		gcp->gc_dma_attr_txbuf.dma_attr_align = gcp->gc_tx_buf_align+1;
		gcp->gc_dma_attr_txbuf.dma_attr_sgllen = gcp->gc_tx_max_frags;

		if (gani_64bit_addr) {
			gcp->gc_dma_attr_rxbuf = gani_dma_attr_buf64;
		} else {
			gcp->gc_dma_attr_rxbuf = gani_dma_attr_buf32;
		}
		gcp->gc_dma_attr_rxbuf.dma_attr_align = gcp->gc_rx_buf_align+1;
		gcp->gc_dma_attr_rxbuf.dma_attr_sgllen = gcp->gc_rx_max_frags;

		/* time out parameters */
		gcp->gc_tx_timeout = GEM_TX_TIMEOUT;
		gcp->gc_tx_timeout_interval = ONESEC;

		/* flow control capability */
		/* XXX - rx hangs for 8168 when flow control is enabled */
		gcp->gc_flow_control = FLOW_CONTROL_NONE;

		/* MII timeout parameters */
		gcp->gc_mii_link_watch_interval = ONESEC;
		gcp->gc_mii_an_watch_interval = ONESEC/10;
		gcp->gc_mii_reset_timeout = 25*(ONESEC/10);
		gcp->gc_mii_an_timeout = MII_AN_TIMEOUT;
		gcp->gc_mii_an_wait = ONESEC;
		gcp->gc_mii_linkdown_timeout = MII_LINKDOWN_TIMEOUT;

		/* rtl8169 seems to delay to recognize PHY status */
		gcp->gc_mii_an_delay = ONESEC/10;	/* 100mS */
		/* we must reset rtl8169 phy to update advert register */
		gcp->gc_mii_linkdown_action = MII_ACTION_RESET;

		/*
		 * XXX - MAC_VER_8169S or later chipset requires reset
		 * on link down timeout for 1Gbps connection
		 */
		gcp->gc_mii_linkdown_timeout_action = MII_ACTION_RESET;
		gcp->gc_mii_dont_reset = B_FALSE;
		gcp->gc_mii_an_oneshot = B_FALSE;
		gcp->gc_mii_hw_link_detection = B_TRUE;

		/* I/O methods */

		/* mac operation */
		gcp->gc_attach_chip = &gani_attach_chip;
#ifdef GEM_CONFIG_JUMBO_FRAME
		gcp->gc_fixup_params = &gani_fixup_params;
#endif
		gcp->gc_reset_chip = &gani_reset_chip;
		gcp->gc_init_chip = &gani_init_chip;
		gcp->gc_start_chip = &gani_start_chip;
		gcp->gc_stop_chip = &gani_stop_chip;
		gcp->gc_multicast_hash = &gani_mcast_hash;
		gcp->gc_set_rx_filter = &gani_set_rx_filter;
		gcp->gc_set_media = &gani_set_media;
		gcp->gc_get_stats = &gani_get_stats;
		gcp->gc_interrupt = &gani_interrupt;

		/* descriptor operation */
		gcp->gc_tx_desc_write = &gani_tx_desc_write;
		gcp->gc_rx_desc_write = &gani_rx_desc_write;
		gcp->gc_tx_start = &gani_tx_start;
		gcp->gc_rx_start = NULL;
		gcp->gc_tx_desc_stat = &gani_tx_desc_stat;
		gcp->gc_rx_desc_stat = &gani_rx_desc_stat;
		gcp->gc_tx_desc_init = &gani_tx_desc_init;
		gcp->gc_rx_desc_init = &gani_rx_desc_init;
		gcp->gc_tx_desc_clean = &gani_tx_desc_init;
		gcp->gc_rx_desc_clean = &gani_rx_desc_init;

		/* mii operations */
		gcp->gc_mii_probe = &gani_mii_probe;
		gcp->gc_mii_init = &gani_mii_init;
		gcp->gc_mii_config = &gani_mii_config;
		gcp->gc_mii_sync = &gani_mii_sync;
		gcp->gc_mii_read = &gani_mii_read;
		gcp->gc_mii_write = &gani_mii_write;
		gcp->gc_mii_tune_phy = NULL;

		/* MSI/MSIX interrupts */
#ifdef CONFIG_INTR_MSI
		gcp->gc_nintrs_req = 1;
#else
		gcp->gc_nintrs_req = 0;
#endif

		/* offload and jumbo frame */
		gcp->gc_max_lso = 16 * 1024 - 1;
		gcp->gc_max_mtu = p->max_mtu;
		gcp->gc_default_mtu = ETHERMTU;
		gcp->gc_min_mtu = ETHERMIN - sizeof (struct ether_header);

		dp = gem_do_attach(dip, 0, gcp, base, &regs_ha,
		    lp, sizeof (struct gani_dev));
		kmem_free(gcp, sizeof (*gcp));

		if (dp == NULL) {
			goto err_free_mem;
		}

		gani_stat_timeout(dp);

		return (DDI_SUCCESS);

err_free_mem:
		kmem_free(lp, sizeof (struct gani_dev));
err:
		return (DDI_FAILURE);
	}

	return (DDI_FAILURE);
}

static int
ganidetach(dev_info_t *dip, ddi_detach_cmd_t cmd)
{
	struct gem_dev  *dp;
	struct gani_dev	*lp;

	dp = GEM_GET_DEV(dip);
	lp = dp->private;

	switch (cmd) {
	case DDI_DETACH:
	case DDI_SUSPEND:
		/* Ensure statistics timer routine stopped */
		mutex_enter(&lp->stat_lock);
		if (lp->stat_to_id != 0) {
			timeout_id_t	old_id;

			do {
				old_id = lp->stat_to_id;
				mutex_exit(&lp->stat_lock);
				(void) untimeout(old_id);
				mutex_enter(&lp->stat_lock);
			} while (old_id != lp->stat_to_id);
			lp->stat_to_id = 0;
		}
		mutex_exit(&lp->stat_lock);

		switch (cmd) {
		case DDI_DETACH:
			mutex_destroy(&lp->stat_lock);
			return (gem_do_detach(dip));

		case DDI_SUSPEND:
			return (gem_suspend(dip));
		}
	}

	return (DDI_FAILURE);
}

/* ======================================================== */
/*
 * OS depend (loadable streams driver) routine
 */
/* ======================================================== */
#ifdef GEM_CONFIG_GLDv3
GEM_STREAM_OPS(gani_ops, ganiattach, ganidetach);
#else
static	struct module_info ganiminfo = {
	0,			/* mi_idnum */
	"gani",			/* mi_idname */
	0,			/* mi_minpsz */
	INFPSZ,			/* mi_maxpsz */
	64*1024,		/* mi_hiwat */
	1,			/* mi_lowat */
};

static	struct qinit ganirinit = {
	(int (*)()) NULL,	/* qi_putp */
	gem_rsrv,		/* qi_srvp */
	gem_open,		/* qi_qopen */
	gem_close,		/* qi_qclose */
	(int (*)()) NULL,	/* qi_qadmin */
	&ganiminfo,		/* qi_minfo */
	NULL			/* qi_mstat */
};

static	struct qinit ganiwinit = {
	gem_wput,		/* qi_putp */
	gem_wsrv,		/* qi_srvp */
	(int (*)()) NULL,	/* qi_qopen */
	(int (*)()) NULL,	/* qi_qclose */
	(int (*)()) NULL,	/* qi_qadmin */
	&ganiminfo,		/* qi_minfo */
	NULL			/* qi_mstat */
};

static struct streamtab	gani_info = {
	&ganirinit,	/* st_rdinit */
	&ganiwinit,	/* st_wrinit */
	NULL,		/* st_muxrinit */
	NULL		/* st_muxwrinit */
};

static	struct cb_ops cb_gani_ops = {
	nulldev,	/* cb_open */
	nulldev,	/* cb_close */
	nodev,		/* cb_strategy */
	nodev,		/* cb_print */
	nodev,		/* cb_dump */
	nodev,		/* cb_read */
	nodev,		/* cb_write */
	nodev,		/* cb_ioctl */
	nodev,		/* cb_devmap */
	nodev,		/* cb_mmap */
	nodev,		/* cb_segmap */
	nochpoll,	/* cb_chpoll */
	ddi_prop_op,	/* cb_prop_op */
	&gani_info,	/* cb_stream */
	D_MP,		/* cb_flag */
};

static	struct dev_ops gani_ops = {
	DEVO_REV,	/* devo_rev */
	0,		/* devo_refcnt */
	gem_getinfo,	/* devo_getinfo */
	nulldev,	/* devo_identify */
	nulldev,	/* devo_probe */
	ganiattach,	/* devo_attach */
	ganidetach,	/* devo_detach */
	nodev,		/* devo_reset */
	&cb_gani_ops,	/* devo_cb_ops */
	NULL,		/* devo_bus_ops */
	gem_power,	/* devo_power */
#if DEVO_REV >= 4
	gem_quiesce,	/* devo_quiesce */
#endif
};
#endif /* GEM_CONFIG_GLDv3 */

static struct modldrv modldrv = {
	&mod_driverops,	/* Type of module.  This one is a driver */
	ident,
	&gani_ops,	/* driver ops */
};

static struct modlinkage modlinkage = {
	MODREV_1, &modldrv, NULL
};

/* ======================================================== */
/*
 * _init : done
 */
/* ======================================================== */
int
_init(void)
{
	int	status;

	DPRINTF(2, (CE_CONT, "!gani: _init: called"));

	status = gem_mod_init(&gani_ops, "gani");
	if (status != DDI_SUCCESS) {
		return (status);
	}
	status = mod_install(&modlinkage);
	if (status != DDI_SUCCESS) {
		gem_mod_fini(&gani_ops);
	}
	return (status);
}

/*
 * _fini : done
 */
int
_fini(void)
{
	int	status;

	DPRINTF(2, (CE_CONT, "!gani: _fini: called"));
	status = mod_remove(&modlinkage);
	if (status == DDI_SUCCESS) {
		gem_mod_fini(&gani_ops);
	}
	return (status);
}

int
_info(struct modinfo *modinfop)
{
	return (mod_info(&modlinkage, modinfop));
}
